/******************************************************************************
* Filename: Combatant.cpp
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/
#include "..\Shared\stdafx.h"

//#include "externs.h"
#include "Dungeon.h"

#include "char.h"
#include "GlobalData.h"
#include "party.h"
#include "DispText.h"
#include "Combatant.h"
#include "Combatants.h"
#include "DrawTile.h"
#include "Viewport.h"
#include "Screen.h"
#include "GameMenu.h"
#include "FormattedText.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char *CombatantStateText[NUM_COMBATANT_STATES] = 
{
  "ICS_None",
  "ICS_Casting",
  "ICS_Attacking",
  "ICS_Guarding",
  "ICS_Bandaging",
  "ICS_Using",
  "ICS_Moving",
  "ICS_Turning",
  "ICS_Fleeing",
  "ICS_Fled",
};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
//
//  COMBATANT functions
//
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

//*****************************************************************************
//    NAME: COMBATANT::Clear
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::Clear()
{
  CHARACTER::Clear();
  self=-1;
  x=-1; y=-1;
  hPath=-1;
  initiative=0;
  target=-1;
  m_iFacing=0;
  movement=0;
  moveX=-1;moveY=-1;
  lastAttackRound=0;
  availAttacks=0.0;
  continueAttack=FALSE;
  turnIsDone=FALSE;
  isBandaged=FALSE;
  bandageWho=-1;
  origKey=-1;
  didMove=FALSE;
  state=ICS_None;
  friendly=TRUE;
  blitDeathTile=FALSE;
  width=0;height=0;
  targetPos=NULL;  
  m_iTmpAttDiceBonus=0;
  m_iTmpDmgDiceBonus=0;
  //partyMember=FALSE;
  iFleeingFlags = 0;
  isTurned=FALSE;
  hasTurnedUndead=FALSE;
  m_iLastAttacker=-1;
  m_iLastAttacked=-1;
  m_eLastAction = LA_None;
  RemoveAllTargets();
  targeters.RemoveAll();
  m_spellKeyBeingCast.Clear();
  pendingSpellKey=-1;
  activeSpellKey=-1;
  forceAttackPose=FALSE;
  m_iAttDiceSides=0;
  m_iAttDiceNum=0;
  m_iAttDiceRoll=0;
  m_isCombatReady = -1;
}

//*****************************************************************************
//    NAME: COMBATANT::operator=
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT& COMBATANT::operator=(COMBATANT& src)
{
  if (&src == this)
    return *this;

  CHARACTER::operator =(src);
  self = src.self;
	x = src.x;
  y = src.y;
  hPath = src.hPath;
	initiative = src.initiative; // 1..10
	target = src.target; // index of target
	m_iFacing = src.m_iFacing;
  movement = src.movement;
  moveX = src.moveX;
  moveY = src.moveY;
  lastAttackRound = src.lastAttackRound;
  availAttacks = src.availAttacks;
  iFleeingFlags = src.iFleeingFlags;
  isTurned=src.isTurned;
  hasTurnedUndead=src.hasTurnedUndead;
  m_iLastAttacker=src.m_iLastAttacker;
  m_iLastAttacked=src.m_iLastAttacked;
  m_eLastAction = src.m_eLastAction;
	turnIsDone = src.turnIsDone;
  isBandaged = src.isBandaged;
  bandageWho = src.bandageWho;
  origKey=src.origKey;
  didMove = src.didMove;
  state = src.state;
	friendly = src.friendly;
  blitDeathTile = src.blitDeathTile;
  width = src.width;
  height = src.height;
  targetPos = src.targetPos;
  //partyMember = src.partyMember;
  m_iTmpAttDiceBonus = src.m_iTmpAttDiceBonus;
  m_iTmpDmgDiceBonus = src.m_iTmpDmgDiceBonus;
  continueAttack=src.continueAttack;
  m_spellKeyBeingCast=src.m_spellKeyBeingCast;
  pendingSpellKey=src.pendingSpellKey;
  activeSpellKey=src.activeSpellKey;
  forceAttackPose=src.forceAttackPose;
  m_iAttDiceSides=src.m_iAttDiceSides;
  m_iAttDiceNum=src.m_iAttDiceNum;
  m_iAttDiceRoll=src.m_iAttDiceRoll;
  m_isCombatReady=src.m_isCombatReady;


  combattargets.RemoveAll();
  POSITION pos = src.combattargets.GetHeadPosition();
  while (pos != NULL)
  {
    DWORD data = src.combattargets.GetNext(pos);
    int dude = GetTargetKeyDude(data);
    combattargets.Insert(dude, data);
  }

  combatantSA.Copy(src.combatantSA);
  //targeters is a temp list that should not need to be copied

  return *this;
}

//*****************************************************************************
//    NAME: COMBATANT::GetContext
//
// PURPOSE: 
//
//*****************************************************************************
ActorType COMBATANT::GetContext(WORD ClassFlag)
{
  ActorType data;
  data.Clear();

  if (IsPartyMember())
  {
    data.SetCombatPartySrc(friendly);
  }
  else
  {
    if (GetType() == MONSTER_TYPE)
      data.SetCombatMonsterSrc(friendly);
    else if (GetType() == NPC_TYPE)
      data.SetCombatNPCSrc(friendly);
  }  

  // This character will not always have a
  // level value for ClassFlag. For instance,
  // a cleric casts a spell on a fighter. The
  // ClassFlag will be ClericFlag, and the fighter's
  // target context will have a zero cleric
  // level.
  data.Level = GetCurrentLevel(ClassFlag);
  ASSERT( (data.Level>=0) && (data.Level<=40) );

  data.Race = race();
  data.Class = GetClass();  
  data.Instance = self;

  return data;
}

//*****************************************************************************
// NAME: COMBATANT::CanCast
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanCast()
{
  if (GetConfigNoMagic()) return FALSE;
  if ((!IsPartyMember()) && (GetConfigMonsterNoCast())) return FALSE;
  if (IsDone()) return FALSE;
  return canCastSpells();
}

//*****************************************************************************
//    NAME: COMBATANT::IsCasting
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::IsCasting()
{
  return (state==ICS_Casting);
}
//*****************************************************************************
//    NAME: COMBATANT::IsSpellPending
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::IsSpellPending() 
{ 
  return ((IsCasting()) && (pendingSpellKey>=0)); 
}

//*****************************************************************************
//    NAME: COMBATANT::StartInitialSpellCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::StartInitialSpellCasting(GLOBAL_SPELL_ID skey)
{
  WriteDebugString("StartInitialSpellCasting for char %i, sbkey %i, turn %i, round %i, initiative %i\n", 
                    self,skey, combatData.m_iCurrTurn, combatData.m_iCurrRound, combatData.m_iCurrInitiative);
  forceAttackPose=FALSE;
  activeSpellKey=-1;
  targets.Clear();
  SetSpellKeyBeingCast(skey);
  state = ICS_Casting;  
  if (!CastSpell(GetSpellBookKey(skey, FALSE)))
  {
    WriteDebugString("StartInitialSpellCasting: CastSpell() FAILED for %i, sbkey %i\n", self,skey);
    m_spellKeyBeingCast.Clear();
    EndTurn();
  }
}
//*****************************************************************************
//    NAME: COMBATANT::EndInitialSpellCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::EndInitialSpellCasting()
{
  WriteDebugString("EndInitialSpellCasting() for %i\n", self);
  EndTurn(ICS_Casting);
}

//*****************************************************************************
//    NAME: COMBATANT::StopCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::StopCasting(BOOL EndYourTurn)
{
  //WriteDebugString("StopCasting() for %i, EndYourTurn %i\n", self,EndYourTurn);
  targets.Clear();  
  m_spellKeyBeingCast.Clear();
  
  if (state == ICS_Casting)
    state = ICS_None;

  if (pendingSpellKey >= 0)
  {
    pendingSpellData.Remove(pendingSpellKey);
    pendingSpellKey=-1;
  }
  activeSpellKey=-1;
  forceAttackPose=FALSE;
  if (EndYourTurn)
    EndTurn(state);
}



// queue up this spell on the pending list
//*****************************************************************************
//    NAME: COMBATANT::CastSpell
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::CastSpell(GLOBAL_SPELL_ID spell)
{ 
  WriteDebugString("CastSpell() for %i, sbkey %i\n", self,spell);
  ASSERT (IsCombatActive() == TRUE);

  CHARACTER_SPELL cspell;
  if (!m_spellbook.GetSpell(spell, cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;

  m_spellbook.DecMemorized(spell, 1);

  int rnd = CurrCombatRound();

  PENDING_SPELL data;
  data.caster = GetContext();
  data.gsID = spell;
  data.waitUntil = max(rnd-1, 0);
  data.flags = pSdata->Casting_Time_Type;

  // spellCastingTimeType { sctImmediate, sctInitiative, sctRounds, sctTurns }
  // 
  // long Casting_Time;
  // spellCastingTimeType Casting_Time_Type;
  //
  // Casting Time (non-combat):
  //   
  //   not used, spells are activated immediately
  //  
  // Casting Time (combat):
  //  
  //   Each round has 10 initiatives
  //   1 round = 1 minute
  //   10 rounds = 1 Turn
  //   Spells requiring 1 or more rounds/turns always get cast at the end of a round/turn
  //   Any hit on caster during casting time voids the spell.
  //
  //   sctImmediate:  Spell activated immediately
  //   sctInitiative: Casting_Time is added to current initiative. Spell is activated
  //                  when initiative time is reached. Cannot go past current round.
  //   sctRounds:     Casting_Time is added to current round. Spell is activated
  //                  when result round is reached, at end of round.
  //   sctTurns:      Casting_Time*10 is added to current round. Spell is activated
  //                  when result turn is reached, at end of that round.
  //

  pSdata->Casting_Time = max(0, pSdata->Casting_Time);
  switch (pSdata->Casting_Time_Type)
  {
  case sctImmediate:
    data.waitUntil = initiative; // immediate activation
    break;
  case sctInitiative:
    data.waitUntil = initiative+pSdata->Casting_Time;
    if (data.waitUntil > 10) 
    {
      data.flags = sctRounds;
      data.waitUntil += (rnd+1); // wait until next round
    }
    if (data.waitUntil == (DWORD)initiative)
    {
      data.flags = sctImmediate;
      data.waitUntil = initiative;
    }
    break;
  case sctRounds:
    data.waitUntil = (rnd+pSdata->Casting_Time);
    if (data.waitUntil == (DWORD)rnd)
    {
      data.flags = sctImmediate;
      data.waitUntil = initiative;
    }
    break;
  case sctTurns:
    data.waitUntil = (rnd+(pSdata->Casting_Time*10));
    if (data.waitUntil == (DWORD)rnd)
    {
      data.flags = sctImmediate;
      data.waitUntil = initiative;
    }
    break;
  default:
    ASSERT(FALSE);
    data.flags = sctImmediate;
    data.waitUntil = initiative;
    break;
  }

  if (pSdata->Casting_Time_Type!=sctImmediate)
  {
    WriteDebugString("CastSpell() add pending spell for %i, sdbkey %i\n", self,data.gsID);
    pendingSpellKey = pendingSpellData.Add(data);
  }
  else
    pendingSpellKey=-1;
  return TRUE;
}


// pending spell is ready for activation
//*****************************************************************************
//    NAME: COMBATANT::SpellActivate
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::SpellActivate(PENDING_SPELL &data)
{
  WriteDebugString("SpellActivate() for %i, sdbkey %i\n", self,data.gsID);
  ASSERT(state==ICS_Casting);
  turnIsDone=false;
  QueuedCombatantData &qcomb = GetQueuedCombatants();
  qcomb.PushTail(self, TRUE);
  // no longer valid
  pendingSpellKey=-1;
}

// doesn't come from spell book
//*****************************************************************************
//    NAME: COMBATANT::InstantSpellActivate
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InstantSpellActivate(GLOBAL_SPELL_ID spelldbkey, int target)
{
  if (!spelldbkey.IsValidSpell()) return;
  if (target == NO_DUDE) return;

  if (GetType() != MONSTER_TYPE)
  {
    ASSERT(FALSE);
    return;
  }

  COMBATANT *pTarget = GetCombatantPtr(target);
  if (pTarget==NULL)
  {
    ASSERT(FALSE);
    return;
  }
    
  SPELL_DATA *pSdata = spellData.GetSpellData(spelldbkey);
  if (pSdata==NULL) 
  {
    ASSERT(FALSE);
    return;
  } 

  // PRS 19 Dec 2009
  //m_spellKeyBeingCast = GetSpellBookKey(spelldbkey, FALSE);
  m_spellKeyBeingCast = spelldbkey;

  InvokeSpellOnCombatant(self, target, -1);
}

//*****************************************************************************
// NAME: COMBATANT::UpdateCombatant
//
// PURPOSE:  
//*****************************************************************************
void /* PRS July 2009 BOOL */ COMBATANT::UpdateCombatant()
{
  /* this will fill up the log window if used
  WriteDebugString("updating %s combatant %u (%s)\n", 
                   OnAuto() ? "auto" : "manual",
                   self, 
                   CombatantStateText[state]);
  */

  individualCombatantState oldState = state;

  if (OnAuto())
  {
    if (!Think()) 
      return; // PRS July 2009 FALSE; 
  }

  //HandleCurrState();

  if ((state != ICS_None)&&(state!=oldState))
  {
    TRACE("done updating %s combatant %u (%s)\n", 
                     OnAuto() ? "auto" : "manual",
                     self, 
                     CombatantStateText[state]);
  }
  return; // PRS July 2009TRUE;
}

//*****************************************************************************
// NAME:    COMBATANT::Think
//
// PURPOSE: 
//*****************************************************************************
// 
// can return one of these states
/*
  CS_Moving
  CS_Guarding
  CS_Attacking
*/
BOOL COMBATANT::Think()
{
  // no user input allowed while dude is on auto
  if (IsPartyMember())
    EnableUserInput(TRUE);
  else
    EnableUserInput(FALSE);

  TRACE("thinking for auto combatant %u (%s)\n", self, CombatantStateText[state]);

  // make sure current path has not been exhausted
  if (hPath >= 0)
  {
    if (pathMgr.GetStepsRemaining(hPath)==0)
      ClearPath();
  }  

  if (iFleeingFlags != 0)
  {
    state = ICS_Moving;
    TRACE("thinking result for auto combatant %u (%s=fleeing)\n", self, CombatantStateText[state]);
    // find path to closest map edge

    BOOL fled=FALSE;
    // if already on map edge, flee off the map
    if (   (x==0) || (x==(MAX_TERRAIN_WIDTH-1))
        || (y==0) || (y==(MAX_TERRAIN_HEIGHT-1)) )
    {
      fled=TRUE;
    }

    if (fled) 
    {
      if (friendly)
        IncNumFriendFlee(); //combatData.numFrndFlee++;
      else
        IncNumMonsterFlee(); //combatData.numMonFlee++;

      menu.setMenu(EmptyMenuData); // starts empty
      menu.setHorzOrient();
      menu.setUseActive(FALSE); // no highlighting      

      CString msg("");
      msg.Format("%s has fled...", GetName());
      menu.changeMenuItem(1, msg);
      UpdateCombatTimeDelayMsg();
      theApp.AppSleep(GetMsgTimeDelay()); // give time to see normal icon      

      EndTurn();
      SetStatus(Fled);
      placeCombatant(x, y, NO_DUDE, width, height);
      TRACE("combatant %i has fled the map\n", self);
      return TRUE;
    }
    if (CanMove())
    {
      if (hPath < 0)
      {
        // find path to farthest corner of room away from cleric that turned me
        if (!FindPathAwayFrom(m_iLastAttacker))    
        {
          WriteDebugString("Fleeing combatant %i failed to find exit path\n", self);
          state = ICS_Guarding;
        }
      }
    }
    else
      state = ICS_Guarding;    
    return TRUE;
  }

  if (isTurned)
  {
    state = ICS_Moving;
    TRACE("thinking result for auto combatant %u (%s=turned/fleeing %i)\n", 
                     self, CombatantStateText[state], m_iLastAttacker);

    BOOL fled=FALSE;
    // if already on map edge, flee off the map
    if (   (x==0) || (x==(MAX_TERRAIN_WIDTH-1))
        || (y==0) || (y==(MAX_TERRAIN_HEIGHT-1)) )
    {
      fled=TRUE;
    }

    if (fled) 
    {
      if (friendly)
        IncNumFriendFlee(); //combatData.numFrndFlee++;
      else
        IncNumMonsterFlee(); //combatData.numMonFlee++;

      menu.setMenu(EmptyMenuData); // starts empty
      menu.setHorzOrient();
      menu.setUseActive(FALSE); // no highlighting      

      CString msg("");
      msg.Format("%s has fled...", GetName());
      menu.changeMenuItem(1, msg);
      UpdateCombatTimeDelayMsg();
      theApp.AppSleep(GetMsgTimeDelay()); // give time to see normal icon      

      // BUGFIX EndTurn(ICS_Fled);
      SetStatus(Fled);
      state = ICS_Fled; // BUGFIX
      placeCombatant(x, y, NO_DUDE, width, height);
      TRACE("Turned Undead %i has fled the map\n", self);
      return TRUE;
    }
    if (CanMove())
    {
      if (hPath < 0)
      {
        // find path to farthest corner of room away from cleric that turned me
        if (!FindPathAwayFrom(m_iLastAttacker))    
        {
          WriteDebugString("Fleeing combatant %i failed to find exit path\n", self);
          state = ICS_Guarding;
        }
      }
    }
    else
      state = ICS_Guarding;
    return TRUE;
  }

  // make sure current target is still on the map
  int dude = GetCurrTarget();
  COMBATANT *tempCOMBATANT=NULL;
  if (dude != NO_DUDE)
  {
    tempCOMBATANT = GetCombatantPtr(dude);
    ASSERT(tempCOMBATANT!=NULL);
    if (tempCOMBATANT==NULL) return FALSE;

    if (!canAttack(dude))
    {
      // target is not attackable for some reason,
      // so force new target acquisition
      dude = NO_DUDE;
    }
  }

  if (   (dude == NO_DUDE) 
      || ((tempCOMBATANT != NULL) && (!tempCOMBATANT->charOnCombatMap())))
  {
    // if no target or it's gone, acquire new target (if any)
    RemoveAllTargets();

    int i;
    // try for combat targets with line of sight first
    // combat targets are ordered by distance, but the closest
    // target in terms of distance may be on the other side
    // of a wall. Using line of sight helps to ensure we will attack
    // closest target that is also the shortest path distance.
    for (i=0; i < GetNumCombatants()/*combatData.numCombatants*/; i++)
    {
      tempCOMBATANT = GetCombatantPtr(i);
      ASSERT(tempCOMBATANT!=NULL);
      if (tempCOMBATANT==NULL) return FALSE;
      if (   (i != self) 
          && (tempCOMBATANT->friendly != GetIsFriendly())
          && (tempCOMBATANT->charOnCombatMap()))
      {
        if (HaveLineOfSight(GetCenterX(),GetCenterY(), tempCOMBATANT->GetCenterX(), tempCOMBATANT->GetCenterY()))
          AddTarget(i);
      }
    }

    if (GetCurrTarget() == NO_DUDE)
    {
      // no combat targets within line of sight
      // so use any avail target
      for (i=0; i < GetNumCombatants()/*combatData.numCombatants*/; i++)
      {
        tempCOMBATANT = GetCombatantPtr(i);
        ASSERT(tempCOMBATANT!=NULL);
        if (tempCOMBATANT==NULL) return FALSE;
        if (   (i != self) 
            && (tempCOMBATANT->friendly != GetIsFriendly())
            && (tempCOMBATANT->charOnCombatMap()))
          AddTarget(i);
      }
    }
  }  

  // might use current target when determining best
  // items to make ready.
  ReadyBestArmor();
  ReadyBestShield();
  ReadyBestWpn(GetCurrTarget());

  dude = GetCurrTarget();

  // no combattargets, sit tight and guard
  if (dude == NO_DUDE)
  {
    state = ICS_Guarding;
    TRACE("thinking result for auto combatant %u (%s) - no targets\n", self, CombatantStateText[state]);
    return TRUE;
  }
  
  BOOL CanAttack = canAttack(dude);
  tempCOMBATANT = GetCombatantPtr(dude);
  ASSERT(tempCOMBATANT!=NULL);
  if (tempCOMBATANT==NULL) return FALSE;
  BOOL found = FALSE;

  // no need to find a path if we can attack from current spot
  if (!CanAttack)
  {    
    BOOL repath = TRUE;

    // check to see if existing path to target needs to change
    CPathFinder* pathPtr = pathMgr.GetPath(hPath);

    if (pathPtr != NULL) 
    {
      CPoint stepPtr = pathPtr->GetLastStep();    

      if (   (tempCOMBATANT->x == stepPtr.x)
          && (tempCOMBATANT->y == stepPtr.y))
      {
        // use same path to target
        repath = FALSE;
        found = TRUE;
        TRACE("Using same path for combatant %i to target %i\n", self, dude);
      }
      else
        TRACE("Existing path for combatant %i does not end at curr target, re-pathing\n", self);
    }        

    // find path to closest enemy
    // need to change to closest enemy with shortest path!
    if ((repath) && (CanMove()))
    {
      ClearPath();
      SetCurrTarget(); // setup for iteration
      int targetCount = 0;
      dude = GetCurrTarget();

      while (   (!found) 
             && (dude != NO_DUDE) 
             && (targetCount < GetNumTargets()))
      {
        tempCOMBATANT = GetCombatantPtr(dude);
        ASSERT(tempCOMBATANT!=NULL);
        if (tempCOMBATANT==NULL) return FALSE;
        if (FindPathTo(tempCOMBATANT->x,
                       tempCOMBATANT->y, TRUE))
        {
          found = TRUE;
          RemoveAllTargets();
          AddTarget(dude);
        }
        else
        {
          dude = GetNextTarget();
        }
        targetCount++;
      }
    }
  }

  if (!found) // do we have a path to take?
  {
    if (GetCurrTarget() == NO_DUDE)
    {
      state = ICS_Guarding;
    }
    else
    {
      if (CanAttack)
      {
        state = ICS_Attacking;
        GLOBAL_ITEM_ID giID;
        int itemIdx;
        itemIdx = myItems.GetReady(WeaponHand);
        if (itemIdx != NO_READY_ITEM)
        {
          giID = myItems.GetItem(itemIdx);
          weaponClassType weaponType;
          weaponType = itemData.GetWpnType(giID);
          if (weaponType == SpellCaster)
          {
            if (myItems.GetCharges(itemIdx) > 0)
            {
              ITEM_DATA *pItem;
              GLOBAL_SPELL_ID gsID;
              pItem = itemData.GetItemData(giID);
              gsID = pItem->m_gsID;
              if (gsID.IsValidSpell())
              {
                POSITION pos;
                pos = combattargets.GetHeadPosition();
                if (pos != NULL)
                {
                  SPELL_DATA *pSpData;
                  COMBATANT *pTarget;
                  CHARACTER_SPELL newSpell;
                  DWORD target;
                  PATH_DIR dir;
                  target = combattargets.GetAt(pos);
                  pTarget = GetCombatantPtr(dude);
                  combattargets.RemoveAll();
                  pSpData = spellData.GetSpellData(gsID);
                  DWORD targs = EvalDiceAsClass(pSpData->NumTargets, pSpData->ClassMask);
                  DWORD range = EvalDiceAsClass(pSpData->TargetRange, pSpData->ClassMask);
                  InitTargeting(pSpData->Targeting,
                                pSpData->Friendly_Fire_Ok,
                                FALSE,
                                targs,
                                range);
                  this->m_spellKeyBeingCast = gsID;
                  dir = GetSpellTargetingDir(GetCenterX(),GetCenterY(), pTarget->x,pTarget->y);
                  AddMapTarget(pTarget->x, pTarget->y, dir);
                  state = ICS_Casting;
                  this->m_spellbook.Clear();
                  this->m_spellbook.spellLimits.UseLimits = FALSE;
                  this->m_spellbook.KnowSpell(MagicUserFlag,MagicUserFlag,gsID,1,TRUE);

                  myItems.SetCharges(itemIdx, myItems.GetCharges(itemIdx) - 1);
                  if (myItems.GetCharges(itemIdx) <= 0)
                  {
                    myItems.DeleteItem(itemIdx);
                  };
                };
              };
            }
            else
            {
              myItems.DeleteItem(itemIdx);
              state = ICS_Guarding;
            };
          };
        };
      }
      else
        state = ICS_Guarding;
    }
  }
  else
    state = ICS_Moving;

  TRACE("thinking result for auto combatant %u (%s)\n", self, CombatantStateText[state]);
  return TRUE;
}


//*****************************************************************************
// NAME: COMBATANT::CheckMorale
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::CheckMorale()
{
  if ((iFleeingFlags != 0)||isTurned)
    return;

  // monsters with no INT never flee
  if ((GetType() == MONSTER_TYPE) && (GetAdjInt() <= 1))
    return;

  static int fflee = 0;
  static int fslain =0;
  static int mflee = 0;
  static int mslain = 0;
  static BOOL SuperiorForceModMade = FALSE;

  int mod = 0;
  int numParty, numMons;
  BOOL Flee = FALSE;

  GetNumCombatants(&numParty, &numMons);
  //combatData.getNumCombatants(numParty, numMons);

  if (friendly)
  {
    if (fflee != GetNumFriendFlee() /*combatData.numFrndFlee*/)
      mod += ((GetNumFriendFlee()-fflee) * 5);
    if (fslain != GetNumFriendSlain())
      mod += ((GetNumFriendSlain()-fslain) * 10);
    if (((numParty * 3) <= numMons) && (!SuperiorForceModMade))
    {
      mod += 20;
      SuperiorForceModMade = TRUE;
    }
  }
  else
  {
    if (mflee != GetNumMonsterFlee()/*combatData.numMonFlee*/)
      mod += ((GetNumMonsterFlee()-mflee) * 5);
    if (mslain != GetNumMonsterSlain())
      mod += ((GetNumMonsterSlain()-mslain) * 10);
    if (((numMons * 3) <= numParty) && (!SuperiorForceModMade))
    {
      mod += 20;
      SuperiorForceModMade = TRUE;
    }
  }

  SetMorale( GetMorale()-mod );

  //int cur_morale = GetAdjMorale();
  Flee = FALSE; //(rollDice(100, 1, 0) > cur_morale);

  if (Flee)
  { 
    // only auto combatants choose to flee
    if (OnAuto())
    {      
      if (!IsPartyMember())
      {
        if (!GetConfigMonsterNoFlee())
          iFleeingFlags |= fleeBecauseImpossible;
      }
      else
        iFleeingFlags |= fleeBecauseImpossible;
    }
  }

  if (iFleeingFlags != 0)
  {
    ClearPath();
    RemoveAllTargets();
    SetStatus(Running);
    state = ICS_Moving;
    // stop any casting in progress
    StopCasting(FALSE);
  }

  fflee = GetNumFriendFlee();
  fslain = GetNumFriendSlain();
  mflee = GetNumMonsterFlee();
  mslain = GetNumMonsterSlain();
}

//*****************************************************************************
// NAME: COMBATANT::toggleReadyItem
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::toggleReadyItem(int item)
{  
  // nbr attacks remaining
  double attacksRemaining = availAttacks;

  // nbr attacks with current weapon
  determineAvailAttacks();
  double oldAttacks = availAttacks;

  // how many attacks made prior to changing to new item
  double attacksUsed = oldAttacks-attacksRemaining;

  if (CHARACTER::toggleReadyItem(item))
  {
    determineAvailAttacks();
    availAttacks -= attacksUsed;
    if (availAttacks < 0.0) availAttacks = 0.0;
    return TRUE;
  }
  else // nothing changed, restore original data
    availAttacks = attacksRemaining;

  return FALSE;
}

//*****************************************************************************
//    NAME: COMBATANT::delCharacterItem
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::delCharacterItem(int index, int qty)
{
  if (CHARACTER::delCharacterItem(index, qty))
  {
    determineAvailAttacks();
    return TRUE;
  }
  return FALSE;
}

//*****************************************************************************
//    NAME: COMBATANT::CurrMoveWouldFleeMap
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::CurrMoveWouldFleeMap()
{
  if ((moveX>=0)&&(moveY>=0))
    return (coordsOnMap(moveX, moveY, width, height) == FALSE);
  return FALSE;
}

//*****************************************************************************
//    NAME: COMBATANT::HandleCurrState
//
// PURPOSE: 
//
// Returns non-zero if scrren needs to be redrawn
//
//*****************************************************************************
int COMBATANT::HandleCurrState()
{
  int dude;
  int updateScreen = 0;
  switch (state)
  {
  case ICS_None:
    break;
  case ICS_Fled:
    EndTurn();
    break;
  case ICS_Casting:
    break;   
  case ICS_Using:
    break;
  case ICS_Guarding:
    //if (!CanGuard())
    //  EndTurn();
    break;
  case ICS_Bandaging:    
    //if (!CanBandage())
    //  EndTurn();
    break;
  case ICS_Turning:
    //if (!CanTurnUndead())
    //  EndTurn();
    break;

  case ICS_Attacking:
    dude = GetCurrTarget();
    if (dude == NO_DUDE)
      EndTurn();
    else if (!canAttack(dude))
      EndTurn();
    break;

  case ICS_Moving:    
    if (hPath < 0) 
    {
      // if actually have a destination
      if ((moveX>=0)&&(moveY>=0))
      {
        if (coordsOnMap(moveX, moveY, width, height))
        {
          // find path to moveX, moveY
          FindPathTo(moveX, moveY, TRUE);
        }
        else 
        {
          // This guy is fleeing the map. Paths of length 1 result
          // in instant flee. Longer paths need to move dude to
          // edge of map before flee is performed.
          iFleeingFlags |= fleeBecauseOffMap;
          
          if (friendly)
            IncNumFriendFlee(); //combatData.numFrndFlee++;
          else
            IncNumMonsterFlee(); //combatData.numMonFlee++;
          
          menu.setMenu(EmptyMenuData); // starts empty
          menu.setHorzOrient();
          menu.setUseActive(FALSE); // no highlighting      
          
          CString msg("");
          msg.Format("%s has fled...", GetName());
          menu.changeMenuItem(1, msg);
          UpdateCombatTimeDelayMsg();
          theApp.AppSleep(GetMsgTimeDelay()); // give time to see flee message      
          
          EndTurn();
          SetStatus(Fled);
          placeCombatant(x, y, NO_DUDE, width, height);
          TRACE("%i has fled the map\n", self);
        }
      }
    }

    if (hPath >= 0)
    {
      if (!TakeNextStep())
      {
        // unable to move/attack target
        if ((OnAuto()) && (state == ICS_Moving))
        {
          // end your turn but continue
          // guarding if possible          
          if (CanGuard())
          {
            TRACE("Auto combatant %i can't move, ending turn and guarding\n", self);
            EndTurn(ICS_Guarding);
          }
          else
          {
            TRACE("Auto combatant %i can't move, ending turn\n", self);
            EndTurn();
          }
        }
      }
      else
      {
        updateScreen = 1;
      };
    }
//    else // can't find a path
//      EndTurn();

    moveX = -1; moveY = -1;
    if (state != ICS_Moving)
       ClearPath();
    break;

  default:
    ASSERT(false);
    break;
  }
  return updateScreen;
}

//*****************************************************************************
// NAME: COMBATANT::TakeNextStep
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::TakeNextStep()
{
  BOOL isUpdate = FALSE;
  CPathFinder *pathPtr = pathMgr.GetPath(hPath);

  if (pathPtr != NULL) 
  {
    CPoint stepPtr = pathPtr->GetNextStep();

    if (stepPtr.x >= 0)
    {
      TRACE("%u taking next step of path %u to %u,%u\n", 
                  self, hPath, stepPtr.x, stepPtr.y);
      isUpdate = MoveCombatant(stepPtr.x, stepPtr.y);
    }
    else
    {
      WriteDebugString("Bogus step data for path %u in TakeNextStep()\n", hPath);
      ClearPath();
    }

    if ((pathPtr->GetStepCount() <= 0) || (!isUpdate))
      ClearPath();
  }
  else
  {
    WriteDebugString("Bogus hPath %u in TakeNextStep()\n", hPath);
    ClearPath();
  }

  return isUpdate;
}

//*****************************************************************************
// NAME: COMBATANT::ReadyBestWpn
//
// PURPOSE: Select best weapon for auto dudes
//*****************************************************************************
void COMBATANT::ReadyBestWpn(int targ)
{
  BOOL isLargeTarget=FALSE;
  int dist=1;

  // get distance to target
  if (targ == NO_DUDE)
  {
    // no combat targets, nobody to shoot at, so just ready 
    // a hand-to-hand weapon
    dist = 1;
    isLargeTarget = FALSE;
  }
  else
  {
    COMBATANT *targCOMBATANT;
    targCOMBATANT = GetCombatantPtr(targ);
    ASSERT(targCOMBATANT!=NULL);
    if (targCOMBATANT!=NULL)
    {
      isLargeTarget = targCOMBATANT->isLargeDude();
      dist = distance(self, x, y, 
                      targCOMBATANT->self, targCOMBATANT->x, targCOMBATANT->y);  
    }
  }  

  CHARACTER::ReadyBestWpn(dist, isLargeTarget);

  // if target is distant but don't have distance weapon
  // then ready hand-to-hand weapon
  if ((dist > 1) && (myItems.GetReady(WeaponHand) == NO_READY_ITEM))
    CHARACTER::ReadyBestWpn(1, isLargeTarget);
}

//*****************************************************************************
// NAME:    COMBATANT::PostCombatScreenUpdate
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::PostCombatScreenUpdate()
{
  // this is called after primary screen has been
  // blitted
}

//*****************************************************************************
// NAME:    COMBATANT::PlayCombatMove
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::PlayCombatMove()
{
  if (GetType() == MONSTER_TYPE)
    monsterData.PlayMove(origKey);
  else
    ::PlayCombatMove(); //combatData.PlayCombatMove();
}

//*****************************************************************************
//    NAME: COMBATANT::PlayCombatTurnUndead
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::PlayCombatTurnUndead()
{
  if (GetType() == MONSTER_TYPE) return;
  ::PlayCombatTurnUndead(); //combatData.PlayCombatTurnUndead();
}

//*****************************************************************************
// NAME:    COMBATANT::PlayCombatDeath
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::PlayCombatDeath()
{
  if (GetType() == MONSTER_TYPE)
    monsterData.PlayDeath(origKey);
  else
    ::PlayCombatDeath();//combatData.PlayCombatDeath();
}

//*****************************************************************************
// NAME: COMBATANT::ClearPath
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::ClearPath()
{
  //TRACE("Combatant %i ClearPath(%i)\n", self, hPath);
  if (hPath >= 0) 
    pathMgr.FreePath(hPath); 
  hPath = -1; 
}


//*****************************************************************************
// NAME: COMBATANT::GetCurrTarget
//
// PURPOSE:  
//*****************************************************************************
int COMBATANT::GetCurrTarget()
{ 
  if (target != NO_DUDE)
  {    
    COMBATANT *targCOMBATANT;
    targCOMBATANT = GetCombatantPtr(target);

    if (targCOMBATANT==NULL) return NO_DUDE;
    if (!targCOMBATANT->charOnCombatMap())
      RemoveCurrTarget();
    if (!combatData.IsValidTarget(this, targCOMBATANT)) return NO_DUDE;
  };

  return target; 
}

//*****************************************************************************
// NAME: StartAttack
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::StartAttack(int targ)
{
   ASSERT(self != NO_DUDE);
   continueAttack = FALSE;
   if (targ == NO_DUDE)
   {
     ASSERT(FALSE);
     WriteDebugString("StartAttack() for %i called with invalid target\n", self);
     return FALSE;
   }
   if (availAttacks <= 0) 
   {
     ASSERT(FALSE);
     WriteDebugString("StartAttack() for %i called with availAttacks=0\n", self);
     return FALSE;
   }
   if (!charOnCombatMap())
   {
     ASSERT(FALSE);
     WriteDebugString("StartAttack() for %i called when combatant not on map\n", self);
     return FALSE;
   }

   state = ICS_Attacking;
   StopCasting(FALSE);
   //PlaceCursorOnCurrentDude();
   EnsureVisibleTarget(targ);
   continueAttack=TRUE;
   PlayLaunch();
   return TRUE;
}

//*****************************************************************************
//    NAME: COMBATANT::NeedMissileAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedMissileAnimation()
{
  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(target);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return FALSE;
  BOOL result = (distance(self, x, y, 
                   target, targCOMBATANT->x, targCOMBATANT->y) > 1);
  TRACE("NeedMissileAnimation() %i for combatant %i, target %i\n", result,self,target);
  return result;
}

//*****************************************************************************
//    NAME: COMBATANT::InitMissileAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitMissileAnimation()
{
  combatSprite.Clear();
  // lookup itemData key to determine missile animation image for
  // the weapon being used.
  GLOBAL_ITEM_ID giID = myItems.GetItem(myItems.GetReady(WeaponHand));

  if (WpnUsesAmmo(giID))
    giID = myItems.GetItem(myItems.GetReady(AmmoQuiver));

  ITEM_DATA *data = itemData.GetItemData(giID);
  if (data == NULL)
  {
    WriteDebugString("InitMissileAnimation() for combatant %i failed item data lookup\n", self);
    return;  
  }

  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(target);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return;
  combatSprite.Init(data->MissileArt, 
                    TRUE, // use path
                    "it_",
                    GetCenterX(), GetCenterY(), 
                    targCOMBATANT->GetCenterX(), targCOMBATANT->GetCenterY()); 
  EnsureVisibleTarget(self);
}

//*****************************************************************************
//    NAME: COMBATANT::NeedHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedHitAnimation()
{
  int wpn = myItems.GetReady(WeaponHand);
  if (wpn == NO_READY_ITEM) return FALSE;
  
  GLOBAL_ITEM_ID giID = myItems.GetItem(wpn);
  if (WpnUsesAmmo(giID))
    giID = myItems.GetItem(myItems.GetReady(AmmoQuiver));
  ITEM_DATA *data = itemData.GetItemData(giID);
  if (data == NULL)
  {
    WriteDebugString("NeedHitAnimation() for combatant %i failed item data lookup\n", self);
    return FALSE;  
  }
  return (data->HitArt.filename.IsEmpty() == FALSE);
}
//*****************************************************************************
//    NAME: COMBATANT::InitHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitHitAnimation()
{
  combatSprite.Clear();
  // lookup itemData key to determine hit animation image for
  // the weapon being used.
  GLOBAL_ITEM_ID giID = myItems.GetItem(myItems.GetReady(WeaponHand));

  if (WpnUsesAmmo(giID))
    giID = myItems.GetItem(myItems.GetReady(AmmoQuiver));

  ITEM_DATA *data = itemData.GetItemData(giID);
  if (data == NULL)
  {
    WriteDebugString("InitHitAnimation() for combatant %i failed item data lookup\n", self);
    return;  
  }

  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(target);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return;
  combatSprite.Init(data->HitArt, 
                    FALSE, // is stationary
                    "it_",
                    targCOMBATANT->GetCenterX(), targCOMBATANT->GetCenterY(),
                    0,0);

  EnsureVisibleTarget(target);
}

//*****************************************************************************
//    NAME: COMBATANT::NeedSpellCastAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedSpellCastAnimation()
{
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;
  return (pSdata->CastArt.filename.IsEmpty() == FALSE);
}

//*****************************************************************************
//    NAME: COMBATANT::InitSpellCastAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitSpellCastAnimation(int targ)
{
  combatSprite.Clear();

  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return;

  combatSprite.Init(pSdata->CastArt, 
                    FALSE, // is stationary
                    "spl_",
                    GetCenterX(), GetCenterY(),
                    0,0);

  EnsureVisibleTarget(self, TRUE);
}

//*****************************************************************************
//    NAME: COMBATANT::NeedSpellInRouteAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedSpellInRouteAnimation(int mapx, int mapy)
{
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;
  if ((x==mapx)&&(y==mapy)) return FALSE;
  return (pSdata->MissileArt.filename.IsEmpty() == FALSE);  
}

//*****************************************************************************
//    NAME: COMBATANT::InitSpellInRouteAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitSpellInRouteAnimation(int mapx, int mapy)
{
  combatSprite.Clear();
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return;

  FaceLocation(mapx,mapy);

  if (targets.m_type != AreaLinePickStart)
  {
    // animation starts at caster
    combatSprite.Init(pSdata->MissileArt, 
                      TRUE, // use path
                      "spl_",
                      GetCenterX(), GetCenterY(), 
                      mapx, mapy); 

    EnsureVisibleTarget(self);
  }
  else
  {
    // animation starts at mapx,mapy
    int sx = mapx;
    int sy = mapy;
    int length = targets.m_MaxRange;
    PATH_DIR dir = GetSpellTargetingDir(GetCenterX(),GetCenterY(), mapx,mapy);
    int dx=sx,dy=sy;

    switch (dir)
    {
    case PathNorth:
      dx = sx;
      dy = sy-length;
      break;
    case PathNE:
      dx = sx+length;
      dy = sy-length;
      break;
    case PathEast:
      dx = sx+length;
      dy = sy;
      break;
    case PathSE:
      dx = sx+length;
      dy = sy+length;
      break;
    case PathSouth:
      dx = sx;
      dy = sy+length;
      break;
    case PathSW:
      dx = sx-length;
      dy = sy+length;
      break;
    case PathWest:
      dx = sx-length;
      dy = sy;
      break;
    case PathNW:
      dx = sx-length;
      dy = sy-length;
      break;
    }

    combatSprite.Init(pSdata->MissileArt, 
                      TRUE, // use path
                      "spl_",
                      sx, sy, 
                      dx, dy); 

    EnsureVisible(mapx, mapy, FALSE);
  }  
}

//*****************************************************************************
//    NAME: COMBATANT::NeedSpellCoverageAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedSpellCoverageAnimation()
{
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;
  return (pSdata->TargCoverageArt.filename.IsEmpty() == FALSE);
}

//*****************************************************************************
//    NAME: COMBATANT::InitSpellCoverageAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitSpellCoverageAnimation(int mapx, int mapy)
{
  combatSprite.Clear();

  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return;

  if (pSdata->TargCoverageArt.style == PIC_DATA::AS_Directional)
  {
    combatSprite.Init(pSdata->TargCoverageArt, 
                      "spl_",
                      mapx, mapy,
                      GetSpellTargetingDir(GetCenterX(), GetCenterY(), mapx,mapy));
    combatSprite.picData.AllowCentering=FALSE;
  }
  else
  {
    combatSprite.Init(pSdata->TargCoverageArt, 
                      FALSE, // is stationary
                      "spl_",
                      mapx, mapy,
                      0,0);
  }

  EnsureVisible(mapx, mapy, TRUE);
}

//*****************************************************************************
//    NAME: COMBATANT::NeedSpellHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedSpellHitAnimation()
{
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;
  return (pSdata->TargHitArt.filename.IsEmpty() == FALSE);
}

//*****************************************************************************
//    NAME: COMBATANT::InitSpellHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitSpellHitAnimation(int targ)
{
  combatSprite.Clear();

  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return;

  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(targ);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return;
  combatSprite.Init(pSdata->TargHitArt, 
                    FALSE, // is stationary
                    "spl_",
                    targCOMBATANT->GetCenterX(), targCOMBATANT->GetCenterY(),
                    0,0);

  EnsureVisibleTarget(targ);
}

//*****************************************************************************
//    NAME: COMBATANT::NeedSpellLingerAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::NeedSpellLingerAnimation()
{
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return FALSE;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return FALSE;
  if (!pSdata->Lingers) return FALSE;
  return (pSdata->TargLingerArt.filename.IsEmpty() == FALSE);
}

//*****************************************************************************
//    NAME: COMBATANT::InitSpellLingerAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::InitSpellLingerAnimation(int mapx, int mapy)
{
  //combatSprite.Clear();
  
  CHARACTER_SPELL cspell;
  if (!GetSpellBeingCast(cspell)) return;
  SPELL_DATA *pSdata = spellData.GetSpellData(cspell.spell);
  if (pSdata==NULL) return;
  if (targets.m_maptarglist.GetCount() == 0)
  {
    ASSERT(FALSE);
    return;
  }

  POSITION pos = targets.m_maptarglist.GetHeadPosition();
  while (pos != NULL)
  {
    COMBAT_SPRITE_DATA *pSprite = new COMBAT_SPRITE_DATA;
    pSprite->Init(pSdata->TargLingerArt, 
                  "spl_",
                  targets.m_maptarglist.GetAt(pos).x, targets.m_maptarglist.GetAt(pos).y);

    WriteDebugString("Adding spell animation with active spell key %i\n", activeSpellKey);
    pSprite->activeSpellKey = activeSpellKey;
    AnimateList.AddTail(pSprite);

    targets.m_maptarglist.GetNext(pos);
  }
  
  EnsureVisible(targets.m_MapTargetX,
                targets.m_MapTargetY, 
                TRUE);
}

//*****************************************************************************
// NAME  DetermineIfBackStab
//
// PURPOSE: Determine if 'self' has the proper weapon and
// proper position to attempt a backstab on an opponent
// who is unaware of our presence.
//*****************************************************************************
BOOL COMBATANT::DetermineIfBackStab(int wpn, int targ)
{
  GLOBAL_ITEM_ID giID;
  int dx, dy;
  int distraction;
  COMBATANT *pTarget;
  COMBATANT *pDistraction;
  weaponClassType weaponType;
  if (!isThief(GetClass())) return FALSE;
  if (wpn == NO_READY_ITEM) return FALSE;
  giID = myItems.GetItem(wpn);
  if (giID.IsNoItem()) return FALSE;
  weaponType = itemData.GetWpnType(giID);
  if (    (weaponType != HandBlunt) 
       && (weaponType != HandCutting)
       && (weaponType != HandThrow)
     ) return FALSE;
  //
  // Now decide if we are standing adjacent to our enemy.
  pTarget = GetCombatantPtr(targ);
  dx = abs(x - pTarget->x);
  dy = abs(y - pTarget->y);
  if (dx > 1) return FALSE;
  if (dy > 1) return FALSE;
  //
  // OK.  Now let us determine if our target is quite busy
  // fighting off one of our friends.
  // We do this by checking that he has just attacked
  // with a combatant that is within 1 square of himself.
  if (pTarget->m_eLastAction == LA_None) return FALSE;
  //
  distraction = NO_DUDE;
  if (pTarget->m_eLastAction == LA_Attack) 
  {
      distraction = pTarget->m_iLastAttacked;
  }
  else
  {
    if (pTarget->m_iLastAttacked == pTarget->m_iLastAttacker)
    {
      distraction = pTarget->m_iLastAttacked;
    };
  }
  //
  if (distraction == NO_DUDE) return FALSE;
  pDistraction = GetCombatantPtr(distraction);
  if (pDistraction->GetAdjStatus() != Okay) return FALSE;
  //
  // Now see if we are on the opposite side of the target
  // from the distraction at the same distance.
  if ( (x - pTarget->x != pTarget->x - pDistraction->x) ) return FALSE;
  if ( (y - pTarget->y != pTarget->y - pDistraction->y) ) return FALSE;
  return TRUE;
}

extern COMBAT_DATA combatData;

//*****************************************************************************
// NAME: makeAttack
//
// PURPOSE:  
//*****************************************************************************
int COMBATANT::makeAttack(int targ)
{
  ASSERT(self != NO_DUDE);
  if (IsDone()) return 1;
  CombatMsg="";
  ClearFormattedText(combatTextData);
    
  if (targ == NO_DUDE) return 1;
  if (availAttacks <= 0) return 1;

  int thaco;//, ac;
  int dmgDiceQty, dmgDiceSides, dmgDiceBonus;
  int attDiceBonus;
  BOOL IsNonLethal;
  BOOL isBackStab;

  {
    CString msg;
    int nbrattacks = GetNbrAttacks();
    int availattacks = availAttacks;
    int currAttack = nbrattacks - availattacks;

    ASSERT( (currAttack>=0) && (currAttack<nbrattacks) );
    if ((currAttack < 0)||(currAttack>=nbrattacks)) currAttack=0;
    monsterData.GetMonsterAttackMsg(origKey, currAttack, msg);
    if (msg == "*NoAttack*") return 1;
  };
   
  EnsureVisibleTarget(targ);

  lastAttackRound = GetCurrentRound();
  state = ICS_Attacking;
  StopCasting(FALSE);
 
  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(targ);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return 1;
  if (targCOMBATANT->GetStatus()==Dead) return 1; // not on the map anymore


  BOOL decQty = FALSE;
  int wpn;
  wpn = myItems.GetReady(WeaponHand);
  GLOBAL_ITEM_ID giID;
  giID.Clear();

  if (wpn != NO_READY_ITEM)
  {
    giID = myItems.GetItem(wpn);
    int dist = distance(self, x, y, 
                        targCOMBATANT->self, targCOMBATANT->x, targCOMBATANT->y);  
    decQty = WpnConsumesAmmoAtRange(giID, dist);
    
    // do we decrement weapon qty or ammo qty?
    if ((decQty) && (!WpnConsumesSelfAsAmmo(giID)))
    {
      // ammo is readied and must be decremented
      wpn = myItems.GetReady(AmmoQuiver);
      giID = myItems.GetItem(wpn);
    }
  }

  isBackStab = DetermineIfBackStab(wpn, targ);
  
  attDiceBonus = GetAttackBonus() + m_iTmpAttDiceBonus;
  if (isBackStab) attDiceBonus += getThiefBackstabAttackAdjustment();

  SetCharContext(this->GetContext());
  SetTargetContext(targCOMBATANT->GetContext());

  m_iAttDiceSides=20;
  m_iAttDiceNum=1;

  ModifyAttackRollDice ( 
                            targCOMBATANT,
                          & m_iAttDiceNum,
                          & m_iAttDiceSides,
                          & attDiceBonus);

  ModifyAttackRollDiceForItem(  
                            targCOMBATANT, 
                            *(GLOBAL_ITEM_ID *)&wpn,  // Investigate this!!
                          & m_iAttDiceNum,
                          & m_iAttDiceSides,
                          & attDiceBonus);

  targCOMBATANT->ModifyAttackRollDiceAsTarget(  
                            this,
                          & m_iAttDiceNum,
                          & m_iAttDiceSides,
                          & attDiceBonus);

  targCOMBATANT->ModifyAttackRollDiceForItemAsTarget(  
                            this, 
                            *(GLOBAL_ITEM_ID *)&wpn,  //Investigate this!!
                          & m_iAttDiceNum,
                          & m_iAttDiceSides,
                          & attDiceBonus);

  thaco = GetAdjTHACO() - attDiceBonus;
   
  int targAC = targCOMBATANT->GetAdjAC();
  targCOMBATANT->ModifyACAsTarget(this, &targAC, giID);
  targAC += targCOMBATANT->myItems.GetProtectModForRdyItems();
  thaco -= targAC;
  ModifyAttackThaco(targCOMBATANT, &thaco);
  targCOMBATANT->ModifyAttackThacoAsTarget(this, &thaco);

  if (thaco < 0)
    thaco = 0;

  int dmg = 0;  
 
  m_iAttDiceRoll = rollDice(m_iAttDiceSides, m_iAttDiceNum, attDiceBonus);

  if (m_iAttDiceRoll < 1)  m_iAttDiceRoll = 1;
  if (m_iAttDiceRoll > 20) m_iAttDiceRoll = 20;
   
  TRACE("%i attacks %i: rolls %i, needs %i\n", 
                   self, targ, m_iAttDiceRoll, thaco);

  if (friendly)
  {
    if      (GetConfigPartyAlwaysMiss()) m_iAttDiceRoll = thaco - 1;
    else if (GetConfigPartyAlwaysHit())  m_iAttDiceRoll = thaco;
  }
  else
  {
    if      (GetConfigMonsterAlwaysMiss()) m_iAttDiceRoll = thaco - 1;
    else if (GetConfigMonsterAlwaysHit())  m_iAttDiceRoll = thaco;
  };
  int DidHit = -1;
  {
    CString result, itemName;
    ITEM_DATA *pItem;
    //SetItemContext(giID);
    scriptContext.SetItemContext(giID);  // Includes spell Context!
    scriptContext.SetTargetContext(targCOMBATANT);
    scriptContext.SetAttackerContext(this);
    if (giID.IsValidItem())
    {
      itemName = scriptContext.GetItemContext()->Name;
    }
    else
    {
      itemName = "Bogus Item";
    };
    PrepareHooks();
    hookParameters[5].Format("%d",m_iAttDiceRoll);
    hookParameters[6].Format("%d",thaco);
    result = this->combatantSA.RunScripts
                (DOES_ATTACK_SUCCEED,
                 ScriptCallback_LookForChar,
                 "YN",
                 this->GetName(),
                 itemName);
    if (result.IsEmpty())
    {
      result = this->specAbs.RunScripts
                  (DOES_ATTACK_SUCCEED,
                   ScriptCallback_LookForChar,
                   "YN",
                   this->GetName(),
                   itemName);
    };
    if (result.IsEmpty())
    {
      result = targCOMBATANT->combatantSA.RunScripts
                  (DOES_ATTACK_SUCCEED,
                   ScriptCallback_LookForChar,
                   "YN",
                   targCOMBATANT->GetName(),
                   itemName);
    };
    if (result.IsEmpty())
    {
      result = targCOMBATANT->specAbs.RunScripts
                  (DOES_ATTACK_SUCCEED,
                   ScriptCallback_LookForChar,
                   "YN",
                   targCOMBATANT->GetName(),
                   itemName);
    };
    if (result.IsEmpty() && giID.IsValidItem())
    {
      pItem = itemData.GetItemData(giID);
      if (pItem != NULL)
      {
        result = pItem->specAbs.RunScripts
                    (DOES_ATTACK_SUCCEED,
                     ScriptCallback_LookForChar,
                     "YN",
                     targCOMBATANT->GetName(),
                     itemName);
        if (result.IsEmpty())
        {
          if (scriptContext.GetSpellContext() != NULL)
          {
            result = scriptContext.GetSpellSAContext()->RunScripts
                        (DOES_ATTACK_SUCCEED,
                         ScriptCallback_LookForChar,
                         "YN",
                         targCOMBATANT->GetName(),
                         itemName);
          };
        };
      };
    };
    scriptContext.Clear();
    //ClearItemContext();
    if (!result.IsEmpty())
    {
      if (result[0] == 'Y') DidHit = 1;
      else if (result[0] == 'N') DidHit = 0;
    };
  };

  if (DidHit < 0) DidHit = (m_iAttDiceRoll >= thaco);
  GLOBAL_SPELL_ID SpellDbKey; //=-1;

  if (DidHit)
  {     
    GetDamageDice(
                wpn, 
              & dmgDiceQty, 
              & dmgDiceSides, 
              & dmgDiceBonus, 
              & IsNonLethal, 
                targCOMBATANT->isLargeDude(),
              & SpellDbKey);

    dmgDiceBonus += m_iTmpDmgDiceBonus; // for backstabbing
    
    ModifyAttackDamageDice(
                    targCOMBATANT,   
                  & dmgDiceQty, 
                  & dmgDiceSides, 
                  & dmgDiceBonus, 
                  & IsNonLethal);

    ModifyAttackDamageDiceForItem(
                    targCOMBATANT,
                    *(GLOBAL_ITEM_ID *)&wpn,  // Investigate this!!
                  & dmgDiceQty,  
                  & dmgDiceSides,  
                  & dmgDiceBonus,  
                  & IsNonLethal);

    targCOMBATANT->ModifyAttackDamageDiceAsTarget(
                    this,
                  & dmgDiceQty,
                  & dmgDiceSides,
                  & dmgDiceBonus,
                  & IsNonLethal);

    targCOMBATANT->ModifyAttackDamageDiceForItemAsTarget(
                    this,
                    *(GLOBAL_ITEM_ID *)&wpn,  // Investigate this!!
                  & dmgDiceQty,
                  & dmgDiceSides,
                  & dmgDiceBonus,
                  & IsNonLethal);

    dmg = rollDice(dmgDiceSides, dmgDiceQty, dmgDiceBonus);
     
    if (dmg <= 0) 
      dmg = 1;

    TRACE("%i rolls %i damage using %i sides, %i qty, %i bonus (%s)\n",
                     self, dmg, dmgDiceSides, dmgDiceQty, dmgDiceBonus,
                     (IsNonLethal?"NonLethal":"Lethal"));

    {
      ITEM_DATA *pWeapon;
      if (!giID.IsNoItem())
      {
        pWeapon = itemData.GetItemData(giID);

        //InstantSpellActivate(SpellDbKey, targ);
        if (pWeapon->IsUsable() && (pWeapon->Wpn_Type == SpellCaster))
        {
          InstantSpellActivate(pWeapon->m_gsID, targ);
        };
      };
    };

    targCOMBATANT->UpdateSpellForAttacks(1);

    PlayHit();

    if (dmg > 0)
    {        
      if (isBackStab)
      {
        dmg *= getThiefBackstabDamageMultiplier();
      };

      {
        CString result;
        ITEM_DATA *pWeapon=NULL;
        scriptContext.SetItemContext(giID);
        if (!giID.IsNoItem())
        {
          pWeapon = itemData.GetItemData(giID);
        };
        PrepareHooks();
        hookParameters[5].Format("%d",dmg);
        hookParameters[6].Format("%d",IsNonLethal);
        scriptContext.SetAttackerContext(this);
        scriptContext.SetTargetContext(targCOMBATANT);
        scriptContext.SetItemContext(pWeapon);
        result = targCOMBATANT->combatantSA.RunScripts(
                COMPUTE_DAMAGE,
                ScriptCallback_RunAllScripts,
                NULL,
                targCOMBATANT->GetName(),
                scriptContext.GetItemContext()->Name);
        if (!result.IsEmpty())
        {
          int temp;
          if (sscanf(result,"%d",&temp) == 1) dmg = temp;
        };
        scriptContext.Clear();
      };

      targCOMBATANT->TakeDamage(dmg, IsNonLethal);
       
      if (targCOMBATANT->GetStatus()==Dead)
      {
        // no longer a valid target
        RemoveCurrTarget();
      }
    }

    continueAttack=FALSE;
  } 
  else
  {
    PlayMiss();
    continueAttack=TRUE;
  }

  if (DidHit)
  {
    if (SpellDbKey.IsValidSpell())
    {
      CString samsg;
      SPELL_DATA *pSpell = spellData.GetSpellData(SpellDbKey);
      ASSERT(pSpell!=NULL);  
      if (pSpell!=NULL)
      {       
        samsg = pSpell->CastMsg;
        samsg.Replace("/s", pSpell->Name);
        samsg.Replace("/c", GetName());
        if (targCOMBATANT!=NULL)
          samsg.Replace("/t", targCOMBATANT->GetName());
        else
          samsg.Replace("/t", ""); 
        
        CombatMsg = samsg;
      }
    }
    else
    {
      if (isBackStab)
      {
        CombatMsg.Format("Rolls %i and BackStabs for %i dmg",
                       m_iAttDiceRoll, dmg);
      }
      else
      {
        CombatMsg.Format("Rolls %i and hits for %i dmg",
                       m_iAttDiceRoll, dmg);
      };
    }
  }
  else
    if (isBackStab)
    {
      CombatMsg.Format("Rolls %i amd fails BackStab", m_iAttDiceRoll);
    }
    else
    {
      CombatMsg.Format("Rolls %i and misses", m_iAttDiceRoll);
    };

  FormatCombatText(combatTextData, CombatMsg);
  //combatMsgs.msgs.RemoveAll();
  //combatMsgs.Add(CombatMsg);

  if (decQty)
  {
    // dec item qty by 1, item won't show up in post-combat treasure list
    if (wpn != NO_READY_ITEM)
    {
      myItems.AdjustQty(wpn, -1);
      if (!myItems.HaveItem(*(GLOBAL_ITEM_ID*)&wpn)) // if deleted because of zero quantity
      {
        // item removed, disable special abilities granted by it (if any)
        ITEM_DATA *pItem = itemData.GetItemData(giID);
        if (pItem!=NULL) 
        {
          SetCharContext(GetContext());
          PrepareHooks();
          scriptContext.SetCharacterContext(this);
          specAbs.RunScripts(ON_USE_LAST_WEAPON, ScriptCallback_RunAllScripts, NULL, name, " Used last weapon");
          scriptContext.Clear();
          ClearCharContext();
          //pItem->specAbs.DisableAllFor(this);
        };
      };

      // re-calculate movement/encumbrance
      SetEncumbrance(determineEffectiveEncumbrance());
      determineMaxMovement();    
    }
  }


  // make sure combatants are facing towards each other
  FaceOpponent(targ);
  if (!isBackStab) targCOMBATANT->FaceOpponent(self);   


  targCOMBATANT->m_iLastAttacker = self;
  targCOMBATANT->m_eLastAction = LA_Defend;
  m_iLastAttacked=targCOMBATANT->self;
  m_eLastAction = LA_Attack;


  m_iTmpAttDiceBonus = 0;
  m_iTmpDmgDiceBonus = 0;

  ClearTargetContext();
  ClearCharContext();
  return -1;
}

//*****************************************************************************
// NAME:    COMBATANT::AttackOver
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::AttackOver()
{
  availAttacks -= 1.0;
  float temp = availAttacks;

  ClearFormattedText(combatTextData);
  TRACE("Attack over for %u, attacks remaining %f\n",
                   self, temp);

  if (availAttacks <= 0)
  {
    continueAttack=FALSE;
    availAttacks = 0;
    EndTurn();
    return;
  }

  PlaceCursorOnCurrentDude();
  // if manual control, and first attack hits, then stop
  // attacks if multiple allowed
  if ((!continueAttack) && (!OnAuto()))
  {
    StopAttack();
    return;
  }
}

//*****************************************************************************
// NAME:    COMBATANT::AttackOver
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::StopAttack()
{
  // stop casting spells if in progress
  StopCasting(FALSE);
  state = ICS_None;
}

//*****************************************************************************
// NAME:    COMBATANT::HandleTimeDelayMsgExpired
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::HandleTimeDelayMsgExpired()
{  
  switch (state)
  {
  case ICS_Attacking:
    TRACE("Handle TimeDelayMsgExpired: AttackOver\n");
    AttackOver();
    break;
  case ICS_Bandaging:
    TRACE("Handle TimeDelayMsgExpired: Bandage Over\n");    
    break;
  case ICS_Guarding:
    TRACE("Handle TimeDelayMsgExpired: Guarding continues\n");
    EndTurn(ICS_Guarding);
    break;
  case ICS_Turning:
    TRACE("Handle TimeDelayMsgExpired: Turn Undead Over\n");
    // message preceeds action in this case
    //EndTurn();
    break;
  }
}

//*****************************************************************************
//    NAME: COMBATANT::HandleCombatRoundMsgExpired
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::HandleCombatRoundMsgExpired()
{
  TRACE("Handle HandleCombatRoundMsgExpired for %i - state %s\n", 
         self,CombatantStateText[state]);
  PlaceCursorOnCurrentDude();  
  // do nothing yet
}

//*****************************************************************************
// NAME:    COMBATANT::HandleTimeDelayMsgBegin
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
int COMBATANT::HandleTimeDelayMsgBegin()
{
  switch (state)
  {
  case ICS_Attacking:
    return makeAttack(GetCurrTarget());

  default:
    //PlaceCursorOnCurrentDude();
    break;
  }
  return -1;
}

//*****************************************************************************
//    NAME: COMBATANT::UseAttackPose
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::UseAttackPose()
{
  if (state==ICS_Attacking) return TRUE;
  if (state==ICS_Using) return TRUE;
  if (state==ICS_Casting) return forceAttackPose;
  return FALSE;
}

//*****************************************************************************
// NAME:    displayCombatSprite
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::displayCombatSprite()
{  
//   charStatusType cstat = GetAdjStatus();
   if (charUnconscious())
   {
      // blit dying combat tile
      blitDyingSprite();
   }
   else
   {
     if (!charOnCombatMap()) return;

     if (!GraphicsMgr.ValidSprite(icon.key))
     {
        LoadCombatIcon();
        if (!GraphicsMgr.ValidSprite(icon.key))
        {
          if (!debugStrings.AlreadyNoted(CString("NISTUID")))
          {
            WriteDebugString("No icon surface to use in displayCombatSprite\n");
          };
          return;
        }
     }
   
     int startTerX, startTerY;
     GetStartTerrain(&startTerX,  &startTerY);
     if (   (x < startTerX + TILES_HORZ)
         && (y < startTerY + TILES_VERT))
     {
        int tx = x;
        int ty = y;

        // Check for partial icon display
        //
        // For icons larger than 1x1, the starting x,y may
        // be off of the left,top but still the partially visible
        // part of the icon should be displayed.
        //
        // This isn't a problem for the right,bottom since
        // if the x,y isn't within those bounds then none
        // of the icon will be either.
        if (tx < startTerX)
        {
          int ex = (tx+width)-1;
          // none of the icon is visible
          if (ex < startTerX)
            return;
          BOOL visible = FALSE;
          while ((tx <= ex) && (!visible))
          {
            if (tx >= startTerX)
              visible = TRUE;
            else
              tx++;
          }
          if (!visible)
            return;
        }
           
        if (ty < startTerY)
        {
          int ey = (ty+height)-1;
          // none of the icon is visible
          if (ey < startTerY)
            return;
          BOOL visible = FALSE;
          while ((ty <= ey) && (!visible))
          {
            if (ty >= startTerY)
              visible = TRUE;
            else
              ty++;
          }
          if (!visible)
            return;
        }

        RECT srcRect;  // clipping rect for source bitmap
        RECT dstRect;

        // width and height in pixels of a single tile
        int iw = width * COMBAT_TILE_WIDTH;
        int ih = height * COMBAT_TILE_HEIGHT;

        // offset in pixels to icon pair indicated by iconIndex
        int offset = (iconIndex-1)*(iw*2);
        if (offset < 0) offset=0;
                  
        if (IsCurrentCombatant(self))
        {
          if (UseAttackPose())
          {
            // display attack pose
            srcRect.left = offset+iw;
          }
          else
          {
            // display ready pose
            srcRect.left = offset;
          }
        }
        else
          srcRect.left = offset;
    
        srcRect.right  = srcRect.left + iw;
        srcRect.top    = 0;
        srcRect.bottom = ih;
        
        BOOL flip = (m_iFacing == FACE_WEST);

        // take into account any clipping done against
        // top, left for partially visible icons
        if (tx > x)
        {
          if (flip)
            srcRect.right -= ((tx-x)*COMBAT_TILE_WIDTH);
          else
            srcRect.left += ((tx-x)*COMBAT_TILE_WIDTH);
        }
        if (ty > y)
          srcRect.top += ((ty-y)*COMBAT_TILE_HEIGHT);

        dstRect.left=TerrainToScreenCoordX(tx);
        dstRect.top=TerrainToScreenCoordY(ty);
        dstRect.right  = dstRect.left+(srcRect.right-srcRect.left);
        dstRect.bottom = dstRect.top+(srcRect.bottom-srcRect.top);

        /*
        if (dude.GetType() == CHAR_TYPE)
          TRACE("Blitting pc icon\n");
        else if (dude.GetType() == NPC_TYPE)
          TRACE("Blitting npc icon\n");
        else if (dude.GetType() == monster)
          TRACE("Blitting monster icon\n");
        */

        GraphicsMgr.SetClipRect(&GetCombatRect());  

        // blit the source tile from the art bitmap to the back buffer
        GraphicsMgr.BlitSpriteImage(dstRect.left, dstRect.top,
                              icon.key,
                              &srcRect, 
                              flip,
                              BACKBUFFERNUM,
                              &dstRect); 

        GraphicsMgr.SetClipRect(&GetFullRect());
     }
   }
}

//*****************************************************************************
// NAME:    blitDyingSprite
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::blitDyingSprite()
{
  if (!blitDeathTile)
    blitDeadSprite();
  
  if (!IsPartyMember()) return;    
  //UpdateCombatTimeDelayMsg();

  if (!ValidTerrainViewportCoords(x, y)) 
  {
    //ASSERT(FALSE);
    return;
  }
  
  //EnsureVisible(GetCenterX(),GetCenterY());

  int sx = TerrainToScreenCoordX(x);
  int sy = TerrainToScreenCoordY(y);
 
  GraphicsMgr.SetClipRect(&GetCombatRect());  
  GraphicsMgr.BlitImage(sx,sy,CombatDeathIconArt.surface,&DeadCharSrcA);
  GraphicsMgr.SetClipRect(&GetFullRect());
}

//*****************************************************************************
// NAME:    blitDeadSprite
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::blitDeadSprite()
{
   if (blitDeathTile) return; // already done
   blitDeathTile = TRUE;

   placeCombatant(x, y, NO_DUDE, width, height);
   placeDyingCombatant(x,y,self,width,height);
   
   UpdateCombatTimeDelayMsg();

   PlayCombatDeath();

   if (!ValidTerrainViewportCoords(x, y)) 
   {
     //ASSERT(FALSE);
     return;
   }

   EnsureVisible(GetCenterX(),GetCenterY());
   GraphicsMgr.SetClipRect(&GetCombatRect());

   int i,j;
   for (i=0;i<height;i++)
   {
     for (j=0;j<width;j++)
     {
       GraphicsMgr.BlitImage(TerrainToScreenCoordX(x+j),
                             TerrainToScreenCoordY(y+i),
                             CombatDeathArt.surface,
                             &DeadCharSrcA,
                             FALSE,FRONTBUFFERNUM);
     }
   }      

   theApp.AppSleep(200);

   for (i=0;i<height;i++)
   {
     for (j=0;j<width;j++)
     {
       GraphicsMgr.BlitImage(TerrainToScreenCoordX(x+j),
                             TerrainToScreenCoordY(y+i),
                             CombatDeathArt.surface,
                             &DeadCharSrcB,
                             FALSE,FRONTBUFFERNUM);
     }
   } 
   
   theApp.AppSleep(200);

   for (i=0;i<height;i++)
   {
     for (j=0;j<width;j++)
     {
       GraphicsMgr.BlitImage(TerrainToScreenCoordX(x+j),
                             TerrainToScreenCoordY(y+i),
                             CombatDeathArt.surface,
                             &DeadCharSrcA,
                             FALSE,FRONTBUFFERNUM);
     }
   }      

   theApp.AppSleep(200);

   for (i=0;i<height;i++)
   {
     for (j=0;j<width;j++)
     {
       GraphicsMgr.BlitImage(TerrainToScreenCoordX(x+j),
                             TerrainToScreenCoordY(y+i),
                             CombatDeathArt.surface,
                             &DeadCharSrcB,
                             FALSE,FRONTBUFFERNUM);
     }
   }      

   theApp.AppSleep(200);

   for (i=0;i<height;i++)
   {
     for (j=0;j<width;j++)
     {
       GraphicsMgr.BlitImage(TerrainToScreenCoordX(x+j),
                             TerrainToScreenCoordY(y+i),
                             CombatDeathArt.surface,
                             &DeadCharSrcA,
                             FALSE,FRONTBUFFERNUM);
     }
   }      

   GraphicsMgr.SetClipRect(&GetFullRect());
   theApp.AppSleep(500);
}

//*****************************************************************************
// NAME: COMBATANT::EndTurn
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::EndTurn(individualCombatantState newState)
{   
  state = newState;
  QueuedCombatantData &qcomb = GetQueuedCombatants();

  if (qcomb.Top() == self)
  {
    if (qcomb.ChangeStats())
    {
      TRACE("EndTurn(%s), %i done\n", CombatantStateText[state], self);
      turnIsDone = TRUE;
    }
    else
      TRACE("EndTurn(%s), %i (no change to done)\n", CombatantStateText[state], self);

    // input was disabled if auto combatant
    EnableUserInput(TRUE);
    qcomb.Pop();
  }
  else
  {
    qcomb.Remove(self);
    TRACE("Forced EndTurn(%s), %i done\n", CombatantStateText[state], self);
    turnIsDone = TRUE;
  }
  forceAttackPose=FALSE;
  OnEndTurn();
  //combatMsgs.msgs.RemoveAll(); // not set up for scrolling msgs yet
}


//*****************************************************************************
// NAME: COMBATANT::IsDone
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::IsDone()
{ 
  QueuedCombatantData &qcomb = GetQueuedCombatants();
  if (   (qcomb.Top() == self)
      && (!qcomb.ChangeStats()))
  {
    return FALSE; // queued up for free attack
  }

  if (m_isCombatReady == -1)
  {
    m_isCombatReady = 1;
    SetCharContext(GetContext());
    scriptContext.SetCharacterContext(this);
    scriptContext.SetCombatantContext(this);
    PrepareHooks();
    if (!specAbs.RunScripts(IS_COMBAT_READY, ScriptCallback_LookForChar, "N", name, "").IsEmpty())
    {
      m_isCombatReady = 0;
    }
    else
    {
      if (!combatantSA.RunScripts(IS_COMBAT_READY, ScriptCallback_LookForChar, "N", name, "").IsEmpty())
      {
        m_isCombatReady = 0;
      }
    };
    scriptContext.Clear();
    ClearCharContext();
    /*
    if (GetAdjSpecAb(SA_Sleep))
    {
      m_incapacitated = 1;
    }
  
    if (GetAdjSpecAb(SA_HoldPerson))
    {
      m_incapacitated = 1;
    }
    */
  };

  if (m_isCombatReady == 0) return TRUE;

  if (!charOnCombatMap())
    turnIsDone = TRUE;

  return turnIsDone; 
}

//*****************************************************************************
// NAME: COMBATANT::Quick
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::Quick(BOOL Enable)
{
  // can't change quick status if not allowed to control
  if (GetAdjAllowPlayerControl())
  {
    // if turning off auto, stop doing whatever
    // the AI had you doing.
    if (!Enable)
    {
      ClearPath();
      combattargets.RemoveAll();
      state = ICS_None;
      StopCasting(FALSE);
    }
    SetAutomatic(Enable);
    if (!IsPartyMember())
      EnableUserInput(!GetAdjAutomatic());
    else
      EnableUserInput(TRUE);
  }
}

//*****************************************************************************
// NAME: COMBATANT::TurnUndead
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::TurnUndead()
{
  // end any spell casting in progress
  StopCasting(FALSE);
  hasTurnedUndead=TRUE;
  state = ICS_Turning;
}

//*****************************************************************************
// NAME: COMBATANT::CanBandage
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanBandage()
{
  return (!IsDone());
}

//*****************************************************************************
// NAME:    COMBATANT::Bandage
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::Bandage()
{ 
  // end any spell casting in progress
  StopCasting(FALSE);
  state = ICS_Bandaging;
}

//*****************************************************************************
// NAME:    COMBATANT::CanGuard
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBATANT::CanGuard()
{
  if (IsDone()) return FALSE;

  // can guard only when melee weapon is readied, if is a
  // party member. Monsters can guard always
  if (!IsPartyMember())
    return TRUE;

  int mywpnidx = myItems.GetReady(WeaponHand);
  if (mywpnidx != NO_READY_ITEM)
  {  
    // using a weapon rather than natural attack types (ie claws/jaws/fists)
     if (myItems.GetQty(mywpnidx) <= 0)
        return FALSE;    
 
     return (WpnCanAttackAtRange(myItems.GetItem(mywpnidx), 1));
  }
  // no weapon readied
  return FALSE;
}

//*****************************************************************************
// NAME: COMBATANT::Guard
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::Guard()
{
  // end any spell casting in progress
  StopCasting(FALSE);
  state = ICS_Guarding;
}

//*****************************************************************************
// NAME: COMBATANT::CanMove
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanMove()
{
  if (GetType() == MONSTER_TYPE)
  {
    if (GetConfigMonsterNoMove())
      return FALSE;
  }

  if (IsDone()) 
    return FALSE;
  if (movement >= GetAdjMaxMovement()) 
    return FALSE;
  return TRUE;
}

//*****************************************************************************
// NAME:    COMBATANT::FindPathTo
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBATANT::FindPathTo(int dx, int dy, BOOL CheckOccupants)
{  
  if (!OnAuto())
  {
    if ((abs(dx-x) > 1) || (abs(dy-y) > 1))
      TRACE("Moving more than 1 square\n");
  }

  //WriteDebugString("FindPathTo(%u,%u)\n", dx, dy);
  ClearPath();

  if (!CanMove()) return FALSE;

  pathMgr.SetPathSize((BYTE)width, (BYTE)height);
  hPath = pathMgr.GetPath(x, y, dx, dy, CheckOccupants);
  return (hPath >= 0);
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathAwayFrom
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::FindPathAwayFrom(int fleeFrom)
{  
  ClearPath();
  if (!CanMove()) return FALSE;
  if (fleeFrom < 0) return FALSE;

//  /*
  pathMgr.SetPathSize((BYTE)width, (BYTE)height);

  // locate path to position in room that is farthest
  // from indicated fleeFrom character

  //int fx = combatData.combatants[fleeFrom].x;
  //int fy = combatData.combatants[fleeFrom].y;
  int fx = GetCombatantX(fleeFrom);
  int fy = GetCombatantY(fleeFrom);

  // get direction for moving away from 'fleeFrom'
  PATH_DIR dir = GetDir(x, y, fx, fy);
  switch (dir)
  {
  case PathNorth: dir = PathSouth; break;
  case PathNE: dir = PathSW; break;
  case PathEast: dir = PathWest; break;
  case PathSE: dir = PathNW; break;
  case PathSouth: dir = PathNorth; break;
  case PathSW: dir = PathNE; break;
  case PathWest: dir = PathEast; break;
  case PathNW: dir = PathSE; break;
  default: dir = PathNorth; break;
  }

  PATH_DIR pleft = PathDirLeft(dir);
  PATH_DIR pleft2 = PathDirLeft(pleft);
  PATH_DIR pleft3 = PathDirLeft(pleft2);
  PATH_DIR pright = PathDirRight(dir);
  PATH_DIR pright2 = PathDirRight(pright);
  PATH_DIR pright3 = PathDirRight(pright2);

  // max movement distance may not allow straight-line pathing
  // but go with it for now.
  int maxmove = GetMaxMovement();
  if (FindPathAlongLine(dir, maxmove))
    return TRUE;
  if (FindPathAlongLine(pleft, maxmove))
    return TRUE;
  if (FindPathAlongLine(pright, maxmove))
    return TRUE;
  if (FindPathAlongLine(pleft2, maxmove))
    return TRUE;
  if (FindPathAlongLine(pright2, maxmove))
    return TRUE;
  if (FindPathAlongLine(pleft3, maxmove))
    return TRUE;
  if (FindPathAlongLine(pright3, maxmove))
    return TRUE;
//  */
  // no corner in room to flee to, just flee the map if possible
  if (FindPathToMapEdge())
    return TRUE;

  return FALSE;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathAlongLine
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::FindPathAlongLine(PATH_DIR dir, int dist)
{
  BOOL found = FALSE;
  int maxdist = dist;
  int halfdist = maxdist/2;
  int dx=0;
  int dy=0;

  switch (dir)
  {
    case PathNorth: dx=x;dy=y-maxdist; break;
    case PathNE: dx=x+halfdist;dy=y-halfdist; break;
    case PathEast: dx=x+maxdist;dy=y; break;
    case PathSE: dx=x+halfdist;dy=y+halfdist; break;
    case PathSouth: dx=x;dy=y+maxdist; break;
    case PathSW: dx=x-halfdist;dy=y+halfdist; break;
    case PathWest: dx=x-maxdist;dy=y; break;
    case PathNW: dx=x-halfdist;dy=y-halfdist; break;
    default: return FALSE;
  }

  if (dx < 0) dx=0;
  if (dx >= MAX_TERRAIN_WIDTH) dx = MAX_TERRAIN_WIDTH-1;
  if (dy < 0) dy=0;
  if (dy >= MAX_TERRAIN_HEIGHT) dy = MAX_TERRAIN_HEIGHT-1;

  if (HaveLineOfSight(GetCenterX(),GetCenterY(),dx,dy))
  {
    hPath = pathMgr.GetPath(x, y, dx, dy, TRUE);
    found = (hPath>=0);
  }

  return found;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathToMapNorthEdge
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::FindPathToMapNorthEdge(void)
{
  int path=-1;
  BOOL found = FALSE;

  // find a path to one of the four map sides
  int dx=0;
  int dy=0;
  pathMgr.SetPathSize((BYTE)width, (BYTE)height);

  // north side, find empty, non-blocked, 
  // square along north map edge.
  // start with center and move towards edges
  int count = 0;
  int max = MAX_TERRAIN_WIDTH/2;
  int center = MAX_TERRAIN_WIDTH/2;

  dy=0;
  while ((!found) && (count<=max))
  {
    if (!isObstical(center+count, dy, 1, 1, TRUE, TRUE))
    {
      dx=center+count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    else if (!isObstical(center-count, dy, 1, 1, TRUE, TRUE))
    {
      dx=center-count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    count++;
  }
  return path;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathToMapEastEdge
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::FindPathToMapEastEdge(void)
{
  int path=-1;
  BOOL found = FALSE;

  // find a path to one of the four map sides
  int dx=0;
  int dy=0;
  pathMgr.SetPathSize((BYTE)width, (BYTE)height);

  // north side, find empty, non-blocked, 
  // square along north map edge.
  // start with center and move towards edges
  int count = 0;
  int max = MAX_TERRAIN_HEIGHT/2;
  int center = MAX_TERRAIN_HEIGHT/2;

  dx=MAX_TERRAIN_WIDTH-1;
  while ((!found) && (count<=max))
  {
    if (!isObstical(dx, center+count, 1, 1, TRUE, TRUE))
    {
      dy=center+count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    else if (!isObstical(dx, center-count, 1, 1, TRUE, TRUE))
    {
      dy=center-count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    count++;
  }
  return path;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathToMapSouthEdge
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::FindPathToMapSouthEdge(void)
{
  int path=-1;
  BOOL found = FALSE;

  // find a path to one of the four map sides
  int dx=0;
  int dy=0;
  pathMgr.SetPathSize((BYTE)width, (BYTE)height);

  // north side, find empty, non-blocked, 
  // square along north map edge.
  // start with center and move towards edges
  int count = 0;
  int max = MAX_TERRAIN_WIDTH/2;
  int center = MAX_TERRAIN_WIDTH/2;

  dy=MAX_TERRAIN_HEIGHT-1;
  while ((!found) && (count<=max))
  {
    if (!isObstical(center+count, dy, 1, 1, TRUE, TRUE))
    {
      dx=center+count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    else if (!isObstical(center-count,dy, 1, 1, TRUE, TRUE))
    {
      dx=center-count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    count++;
  }
  return path;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathToMapWestEdge
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::FindPathToMapWestEdge(void)
{
  int path=-1;
  BOOL found = FALSE;

  // find a path to one of the four map sides
  int dx=0;
  int dy=0;
  pathMgr.SetPathSize((BYTE)width, (BYTE)height);

  // north side, find empty, non-blocked, 
  // square along north map edge.
  // start with center and move towards edges
  int count = 0;
  int max = MAX_TERRAIN_HEIGHT/2;
  int center = MAX_TERRAIN_HEIGHT/2;

  dx=0;
  while ((!found) && (count<=max))
  {
    if (!isObstical(dx, center+count, 1, 1, TRUE, TRUE))
    {
      dy=center+count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    else if (!isObstical(dx,center-count,1, 1, TRUE, TRUE))
    {
      dy=center-count;
      path = pathMgr.GetPath(x, y, dx, dy, TRUE);
      found = (path >= 0);      
    }
    count++;
  }
  return path;
}

//*****************************************************************************
//    NAME: COMBATANT::FindPathToMapEdge
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::FindPathToMapEdge(void)
{    
  ClearPath();
  if (!CanMove()) return FALSE;
  int shortestpath=-1;
  int shortestdist=INT_MAX;
  int ndist=INT_MAX;
  int edist=INT_MAX;
  int sdist=INT_MAX;
  int wdist=INT_MAX;

  int npath = FindPathToMapNorthEdge();
  if (npath >= 0) ndist = pathMgr.GetStepsRemaining(npath); 
  if (ndist <= 10)
  {
    // close enough, stop searching
    hPath = npath;
    return TRUE;
  }

  int epath = FindPathToMapEastEdge();
  if (epath >= 0) edist = pathMgr.GetStepsRemaining(epath); 
  if (edist <= 10)
  {
    // close enough, stop searching
    hPath = epath;
    return TRUE;
  }
  
  int spath = FindPathToMapSouthEdge();
  if (spath >= 0) sdist = pathMgr.GetStepsRemaining(spath); 
  if (sdist <= 10)
  {
    // close enough, stop searching
    hPath = spath;
    return TRUE;
  }

  int wpath = FindPathToMapWestEdge(); 
  if (wpath >= 0) wdist = pathMgr.GetStepsRemaining(wpath); 
  if (wdist <= 10)
  {
    // close enough, stop searching
    hPath = wpath;
    return TRUE;
  }

  // all are more than 10 squares away, pick the shortest
  if (ndist < shortestdist)
  {
    shortestdist = ndist;
    shortestpath = npath;
  }
  if (edist < shortestdist)
  {
    shortestdist = edist;
    shortestpath = epath;
  }
  if (sdist < shortestdist)
  {
    shortestdist = sdist;
    shortestpath = spath;
  }
  if (wdist < shortestdist)
  {
    shortestdist = wdist;
    shortestpath = wpath;
  }

  if (shortestpath != npath) pathMgr.FreePath(npath);
  if (shortestpath != epath) pathMgr.FreePath(epath);
  if (shortestpath != spath) pathMgr.FreePath(spath);
  if (shortestpath != wpath) pathMgr.FreePath(wpath);

  if (shortestpath >= 0)
  {
    hPath = shortestpath;
    return TRUE;
  }
  return FALSE;
}

//*****************************************************************************
// NAME: COMBATANT::CanUse
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanUse()
{
  if (IsDone()) return FALSE;
  return TRUE;
}


//*****************************************************************************
// NAME: COMBATANT::CanTurnUndead
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanTurnUndead()
{
  if (hasTurnedUndead) return FALSE;
  if (IsDone()) return FALSE;

  if (isCleric(GetClass()))
    return TRUE;

  if ((isPaladin(GetClass())) && (GetCurrentLevel(PaladinFlag) >= 3))
    return TRUE;

  return FALSE;
}


//*****************************************************************************
// NAME: COMBATANT::CanDelay
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanDelay()
{
  if (IsDone()) return FALSE;
  return (initiative < 10);
}
  
 
//*****************************************************************************
// NAME: COMBATANT::DelayAction
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::DelayAction()
{
  if (CanDelay())
  {
    initiative = 10;
    state = ICS_None;
    QueuedCombatantData &qcomb = GetQueuedCombatants();

    if (qcomb.Top() == self)
    {
      TRACE("DelayAction() for %i\n", self);
      // input was disabled if auto combatant
      EnableUserInput(TRUE);
      qcomb.Pop();
    }
  }
}

//*****************************************************************************
// NAME: COMBATANT::GetMaxTargets
//
// PURPOSE:  
//*****************************************************************************
int COMBATANT::GetMaxTargets()
{
  int max = 0;
  int mywpnidx;
  switch (state)
  {
  case ICS_Casting:
    ASSERT(FALSE); // handled elsewhere
    break;

  default:
    mywpnidx = myItems.GetReady(WeaponHand);
    if (mywpnidx != NO_READY_ITEM)
    {
      // item is readied    
     // check for spell abilities
      if (   (state == ICS_Using)
          && (myItems.GetCharges(mywpnidx) > 0))
      {
        ITEM_DATA *data = itemData.GetItemData(myItems.GetItem(mywpnidx));
        if (data != NULL)
        {
          if (data->m_gsID.IsValidSpell())
          {
            // get spell combattargets
          }
        }
      }
      else
      {
        // make sure it is a weapon
        if (isWeapon(myItems.GetItem(mywpnidx)))
        {
          if (GetType() != MONSTER_TYPE)
          {
            // check for sweeps
          }
          else
            max = 1;
        }
      }
    }
    else // using fists, claws, etc.
      max = 1;
    break;
  }

  return max;
}


//*****************************************************************************
// NAME: COMBATANT::CanTarget
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanTarget()
{
  if (IsDone()) return FALSE;

  if ((state == ICS_Casting) || (state == ICS_Using))
  {
    ASSERT(FALSE); // spell targeting uses different data structures
    return FALSE;
  }
  else
    return TRUE;
}

//*****************************************************************************
// NAME: COMBATANT::HaveTarget
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::HaveTarget(int targ)
{
  if (combattargets.GetCount() == 0)
    return FALSE;

  POSITION pos = combattargets.GetHeadPosition();
  while (pos != NULL)
  {
    if (combattargets.GetNext(pos) == targ)
      return TRUE;
  }
  return FALSE;
}


//*****************************************************************************
// NAME: COMBATANT::RemoveTarget
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::RemoveTarget(int targ)
{
  if (combattargets.GetCount() == 0)
    return;

  POSITION pos = combattargets.GetHeadPosition();
  while (pos != NULL)
  {
    if (combattargets.GetAt(pos) == targ)
    {
      combattargets.RemoveAt(pos);
      return;
    }
    else
      combattargets.GetNext(pos);
  }
}

//*****************************************************************************
// NAME: COMBATANT::AddTarget
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::CanAddTarget(int target)
{

  // Probably should add Special Abilities scripts at the end of this function.
  COMBATANT *pTarget = combatData.getCombatantPtr(target);
  if (pTarget->GetIsFriendly() && !targets.m_FriendlyFireOk)
  {
    return FALSE;
  }
  else
  {
    int targrange = distance(self, x, y, 
                             pTarget->self, pTarget->x,pTarget->y);
    if (!targets.CanAddTarget(pTarget->GetNbrHD(), targrange))
    {
      return FALSE;
    }
    else
    {
      if (targets.IsAlreadyTargeted(target))
      {
        return FALSE;
      };
    };
  };
  return TRUE;
}



void COMBATANT::AddTarget(int newTarget) 
{ 
  if (newTarget == NO_DUDE)
    return;

  RemoveTarget(newTarget);

  if (CanTarget())
  {
    // if using weapon, change to new target (only one allowed)
    //if (!combattargets.IsEmpty())
    //  combattargets.RemoveTail();

    COMBATANT *targCOMBATANT;
    targCOMBATANT = GetCombatantPtr(newTarget);
    ASSERT(targCOMBATANT!=NULL);
    if (targCOMBATANT==NULL) return;

    int dist = distance(self, x, y, 
                        targCOMBATANT->self, targCOMBATANT->x, targCOMBATANT->y);  

    combattargets.Insert(newTarget, 
                   MakeTargetKey(newTarget, dist));
  }

  SetCurrTarget(); 
}

//*****************************************************************************
//    NAME: COMBATANT::AddTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::AddTarget(COMBATANT &targ, int range)
{
  ASSERT( IsCombatActive() == TRUE ); // non-combat magic handled in char.cpp
  if (targets.m_SelectingUnits==FALSE) 
  {
    WriteDebugString("AddTarget() fails, SelectingUnits is FALSE\n");
    return FALSE;
  }
  if ((targets.m_FriendlyFireOk==FALSE) && (targ.GetIsFriendly())) 
  {
    WriteDebugString("AddTarget() fails, IsFriendly && FriendlyFireNotAllowed\n");
    return FALSE;
  }
  if (!HaveLineOfSight(GetCenterX(), GetCenterY(),targ.GetCenterX(), targ.GetCenterY()))
  {
    WriteDebugString("AddTarget() fails, no line of sight from caster to target\n");
    return FALSE;    
  }
  return (targets.AddTarget(targ.self, targ.GetNbrHD(), range));
}

//*****************************************************************************
//    NAME: COMBATANT::AddMapTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::AddMapTarget(int mapx, int mapy, PATH_DIR dir)
{
  ASSERT( IsCombatActive() == TRUE ); // non-combat magic handled in char.cpp
  if (targets.m_SelectingUnits==TRUE) return FALSE;

  if (!HaveLineOfSight(GetCenterX(), GetCenterY(), mapx, mapy))
  {
    WriteDebugString("AddMapTarget() fails, no line of sight from caster to mapx,mapy\n");
    return FALSE;    
  }
  
  targets.m_maptarglist.RemoveAll();

  //int range = targets.m_MaxRange;
  if (targets.AddMapTarget(mapx, mapy))
  {    
    // figure out which targets are in range
    MAPTARGETLIST targlist;
    MAPLOCATIONLIST maplist;
    int maxtargs=100000;
    int VisCheckX=0,VisCheckY=0;
    bool screenself=false;
    switch (targets.m_type)
    {
    case AreaCircle: // circle is centered at x,y
      maxtargs = targets.m_MaxTargets;
      GetCombatantsInCircle(targlist, 
                            mapx,mapy, 
                            targets.m_MaxRange);
      GetMapTilesInCircle(maplist,//targets.m_maptarglist, 
                          mapx,mapy, 
                          targets.m_MaxRange);      
      VisCheckX=mapx;
      VisCheckY=mapy;
      break;

    case AreaLinePickStart: // line starts at mapx,mapy, dir is from caster to mapx,mapy
      screenself=true;
      GetCombatantsInLine(targlist,  
                          mapx,mapy,            // start
                          targets.m_MaxRange,   // length
                          targets.m_MaxTargets, // width
                          dir);
      GetMapTilesInLine(maplist,  
                        mapx,mapy,            // start
                        targets.m_MaxRange,   // length
                        targets.m_MaxTargets, // width
                        dir);      
      VisCheckX=mapx;
      VisCheckY=mapy;
      break;

    case AreaLinePickEnd: // line starts at caster, towards selected map location
      screenself=true;
      GetCombatantsInLine(targlist, 
                          this->x,this->y,       // start
                          mapx,mapy,             // end
                          targets.m_MaxTargets); // width
      GetMapTilesInLine(maplist, 
                        this->x,this->y,       // start
                        mapx,mapy,             // end
                        targets.m_MaxTargets); // width      
      VisCheckX=this->x;
      VisCheckY=this->y;
      break;

    case AreaSquare: // square is centered at x,y
      GetCombatantsInRectangle(targlist, 
                               mapx,mapy, 
                               targets.m_MaxTargets,  // width
                               targets.m_MaxRange);   // height                          
      GetMapTilesInRectangle(maplist, 
                             mapx,mapy, 
                             targets.m_MaxTargets,  // width
                             targets.m_MaxRange);   // height                                
      VisCheckX=mapx;
      VisCheckY=mapy;
      break;

    case AreaCone: // cone starts at x,y, extends in 'dir' direction
      screenself=true;
      GetCombatantsInCone(targlist, 
                          mapx,mapy,            // start
                          targets.m_MaxRange,   // length
                          targets.m_MaxTargets, // width at end of cone
                          dir);      
      GetMapTilesInCone(maplist, 
                        mapx,mapy,            // start
                        targets.m_MaxRange,   // length
                        targets.m_MaxTargets, // width at end of cone
                        dir);            
      VisCheckX=mapx;
      VisCheckY=mapy;
      break;

    default: ASSERT(FALSE); break;
    }

    // now add them to the target list
    POSITION pos=targlist.GetHeadPosition();
    int targcount=0;
    while ((pos!=NULL)&&(targcount < maxtargs))
    {
      int index = targets.m_targets.GetSize();
      int dude = targlist.GetAt(pos);

      // target selection code does not care about walls or other map
      // obstructions, so weed out the targets that are not visible from
      // the source location of the spell.
      if (HaveLineOfSight(VisCheckX,VisCheckY, GetCombatantPtr(dude)->x,GetCombatantPtr(dude)->y))
      {
        if (screenself)
        {
          if (dude!=self)
          {
            targets.m_targets.SetAtGrow(index, dude);
            targcount++;
          }
        }
        else
        {
          targets.m_targets.SetAtGrow(index, dude);
          targcount++;
        }
      }

      targlist.GetNext(pos);      
    }

    // now add them to the map target list
    pos=maplist.GetHeadPosition();
    while (pos!=NULL)
    {
      POINT pt = maplist.GetNext(pos);
      
      // target selection code does not care about walls or other map
      // obstructions, so weed out the targets that are not visible from
      // the source location of the spell.
      if (HaveLineOfSight(VisCheckX,VisCheckY, pt.x, pt.y))
        targets.m_maptarglist.AddTail(pt);     
    }

    if (targcount > 0)
    {
      ASSERT( targets.m_maptarglist.GetCount() > 0 );
    }
  }
  return (targets.m_targets.GetSize()>0);
}

//*****************************************************************************
//    NAME: COMBATANT::AddTargetSelf
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::AddTargetSelf()
{
  return (AddTarget(*this, 0));
}

//*****************************************************************************
//    NAME: COMBATANT::AutoChooseSpellTargets
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::AutoChooseSpellTargets()
{

}

//*****************************************************************************
// NAME: COMBATANT::RemoveAllTargets
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::RemoveAllTargets() 
{ 
  combattargets.RemoveAll();
  SetCurrTarget();
}

//*****************************************************************************
// NAME: COMBATANT::SetCurrTarget
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::SetCurrTarget()
{  
  if (!combattargets.IsEmpty())
  {
    targetPos = combattargets.GetHeadPosition();
    target = combattargets.GetAt(targetPos);
  }
  else
  {
    target = NO_DUDE;
    targetPos = NULL;
  }
}

//*****************************************************************************
// NAME: COMBATANT::RemoveCurrTarget
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::RemoveCurrTarget()
{
  if (!combattargets.IsEmpty())
    combattargets.RemoveHead();

  SetCurrTarget();
}


//*****************************************************************************
// NAME: COMBATANT::GetNextTarget
//
// PURPOSE:  
//*****************************************************************************
int COMBATANT::GetNextTarget()
{
  int dude = NO_DUDE;

  if (combattargets.IsEmpty())
    return NO_DUDE;

  if (targetPos == NULL)
  {
    targetPos = combattargets.GetHeadPosition();
  }
  else
  {
    combattargets.GetNext(targetPos);
    if (targetPos == NULL)
      targetPos = combattargets.GetHeadPosition();
  }

  dude = combattargets.GetAt(targetPos);
  return dude;
}

//*****************************************************************************
// NAME:    COMBATANT::FaceOpponent
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::FaceOpponent(int opponent)
{
  COMBATANT *pOpponent;
  pOpponent = GetCombatantPtr(opponent);
  ASSERT(pOpponent != NULL);
  if (pOpponent == NULL) return;
  int ax = pOpponent->x;
  int ay = pOpponent->y;
  PATH_DIR dir;

  // get direction to attacker
  if (x > ax) // nw, w, sw
  {
    if (y > ay) dir = PathNW;
    else if (y < ay) dir =  PathSW;
    else dir = PathWest;
  }
  else if (x < ax) // ne, e, se
  {
    if (y > ay) dir = PathNE;
    else if (y < ay) dir = PathSE;
    else dir = PathEast;
  }
  else // x == ax
  {
    if (y > ay) dir = PathNorth;
    else if (y < ay) dir = PathSouth;
    else  dir = PathBAD; // curr == dest
  }

  // if backstabbing, don't face attacker

  FaceDirection(dir);
}

//*****************************************************************************
//    NAME: COMBATANT::FaceLocation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::FaceLocation(int fx, int fy)
{
  int ax = fx;
  int ay = fy;
  PATH_DIR dir;

  // get direction to x,y
  if (x > ax) // nw, w, sw
  {
    if (y > ay) dir = PathNW;
    else if (y < ay) dir =  PathSW;
    else dir = PathWest;
  }
  else if (x < ax) // ne, e, se
  {
    if (y > ay) dir = PathNE;
    else if (y < ay) dir = PathSE;
    else dir = PathEast;
  }
  else // x == ax
  {
    if (y > ay) dir = PathNorth;
    else if (y < ay) dir = PathSouth;
    else  dir = PathBAD; // curr == dest
  }

  FaceDirection(dir);
}

//*****************************************************************************
//    NAME: COMBATANT::FaceDirection
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::FaceDirection(PATH_DIR dir)
{
  // translate attacker direction into
  // east/west icon facing
  switch (dir)
  {
  case PathNW:
  case PathWest:
  case PathSW:
    m_iFacing = FACE_WEST;
    break;
  case PathNE:
  case PathEast:
  case PathSE:
    m_iFacing = FACE_EAST;
    break;

  default:
    // if north/south attacker, no need to change facing.
    break;
  }
}

//*****************************************************************************
// NAME:    COMBATANT::GetDamageDice
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::GetDamageDice(int wpn, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal, BOOL IsLarge, GLOBAL_SPELL_ID *pSpellDbKey)
{
  *pNonLethal=FALSE;
  pSpellDbKey->Clear();
  
  if (wpn != NO_READY_ITEM)
  {  
     ITEM_DATA *idata = itemData.GetItemData(myItems.GetItem(wpn));

     if (idata != NULL)
     {      
        *pNonLethal = idata->IsNonLethal;

        if (IsLarge)
        {
           *pNum = idata->Nbr_Dice_Lg;
           *pSides = idata->Dmg_Dice_Lg;
           *pBonus = idata->Dmg_Bonus_Lg + GetAdjDmgBonus() + idata->Attack_Bonus;
        }
        else 
        {
           *pNum = idata->Nbr_Dice_Sm;
           *pSides = idata->Dmg_Dice_Sm;
           *pBonus = idata->Dmg_Bonus_Sm + GetAdjDmgBonus() + idata->Attack_Bonus;
        }
     }
     else 
     {
        WriteDebugString("Bogus item num in GetDamageDice()\n");
     }
  }
  else 
  {
    if (GetType()==MONSTER_TYPE)
    {
      int nbrAttacks = GetNbrAttacks();
      int currAttack = nbrAttacks - availAttacks;
      ASSERT( (currAttack>=0) && (currAttack<nbrAttacks) );
      
      MONSTER_DATA *pData = monsterData.GetMonsterData(origKey);
      ASSERT( pData!=NULL );

      if (pData == NULL) 
      {
        *pNum   = unarmedNbrDieL;
        *pSides = unarmedDieL;
        *pBonus = GetAdjDmgBonus();
        return;
      }

      if (pData->attackData.GetCount() == 0)
      {
        WriteDebugString("Monster %s has zero attacks defined\n", pData->Name);
        *pNum   = unarmedNbrDieL;
        *pSides = unarmedDieL;
        *pBonus = GetAdjDmgBonus();
        return;
      }

      if ((currAttack < 0)||(currAttack>=pData->attackData.GetCount()))
        currAttack=0;

      *pNum = pData->attackData.GetAtIndex(currAttack).nbr;
      *pSides = pData->attackData.GetAtIndex(currAttack).sides;
      *pBonus = pData->attackData.GetAtIndex(currAttack).bonus;
      *pSpellDbKey = pData->attackData.GetAtIndex(currAttack).spelldbkey;
    }
    else
    {
      if (IsLarge)
      {
        *pNum   = unarmedNbrDieL;
        *pSides = unarmedDieL;
        *pBonus = GetAdjDmgBonus();
      }
      else 
      {
        *pNum   = unarmedNbrDieS;
        *pSides = unarmedDieS;
        *pBonus = unarmedBonus + GetAdjDmgBonus();
      }
    }
  }
}

//*****************************************************************************
// NAME:    COMBATANT::LoadCombatIcon
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::LoadCombatIcon()
{
  if (icon.filename.IsEmpty()) return;

  icon.FreePic();
  CString prefix;
  switch (GetType())
  {
  case MONSTER_TYPE: prefix="cm_"; break;
  default: prefix="cn_";break;
  }

  // load icon data as a sprite
  if (!icon.LoadPicSurfaces(prefix))
  {
    if (!debugStrings.AlreadyNoted(CString("FTLIF01") + icon.filename))
    {
      WriteDebugString("Failed to load icon file %s in LoadCombatIcon\n", icon.filename);
    };

    // one last try
    icon.filename = DEFAULT_MONSTER_ICON;
    icon.picType=IconDib;
    icon.SetDefaults();
    if (!icon.LoadPicSurfaces(prefix))
    {
      SetMiscError(MissingMonsterCombatIcons);
      if (!debugStrings.AlreadyNoted(CString("FTLDCI01") + icon.filename))
      {
        WriteDebugString("Failed to load default combat icon %s in LoadCombatIcon()\n",
                          icon.filename);
      };
      icon.FreePic();
    }
  }
}

//*****************************************************************************
// NAME:    COMBATANT::determineIconSize
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::determineIconSize()
{
  int w,h,bits;
  if (GraphicsMgr.GetSpriteSurfaceStats(icon.key, w, h, bits))
  {
    int imagewidth=w; // pixel width

    // (width / frame_width) / 2_frames_per_icon
    w = (w / COMBAT_TILE_WIDTH) / 2;

    // 2 frame minimum (one of each pose, rdy/attack)
    // frame count is multiple of 2
    // the icon frame pair used is indicated by iconIndex
    // adjust for number of frames in icon
    w /= (icon.NumFrames/2);
    
    h = h / COMBAT_TILE_HEIGHT;
    
    if (w < 1) w = 1;
    if (h < 1) h = 1;

    width = w;
    height= h;
    int offset = (iconIndex-1)*((width*COMBAT_TILE_WIDTH)*2);
    if ((offset+(width*COMBAT_TILE_WIDTH)) >= (imagewidth-width))
      iconIndex=1;
  }
  else
    icon.FreePic();
}

//*****************************************************************************
// NAME:    COMBATANT::GetAttackBonus
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
int COMBATANT::GetAttackBonus()
{
  int bonus = 0;

  if (myItems.GetReady(WeaponHand) != NO_READY_ITEM)
  {  
     ITEM_DATA *idata = itemData.GetItemData(myItems.GetItem(myItems.GetReady(WeaponHand)));
     if (idata != NULL)
     {      
       bonus += idata->Attack_Bonus;
     }
     else 
     {
        WriteDebugString("Bogus item num in GetAttackBonus()\n");
     }
  }

  bonus += GetAdjHitBonus();

  return bonus;
}

//*e****************************************************************************
// NAME:    COMBATANT::TakeDamage
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::TakeDamage(int dmg, BOOL IsNonLethal)
{  
  // don't beat a dead horse!
  // *but!* allow dying to take 1 hp damage per turn if not yet bandaged
  //
  charStatusType stype = GetAdjStatus();
  if (   (stype == Fled)
      || (stype == Gone)
      || (stype == TempGone)
      || (stype == Dead))
    return;

  if (IsNonLethal) dmg=0;

  // adjust raw hitpoints
  SetHitPoints(GetHitPoints()-dmg);

  int trueHp = GetHitPoints();
  int startTrueHp = trueHp;

  // can't go below -10
  // and 10 = dead
  if (trueHp < -10)
    trueHp = -10;

  if (!IsPartyMember())
  {
    // monster die when hp reaches zero
    if (trueHp <= 0)
      trueHp = -10;
    // unless configured to not die
    if ((trueHp < 1) && (GetConfigMonsterNoDeath())) // debug flag
      trueHp = 1;
    if ((trueHp < 1) && (HasDeathImmunity())) // monster immunity flag
      trueHp = -10; 
  }

  // set to modified value if needed
  if (trueHp != startTrueHp)
    SetHitPoints(trueHp);

  // now get adjusted hp value, which can
  // magically change the current hp
  //
  int adjHp = GetAdjHitPoints();

  if (!IsPartyMember())
  {
    if (adjHp < 0)
      adjHp = -10;

    // still can't let monsters die, even if hp is 
    // magically below 0
    if ((adjHp < 1) && (GetConfigMonsterNoDeath()))
      adjHp = 1;
    if ((adjHp < 1) && (HasDeathImmunity())) // monster immunity flag
      adjHp = -10;     
  }

  if (adjHp <= -10)
  {
    SetStatus(Dead);
    placeCombatant(x, y, NO_DUDE, width, height);
    placeDyingCombatant(x,y,NO_DUDE,width,height);
    TRACE("%i takes %i hp damage, -10 left, and is dead\n", self, dmg);
    blitDeadSprite();
  }
  else if (adjHp <= 0)
  {
    ASSERT( IsPartyMember() );
    placeCombatant(x, y, NO_DUDE, width, height);
    placeDyingCombatant(x,y,self,width,height);
    SetStatus(Dying);
    isBandaged = FALSE;
    bandageWho=-1;
    TRACE("%i takes %i hp damage, %i left, and is dying\n", self, dmg, adjHp);
    blitDyingSprite();
  }

  if (adjHp <= 0)
  {
    // update stats used for monster morale
    if (friendly)
     IncNumFriendSlain();
    else
     IncNumMonsterSlain();

    // if dead, this dude can't do anything else    
    StopCasting(TRUE); // also calls EndTurn()
    movement = GetAdjMaxMovement();          
  }
  else if (dmg > 0) // cancel any pending spell casting if damage taken
    StopCasting(FALSE); // does not call EndTurn()

  UpdateSpellForDamage(dmg);
}

//*****************************************************************************
// NAME:    COMBATANT::PlayHit
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::PlayHit()
{
  if (myItems.GetReady(WeaponHand) != NO_READY_ITEM)
  {  
     itemData.PlayHit(myItems.GetItem(myItems.GetReady(WeaponHand)));
  }
  else
  { 
    if (GetType() == MONSTER_TYPE)
      monsterData.PlayHit(origKey);
    else
      PlayCharHit();
  }
}

//*****************************************************************************
// NAME:    COMBATANT::PlayMiss
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::PlayMiss()
{
  if (myItems.GetReady(WeaponHand) != NO_READY_ITEM)
  {  
     itemData.PlayMiss(myItems.GetItem(myItems.GetReady(WeaponHand)));
  }
  else
  { 
    if (GetType() == MONSTER_TYPE)
      monsterData.PlayMiss(origKey);
    else
      PlayCharMiss();
  }
}

//*****************************************************************************
//    NAME: COMBATANT::PlayLaunch
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::PlayLaunch()
{
  if (myItems.GetReady(WeaponHand) != NO_READY_ITEM)
  {  
     itemData.PlayLaunch(myItems.GetItem(myItems.GetReady(WeaponHand)));
  }
}

//*****************************************************************************
// NAME: canAttack
//
// PURPOSE:  
// check non-magical attack capabilities
//*****************************************************************************
BOOL COMBATANT::canAttack(int targ)
{  
  if (availAttacks <= 0)
    return FALSE;

  if (targ == NO_DUDE)
    return FALSE;

  if (   (availAttacks < 1.0)
     && (CurrCombatRound() - lastAttackRound <= 1))
    return FALSE;

  // cannot attack yourself
  if (targ == self)
    return FALSE;  

  COMBATANT *targCOMBATANT;
  targCOMBATANT = GetCombatantPtr(targ);
  ASSERT(targCOMBATANT!=NULL);
  if (targCOMBATANT==NULL) return FALSE;
  int aX = targCOMBATANT->x;
  int aY = targCOMBATANT->y;

  // if attacking dude on same side of battle
  if (targCOMBATANT->GetIsFriendly() == GetIsFriendly())
  {
    // AI will never attack dudes on same side
    if (OnAuto()) // includes all monsters (always auto), and auto dudes.
    {
      return FALSE;
    }
    else if (targCOMBATANT->GetType() == CHAR_TYPE)
    {
      // cannot attack pc party members with weapon
      return FALSE;
    }
    else if ((targCOMBATANT->GetType() == NPC_TYPE)&&(!targCOMBATANT->IsPreGen))
    {      
      // if player chooses to attack friendly npc, the npc changes sides,
      // even if unable to perform attack due to following checks.
      targCOMBATANT->friendly = !targCOMBATANT->friendly;
      // npc will leave the battle (and the party after combat)?
      //targCOMBATANT.SetMorale(0);
    }
  }

  int dis = distance(self, x, y, targ, aX, aY);  
  int mywpnitemidx = myItems.GetReady(WeaponHand);
  if (mywpnitemidx != NO_READY_ITEM)
  {       
     // using a weapon rather than natural attack types (ie claws/jaws/fists)
     GLOBAL_ITEM_ID wpn_giID = myItems.GetItem(mywpnitemidx);
 
     if (myItems.GetQty(mywpnitemidx) <= 0)
        return FALSE;
 
     if (!WpnCanAttackAtRange(wpn_giID, dis))
       return FALSE;

     switch (itemData.GetWpnType(wpn_giID)) 
     {
     case NotWeapon: // not weapon, must be used, invokes special abilities
        if (isMagical(wpn_giID))
        {
           if (myItems.GetCharges(mywpnitemidx) <= 0)
              return FALSE;
        }
        else
           return FALSE;
        break;

     case HandBlunt: // hand, no throw, blunt
     case HandCutting: // hand, no throw, cutting
     case HandThrow: // hand or throw
     case SlingNoAmmo: // sling
     case SpellCaster:
       //
       // handled by WpnCanAttackAtRange()
       //
       break;

     case Bow: // bow
     case Crossbow: // crossbow
       {
         int myammoitemidx = myItems.GetReady(AmmoQuiver);         

         // arrows or bolts must be readied in AmmoQuiver
         if (myammoitemidx == NO_READY_ITEM)
           return FALSE;

         GLOBAL_ITEM_ID ammo_giID = myItems.GetItem(myammoitemidx);

         if (itemData.GetWpnType(ammo_giID) != Ammo)
           return FALSE;

         // ammo class must match between weapon and ammo
         CString myammoclass(myItems.GetAmmoClass(myammoitemidx));
         if (myammoclass.IsEmpty()) return FALSE; // might be wand,potion,amulet,etc

         if (myammoclass != myItems.GetAmmoClass(mywpnitemidx))
           return FALSE;

         // must have non-zero qty of ammo
         if (myItems.GetQty(myammoitemidx) <= 0)
           return FALSE;
       }
       break;

     case Throw:
       //
       // handled by WpnCanAttackAtRange()
       //
       /*
        if (dis < 2)
           return FALSE;
        */
        break;

     case Ammo: // ammo - arrows or bolts
        return FALSE;

     default:
        WriteDebugString("Unhandled item Wpn_Type in canAttack()\n");
        return FALSE;
     }
  }
  else
  {
     // use natural attack weapons (ie claws, jaws, paws, fists)
     if (dis > 1)
        return FALSE;
  }
 
  // passed all tests so far, now check for line of sight
  if (!HaveLineOfSight(GetCenterX(), GetCenterY(), aX, aY))
    return FALSE;

  if (dis > 1)
  {
    if (!GetAdjDetectingInvisible())
    {
      // cannot attack invisible targets with ranged weapons
      if (targCOMBATANT->GetAdjSpecAb(SA_Invisible))
        return FALSE;

      if (GetUndeadType() != NotUndead)
      {
        if (targCOMBATANT->GetAdjSpecAb(SA_InvisibleToUndead))
          return FALSE;
      }

      if (IsAnimal())
      {
        if (targCOMBATANT->GetAdjSpecAb(SA_InvisibleToAnimals))
          return FALSE;
      }
    }
  }

  return TRUE;
}

//*****************************************************************************
// NAME:    COMBATANT::moveNW
//
// PURPOSE: Change the facing of the icon to the direction of the keypress
//			and move the icon in the stated direction
//*****************************************************************************
void COMBATANT::moveNW()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_WEST;
   moveX = x-1;
   moveY = y-1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveNE
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveNE()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_EAST;
   moveX = x+1;
   moveY = y-1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveSE
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveSE()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_EAST;
   moveX = x+1;
   moveY = y+1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveSW
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveSW()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_WEST;
   moveX = x-1;
   moveY = y+1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveForward
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveForward()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();
   moveX = x;
   moveY = y-1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveBackward
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveBackward()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();
   moveX = x;
   moveY = y+1;
}

//*****************************************************************************
// NAME:    COMBATANT::moveLeft
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveLeft()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_WEST;
   moveX = x-1;
   moveY = y;
}

//*****************************************************************************
// NAME:    COMBATANT::moveRight
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::moveRight()
{
   StopCasting(FALSE);
   state = ICS_Moving;
   //combatData.SetUpdateFlag();

   m_iFacing = FACE_EAST;
   moveX = x+1;
   moveY = y;
}

//*****************************************************************************
// NAME:    COMBATANT::MoveCombatant
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBATANT::MoveCombatant(int newX, int newY)
{
  int dude;
  BOOL isUpdate = FALSE;

  // if already at destination
  if ((newX == x) && (newY == y))
    return FALSE;

  if (GetType() == MONSTER_TYPE)
  {
    if (GetConfigMonsterNoMove())
      return FALSE;
  }

  PATH_DIR dir = GetDir(x, y, newX, newY);
  int movePoints = GetDist(dir);
  FaceDirection(dir);

  if (movement < GetAdjMaxMovement()-(movePoints-1))
  {     
    if (coordsOnMap(newX, newY, width, height))
    {     
      dude = getCombatantInCell(newX, newY, width, height, self);      

      if (dude != NO_DUDE)
      {        
        if (canAttack(dude))
        {
          AddTarget(dude);
          StartAttack(dude);
          isUpdate = TRUE;
        }
        /*
        else
        {
          // unable to take step, or to attack
          // dude blocking path, or no attacks remaining
          if (OnAuto())
          {
            // maybe should call Think() here?
            TRACE("Auto combatant %i cannot attack targ in path, end turn + guard\n", self);
            EndTurn(CS_Guarding); // sets state to none, done=true
          }
        }
        */
      }
      // check for walls and such
      else if (!isObstical(newX, newY, width, height, FALSE, TRUE))
      {        
        // if moving away from an enemy that has no target,
        // the enemy gets to make an attack at this dude's
        // retreating backside.

        PlayCombatMove();        
        movement += movePoints;

        // remove dude from old location
        placeCombatant(x, y, NO_DUDE, width, height);

        TRACE("MoveCombatant() from %u,%u to %u,%u\n",
                    x,y,newX,newY);
      
        if ((abs(newX-x) > 1) || (abs(newY-y) > 1))
          TRACE("Moving more than 1 square\n");

        x = newX;
        y = newY;

        // place dude at new location
        placeCombatant(x, y, self, width, height);
        {

          SetCharContext(GetContext());

          PrepareHooks();
          //hookParameters[5].Format("%d",dmg);
          //hookParameters[6].Format("%d",IsNonLethal);
          scriptContext.SetCombatantContext(this);


          combatantSA.RunScripts(
                        ON_STEP,
                        ScriptCallback_LookForChar,
                        "F",
                        "Combatant Move",
                        "");
          scriptContext.Clear();
          ClearCharContext();
        };

        if (activeSpellData.LingerSpellAffectsTarget(self, x,y))
        {
          // A lingering spell is active at the location 
          // just moved to. 
          TRACE("%i moved into at least one active linger spell at location %i,%i\n",
                self, x,y);

          activeSpellData.ActivateLingerSpellsOnTarget(self, x,y);
        }

        // if just moved into square with adjacent guarding
        // enemy, the enemy gets to make an attack
        if (CheckForGuardingEnemy())
        {
          // This guys turn isn't over, but once the free
          // attacks are over, he will not start out in CS_Move.
          state = ICS_None;
        }
        isUpdate = TRUE;
      }      
    }
    else 
    { 
       isUpdate = TRUE;
       // dude fled map
       SetStatus(Fled);
       if (friendly)
         IncNumFriendFlee(); //combatData.numFrndFlee++;
       else
         IncNumMonsterFlee(); //combatData.numMonFlee++;
       EndTurn();
    }
  }

  if ((isUpdate) && (charOnCombatMap()))
  {    
    ForceMapToMoveAlong(x, y, dir, max(width, height));
    PlaceCursorOnCurrentDude();  
  }

  return isUpdate;
}

//*****************************************************************************
//    NAME: COMBATANT::CheckForGuardingEnemy
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::CheckForGuardingEnemy()
{
  // check clockwise around current location
  // looking for enemy in guarding state that
  // can make an attack.
  BOOL found = FALSE;
  int dude;
  POSITION pos;
  int d = PathNorth;

  COMBATANT *tempCOMBATANT;
  QueuedCombatantData &qcomb = GetQueuedCombatants();

  while (d <= PathNW)
  {
    FillTargeterList((PATH_DIR)d);
    pos = targeters.GetHeadPosition();
    while (pos != NULL)
    {
      dude = targeters.GetAt(pos);
      tempCOMBATANT = GetCombatantPtr(dude);
      ASSERT(tempCOMBATANT!=NULL);
      if (tempCOMBATANT==NULL) return FALSE;

      if (tempCOMBATANT!=NULL)
      {
        if (   (tempCOMBATANT->GetIsFriendly() != GetIsFriendly())          
            && (tempCOMBATANT->state == ICS_Guarding)
            && (tempCOMBATANT->canAttack(self)))
        {
          // queue up the attackers
          qcomb.Push(dude, FALSE);
          // give self as target to each attacker
          tempCOMBATANT->AddTarget(self);        
          // set each attacker in attack mode (if possible)
          tempCOMBATANT->StartAttack(self);
          found = TRUE;
        }
      }
      targeters.GetNext(pos);
    }
    d++;
  }
  return found;
}

//*****************************************************************************
//    NAME: COMBATANT::FillTargeterList
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::FillTargeterList(PATH_DIR dir)
{
  targeters.RemoveAll();
  int i;
  int dude;

  switch (dir)
  {
  case PathNorth:
    for (i=x; i < x+width; i++)
    {
      if (coordsOnMap(i, y-1, 1, 1))
      {      
        if ((dude = getCombatantInCell(i, y-1, 1, 1, self)) != NO_DUDE)
          targeters.AddTail(dude);
      }
    }
    break;
  case PathNE:
    if (coordsOnMap(x+width, y-1, 1, 1))
    {      
      if ((dude = getCombatantInCell(x+width, y-1, 1, 1, self)) != NO_DUDE)
        targeters.AddTail(dude);
    }
    break;

  case PathEast:
    for (i=y; i < y+height; i++)
    {
      if (coordsOnMap(x+width, i, 1, 1))
      {      
        if ((dude = getCombatantInCell(x+width, i, 1, 1, self)) != NO_DUDE)
          targeters.AddTail(dude);
      }
    }
    break;
  case PathSE:
    if (coordsOnMap(x+width, y+height, 1, 1))
    {      
      if ((dude = getCombatantInCell(x+width, y+height, 1, 1, self)) != NO_DUDE)
        targeters.AddTail(dude);
    }
    break;
  case PathSouth:
    for (i=x; i < x+width; i++)
    {
      if (coordsOnMap(i, y+height, 1, 1))
      {      
        if ((dude = getCombatantInCell(i, y+height, 1, 1, self)) != NO_DUDE)
          targeters.AddTail(dude);
      }
    }
    break;
  case PathSW:
    if (coordsOnMap(x-1, y+height, 1, 1))
    {      
      if ((dude = getCombatantInCell(x-1, y+height, 1, 1, self)) != NO_DUDE)
        targeters.AddTail(dude);
    }
    break;
  case PathWest:
    for (i=y; i < y+height; i++)
    {
      if (coordsOnMap(x-1, i, 1, 1))
      {      
        if ((dude = getCombatantInCell(x-1, i, 1, 1, self)) != NO_DUDE)
          targeters.AddTail(dude);
      }
    }
    break;
  case PathNW:
    if (coordsOnMap(x-1, y-1, 1, 1))
    {      
      if ((dude = getCombatantInCell(x-1, y-1, 1, 1, self)) != NO_DUDE)
        targeters.AddTail(dude);
    }
    break;
  default:
    break;
  }
}
//*****************************************************************************
// NAME: charOnCombatMap
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::charOnCombatMap()
{
  charStatusType stype = GetAdjStatus();
   if (   (charUnconscious())
       || (stype == Fled)
       || (stype == Gone)
       || (stype == TempGone)
       || (stype == Dead))
     return FALSE;

  return TRUE;
}

//*****************************************************************************
//    NAME: COMBATANT::charUnconscious
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::charUnconscious()
{
  charStatusType stype = GetAdjStatus();
  if ((stype == Dying) || (stype == Unconscious)) return TRUE;
  return FALSE;
}

//*****************************************************************************
//    NAME: COMBATANT::charCanTakeAction
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::charCanTakeAction()
{
  charStatusType stype = GetAdjStatus();
   if ((stype == Okay) || (stype == Running) || (stype == Animated))
     return TRUE;
  return FALSE;  
}

//*****************************************************************************
// NAME:    COMBATANT::RestoreToParty
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::RestoreToParty()
{
  if (!IsPartyMember())
    return;

  if (   ((GetType() == CHAR_TYPE) || (GetType() == NPC_TYPE))
      && (origKey >= 0))
  {     
    // need to restore original morale to characters
    // so that they begin each battle with normal morale.
    int origMorale = party.characters[origKey].GetMorale();

    // unless of course if the NPC has decided to leave
    // the party
    if ((GetType() == NPC_TYPE) && (!IsPreGen) && (GetMorale() <= 0)) 
      origMorale = 0;

    // if PartyNeverDies flag is set and dude has 0 or less hp,
    // restore to original hp with Okay status
    if (GetConfigPartyNeverDies())
    {
      SetStatus(party.characters[origKey].GetStatus());
      if (GetHitPoints() <= 0) SetHitPoints(1);
    }

    party.characters[origKey] = *((CHARACTER*)(this));
    party.characters[origKey].SetMorale(origMorale);
    
    // if dying when combat ends (between -10 and 0 hitpoints)
    // set to 0 hp and unconscious status
    if (   (party.characters[origKey].GetStatus() == Dying)
        || (party.characters[origKey].GetStatus() == Unconscious))
    {
      party.characters[origKey].SetStatus(Unconscious);
      if (GetHitPoints() < 0) 
        SetHitPoints(0);
    }

    // spells may have altered control status
    party.characters[origKey].SetAllowPlayerControl((GetType()==CHAR_TYPE) || (IsPreGen)); 
  }
}


//*****************************************************************************
// NAME: COMBATANT::DetermineInitialFacing
//
// PURPOSE:  
//*****************************************************************************
int COMBATANT::DetermineInitialFacing(int PartyFacing)
{   
  switch (PartyFacing)
  {
  case FACE_NORTH:
  case FACE_SOUTH:
    if (rollDice(100, 1, 0) <= 50) 
      return FACE_EAST;
    else 
      return FACE_WEST;

  case FACE_EAST:
    return FACE_EAST;
  case FACE_WEST:
    return FACE_WEST;
  }
  return FACE_EAST;
}

//*****************************************************************************
// NAME:    COMBATANT::InitFromCharData
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::InitFromCharData(int dude)
{
  *((CHARACTER*)(this)) = party.characters[dude];

  SetPartyMember(TRUE);
  state = ICS_None;
  origKey = dude;
  //WriteDebugString("add 1: origIndex %i, origKey %i\n", origIndex, origKey);
  movement = 0;

  friendly = TRUE;  
  // NPC in party cannot be controlled in combat unless it is a 
  // pre-generated character
  SetAllowPlayerControl( (GetType()==CHAR_TYPE) || (IsPreGen) );
  m_iFacing = DetermineInitialFacing(party.facing);

  determineNbrAttacks(); // lookup max possible defaults
  if (GetAdjAutomatic())
  {
    ReadyBestArmor();
    ReadyBestWpn(NO_DUDE);
    ReadyBestShield();  
  }
  determineAvailAttacks(); // now take ready wpn into account
  //WriteDebugString("add 2: origIndex %i, origKey %i\n", origIndex, origKey);
}

//*****************************************************************************
// NAME:    COMBATANT::InitFromMonsterData
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::InitFromMonsterData(int dude, BOOL IsFriendly, ITEM_LIST &items, MONEY_SACK &msack)
{
  MONSTER_DATA *data;

  state = ICS_None;
  if ((data = monsterData.GetMonsterData(dude)) == NULL)
  {
    WriteDebugString("Bogus monster index %u in InitFromMonsterData()\n", dude);    
    Clear();
    origKey = NO_DUDE;
    return;
  }

  //partyMember = FALSE;
  SetPartyMember(FALSE);
  SetType(MONSTER_TYPE);
  origKey =  dude;
  origIndex = dude;
  friendly = IsFriendly; 

  generateNewCharacter(data->XP_Value, START_EXP_VALUE); // determines monster AC,THACO,Hit Points,etc
  SetStatus(Okay);
  //SetAutomatic(!combatData.MonsterPlyrControl);
  SetAllowPlayerControl(GetConfigMonsterPlyrControl());

  SetName(data->Name);
  SetInt(data->Intelligence);
  SetMagicResistance(data->Magic_Resistance);
  SetSize(data->Size);
  SetMorale(data->Morale);  
  ItemMask = data->ItemMask;
  specAbs.Copy(data->specAbs);  
  SetUndead(data->undeadStatus);

  // items specified in combat event for this monster
  myItems = items;

  // default monster items in monster database
  POSITION pos = data->myItems.GetHeadPosition();
  while (pos!=NULL)
  {
    myItems.AddItem(data->myItems.GetNext(pos));
  }

  money = msack; // combat additional money
  money.Add(data->money); // monster default money

  icon = data->Icon;
  determineNbrAttacks();
  ReadyBestArmor();
  ReadyBestWpn(NO_DUDE);
  ReadyBestShield();  
  determineAvailAttacks();
  determineMaxMovement();

  if (GetConfigMonsterNoMove())
  {
    SetMaxMovement(0);
    movement = 0;
  }

  ASSERT(GetAdjHitPoints() > 0);
}

//*****************************************************************************
// NAME: COMBATANT::InitFromNPCData
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::InitFromNPCData(int dude, BOOL IsFriendly, ITEM_LIST &items, MONEY_SACK &msack)
{
  CHARACTER data;
  if (!globalData.charData.GetCharacterData(dude, data))
  {
    WriteDebugString("invalid NPC combatant index %i in InitFromNPCData()\n", dude);
    Clear();
    origKey = NO_DUDE;
    return;
  }

  *((CHARACTER*)(this)) = data;

  POSITION pos = items.GetHeadPosition();
  while (pos != NULL)
    myItems.AddItem(items.GetNext(pos));

  money = msack;
  //partyMember = FALSE;
  SetPartyMember(FALSE);
  state = ICS_None;
  origKey = dude;
  origIndex = dude;
  SetType(NPC_TYPE);
  friendly = IsFriendly;
  movement = 0;
  SetStatus(Okay);
  //SetAutomatic(!combatData.MonsterPlyrControl);
  SetAllowPlayerControl(GetConfigMonsterPlyrControl());
  determineNbrAttacks();
  ReadyBestArmor();
  ReadyBestWpn(NO_DUDE);
  ReadyBestShield();  
  determineAvailAttacks();
  determineMaxMovement();

  if (GetConfigMonsterNoMove())
  {
    SetMaxMovement(0);
    movement = 0;
  }
}

//*****************************************************************************
// NAME: determineAvailAttacks
//
// PURPOSE:  
//*****************************************************************************
void COMBATANT::determineAvailAttacks()
{
  determineNbrAttacks(); // lookup max attacks
  availAttacks = GetNbrAttacks();

  float temp = availAttacks;
  TRACE("Setting num attacks for %i to %f\n", self, temp);     
}

//*****************************************************************************
// NAME: isLargeDude
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBATANT::isLargeDude()
{
  creatureSizeType ctype = GetAdjSize();

  if (ctype == Large) return TRUE;
  if ((width > 1) || (height > 1)) return TRUE;
  if (IsAlwaysLarge()) return TRUE;
  
  return FALSE;
}

//*****************************************************************************
// NAME:    COMBATANT::RollInitiative
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBATANT::RollInitiative(eventSurpriseType eSurprise)
{
  BOOL partymember = (IsPartyMember() || friendly);
  switch (eSurprise)
  {
  case Neither:
    initiative = rollDice(10,1,0);
    if (GetAdjSpecAb(SA_Blindness))
      initiative += 2;
    break;
  case PartySurprised: // monsters go first
    initiative = (partymember ? 10 : 1);
    break;
  case MonsterSurprised: // party members go first
    initiative = (partymember ? 1 : 10);
    break;
  }

  if (initiative < 1) initiative = 1;
  if (initiative > 10) initiative = 10;
}

//*****************************************************************************
//    NAME: COMBATANT::GetCenterX
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::GetCenterX()
{
  if (width<=1) return x;
  if (m_iFacing == FACE_WEST)
    return (x+(width/2)-1);
  else
    return (x+(width/2));
}

//*****************************************************************************
//    NAME: COMBATANT::GetCenterY
//
// PURPOSE: 
//
//*****************************************************************************
int COMBATANT::GetCenterY()
{
  if (height<=1) return y;
  return (y+(height/2)-1);
}

CBRESULT ScriptCallback_CharCombatStatus(CBFUNC func, CString *str, void *pkt)
{
  CString *pResult;
  pResult = (CString *)pkt;
  if (func == CBF_EXAMINESCRIPT)
  {
    if (!str->IsEmpty())
    {
      *pResult = *pResult + *str;
      pResult += '\n';
    };
  };
  return CBR_CONTINUE;
}

//*****************************************************************************
//    NAME: COMBATANT::OnStartCombatantAction
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::OnStartCombatantAction()
{
  TRACE("OnStartCombatantAction for %i\n", self);
  ClearQueuedSpecAb();
  PlaceCursorOnCurrentDude();
  
  CombatMsg="";
  ClearFormattedText(combatTextData);
  CString tmp;

  SetCharContext(GetContext());
  PrepareHooks();
  scriptContext.SetCharacterContext(this);
  scriptContext.SetCombatantContext(this);
  specAbs.RunScripts(CHAR_COMBAT_STATUS, ScriptCallback_CharCombatStatus, &CombatMsg, name, "");
  combatantSA.RunScripts(CHAR_COMBAT_STATUS, ScriptCallback_CharCombatStatus, &CombatMsg, name, "");
  combatantSA.RunScripts(ON_START_COMBATANT_ACTION, ScriptCallback_RunAllScripts, NULL, "OnStartCombatantAction",name);
  scriptContext.Clear();
  ClearCharContext();

  /*
  
  if (GetAdjSpecAb(SA_Curse))
  {
    tmp.Format(" \nCursed");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_CharmPerson))
  {
    tmp.Format(" \nCharmed");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_Sleep))
  {
    tmp.Format(" \nSleeping");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_Entangle))
  {
    tmp.Format(" \nEntangled");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_HoldPerson))
  {
    tmp.Format(" \nHeld");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_Silence))
  {
    tmp.Format(" \nSilenced");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_Poisoned))
  {
    tmp.Format(" \nPoisoned");
    CombatMsg += tmp;
  }
  if (GetAdjSpecAb(SA_Blindness))
  {
    tmp.Format(" \nBlinded");
    CombatMsg += tmp;
  }
  */
  if (!CombatMsg.IsEmpty())
  {
    FormatCombatText(combatTextData, CombatMsg);
    UpdateCombatTimeDelayMsg();
    theApp.AppSleep(GetMsgTimeDelay());
    //ClearFormattedText(combatTextData);
  }
}

//*****************************************************************************
//    NAME: COMBATANT::OnEndTurn
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::OnEndTurn()
{
  TRACE("OnEndTurn for %i\n", self);
}

//*****************************************************************************
//    NAME: COMBATANT::OnEnsureVisible
//
// PURPOSE: 
//
//*****************************************************************************
void COMBATANT::OnEnsureVisible()
{
  TRACE("OnEnsureVisible for %i\n", self);
}

//*****************************************************************************
//    NAME: COMBATANT::FormatSpecAbMsg
//
// PURPOSE: 
//
//*****************************************************************************
CString COMBATANT::FormatSpecAbMsg(DWORD sa_state)
{
  if (sa_state == SPECAB_MSG_NONE) return "";
  if (specAbQueue.GetCount()==0) return "";

  CString msg("");

  BOOL found=FALSE;
  POSITION pos = specAbQueue.GetHeadPosition();
  while (pos != NULL)
  {
    SQueuedSpecAb data = specAbQueue.GetAt(pos);
    specialAbilitiesType stype = (specialAbilitiesType)(data.sa);

    SPECIAL_ABILITIES *pSpecAbSrc=NULL;
    switch (data.src)
    {
    case SPELL_EFFECTS_DATA::EFFECT_ITEMSPECAB:
      {
        GLOBAL_ITEM_ID giID;
        giID = data.SourceItem_ID();
        ITEM_DATA *pItem = itemData.GetItemData(giID);
        if (pItem!=NULL) pSpecAbSrc = &pItem->specAbs;
      }
      break;
    case SPELL_EFFECTS_DATA::EFFECT_SPELLSPECAB:
      {
        SPELL_DATA *pSpell = spellData.GetSpellData(data.SourceSpell_ID());
        if (pSpell!=NULL) pSpecAbSrc = &pSpell->specAbs;
      }
      break;
    case SPELL_EFFECTS_DATA::EFFECT_CHARSPECAB:
      {
        if (data.SourceChar_ID()==self)
          pSpecAbSrc=&specAbs;
      }
      break;
    }

    if (pSpecAbSrc!=NULL)
    {
      if (pSpecAbSrc->GetMsgTypes(stype) & sa_state)
      {
        msg = pSpecAbSrc->GetMsgFormat(stype, sa_state);        
        if (msg.GetLength()>0)
        {
          specAbQueue.RemoveAt(pos);
          found=TRUE;
          break;
        }
      }
    }
    specAbQueue.GetNext(pos);
  }

  if (!found) return "";

  CString User("");
  CString Target("");

  User = GetName();
  
  int targ = m_iLastAttacked;//GetCurrTarget(); ** targ may be dead by this time
  if (targ != NO_DUDE)
  {
    COMBATANT *tempCOMBATANT=GetCombatantPtr(targ);
    if (tempCOMBATANT!=NULL)
      Target = tempCOMBATANT->GetName();
  }
  else
  {
    Target="the enemy";
  }

  msg.Replace("/u", User);
  msg.Replace("/t", Target);
  return msg;
}

//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackRollDice
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackRollDice(CHARACTER *pTarget,int *pNum, int *pSides, int *pBonus)
{
  return CHARACTER::ModifyAttackRollDice(pTarget,pNum, pSides, pBonus);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackDamageDice
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackDamageDice(CHARACTER *pTarget,int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal)
{
  return CHARACTER::ModifyAttackDamageDice(pTarget, pNum, pSides, pBonus, pNonLethal);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyACAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyACAsTarget(CHARACTER *pAttacker, int *pAC, GLOBAL_ITEM_ID giID)
{
  DWORD src;
  GLOBAL_SPELL_ID skey;
  BOOL modify = CHARACTER::ModifyACAsTarget(pAttacker, pAC, giID);

  if (GetAdjSpecAb(SA_Displacement,&src, &skey))
  {
    COMBATANT *pa = (COMBATANT*)pAttacker;
    QueueUsedSpecAb(SA_Displacement,src,skey);
    modify=TRUE;
    if (pa->self != m_iLastAttacker) // if first attack on me
      *pAC = 21; // force a miss
    else
      (*pAC) -= 2; // +2 bonus to armor class
  }
  
  if (GetAdjSpecAb(SA_MirrorImage,&src, &skey))
  {
    QueueUsedSpecAb(SA_MirrorImage,src,skey);
    // decrement counter used to keep track of number of mirror images
    //AdjustEffectData("$CHAR_MIRRORIMAGE", -1);
    modify=TRUE;
    *pAC = 21; // force a miss
  }

  if (GetAdjSpecAb(SA_Blindness,&src, &skey))
  {
    QueueUsedSpecAb(SA_Blindness,src,skey);
    modify=TRUE;
    (*pAC) += 4;
  }
  
  if (race() == Dwarf)
  {
    if (pAttacker->HasDwarfACPenalty())
    {
      (*pAC) -= 4; // +4 bonus to armor class
      modify=TRUE;
    }
  }
  else if (race() == Gnome)
  {
    if (pAttacker->HasGnomeACPenalty())
    {
      (*pAC) -= 4; // +4 bonus to armor class    
      modify=TRUE;
    }
  }
  return modify;
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackThaco
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackThaco(CHARACTER *pTarget,int *pVal)
{
  DWORD src;
  GLOBAL_SPELL_ID skey;
  BOOL modify = CHARACTER::ModifyAttackThacoAsTarget(pTarget, pVal);

  if (pTarget->GetAdjSpecAb(SA_Sanctuary,&src, &skey))
  {
    if (!didCharacterSaveVs(Sp,0))
    {
      *pVal = 50; // cannot hit target
      RemoveAllTargets();
      pTarget->QueueUsedSpecAb(SA_Sanctuary,src,skey);
      modify=TRUE;
    }
  }
  
  if (GetAdjSpecAb(SA_Blindness,&src, &skey))
  {
    QueueUsedSpecAb(SA_Blindness,src,skey);
    modify=TRUE;
    (*pVal) += 4; // -4 penalty to THACO if blinded
  }

  if (race() == Dwarf)
  {
    if (pTarget->HasDwarfTHACOPenalty())
    {
      (*pVal) -= 1; // +1 bonus to THACO
      modify=TRUE;
    }
  }
  else if (race() == Gnome)
  {
    if (pTarget->HasGnomeTHACOPenalty())
    {
      (*pVal) -= 1; // +1 bonus to THACO
      modify=TRUE;
    }
  }
  
  return modify;
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackRollDiceAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackRollDiceAsTarget(CHARACTER *pAttacker,int *pNum, int *pSides, int *pBonus)
{
  return CHARACTER::ModifyAttackRollDiceAsTarget(pAttacker, pNum, pSides, pBonus);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackDamageDiceAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackDamageDiceAsTarget(CHARACTER *pAttacker, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal)
{
  return CHARACTER::ModifyAttackDamageDiceAsTarget(pAttacker, pNum, pSides, pBonus, pNonLethal);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackThacoAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackThacoAsTarget(CHARACTER *pAttacker,int *pVal)
{
  return CHARACTER::ModifyAttackThacoAsTarget(pAttacker, pVal);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifySaveRoll
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifySaveRoll(CHARACTER *pTarget, int *pRoll)
{
  return CHARACTER::ModifySaveRoll(pTarget, pRoll);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifySaveRollAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifySaveRollAsTarget(CHARACTER *pAttacker, int *pRoll)
{
  return CHARACTER::ModifySaveRollAsTarget(pAttacker, pRoll);
}
//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackRollDiceForItem
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackRollDiceForItem(CHARACTER *pTarget, GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus)
{
  return CHARACTER::ModifyAttackRollDiceForItem(pTarget, giID, pNum, pSides, pBonus);
}

//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackRollDiceForItemAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackRollDiceForItemAsTarget(CHARACTER *pAttacker, GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus)
{
  return CHARACTER::ModifyAttackRollDiceForItemAsTarget(pAttacker, giID, pNum, pSides, pBonus);
}

//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackDamageDiceForItem
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackDamageDiceForItem(CHARACTER *pTarget, GLOBAL_ITEM_ID giID,int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal)
{
  return CHARACTER::ModifyAttackDamageDiceForItem(pTarget, giID, pNum, pSides, pBonus, pNonLethal);
}

//*****************************************************************************
//    NAME: COMBATANT::ModifyAttackDamageDiceForItemAsTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::ModifyAttackDamageDiceForItemAsTarget(CHARACTER *pAttacker, GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal)
{
  ITEM_DATA *pItem = itemData.GetItemData(giID);
  if (pItem==NULL) return FALSE;

  DWORD src;
  GLOBAL_SPELL_ID skey;
  BOOL modify = CHARACTER::ModifyAttackDamageDiceForItemAsTarget(pAttacker, giID, pNum, pSides, pBonus, pNonLethal);
  
  COMBATANT *pa = (COMBATANT*)pAttacker;
  // if vorpal item readied (attacker has ability enabled), and using item that confers ability
  if (   (pa->m_iAttDiceRoll==20)
      && (pa->GetAdjSpecAb(SA_VorpalAttack,&src, &skey))
      && (pItem->specAbs.HaveSpecAb(SA_VorpalAttack)))
  {    
    if (!HasVorpalImmunity())
    {
      *pBonus = GetHitPoints()+1; // make sure this attack kills target
      pa->QueueUsedSpecAb(SA_VorpalAttack,src,skey);
      modify=TRUE;
    }
  }
  return modify;
}

//*****************************************************************************
//    NAME: COMBATANT::GetIsFriendly
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBATANT::GetIsFriendly()
{
  BOOL AdjFriendly = GetAdjSpecAb(SA_CharmPerson);
  if (AdjFriendly) 
    return (!friendly);
  else
    return (friendly);
}