/******************************************************************************
* Filename: Combatant.h
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/
#ifndef __COMBATANT_H__
#define __COMBATANT_H__

#include "SharedQueue.h"
#include "monster.h"
#include "spell.h"
#include "Path.h"

// make sure this enum has the same order as CombatStateType
//
enum individualCombatantState
{ 
  ICS_None, 
  ICS_Casting, 
  ICS_Attacking, 
  ICS_Guarding, 
  ICS_Bandaging, 
  ICS_Using,
  ICS_Moving,
  ICS_Turning,
  ICS_Fleeing,
  ICS_Fled,
};

enum LASTACTION
{
  LA_None = 0,
  LA_Attack,
  LA_Defend
};

const int NUM_COMBATANT_STATES = 10;
extern char *CombatantStateText[NUM_COMBATANT_STATES];

enum FLEEING_FLAGS
{
  fleeBecauseOffMap     = 1,
  fleeBecauseImpossible = 2,  // Never used!
};


struct COMBATANT : public CHARACTER
{
  COMBATANT() : CHARACTER() { Clear(); }
  ~COMBATANT() {  }
  COMBATANT& operator=(COMBATANT& src);
  
  enum { MAX_COMBAT_TARGETS=100 };

  COMBATANT *GetCombatant(void){return this;};

  individualCombatantState GetState() { return state; }
  int GetUniqueId() { return self; }

  void Clear();
  void InitFromCharData(int dude);
  void InitFromMonsterData(int dude, BOOL IsFriendly, ITEM_LIST &items, MONEY_SACK &msack);
  void InitFromNPCData(int dude, BOOL IsFriendly, ITEM_LIST &items, MONEY_SACK &msack);
  void determineAvailAttacks();
  void LoadCombatIcon();
  void determineIconSize();
  BOOL isLargeDude();
  void RollInitiative(eventSurpriseType surprise);
  BOOL charOnCombatMap();
  BOOL charUnconscious();
  BOOL charCanTakeAction();
  void GetDamageDice(int wpn, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal, BOOL IsLarge, GLOBAL_SPELL_ID *pSpellDbKey);
  int GetAttackBonus();
  ActorType GetContext(WORD ClassFlag=0);

  BOOL CastSpell(GLOBAL_SPELL_ID spell);
  void SpellActivate(PENDING_SPELL &data);
  void InstantSpellActivate(GLOBAL_SPELL_ID spelldbkey, int target);    
  BOOL IsSpellPending();

  // one time only, used for temp bonuses on such attacks as backstabbing
  void SetTempAttackBonus(int att, int dmg) { m_iTmpAttDiceBonus=att;m_iTmpDmgDiceBonus=dmg; }

  void FaceOpponent(int opponent);
  void FaceLocation(int x, int y);
  void FaceDirection(PATH_DIR dir);
  BOOL CheckForGuardingEnemy();
  void FillTargeterList(PATH_DIR dir);
  
  void PlayHit();
  void PlayMiss();
  void PlayLaunch();
  void PlayCombatDeath();
  void PlayCombatMove();
  void PlayCombatTurnUndead();

  void TakeDamage(int dmg, BOOL IsNonLethal);
  void RestoreToParty();  
  
  void /* PRS July 2009 BOOL */ UpdateCombatant();
  BOOL Think();
  void HandleCombatRoundMsgExpired();
  void HandleTimeDelayMsgExpired();
  int  HandleTimeDelayMsgBegin();
  void PostCombatScreenUpdate();
  int  HandleCurrState();

  void displayCombatSprite();
  void blitDeadSprite();
  void blitDyingSprite();

  BOOL CanAddTarget(int target);
  void AddTarget(int newTarget);
  BOOL AddTarget(COMBATANT &dude, int range=0);
  BOOL AddMapTarget(int mapx, int mapy, PATH_DIR dir);
  BOOL AddTargetSelf();
  void AutoChooseSpellTargets();
  void RemoveAllTargets();
  void SetCurrTarget();
  void RemoveCurrTarget();
  int  GetCurrTarget();
  int  GetNextTarget();
  int  GetMaxTargets();
  int  GetNumTargets() const { return combattargets.GetCount(); }
  BOOL HaveTarget(int target);
  void RemoveTarget(int target);

  BOOL canAttack(int theAttackee);
  BOOL StartAttack(int target);
  int  makeAttack(int target);
  BOOL DetermineIfBackStab(int wpn, int targ);

  void AttackOver();
  void StopAttack();
  double AttacksRemaining() const { return availAttacks; }
  BOOL UseAttackPose();

  BOOL NeedMissileAnimation();
  void InitMissileAnimation();
  BOOL NeedHitAnimation();
  void InitHitAnimation();
  BOOL NeedSpellCastAnimation();
  void InitSpellCastAnimation(int targ);
  BOOL NeedSpellInRouteAnimation(int x, int y);
  void InitSpellInRouteAnimation(int x, int y);
  BOOL NeedSpellCoverageAnimation();
  void InitSpellCoverageAnimation(int x, int y);
  BOOL NeedSpellHitAnimation();
  void InitSpellHitAnimation(int targ);
  BOOL NeedSpellLingerAnimation();
  void InitSpellLingerAnimation(int mapx, int mapy);

  BOOL CanTarget();
  BOOL CanMove();
  BOOL CanCast();
  BOOL CanUse();
  BOOL CanTurnUndead();
  BOOL CanDelay();
  BOOL CanBandage();
  BOOL CanGuard();

  void Bandage();
  void EndTurn(individualCombatantState newState=ICS_None);
  void Quick(BOOL Enable);
  void Guard();
  void DelayAction(); 
  void TurnUndead();
  void StartInitialSpellCasting(GLOBAL_SPELL_ID key);
  void EndInitialSpellCasting();

  BOOL IsDone(); 
  BOOL IsAttacking() const { return (state == ICS_Attacking); }
  BOOL IsGuarding() const { return (state == ICS_Guarding); }

  BOOL IsCasting();
  void StopCasting(BOOL EndTurn);
  GLOBAL_SPELL_ID  GetSpellKeyBeingCast() const {return m_spellKeyBeingCast;};;
  void SetSpellKeyBeingCast(GLOBAL_SPELL_ID gsID){m_spellKeyBeingCast = gsID;};
  BOOL GetSpellBeingCast(CHARACTER_SPELL& pcSpell){return GetSpell(m_spellKeyBeingCast, pcSpell);};;
 

  BOOL OnAuto() { return GetAdjAutomatic(); }
  void StopMoving() { state = ICS_None; moveX=-1; moveY=-1;}  

  void moveLeft();
  void moveRight();
  void moveForward();
  void moveBackward();
  void moveNW();
  void moveNE();
  void moveSW();
  void moveSE();
  BOOL CurrMoveWouldFleeMap();

  BOOL MoveCombatant(int newX, int newY);
  BOOL FindPathTo(int dx, int dy, BOOL CheckOccupants);
  BOOL FindPathAwayFrom(int fleeFrom);
  int FindPathToMapNorthEdge(void);
  int FindPathToMapEastEdge(void);
  int FindPathToMapSouthEdge(void);
  int FindPathToMapWestEdge(void);
  BOOL FindPathToMapEdge();
  BOOL FindPathAlongLine(PATH_DIR dir, int dist);
  void ClearPath();
  BOOL TakeNextStep();
 
  void ReadyBestWpn(int targ);
  BOOL toggleReadyItem(int item);
  BOOL delCharacterItem(int index, int qty);

  int DetermineInitialFacing(int PartyFacing);
  void CheckMorale();

  DWORD MakeTargetKey(int dude, int dist) { return ((((DWORD)dist) << 16) | ((DWORD)dude)); }
  int   GetTargetKeyDude(DWORD k) const { return (k & 0x0000FFFF); }
  int   GetTargetKeyDist(DWORD k) const { return ((k >> 16) & 0x0000FFFF); }

  int GetCenterX();
  int GetCenterY();

  BOOL GetIsFriendly();
  
  void OnStartCombatantAction();
  void OnEndTurn();
  void OnEnsureVisible();

  CString FormatSpecAbMsg(DWORD sa_state);


  // These ModifyXXX functions dynamically alter character
  // values based on spell effects or special abilities.
  BOOL ModifyAttackRollDice                 
        (CHARACTER *pTarget,                        int *pNum, int *pSides, int *pBonus);
  BOOL ModifyAttackRollDiceForItem          
        (CHARACTER *pTarget,   GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus);
  BOOL ModifyAttackRollDiceAsTarget         
        (CHARACTER *pAttacker,                      int *pNum, int *pSides, int *pBonus);
  BOOL ModifyAttackRollDiceForItemAsTarget  
        (CHARACTER *pAttacker, GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus);
  BOOL ModifyAttackDamageDice               
        (CHARACTER *pTarget,                        int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal);
  BOOL ModifyAttackDamageDiceForItem        
        (CHARACTER *pTarget,   GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal);
  BOOL ModifyAttackDamageDiceAsTarget       
        (CHARACTER *pAttacker,                      int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal);
  BOOL ModifyAttackDamageDiceForItemAsTarget
        (CHARACTER *pAttacker, GLOBAL_ITEM_ID giID, int *pNum, int *pSides, int *pBonus, BOOL *pNonLethal);
  BOOL ModifyAttackThaco                    
        (CHARACTER *pTarget,                        int *pVal);
  BOOL ModifyAttackThacoAsTarget            
        (CHARACTER *pAttacker,                      int *pVal);
  BOOL ModifySaveRoll                       
        (CHARACTER *pTarget,                        int *pRoll);
  BOOL ModifySaveRollAsTarget               
        (CHARACTER *pAttacker,                      int *pRoll);
  BOOL ModifyACAsTarget                     
        (CHARACTER *pAttacker,                      int *pAC, GLOBAL_ITEM_ID giID);

  int self;
	int x,y;
  int hPath;
	int initiative; // 1..10
	int target; // index of target
	int m_iFacing;
  int movement;
  int moveX;
  int moveY;
  int lastAttackRound;
  double availAttacks;
  BOOL continueAttack; // interrupt a series of multiple attacks?
  int  iFleeingFlags;  // See FLEEING_FLAGS
  BOOL isTurned;  // this guy has been turned and is fleeing from the cleric that did it
  BOOL hasTurnedUndead; // only get to turn undead once per combat
  int  m_iLastAttacker; // who tried to hit us most recently?
  int  m_iLastAttacked; // who did we try to hit most recently?
  LASTACTION m_eLastAction; //
	BOOL turnIsDone;
  BOOL isBandaged;
  int  bandageWho;
  BOOL didMove;
  individualCombatantState state;
	BOOL friendly;
  BOOL blitDeathTile;
  GLOBAL_SPELL_ID m_spellKeyBeingCast;
  int width;
  int height;
  POSITION targetPos;
  int m_iTmpAttDiceBonus;  // Valid for one attack then set to 0.
  int m_iTmpDmgDiceBonus;  // Valid for one attack then set to 0.
  int origKey;
  int pendingSpellKey;
  int activeSpellKey;
  BOOL forceAttackPose;
  int m_iAttDiceSides;
  int m_iAttDiceNum;
  int m_iAttDiceRoll;
  int m_isCombatReady;

  SPECIAL_ABILITIES combatantSA;

  OrderedQueue<int, MAX_COMBAT_TARGETS> combattargets; // who am I targeting?
  // temp list to check perimeter of self for attackers.
  CList<int, int> targeters; 
};

struct QueuedCombatant
{
  QueuedCombatant() { Dude=NO_DUDE; AffectStats=TRUE; }
  QueuedCombatant(int dude, BOOL stats) { Dude=dude; AffectStats=stats; }

  BOOL AffectStats;
  int Dude;
};

struct QueuedCombatantData
{
private:
  CList<QueuedCombatant, QueuedCombatant&> m_q;
public:
  QueuedCombatantData() { Clear(); }
  void Clear() { m_q.RemoveAll(); }
  void Push(int dude, BOOL stats)
  {
    //ASSERT(m_q.GetCount() == 0);
    QueuedCombatant temp(dude,stats);
    m_q.AddHead(temp);
  }
  void PushTail(int dude, BOOL stats)
  {
    //ASSERT(m_q.GetCount() == 0);
    QueuedCombatant temp(dude,stats);
    m_q.AddTail(temp);
  }
  void Pop() { m_q.RemoveHead(); }
  
  void Remove(int dud)
  {
    POSITION pos = m_q.GetHeadPosition();
    while (pos != NULL)
    {
      if (m_q.GetAt(pos).Dude == dud)
      {
        m_q.RemoveAt(pos);
        return;
      }
      m_q.GetNext(pos);
    }
  }

  int Top() 
  { 
    if (m_q.GetCount() > 0) 
      return m_q.GetHead().Dude; 
    else 
      return NO_DUDE; 
  }

  BOOL ChangeStats()
  {
    if (m_q.GetCount() > 0) 
      return m_q.GetHead().AffectStats; 
    else 
      return FALSE; 
  }

};


struct COMBAT_MSGS
{
  enum { MAX_COMBAT_MSGS=1 };

  ~COMBAT_MSGS() { msgs.RemoveAll(); }
  void Add(CString &msg) 
  { 
    if (msgs.GetCount() > MAX_COMBAT_MSGS) msgs.RemoveHead();
    msgs.AddTail(msg);
  }
     
  CList<CString, CString> msgs;
};

#endif //__COMBATANT_H__