/******************************************************************************
* Filename: Combatants.cpp
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/
#include "..\Shared\stdafx.h"

//#include "externs.h"
#include "Dungeon.h"

#include <math.h>
#include "char.h"
#include "GameEvent.h"
#include "party.h"
#include "Combatants.h"
#include "DrawTile.h"
#include "FileParse.h"
#include "SoundMgr.h"
#include "Viewport.h"
#include "Screen.h"
#include "GameMenu.h"
#include "FormattedText.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


COMBAT_DATA combatData;
COMBAT_MSGS combatMsgs;

char *CombatantsStateText[NUM_COMBATANTS_STATES] = 
{
  "OCS_None",
  "OCS_Casting",
  "OCS_Attacking", 
  "OCS_Guarding",
  "OCS_Bandaging", 
  "OCS_Using",
  "OCS_Moving",
  "OCS_Turning",
  "OCS_Fleeing",
  "OCS_CombatRoundDelay",
  "OCS_CombatOver",
  "OCS_Delaying",
  "OCS_EndingTurn",
  "OCS_ForcingWin",
  "OCS_StartNewRound",
  "OCS_ActivateSpell",
  "OCS_Fled"
};


//*****************************************************************************
// NAME: InitCombatData
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::InitCombatData(COMBAT_EVENT_DATA *event)
{
   if (event==NULL) return;
   Clear();

   itemData.FreeArt();
   spellData.FreeArt();
   monsterData.ClearSounds();
   itemData.ClearSounds();
   spellData.ClearSounds();
   pathMgr.Clear();
   ClearEmptyCellSearch();
 
   m_bCombatActive = TRUE;
   m_bIsWilderness = event->m_UseOutdoorMap;

   m_combatRect.left   = CombatScreenX;
   m_combatRect.top    = CombatScreenY;
   m_combatRect.right  = COMBAT_SCREEN_WIDTH;
   m_combatRect.bottom = COMBAT_SCREEN_HEIGHT;

   m_fullRect.left = 0;
   m_fullRect.top = 0;
   m_fullRect.right = SCREEN_WIDTH;
   m_fullRect.bottom = SCREEN_HEIGHT;

#ifdef _DEBUG
   ConfigFile.FindToken("Monster_NoMove"     , m_bMonsterNoMove      , false); // don't remove these tokens
   ConfigFile.FindToken("Monster_AlwaysMiss" , m_bMonsterAlwaysMiss  , false);
   ConfigFile.FindToken("Monster_AlwaysHit"  , m_bMonsterAlwaysHit   , false);
   ConfigFile.FindToken("Monster_NoCast"     , m_bMonsterNoCast      , false);
   ConfigFile.FindToken("Monster_PlyrControl", m_bMonsterPlyrControl , false);
   ConfigFile.FindToken("Monster_NoDeath"    , m_bMonsterNoDeath     , false);
   ConfigFile.FindToken("Party_AlwaysMiss"   , m_bPartyAlwaysMiss    , false);
   ConfigFile.FindToken("Party_AlwaysHit"    , m_bPartyAlwaysHit     , false);

   WriteDebugString("Combat Debug Settings:\n");
   WriteDebugString("       MonsNoMove = %u\n"  , m_bMonsterNoMove);
   WriteDebugString("    MonsAlwaysHit = %u\n"  , m_bMonsterAlwaysHit);
   WriteDebugString("   MonsAlwaysMiss = %u\n"  , m_bMonsterAlwaysMiss);
   WriteDebugString("       MonsNoCast = %u\n"  , m_bMonsterNoCast);
   WriteDebugString("      MonsNoDeath = %u\n"  , m_bMonsterNoDeath);
   WriteDebugString("  MonsPlyrControl = %u\n"  , m_bMonsterPlyrControl);
   WriteDebugString("  PartyAlwaysMiss = %u\n"  , m_bPartyAlwaysMiss);
   WriteDebugString("   PartyAlwaysHit = %u\n\n", m_bPartyAlwaysHit);
   WriteDebugString("     MapTileWidth = %u\n"  , TILES_HORZ);
   WriteDebugString("    MapTileHeight = %u\n\n", TILES_VERT);
#endif

   m_bPartyNeverDies     = event->partyNeverDies;
   m_bPartyNoExperience  = event->partyNoExperience;
   m_bNoMagic            = event->noMagic;
   m_eSurprise           = event->m_eTEMPsurprise;
   m_eUndeadTurnModifier = event->turningMod;

   m_pEvent = event;
   LoadSounds();

   m_iPartyStartX = party.Posx;
   m_iPartyStartY = party.Posy;

   // this function adjusts the start x,y to reflect
   // x,y in combat terrain map
   if (!m_bIsWilderness)
     GenerateIndoorCombatMap(m_iPartyStartX, m_iPartyStartY, EventDirToPathDir(event->direction));
   else
     GenerateOutdoorCombatMap(m_iPartyStartX, m_iPartyStartY, EventDirToPathDir(event->direction));

   m_iStartTerrainX = 0;
   m_iStartTerrainY = 0;

   if (isObstical(m_iPartyStartX, m_iPartyStartY,1,1,FALSE, TRUE))
   {
      WriteDebugString("Starting x,y is not empty\n");
      ASSERT(FALSE);
   }

   m_iCurrInitiative = 1;  // Characters start with initiative=0 so we will need
                           // to start a new round before we can begin.
   m_iCurrRound = 0;
   m_iCurrTurn=0;
   m_bLastRound = FALSE;
   m_bCombatOver = FALSE;
  

   // Add all combatants to combatData.m_aCombatants.
   AddCombatants();

   // Position each combatant on the map.
   determineInitCombatPos();
   WriteDebugString("Finished placing combatants on map\n");
   int i;
   // make sure every monster can reach the party
   pathMgr.SetPathSize(1,1); // 1x1 good enough to let party reach
   for (i=0; i<NumCombatants()/*m_aCombatants.GetSize()*/; i++)
   {      
      if (!m_aCombatants[i].IsPartyMember())
      {
        int hPath = pathMgr.GetPath(m_aCombatants[i].x, m_aCombatants[i].y, 
                            m_iPartyStartX, m_iPartyStartY, FALSE);
        if (hPath < 0)
        {
          // no path from monster to party, so remove monster.

          // Remove unreachable monster from combat map.
          placeCombatant(m_aCombatants[i].x, m_aCombatants[i].y,
                         NO_DUDE, 
                         m_aCombatants[i].width,
                         m_aCombatants[i].height);

          // Remove unreachable monster from array combatData.m_aCombatants
          // by marking position negative.
          m_aCombatants[i].x = -1;
        }
        else
          pathMgr.FreePath(hPath);
      }
      //else
      //  WriteDebugString("origIndex %i, origKey %i\n", combatants[i].origIndex, combatants[i].origKey);
   }   

   // some combatants may not have been placed on the map

   // transfer valid combatants to temp
   int count = 0;
   m_aTempCOMBATANTS.RemoveAll();

   for (i=0; i<NumCombatants()/*m_aCombatants.GetSize()*/; i++)
   {
     if (m_aCombatants[i].x >= 0)
     {
       m_aTempCOMBATANTS.SetAtGrow(count, m_aCombatants[i]);
       count++;
     }
   }

   // check to see if any were left out
   if (m_aTempCOMBATANTS.GetSize() != NumCombatants()/*m_aCombatants.GetSize()*/)
   {
     TRACE("Adjusting combatant list from %i to %i\n",
                      NumCombatants()/*m_aCombatants.GetSize()*/, m_aTempCOMBATANTS.GetSize());

     // now we need to clean up the self index
     // and the terrain references to existing combatants
     for (i=0; i<m_aTempCOMBATANTS.GetSize(); i++)
     {
       if (m_aTempCOMBATANTS[i].self != i)
       {
         // this one needs updating
         m_aTempCOMBATANTS[i].self = i;
         placeCombatant(m_aTempCOMBATANTS[i].x, m_aTempCOMBATANTS[i].y,
                        m_aTempCOMBATANTS[i].self, 
                        m_aTempCOMBATANTS[i].width,
                        m_aTempCOMBATANTS[i].height);

       }
     }

     // now move corrected data back into combatants
     count=0;
     m_aCombatants.RemoveAll();
     for (i=0; i<m_aTempCOMBATANTS.GetSize(); i++)
     {
       m_aCombatants.SetAtGrow(count, m_aTempCOMBATANTS[i]);
       TRACE("Start Data for combatant %i: self %i, x %i, y %i, f %i\n",
                        count, 
                        m_aTempCOMBATANTS[i].self,
                        m_aTempCOMBATANTS[i].x,
                        m_aTempCOMBATANTS[i].y,
                        m_aTempCOMBATANTS[i].GetIsFriendly());
       count++;
     }
   }

   m_iNumCombatants = count; //m_aCombatants.GetSize();
   WriteDebugString("Total combatants = %i\n", m_iNumCombatants);

   m_aTempCOMBATANTS.RemoveAll();
   QComb.Clear();
   //StartNewRound();
   //prevRndCombatant = GetCurrCombatant();
   PlaceCursorOnDude(0,TRUE);   
   ClearEmptyCellSearch(); // don't need this data anymore
}

//*****************************************************************************
// NAME:    COMBAT_DATA::AddCombatatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::AddCombatants()
{ 
  // always add party members first!
  m_iNumCombatants = 0;
  AddCharsToCombatants(); 
  AddMonstersToCombatants();

  //m_iNumCombatants = m_aCombatants.GetSize();
  PrepareHooks();
  for (int i=0; i<m_iNumCombatants; i++)
  {
    m_aCombatants[i].LoadCombatIcon();
    m_aCombatants[i].determineIconSize();
    SetCharContext(m_aCombatants[i].GetContext());
    m_aCombatants[i].combatantSA.InsertAbility("Competing","", "Create Combatant ",m_aCombatants[i].GetName());
    scriptContext.SetCharacterContext(&m_aCombatants[i]);
    m_aCombatants[i].combatantSA.RunScripts(ON_START_COMBAT,
                                            ScriptCallback_RunAllScripts,
                                            NULL,
                                            m_aCombatants[i].GetName(),
                                            "");
    scriptContext.Clear();
    ClearCharContext();
  }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::AddCharsToCombatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::AddCharsToCombatants()
{
  COMBATANT temp;  

  for (int i=0; i<party.numCharacters;i++)
  {
    charStatusType stype = party.characters[i].GetAdjStatus();
    if (   ((stype == Okay) || (stype == Dying))
        && (party.characters[i].GetAllowInCombat()))
    {       
       temp.Clear();
       temp.self = m_iNumCombatants;
       temp.InitFromCharData(i); 
       
       // Add new combatant to array of combatants.
       m_aCombatants.SetAtGrow(m_iNumCombatants, temp);     
       m_iNumCombatants++;
    }
  }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::AddMonstersToCombatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::AddMonstersToCombatants()
{
  COMBATANT temp;

  if (m_pEvent->monsters.GetCount() == 0)
  {
    WriteDebugString("No monsters in combat event %i\n", m_pEvent->GetEventId());
    return;
  }

  // how many monsters can we add?
  int MaxMonstersAllowed = MAX_COMBATANTS - NumCombatants()/*m_aCombatants.GetSize()*/ - 1;

  if (m_pEvent->randomMonster)
  {
    // pick just one monster/NPC from the list
    int result = rollDice(m_pEvent->monsters.GetCount(), 1, 0.0);
    result -= 1; // make zero-based index;
    result = max(0, result);

    int count;
    if (m_pEvent->monsters.monsters[result].UseQty == MONSTER_EVENT::meUseQty)
    {
      count = m_pEvent->monsters.monsters[result].qty;
    }
    else
    {
      count = rollDice(m_pEvent->monsters.monsters[result].qtyDiceSides,
                       m_pEvent->monsters.monsters[result].qtyDiceQty,
                       m_pEvent->monsters.monsters[result].qtyBonus);
      WriteDebugString("Random quantity of combat monster: %i\n", count);
    }

    double mod = GetMonsterQtyMod();
    if (mod != 0.0) count += ((mod/100.0)*count);
    if (count > MaxMonstersAllowed) count = MaxMonstersAllowed;
    if (count <= 0) count=1;

    for (int j=0;j<count;j++)
    {     
      temp.Clear();
      temp.self = m_iNumCombatants; // init relies on self index

      if (m_pEvent->monsters.monsters[result].type == MONSTER_TYPE)
        temp.InitFromMonsterData(m_pEvent->monsters.monsters[result].monster,
                                 m_pEvent->monsters.monsters[result].friendly,
                                 m_pEvent->monsters.monsters[result].items,
                                 m_pEvent->monsters.monsters[result].money);
      else
        temp.InitFromNPCData(m_pEvent->monsters.monsters[result].monster,
                             m_pEvent->monsters.monsters[result].friendly,
                             m_pEvent->monsters.monsters[result].items,
                             m_pEvent->monsters.monsters[result].money);

      if (temp.origKey != -1)
      {         
        temp.self = m_iNumCombatants;
        temp.SetMorale(m_pEvent->monsterMorale);
        temp.SetMorale( temp.GetMorale() + m_pEvent->monsters.monsters[result].moraleAdj );

        int PreSpecAbCount = temp.spellEffects.GetCount();
        // turn on the special abilities
        NotImplemented(0xc98f4a,FALSE);//temp.specAbs.EnableAllFor(&temp);

        int PostSpecAbCount = temp.spellEffects.GetCount();
        int Diff = PostSpecAbCount - PreSpecAbCount;
        if (Diff > 0)
        {
          POSITION pos = temp.spellEffects.GetTailPosition();  
          while ((pos != NULL) && (Diff > 0))
          {        
            temp.spellEffects.GetAt(pos).flags |= SPELL_EFFECTS_DATA::EFFECT_CHARSPECAB;
            //temp.spellEffects.GetAt(pos).m_gsID = temp.self;
            temp.spellEffects.GetAt(pos).SourceChar_ID(temp.self);
            temp.spellEffects.GetPrev(pos);
            Diff--;
          }
        }

        m_aCombatants.SetAtGrow(m_iNumCombatants, temp);
        m_iNumCombatants++;
      }
    }
  }
  else
  {
    // add all monsters/NPCs
    for (int i=0; i<m_pEvent->monsters.GetCount(); i++)
    {
      int count;
      if (m_pEvent->monsters.monsters[i].UseQty == MONSTER_EVENT::meUseQty)
      {
        count = m_pEvent->monsters.monsters[i].qty;
      }
      else
      {
        count = rollDice(m_pEvent->monsters.monsters[i].qtyDiceSides,
                         m_pEvent->monsters.monsters[i].qtyDiceQty,
                         m_pEvent->monsters.monsters[i].qtyBonus);
        WriteDebugString("Random quantity of combat monster: %i\n", count);
      }

      double mod = GetMonsterQtyMod();
      if (mod != 0.0) count += ((mod/100.0)*count);
      if (count <= 0) count=1;

      for (int j=0;j<count;j++)
      {     
        temp.Clear();
        temp.self = m_iNumCombatants; // init relies on self index

        if (m_pEvent->monsters.monsters[i].type == MONSTER_TYPE)
          temp.InitFromMonsterData(m_pEvent->monsters.monsters[i].monster,
                                   m_pEvent->monsters.monsters[i].friendly,
                                   m_pEvent->monsters.monsters[i].items,
                                   m_pEvent->monsters.monsters[i].money);
        else
          temp.InitFromNPCData(m_pEvent->monsters.monsters[i].monster,
                               m_pEvent->monsters.monsters[i].friendly,
                               m_pEvent->monsters.monsters[i].items,
                               m_pEvent->monsters.monsters[i].money);

        if (temp.origKey != -1)
        {         
          temp.self = m_iNumCombatants;
          temp.SetMorale(m_pEvent->monsterMorale);
          temp.SetMorale( temp.GetMorale() + m_pEvent->monsters.monsters[i].moraleAdj );

          // int PreSpecAbCount = temp.spellEffects.GetCount();
          // turn on the special abilities
          // temp.specAbs.EnableAllFor(&temp);

          // int PostSpecAbCount = temp.spellEffects.GetCount();
          // int Diff = PostSpecAbCount - PreSpecAbCount;
          // if (Diff > 0)
          // {
          //   POSITION pos = temp.spellEffects.GetTailPosition();  
          //   while ((pos != NULL) && (Diff > 0))
          //   {        
          //     temp.spellEffects.GetAt(pos).flags |= SPELL_EFFECTS_DATA::EFFECT_CHARSPECAB;
          //     temp.spellEffects.GetAt(pos).SourceChar_ID(temp.self);
          //     //temp.spellEffects.GetAt(pos).m_gsID = temp.self;
          //     temp.spellEffects.GetPrev(pos);
          //     Diff--;
          //   }
          // }

          if (NumCombatants()/*m_aCombatants.GetSize()*/ < MaxMonstersAllowed)
          {
            m_aCombatants.SetAtGrow(m_iNumCombatants, temp);
            m_iNumCombatants++;
          }
        }
      }
    }
  }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::AddMonsterToCombatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::AddMonsterToCombatants(const CString& monsterName, int friendly)
{
  int index;
  BOOL reset;
  index = monsterData.FindName(monsterName);
  if (index > 0)
  {
    AddMonsterToCombatants(index);
    index = NumCombatants()/*m_aCombatants.GetSize()*/-1;
    m_aCombatants[index].LoadCombatIcon();
    m_aCombatants[index].friendly = friendly;
    m_aCombatants[index].determineIconSize();
    SetCharContext(m_aCombatants[index].GetContext());
    m_aCombatants[index].combatantSA.InsertAbility("Competing","", "Create Combatant ",m_aCombatants[index].GetName());
    scriptContext.SetCharacterContext(&m_aCombatants[index]);
    PrepareHooks();
    m_aCombatants[index].combatantSA.RunScripts(ON_START_COMBAT,
                                            ScriptCallback_RunAllScripts,
                                            NULL,
                                            m_aCombatants[index].GetName(),
                                            "");
    scriptContext.Clear();
    ClearCharContext();

    reset = false;
    determineInitCombatPos(&m_aCombatants[index], index, &reset, FALSE);
  };
}

//*****************************************************************************
// NAME:    COMBAT_DATA::AddMonsterToCombatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::AddMonsterToCombatants(int monsterDataIndex)
{
  COMBATANT temp;
  MONSTER_DATA *pMonster;

  pMonster = monsterData.GetMonsterData(monsterDataIndex);

  // how many monsters can we add?
 // int MaxMonstersAllowed = MAX_COMBATANTS - m_aCombatants.GetSize() - 1;

  temp.Clear();
  temp.self = m_iNumCombatants; // init relies on self index

  //if (m_pEvent->monsters.monsters[result].type == MONSTER_TYPE)
        temp.InitFromMonsterData(monsterDataIndex,  //m_pEvent->monsters.monsters[result].monster,
                                 FALSE, //m_pEvent->monsters.monsters[result].friendly,
                                 pMonster->myItems, //_pEvent->monsters.monsters[result].items,
                                 pMonster->money); //m_pEvent->monsters.monsters[result].money);
  //    else
  //      temp.InitFromNPCData(m_pEvent->monsters.monsters[result].monster,
  //                           m_pEvent->monsters.monsters[result].friendly,
  //                           m_pEvent->monsters.monsters[result].items,
  //                           m_pEvent->monsters.monsters[result].money);

  if (temp.origKey != -1)
  {         
    temp.self = m_iNumCombatants;
    temp.SetMorale(m_pEvent->monsterMorale);
    temp.SetMorale( temp.GetMorale() + 0);//m_pEvent->monsters.monsters[result].moraleAdj );

    int PreSpecAbCount = temp.spellEffects.GetCount();
    // turn on the special abilities
    //NotImplemented(0xc98f7c,FALSE);//temp.specAbs.EnableAllFor(&temp);

    int PostSpecAbCount = temp.spellEffects.GetCount();
    int Diff = PostSpecAbCount - PreSpecAbCount;
    if (Diff > 0)
    {
      POSITION pos = temp.spellEffects.GetTailPosition();  
      while ((pos != NULL) && (Diff > 0))
      {        
        temp.spellEffects.GetAt(pos).flags |= SPELL_EFFECTS_DATA::EFFECT_CHARSPECAB;
        //temp.spellEffects.GetAt(pos).m_gsID = temp.self;
        temp.spellEffects.GetAt(pos).SourceChar_ID(temp.self);
        temp.spellEffects.GetPrev(pos);
        Diff--;
      }
    }

    m_aCombatants.SetAtGrow(m_iNumCombatants, temp);
    m_iNumCombatants++;
  }
}


//*****************************************************************************
// NAME: getActiveCombatCharacter
//
// PURPOSE:  
//*****************************************************************************
COMBATANT& COMBAT_DATA::getActiveCombatCharacter()
{
  int curr = GetCurrCombatant();
  ASSERT(curr != NO_DUDE);
  
  // coding error
  if (curr==NO_DUDE) return m_aCombatants[0];

  // call getActiveCastingCharacter while casting a spell
  // since the caster may die. The caster info is stored
  // separately to make sure it is available until the 
  // spell effects have been delivered to all targets (if applicable).
  //ASSERT(combatants[curr].state!=ICS_Casting);

  if (m_aCombatants[curr].GetType() == MONSTER_TYPE)
  {
    WriteDebugString("getActiveCombatCharacter() called for monster combatant\n");
  }

  return m_aCombatants[curr];
}

//*****************************************************************************
//    NAME: *COMBAT_DATA::getActiveCastingCharacter
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *COMBAT_DATA::getActiveCastingCharacter()
{
  ASSERT(m_iCurrCaster != NO_DUDE);
  if (m_iCurrCaster==NO_DUDE) return NULL;
  return (&(m_aCombatants[m_iCurrCaster]));
}

//*****************************************************************************
//    NAME: COMBAT_DATA::SetActiveCastingCharacter
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::SetActiveCastingCharacter(int dude)
{
  m_iCurrCaster = dude;
  if (m_iCurrCaster==NO_DUDE) return;
  EnsureVisibleTarget(m_iCurrCaster);
}


//*****************************************************************************
// NAME: COMBAT_DATA::CanBandage
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBAT_DATA::CanBandage()
{
  int curr = GetCurrCombatant();
  if (curr == NO_DUDE)
    return FALSE;

  if (m_aCombatants[curr].CanBandage())
  {
    int i=0;    
    while (i < m_iNumCombatants)
    {
      if ((i != GetCurrCombatant()) && (m_aCombatants[i].GetAdjStatus() == Dying))
        return TRUE;  
      i++;
    }
  }

  return FALSE;
}


//*****************************************************************************
// NAME: COMBAT_DATA::Bandage
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::Bandage()
{
  if (CanBandage())
  {
    int count=0;
    int btarg=-1;
	int mosthurt = 0;

    for (int i=0; i<m_iNumCombatants; i++)
    {
      if (m_aCombatants[i].GetAdjStatus() == Dying)
      {
		  // Figure out which PC is the most hurt, and set them to be bandaged first.
		  if (m_aCombatants[i].GetHitPoints() <= m_aCombatants[mosthurt].GetHitPoints()) {
			mosthurt = i;
			btarg = i;
			count = 1;
		  }

	  }
	}
       
	// Now bandage the most hurt PC
	if (count ==1) 
  {
		m_aCombatants[mosthurt].isBandaged = TRUE;
		m_aCombatants[mosthurt].SetHitPoints(0);
		m_aCombatants[mosthurt].SetStatus(Unconscious);        
        //combatants[i].bandageWho = i;
  }
    
    // sets state to cs_bandage so status message is displayed
  m_aCombatants[GetCurrCombatant()].Bandage();
  // status message displays bandage targ if only one targ
  if (count == 1) 
      m_aCombatants[GetCurrCombatant()].bandageWho = btarg;      
  }
}

//*****************************************************************************
// NAME: COMBAT_DATA::ResetAim
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::ResetAim()
{
  SetUpdateFlag();
  m_iAimPos = GetCurrCombatant();

  if ((m_iAimPos >= 0) && (m_iAimPos < m_iNumCombatants))
  {
    m_iCursorX = m_aCombatants[m_iAimPos].x;
    m_iCursorY = m_aCombatants[m_iAimPos].y;
    EnsureVisibleCursor();
  }
}


BOOL COMBAT_DATA::IsValidTarget(COMBATANT *pAttacker, COMBATANT *pTarget)
{
  CString result;
  if (pTarget==NULL) return FALSE;

  PrepareHooks();

  scriptContext.SetCombatantContext(pTarget);
  scriptContext.SetAttackerContext(pAttacker);
  scriptContext.SetTargetContext(pTarget);

  result = pTarget->combatantSA.RunScripts(
                     IS_VALID_TARGET,
                     ScriptCallback_LookForChar,
                     "YN",
                     pTarget->GetName(),
                     "");
  scriptContext.Clear();
  if (!result.IsEmpty())
  {
    if (result[0] == 'N') return FALSE;
  };
  return TRUE;
}

//*****************************************************************************
// NAME: GetNextAim
//
// PURPOSE:  
//*****************************************************************************
int COMBAT_DATA::GetNextAim()
{
  if (m_iNumCombatants <= 0)
  {
    ASSERT(FALSE);
    m_iAimPos = NO_DUDE;
    return NO_DUDE;
  }
  //combatData.SetUpdateFlag();

  if (m_iAimPos == NO_DUDE)
    m_iAimPos = GetCurrCombatant();

  int count=0;
  while (count < m_iNumCombatants)
  {
    if ((m_iAimPos < 0) || (m_iAimPos >= m_iNumCombatants))
      m_iAimPos = 0;
    else
      m_iAimPos++;
    
    if ((m_iAimPos >= 0) && (m_iAimPos < m_iNumCombatants))
    {
      if (m_aCombatants[m_iAimPos].charOnCombatMap())
      {
        COMBATANT *pTarget, *pAttacker;
        pTarget = &m_aCombatants[m_iAimPos];
        pAttacker = &m_aCombatants[GetCurrCombatant()];
        if (IsValidTarget(pAttacker , pTarget))
        {
          m_iCursorX = pTarget->x;
          m_iCursorY = pTarget->y;
          EnsureVisibleCursor();
          return m_iAimPos;
        };
      }
    }
    count++;
  }
  m_iAimPos = GetCurrCombatant();
  return m_iAimPos;
}

//*****************************************************************************
// NAME: GetPrevAim
//
// PURPOSE:  
//*****************************************************************************
int COMBAT_DATA::GetPrevAim()
{
  if (m_iNumCombatants <= 0)
  {
    ASSERT(FALSE);
    m_iAimPos = NO_DUDE;
    return NO_DUDE;
  }
  //combatData.SetUpdateFlag();

  if (m_iAimPos == NO_DUDE)
    m_iAimPos = GetCurrCombatant();

  int count=0;
  while (count < m_iNumCombatants)
  {
    if ((m_iAimPos < 0) || (m_iAimPos >= m_iNumCombatants))
      m_iAimPos = m_iNumCombatants-1;
    else
      m_iAimPos--;
    
    if ((m_iAimPos >= 0) && (m_iAimPos < m_iNumCombatants))
    {
      if (m_aCombatants[m_iAimPos].charOnCombatMap())
      {
        COMBATANT *pTarget, *pAttacker;
        pTarget = &m_aCombatants[m_iAimPos];
        pAttacker = &m_aCombatants[GetCurrCombatant()];
        if (IsValidTarget(pAttacker , pTarget))
        {
          m_iCursorX = pTarget->x;
          m_iCursorY = pTarget->y;
          EnsureVisibleCursor();
          return m_iAimPos;
        };
      }
    }
    count++;
  }
  m_iAimPos = GetCurrCombatant();
  return m_iAimPos;

}

//*****************************************************************************
// NAME: getNumCombatants
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::getNumCombatants(int* pNumParty, int* pNumMons)
{
   *pNumParty = 0;
   *pNumMons = 0;
 
   // count friendly pc's and npc's in numParty
   // count non-friendly pc's, npc's, and monsters in numMons
   // do not count friendly monsters in numParty
 
   for (int i=0; i < m_iNumCombatants; i++)
   {
     if (m_aCombatants[i].charOnCombatMap())
     {
        if ((m_aCombatants[i].IsPartyMember()) || (m_aCombatants[i].GetIsFriendly())) (*pNumParty)++;
        else (*pNumMons)++;    
     }
   }
}

//*****************************************************************************
// NAME: DetermineCombatInitiative
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::DetermineCombatInitiative(void)
{  
   int i;
   for (i=0; i<m_iNumCombatants; i++)
      m_aCombatants[i].RollInitiative(m_eSurprise);

   // if one side was initially surprised, it only
   // works for the first round so reset it.
    {
      CString line;
      int swap;
      int combatant[MAX_COMBATANTS];
      for (i=0; i<m_iNumCombatants; i++)
      {
        combatant[i] = i;
      };
      for (swap=1; swap!=0; )
      {
        swap = 0;
        for (i=0; i<m_iNumCombatants-1; i++)
        {
          if (m_aCombatants[combatant[i]].initiative > m_aCombatants[combatant[i+1]].initiative)
          {
            int temp;
            temp = combatant[i];
            combatant[i] = combatant[i+1];
            combatant[i+1] = temp;
            swap = 1;
          };
        };
      };
      line = "Initiatives: ";
      for (i=0; i<m_iNumCombatants; i++)
      {
        char initiative[40];
        sprintf(initiative, "%i[%i] ",combatant[i],m_aCombatants[combatant[i]].initiative);
        line += initiative;
      };
      WriteDebugString(line+'\n');
   };
   m_eSurprise = Neither;
}

//*****************************************************************************
// NAME: getNextCombatant
//
// PURPOSE:  
//*****************************************************************************
int COMBAT_DATA::getNextCombatant()
{   
   int dude;   

   while ((dude = QComb.Top()) != NO_DUDE)
   {
     m_aCombatants[dude].m_isCombatReady = -1; // Force check before any moves
     if (m_aCombatants[dude].IsDone())
     {
       QComb.Pop();
     }
     else
     {
       TRACE("getNextCombatant() returns %u (queued)\n", dude);
       return dude;
     }
   }
 
   int i=0;
   BOOL found = FALSE;
   dude = NO_DUDE;

   while ((m_iCurrInitiative <= 10) && (!found))
   {
      if (pendingSpellData.ProcessTimeSensitiveData(0, m_iCurrInitiative))
      {
        dude = QComb.Top();
        if (dude != NO_DUDE)
        {
          m_aCombatants[dude].m_isCombatReady = -1; // Force check before any moves
          if (m_aCombatants[dude].IsDone())
          {
            QComb.Pop();
          }
          else
          {
            TRACE("getNextCombatant() returns %u (queued) for pending spell.\n", dude);
            return dude;
          }

        };
      };
        
      i=0;
      while ((i < m_iNumCombatants) && (!found))
      {
         if (   (m_aCombatants[i].initiative == m_iCurrInitiative) 
            && (!m_aCombatants[i].IsDone()))
         {
            dude = i;
            found = TRUE;
         }         
         i++;
      }
    
      if (!found)
         m_iCurrInitiative++;
   }
 
   if (dude != NO_DUDE)
   {
     QComb.Push(dude, TRUE);

     {
      /* The calculation of availAttacks has been moved from
       * StartNewRound() to getNextCombatant().
       * I did this because Manikus complained that a spell
       * given during a combat round that increased the number
       * of attacks on a target did not take effect until the
       * start of the following round.  PRS 20091215
       */
       int i = dude;
       double leftoverAttacks=m_aCombatants[i].availAttacks;             
       m_aCombatants[i].determineAvailAttacks();
       double maxAttacks = ceil(m_aCombatants[i].availAttacks);
       m_aCombatants[i].availAttacks += leftoverAttacks;
       m_aCombatants[i].availAttacks = min(m_aCombatants[i].availAttacks, maxAttacks);
       m_aCombatants[i].movement = 0;      
     }; 
     
     
     //PlaceCursorOnCurrentDude();
     TRACE("getNextCombatant() returns %u\n", dude);
   }
   else
     TRACE("getNextCombatant() returns NO_DUDE\n");

   return dude;
}

//*****************************************************************************
// NAME: moveCharNW
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveNW()
{  
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveNW(); 
}

//*****************************************************************************
// NAME: moveCharNE
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveNE()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveNE();
}

//*****************************************************************************
// NAME: moveCharSE
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveSE()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveSE();
}

//*****************************************************************************
// NAME: moveCharSW
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveSW()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveSW();
}

//*****************************************************************************
// NAME: moveCharLeft
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveLeft()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveLeft();
}

//*****************************************************************************
// NAME: moveCharRight
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveRight()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveRight();
}

//*****************************************************************************
// NAME: moveCharForward
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveForward()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveForward();
}

//*****************************************************************************
// NAME: moveCharBackward
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveBackward()
{
   ASSERT(GetCurrCombatant() != NO_DUDE);
   m_aCombatants[GetCurrCombatant()].moveBackward();
}

//*****************************************************************************
//    NAME: COMBAT_DATA::CurrMoveWouldFleeMap
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CurrMoveWouldFleeMap()
{
  ASSERT(GetCurrCombatant() != NO_DUDE);
  return (m_aCombatants[GetCurrCombatant()].CurrMoveWouldFleeMap());
}

//*****************************************************************************
// NAME: moveCursorForward
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveCursorForward()
{
   int x = m_iCursorX;
   int y = m_iCursorY-1;
 
   if (y >= 0)
   {
     SetUpdateFlag();
     ForceMapToMoveAlong(x, y, PathNorth, 1);
     m_iCursorY = y;
   }
}


//*****************************************************************************
// NAME: moveCursorBackward
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveCursorBackward()
{
   int x = m_iCursorX;
   int y = m_iCursorY+1;
 
   if (y < MAX_TERRAIN_HEIGHT)
   {
     SetUpdateFlag();
     ForceMapToMoveAlong(x, y, PathSouth, 1);      
     m_iCursorY = y;
   }
}


//*****************************************************************************
// NAME: moveCursorLeft
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveCursorLeft()
{
   int x = m_iCursorX-1;
   int y = m_iCursorY;
 
   if (x >= 0)
   {
     SetUpdateFlag();
     ForceMapToMoveAlong(x, y, PathWest, 1);
     m_iCursorX = x;
   }
}


//*****************************************************************************
// NAME: moveCursorRight
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::moveCursorRight()
{
   int x = m_iCursorX+1;
   int y = m_iCursorY;
 
   if (x < MAX_TERRAIN_WIDTH)
   {
     SetUpdateFlag();
     ForceMapToMoveAlong(x, y, PathEast, 1); 
     m_iCursorX = x;
   }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::determinePartyOrientation
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
int COMBAT_DATA::determinePartyOrientation(int x, int y)
{
   int facing = party.GetPartyFacing();
   int width = getHallWidth(x, y, facing);

   if (width < 2)
   {
      switch (facing) 
      {
      case FACE_NORTH:
      case FACE_SOUTH:
         facing = FACE_EAST;
         width = getHallWidth(x, y, facing);
         break;
      case FACE_EAST:
      case FACE_WEST:
         facing = FACE_NORTH;
         width = getHallWidth(x, y, facing);
         break;
      }
    
      if (width < 2)
         WriteDebugString("Cannot orient combat party properly, space too small!\n");
   }

   return facing;
}


//*****************************************************************************
// NAME: determineInitCombatPos
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::determineInitCombatPos(COMBATANT *pCombatant, int i, BOOL *reset, BOOL ignoreCurrentCombatant)
{
  //if (combatants[i].IsPartyMember())
  if (pCombatant->GetIsFriendly())
  {
     getNextCharCombatPos(pCombatant, reset, ignoreCurrentCombatant);
     if (pCombatant->x >= 0)
     {
       placeCombatant(pCombatant->x, 
                      pCombatant->y,
                      i, 
                      pCombatant->width,
                      pCombatant->height);
     }
  }
  else 
  {
     getNextMonsterCombatPos(*pCombatant);
     if (pCombatant->x >= 0)
     {
       placeCombatant(pCombatant->x, 
                      pCombatant->y, 
                      i, 
                      pCombatant->width,
                      pCombatant->height);
     }
  };
}


//*****************************************************************************
// NAME: determineInitCombatPos
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::determineInitCombatPos()
{
   int i;   
   BOOL reset = TRUE;
 
   // PC's and NPC's should be listed in combatData by marching order
   for (i=0; i<NumCombatants()/*m_aCombatants.GetSize()*/; i++)
   {
     determineInitCombatPos(&m_aCombatants[i], i, &reset, TRUE);
   };
}

//*****************************************************************************
// NAME:    COMBAT_DATA::GetNextDirection
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::GetNextDirection(eventDirType *pCurrDir, int *pFacing)
{
   // Get the next direction to place monsters in
   switch (m_pEvent->direction) 
   {
   case Any:
     switch (*pCurrDir)
     {
     case North:
        *pCurrDir = East;
        break;
     case East:
        *pCurrDir = South;
        break;
     case South:
        *pCurrDir = West;
        break;
     case West:
        *pCurrDir = North;
        break;
     default:
       switch (party.GetPartyFacing()) 
       {
       case FACE_NORTH:
          *pCurrDir = North;
          break;
       case FACE_EAST:
          *pCurrDir = East;
          break;
       case FACE_SOUTH:
          *pCurrDir = South;
          break;
       case FACE_WEST:
          *pCurrDir = West;
          break;
       }
       break;
     }
     break;

   case InFront:
     switch (party.GetPartyFacing()) 
     {
     case FACE_NORTH:
        *pCurrDir = North;
        break;
     case FACE_EAST:
        *pCurrDir = East;
        break;
     case FACE_SOUTH:
        *pCurrDir = South;
        break;
     case FACE_WEST:
        *pCurrDir = West;
        break;
     }
     break;

   case N_W_E:
      if (*pCurrDir == West)
         *pCurrDir = North;
      else if (*pCurrDir == North)
         *pCurrDir = East;
      else
         *pCurrDir = West;
      break;
   case North:
      *pCurrDir = North;
      break;
   case W_S_E:
      if (*pCurrDir == East)
         *pCurrDir = South;
      else if (*pCurrDir == South)
         *pCurrDir = West;
      else
         *pCurrDir = East;
      break;
   case South:
      *pCurrDir = South;
      break;
   case N_S_E:
      if (*pCurrDir == North)
         *pCurrDir = East;
      else if (*pCurrDir == East)
         *pCurrDir = South;
      else
         *pCurrDir = North;
      break;
   case East:
      *pCurrDir = East;
      break;
   case N_S_W:
      if (*pCurrDir == South)
         *pCurrDir = West;
      else if (*pCurrDir == West)
         *pCurrDir = North;
      else
         *pCurrDir = South;
      break;
   case West:
      *pCurrDir = West;
      break;
   case N_S:
      if (*pCurrDir == North)
         *pCurrDir = South;
      else
         *pCurrDir = North;
      break;
   case N_E:
      if (*pCurrDir == North)
         *pCurrDir = East;
      else
         *pCurrDir = North;
      break;
   case N_W:
      if (*pCurrDir == West)
         *pCurrDir = North;
      else
         *pCurrDir = West;
      break;
   case S_E:
      if (*pCurrDir == East)
         *pCurrDir = South;
      else
         *pCurrDir = East;
      break;
   case S_W:
      if (*pCurrDir == South)
         *pCurrDir = West;
      else
         *pCurrDir = South;
      break;
   case E_W:
      if (*pCurrDir == East)
         *pCurrDir = West;
      else
         *pCurrDir = East;
      break;
   }

   switch (*pCurrDir) 
   {
   case North:
     *pFacing = FACE_WEST;
     TRACE("Placing monster NORTH of party\n");
     break;
   case East:
     *pFacing = FACE_WEST;
     TRACE("Placing monster EAST of party\n");
     break;
    
   case South:
     *pFacing = FACE_EAST;
     TRACE("Placing monster SOUTH of party\n");
     break;
   case West:
     *pFacing = FACE_EAST;
     TRACE("Placing monster WEST of party\n");
     break;   
   }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::GetMonsterDistance
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
int COMBAT_DATA::GetMonsterDistance(eventDirType dir)
{
   int dist = 0;
 
   switch (m_pEvent->distance) 
   {
   case UpClose:
      dist = 2;
      break;
   case Nearby:
      if ((dir == East) || (dir == West))
        dist = TILES_HORZ/2;
      else
        dist = TILES_VERT/2;
      break;
   case FarAway:
      if ((dir == East) || (dir == West))
        dist = TILES_HORZ-2;
      else
        dist = TILES_VERT-2;
      break;
   }
   TRACE("Placing monster %u tiles away from party\n", dist);
   return dist;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::GetMonsterCoords
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::GetMonsterCoords(int &x, int &y, int dist, int dir)
{
   // take a guess about start coords
   switch (dir) 
   {
   case North:
      // account for isometric orientation
      x = m_iPartyStartX - (dist+(dist/2));
      if (x < 0)
        x = 0;
      y = m_iPartyStartY-dist;
      if (y < 0)
         y = 0;
      break;
    
   case East:
      x = m_iPartyStartX+dist;
      y = m_iPartyStartY;
      if (x >= MAX_TERRAIN_WIDTH)
         x = MAX_TERRAIN_WIDTH-5;
      break;
    
   case South:
     // account for isometric orientation
      x = m_iPartyStartX + (dist+(dist/2));
      if (x >= MAX_TERRAIN_WIDTH)
         x = MAX_TERRAIN_WIDTH-1;
      y = m_iPartyStartY+dist;
      if (y >= MAX_TERRAIN_HEIGHT)
         y = MAX_TERRAIN_HEIGHT-1;
      break;
    
   case West:
      x = m_iPartyStartX-dist;
      y = m_iPartyStartY;
      if (x < 0)
         x = 0;
      break;   
   }
 // make sure coords are in same room as party
 BOOL look = TRUE;
 
 double dX =  abs(m_iPartyStartX - x);
        dX /= (abs(m_iPartyStartY-y)-1);
 double deltaSum=0;

 while (   (look) 
        && (!isObstical(x, y, 1, 1, FALSE, TRUE))
        && (!HaveLineOfSight(m_iPartyStartX, m_iPartyStartY, x, y)))

 {
   switch (dir) 
   {
   case North:
      deltaSum += dX;
      while (deltaSum >= 1.0)
      {
        x++;
        deltaSum -= 1.0;
      }
        
      y++;
      if (x == m_iPartyStartX)
      {
        x--;
        look = FALSE;
      }
      if (y == m_iPartyStartY)
      {
        y--;
        look = FALSE;
      }
      break;
  
   case East:
      x--;
      if (x == m_iPartyStartX)
      {
        x++;
        look = FALSE;
      }
      break;
  
   case South:
      deltaSum += dX;
      while (deltaSum >= 1.0)
      {
        x--;
        deltaSum -= 1.0;
      }

      y--;
      if (x == m_iPartyStartX)
      {
        x++;
        look = FALSE;
      }
      if (y == m_iPartyStartY)
      {
        y++;
        look = FALSE;
      }
      break;
  
   case West:
      x++;
      if (x == m_iPartyStartX)
      {
        x--;
        look = FALSE;
      }
      break;   
   }     
 
   if (x < 0) 
   {
     x = 0;
     look = FALSE;
   }
   if (y < 0)
   {
     look = FALSE;
     y = 0;
   }
   if (x >= MAX_TERRAIN_WIDTH)
   {
      x = MAX_TERRAIN_WIDTH-1;
      look = FALSE;
   }
   if (y >= MAX_TERRAIN_HEIGHT)
   {
      y = MAX_TERRAIN_HEIGHT-1;
      look = FALSE;
   }
 }

 // no line of sight in specified direction
 if (look == FALSE)
 {
   x=-1;y=-1;
 }
}

//*****************************************************************************
// NAME: getNextMonsterCombatPos
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::getNextMonsterCombatPos(COMBATANT &monster)
{
   static eventDirType currDir = Any;   
 
   eventDirType lastDir;   

   do 
   {
     lastDir = currDir;
     GetNextDirection(&currDir, &monster.m_iFacing);
     GetMonsterCoords(monster.x, monster.y, GetMonsterDistance(currDir), currDir);
   } while ((monster.x == -1) && (currDir != lastDir));


   if (monster.x < 0)
   {
     monster.x = m_iPartyStartX;
     monster.y = m_iPartyStartY;
     TRACE("Setting monster start combat pos at party loc\n");
   }
 
   // find first friendly combatant
   int dude = NO_DUDE;
   int i=0;
   while ((dude == NO_DUDE) && (i < m_iNumCombatants))
   {
     if (m_aCombatants[i].friendly)
       dude = i;
     i++;
   }

   PATH_DIR dir=PathBAD;
   
   // find direction from monster to party to help guide 
   // search for empty cell when specified cell is not empty
   if (dude != NO_DUDE)
     dir = GetDir(monster.x,
                  monster.y,
                  m_aCombatants[dude].x,
                  m_aCombatants[dude].y);

   TRACE("Attempting to place %ux%u monster at %u,%u (%u)\n", 
                    monster.width, monster.height, monster.x, monster.y, dir);

   // calling findEmptyCell() will search for empty cell
   // in expanding circle.

   // try to find something in direction of party first
   if (findEmptyCell(monster.x, monster.y, monster.width, monster.height, dir))
   {
      TRACE("Found empty cell for monster (%u,%u)\n", monster.x, monster.y);
   }
   else
   {
      // now try to put the monster anywhere we can
      // does not check for reachable path to party, just place in first non-obstical location
      TRACE("First call to findEmptyCell() failed - widening search...\n");
      if (findEmptyCell(monster.x, monster.y, monster.width, monster.height, PathBAD))
      {
        TRACE("Found empty cell for monster (%u,%u)\n", monster.x, monster.y);
      }
      else
      {
        WriteDebugString("Did NOT find empty cell for monster\n");
        ASSERT(FALSE);
      }
   }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacter
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacter(COMBATANT* pDude, int x, int y, BOOL ignoreCurrentCombatant)
{
   if (!isObstical(x,y, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
   {
      pDude->x = x;
      pDude->y = y;
      return TRUE;
   }
   return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::CanSeeParty
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanSeeParty(int x, int y)
{
  // determine if at least one party member is in
  // sight on combat map
  BOOL success=FALSE;
  int i=0;
  while ((!success) && (i<m_iNumCombatants))
  {
    if ((m_aCombatants[i].x >= 0) && (m_aCombatants[i].y >= 0))
      success = HaveLineOfSight(m_aCombatants[i].GetCenterX(), m_aCombatants[i].GetCenterY(), x, y);
    i++;
  }

  if (!success)
    success = HaveLineOfSight(m_iPartyStartX, m_iPartyStartY, x, y);

  return success;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterLeft
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterLeft(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
   if ((!isObstical(*px-1, *py, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
       && (CanSeeParty(*px-1, *py)))
   {
      pDude->x = *px-1;
      pDude->y = *py;
      *px = *px - 1;
      return TRUE;
   }
   return FALSE;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterRight
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterRight(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
   if ((!isObstical(*px+1, *py, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
       && (CanSeeParty(*px+1, *py)))
   {
      pDude->x = *px+1;
      pDude->y = *py;
      *px = *px + 1;
      return TRUE;
   }
   return FALSE;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterNorth
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterNorth(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
   if ((!isObstical(*px, *py-1, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
       && (CanSeeParty(*px, *py-1)))
   {
      pDude->x = *px;
      pDude->y = *py-1;
      *py = *py - 1;
      return TRUE;
   }
   return FALSE;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterSouth
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterSouth(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
   if ((!isObstical(*px, *py+1, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
       && (CanSeeParty(*px, *py+1)))
   {
      pDude->x = *px;
      pDude->y = *py + 1;
      *py = *py + 1;
      return TRUE;
   }
   return FALSE;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterHorz
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterHorz(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
  if (PlaceCharacter(pDude, *px, *py, ignoreCurrentCombatant))
    return TRUE;

  // get east/west width
  int width = getHallWidth(*px, *py, FACE_NORTH);
  if (width < 1)
    return FALSE;

  int newX; 
  int newY = *py;
  BOOL found = FALSE;
  int attempts = 0;
  int dist = 0;

  while ((!found) && (attempts < width))
  {
    newX = *px - dist;
    if (PlaceCharacterLeft(pDude, &newX, &newY, ignoreCurrentCombatant))
    {
      found = TRUE;
      *px = newX;
      *py = newY;
    }
    else
      attempts++;

    newX = *px + dist;
    if (PlaceCharacterRight(pDude, &newX, &newY, ignoreCurrentCombatant))
    {
      found = TRUE;
      *px = newX;
      *py = newY;
    }
    else
      attempts++;

    dist++;
  }

  return found;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCharacterVert
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BOOL COMBAT_DATA::PlaceCharacterVert(COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant)
{
  if (PlaceCharacter(pDude, *px, *py, ignoreCurrentCombatant))
    return TRUE;

  // get north/south width
  int width = getHallWidth(*px, *py, FACE_EAST);
  if (width < 1)
    return FALSE;

  int newX = *px; 
  int newY;
  BOOL found = FALSE;
  int attempts = 0;
  int dist = 0;

  while ((!found) && (attempts < width))
  {
    newY = *py - dist;
    if (PlaceCharacterNorth(pDude, &newX, &newY, ignoreCurrentCombatant))
    {
      found = TRUE;
      *px = newX;
      *py = newY;
    }
    else
      attempts++;

    newY = *py + dist;
    if (PlaceCharacterSouth(pDude, &newX, &newY, ignoreCurrentCombatant))
    {
      found = TRUE;
      *px = newX;
      *py = newY;
    }
    else
      attempts++;

    dist++;
  }

  return found;
}
//*****************************************************************************
// NAME: getNextCharCombatPos
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::getNextCharCombatPos(COMBATANT* pDude, BOOL* pReset, BOOL ignoreCurrentCombatant)
{
   static int lastX = 0, lastY = 0;
   static int charsInRow = 0;
   BOOL found = FALSE;
 
   if (*pReset)
   {
      charsInRow = 0;
      lastX = m_iPartyStartX;
      lastY = m_iPartyStartY;
      *pReset = FALSE;
   }
   
   int facing = party.GetPartyFacing();

   switch (facing) 
   {      
   case FACE_NORTH:
   case FACE_SOUTH:
      while ((!found) && (lastY >= 0) && (lastY < MAX_TERRAIN_HEIGHT))
      {
         if (charsInRow < MAX_CHARS_IN_ROW)
         {
            found = PlaceCharacterHorz(pDude, &lastX, &lastY, ignoreCurrentCombatant);            
            if (!found)
               charsInRow = MAX_CHARS_IN_ROW; // advance to next row
         }
         else 
         {           
            charsInRow = 0;
            lastX = m_iPartyStartX;
            if (facing == FACE_NORTH)
               lastY++;
            else
               lastY--;
         }
      }
      break;
    
   case FACE_EAST:
   case FACE_WEST:
      while ((!found) && (lastX >= 0) && (lastX < MAX_TERRAIN_WIDTH))
      {
         if (charsInRow < MAX_CHARS_IN_ROW)
         {
            found = PlaceCharacterVert(pDude, &lastX, &lastY, ignoreCurrentCombatant);            
            if (!found)
               charsInRow = MAX_CHARS_IN_ROW; // advance to next row
         }
         else 
         {
            charsInRow = 0;
            lastY = m_iPartyStartY;
            if (facing == FACE_EAST)
               lastX--;
            else
               lastX++;
         }
      }
      break;
   }
 
   if (isObstical(pDude->x, pDude->y, pDude->width, pDude->height, TRUE, ignoreCurrentCombatant))
     WriteDebugString("getNextCharCombatPos() returns a cell containing a wall\n");
   else
     TRACE("Placing char at combat pos %u,%u\n", pDude->x, pDude->y);
}

//*****************************************************************************
// NAME:    COMBAT_DATA::RestoreToParty
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::RestoreToParty()
{
  for (int i=0; i<m_iNumCombatants; i++)
    m_aCombatants[i].RestoreToParty();
}

//*****************************************************************************
//    NAME: COMBAT_DATA::DetermineIfCombatOver
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::DetermineIfCombatOver()
{
  int dude = GetCurrCombatant();
  int numParty, numMons;
  getNumCombatants(&numParty, &numMons);      
  
  if (numMons == 0)
  {
    // party wins
    m_eCombatVictor = PartyWins;
    
// PRS 24 Oct 2009    if (dude != NO_DUDE)
      m_bLastRound = TRUE; 
// PRS 24 Oct 2009   else
      m_bCombatOver = TRUE;
  }
  else if (numParty == 0)
  {
    // monsters win, or party ran away
    m_eCombatVictor = MonsterWins;
    
    if (dude != NO_DUDE)
      m_bLastRound = TRUE; 
    else
      m_bCombatOver = TRUE;
  }
  else
    m_eCombatVictor = NoWinner;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::DetermineVictoryExpPoints
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::DetermineVictoryExpPoints()
{
  int lcv;
  if (m_eCombatVictor == PartyWins)
  {
    //WriteDebugString("Party wins combat\n");
    m_iExpPointTotal = 0;        
    if (!GetConfigPartyNoExperience())
    {
      for (lcv=0; lcv<m_iNumCombatants; lcv++)
      {
        if ((!m_aCombatants[lcv].GetIsFriendly())&&(!m_aCombatants[lcv].IsPartyMember()))
        {
          charStatusType status = m_aCombatants[lcv].GetAdjStatus();
          if (status == Dead)
            m_iExpPointTotal += m_aCombatants[lcv].getCharExpWorth();
        }
      }
      
      double mod = GetMonsterExpMod();
      if (mod != 0.0)
        m_iExpPointTotal += ((mod/100.0)*m_iExpPointTotal);
      if (m_iExpPointTotal < 0) m_iExpPointTotal=0;
    }
  }
  else if (m_eCombatVictor == MonsterWins)
  {
    // See if party fled, or all died
    lcv = 0;        
    while (lcv < m_iNumCombatants)
    {
      if (m_aCombatants[lcv].IsPartyMember())
      {
        if (m_aCombatants[lcv].GetAdjStatus()==Fled)
        {
          m_eCombatVictor = PartyRanAway;
          break;
        }
      }
      lcv++;
    }
  }
}

//*****************************************************************************
// NAME:    COMBAT_DATA::UpdateCombat
//
// PURPOSE: 
//
// /* PRS July 2009  RETURNS: TRUE if input should be forced by ProcInput.cpp */
//*****************************************************************************
void /* PRS July 2009 BOOL */ COMBAT_DATA::UpdateCombat()
{
  BOOL isUpdate = FALSE;
  static int prevDude = NO_DUDE;

  m_iPrevRndCombatant = prevDude;
  m_bStartingNewCombatRound=FALSE;

  int dude = GetCurrCombatant();  

  //if (prevRndCombatant != dude)
  //  PlaceCursorOnCurrentDude();


  if (   (dude == NO_DUDE) 
      || ((dude != NO_DUDE) && (m_aCombatants[dude].IsDone())))
  {
    getNextCombatant();
    dude = GetCurrCombatant();
  
    if (dude == NO_DUDE)
    {  
      StartNewRound();  // Performs a GetNextCombatant();
      dude = GetCurrCombatant();
      if (dude != NO_DUDE)
      {
        m_aCombatants[dude].OnStartCombatantAction();
      };
      DetermineIfCombatOver();
      return;
    }

    if (dude != NO_DUDE)
    {
      isUpdate = TRUE;
      m_aCombatants[dude].OnStartCombatantAction();
      //PlaceCursorOnCurrentDude(TRUE);
    }
  } // if dude is done with his turn
  else
  {
  };

  DetermineIfCombatOver();  // PRS 24 Oct 2009

  if (CheckIdleTime())
  {
    // force end to combat if nobody does anything for awhile
    m_bCombatOver = TRUE;
  }

  if (IsCombatOver())
  {     
    DetermineVictoryExpPoints();
    isUpdate = TRUE;
  }
  else 
  {
    if (IsNewCombatant()) 
    {
    }
    else
    {
      // there is a delay between combatant updates, if prev combatant is not
      // current combatant     
      if (dude != NO_DUDE)
      {
      // PRS July 2009 isUpdate |= m_aCombatants[dude].UpdateCombatant();
        m_aCombatants[dude].UpdateCombatant();
      };
    };
  };

  isUpdate |= m_bNeedUpdate;
  m_bNeedUpdate = FALSE;

  //ASSERT(GetCurrCombatant() != NO_DUDE);
  prevDude = GetCurrCombatant();
  return; // PRS July 2009 isUpdate;
}

//*****************************************************************************
// NAME: COMBAT_DATA::CheckIdleTime
//
// PURPOSE:  
//*****************************************************************************
BOOL COMBAT_DATA::CheckIdleTime()
{
  int minIdleTime = 1000000;
  int diff;
  for (int i=0; i<m_iNumCombatants; i++)
  {
    diff = m_iCurrRound - m_aCombatants[i].lastAttackRound;
    if (diff < minIdleTime)
      minIdleTime = diff;
  }

  return (minIdleTime > MAX_COMBAT_IDLE_ROUNDS);
}


//*****************************************************************************
// NAME: COMBAT_DATA::CheckMorale
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::CheckMorale()
{
  for (int i=0; i<m_iNumCombatants; i++)
    m_aCombatants[i].CheckMorale();
}

//*****************************************************************************
// NAME:    COMBAT_DATA::StartNewRound
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::StartNewRound()
{
  SetUpdateFlag();

  if (m_bLastRound)
    m_bCombatOver = TRUE;

  //party.incrementClock(1); // handled in ProcessTimeSensitiveData()
  TRACE("Start new combat round\n");

  // battle rages on
  CheckDyingCombatants();
  CheckMorale();

  int i;
  for (i=0; i<m_iNumCombatants; i++)
  {
    m_aCombatants[i].m_isCombatReady = -1; // Force a search of SPECIAL_ABILITIES in IsDone()
    if ((m_aCombatants[i].charCanTakeAction()) && (m_aCombatants[i].IsDone()))
    {      
      if (m_aCombatants[i].state != ICS_Casting)
      {
        m_aCombatants[i].turnIsDone = FALSE;
        // stay guarding until told otherwise
        if (m_aCombatants[i].OnAuto())
        {
          if (m_aCombatants[i].state != ICS_Guarding)
            m_aCombatants[i].state = ICS_None;
        }
        else
          m_aCombatants[i].state = ICS_None;
      }
      /* The calculation of availAttacks has been moved from
       * StartNewRound() to getNextCombatant().
       * I did this because Manikus complained that a spell
       * given during a combat round that increased the number
       * of attacks on a target did not take effect until the
       * start of the following round.  PRS 20091215
       *
       * double leftoverAttacks=m_aCombatants[i].availAttacks;             
       * m_aCombatants[i].determineAvailAttacks();
       * double maxAttacks = ceil(m_aCombatants[i].availAttacks);
       * m_aCombatants[i].availAttacks += leftoverAttacks;
       * m_aCombatants[i].availAttacks = min(m_aCombatants[i].availAttacks, maxAttacks);
       * m_aCombatants[i].movement = 0;      
       */
    }

    if (m_aCombatants[i].charOnCombatMap())
    {
      if (activeSpellData.LingerSpellAffectsTarget(m_aCombatants[i].self, m_aCombatants[i].x,m_aCombatants[i].y))
      {
        // A lingering spell is active at the location 
        // this dude is located at.
        TRACE("%i re-targeted by at least one active linger spell at location %i,%i\n",
              m_aCombatants[i].self, m_aCombatants[i].x, m_aCombatants[i].y);

        activeSpellData.ActivateLingerSpellsOnTarget(m_aCombatants[i].self, m_aCombatants[i].x, m_aCombatants[i].y);
      }    
    }
  }

  m_iCurrInitiative = 1;
  m_iCurrRound++;
  if (m_iCurrRound%10 == 0) 
    m_iCurrTurn++;

  DetermineCombatInitiative();

  PrepareHooks();
  for (i=0; i<m_iNumCombatants; i++)
  {
    scriptContext.SetCharacterContext(&m_aCombatants[i]);
    scriptContext.SetCombatantContext(&m_aCombatants[i]);
    m_aCombatants[i].combatantSA.RunScripts(START_COMBAT_ROUND,
                                            ScriptCallback_RunAllScripts, 
                                            NULL,
                                            m_aCombatants[i].GetName(), 
                                            "");
    scriptContext.Clear();
  };

  getNextCombatant();

  m_iPrevRndCombatant = GetCurrCombatant();
  m_bStartingNewCombatRound=TRUE;  
}

//*****************************************************************************
// NAME:    COMBAT_DATA::CheckDyingCombatants
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::CheckDyingCombatants()
{
  for (int i=0; i<m_iNumCombatants; i++)
  {
    if (m_aCombatants[i].GetAdjStatus() == Dying)        
    {      
      if (!m_aCombatants[i].isBandaged)
        m_aCombatants[i].TakeDamage(1,FALSE);
    }
  }
}


//*****************************************************************************
//    NAME: COMBAT_DATA::CheckForDeadGuys
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::CheckForDeadGuys()
{
  for (int i=0; i<m_iNumCombatants; i++)
  {
    if (m_aCombatants[i].GetAdjHitPoints() <= 0)
      m_aCombatants[i].blitDeadSprite();
  }
}

//*****************************************************************************
// NAME:    displayCursor
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
BYTE aval=100;
void COMBAT_DATA::displayCursor(BOOL coverFullIcon)
{
  int i,j;
  int x = TerrainToScreenCoordX(m_iCursorX);
  int y = TerrainToScreenCoordY(m_iCursorY);

  if (x < 0) return;
  if (y < 0) return;

  // make sure entire cursor will fit in display area
  if (   ((x+(CursorSrc.right-CursorSrc.left)) <= (COMBAT_SCREEN_WIDTH))
      && ((y+(CursorSrc.bottom-CursorSrc.top)) <= (SCREEN_HEIGHT-COMBAT_MENU_HEIGHT)))
  {
    int dude = getCombatantInCell(m_iCursorX, m_iCursorY, 1,1);
  
    if (coverFullIcon)
    {
      if ((dude != NO_DUDE) && (dude < NumCombatants()/*m_aCombatants.GetSize()*/))
      {
        for (i=0;i<m_aCombatants[dude].height;i++)
        {
          for (j=0;j<m_aCombatants[dude].width;j++)
          {
            GraphicsMgr.SetClipRect(&m_combatRect);              
            GraphicsMgr.BlitAlphaImage(aval, // 50%
                                      TerrainToScreenCoordX(m_aCombatants[dude].x+j),
                                      TerrainToScreenCoordY(m_aCombatants[dude].y+i),
                                      CombatCursorArt.surface,
                                      &CursorSrc);
            GraphicsMgr.SetClipRect(&m_fullRect);
          }
        }      
        m_aCombatants[dude].displayCombatSprite();
      }
    }
    else 
    {
      GraphicsMgr.SetClipRect(&m_combatRect);  
      // destX, destY, SrcSurfaceNum, src RECT
      GraphicsMgr.BlitAlphaImage(aval, x,y, CombatCursorArt.surface, &CursorSrc);
      GraphicsMgr.SetClipRect(&m_fullRect);
      
      if ((dude != NO_DUDE) && (dude < NumCombatants()/*m_aCombatants.GetSize()*/))
        m_aCombatants[dude].displayCombatSprite();
    }
  }  
}

//*****************************************************************************
// NAME:    COMBAT_DATA::PlaceCursorOnCurrentDude
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::PlaceCursorOnCurrentDude(BOOL ForceCenter)
{
  int dude = GetCurrCombatant();
  if ((dude < 0) || (dude >= m_iNumCombatants)) return;
  if (    (m_iCursorX != m_aCombatants[dude].x)
       || (m_iCursorY != m_aCombatants[dude].y)
     )
  {
    m_iCursorX = m_aCombatants[dude].x;
    m_iCursorY = m_aCombatants[dude].y;
    EnsureVisibleTarget(dude, ForceCenter);
  };
}

//*****************************************************************************
//    NAME: COMBAT_DATA::PlaceCursorOnDude
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::PlaceCursorOnDude(int dude, BOOL ForceCenter)
{
  if ((dude < 0) || (dude >= m_iNumCombatants)) return;
  m_iCursorX = m_aCombatants[dude].x;
  m_iCursorY = m_aCombatants[dude].y;
  EnsureVisibleTarget(dude, ForceCenter);
}

//*****************************************************************************
// NAME: COMBAT_DATA::DisplayCombatants
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::DisplayCombatants()
{
  int i;
  // display dying dudes first  
  for (i=0; i<NumCombatants()/*m_aCombatants.GetSize()*/; i++)
  {
    if (m_aCombatants[i].charUnconscious())
      m_aCombatants[i].displayCombatSprite();
  }  

  for (i=0; i<NumCombatants()/*m_aCombatants.GetSize()*/; i++)
  {
    if (!m_aCombatants[i].charUnconscious())
      m_aCombatants[i].displayCombatSprite();
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::HandleCurrState
//
// PURPOSE: 
//
// Returns non-zero if screen needs to be redrawn
//
//*****************************************************************************
int COMBAT_DATA::HandleCurrState()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].HandleCurrState();
  return 0;
}

//*****************************************************************************
// NAME:    COMBAT_DATA::HandleTimeDelayMsgExpired
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
void COMBAT_DATA::HandleTimeDelayMsgExpired()
{
  //PlaceCursorOnCurrentDude();
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].HandleTimeDelayMsgExpired();
}

//*****************************************************************************
// NAME:    COMBAT_DATA::HandleTimeDelayMsgBegin
//
// PURPOSE: 
//
// RETURNS: None
//*****************************************************************************
int COMBAT_DATA::HandleTimeDelayMsgBegin()
{  
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].HandleTimeDelayMsgBegin();
  return -1;
}

//*****************************************************************************
// NAME: COMBAT_DATA::getCharWeaponText
//
// PURPOSE:  
//*****************************************************************************
void COMBAT_DATA::getCharWeaponText(int index, CString &wpn, CString &dmg)
{
  m_aCombatants[index].getCharWeaponText(wpn, dmg);
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedMissileAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedMissileAnimation()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].NeedMissileAnimation();
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitMissileAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitMissileAnimation()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitMissileAnimation();  
    //PlaceCursorOnCurrentDude();
  }
}
//*****************************************************************************
//    NAME: COMBAT_DATA::NeedHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedHitAnimation()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].NeedHitAnimation();
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitHitAnimation()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitHitAnimation();  
    //PlaceCursorOnDude(m_aCombatants[curr].target, FALSE);
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedSpellCastAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedSpellCastAnimation(int targ)
{
  ASSERT( (targ>=0) || (targ==-1) );
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);
  if (curr != NO_DUDE)
  {
    if (targ < 0) return FALSE;
    if (!m_aCombatants[targ].charOnCombatMap()) return FALSE;
    return (m_aCombatants[curr].NeedSpellCastAnimation());
  }
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitSpellCastAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitSpellCastAnimation(int targ)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);
  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitSpellCastAnimation(targ);  
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedSpellInRouteAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedSpellInRouteAnimation(int x, int y)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);
  if (curr != NO_DUDE)
    return (m_aCombatants[curr].NeedSpellInRouteAnimation(x,y));
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitSpellInRouteAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitSpellInRouteAnimation(int x, int y)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitSpellInRouteAnimation(x, y);  
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedSpellCoverageAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedSpellCoverageAnimation(int x, int y)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
    return m_aCombatants[curr].NeedSpellCoverageAnimation();
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitSpellCoverageAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitSpellCoverageAnimation(int x, int y)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitSpellCoverageAnimation(x, y);  
    //PlaceCursorOnCurrentDude();
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedSpellHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedSpellHitAnimation(int targ)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
    return m_aCombatants[curr].NeedSpellHitAnimation();
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitSpellHitAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitSpellHitAnimation(int targ)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitSpellHitAnimation(targ);  
    //PlaceCursorOnCurrentDude();
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::NeedSpellLingerAnimation
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::NeedSpellLingerAnimation()
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
    return m_aCombatants[curr].NeedSpellLingerAnimation();
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InitSpellLingerAnimation
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::InitSpellLingerAnimation(int mapx, int mapy)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr != NO_DUDE);

  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].InitSpellLingerAnimation(mapx,mapy);  
    //PlaceCursorOnCurrentDude();
  }
}

//*****************************************************************************
//    NAME: FormatCombatDelayText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatDelayText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  temp.Format("%s is delaying...",
              combatData.GetCombatant(dude).GetName());
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatEndTurnText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatEndTurnText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  temp.Format("%s ends turn...",
              combatData.GetCombatant(dude).GetName());
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatForceWinText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatForceWinText()
{
  CString temp;
  temp = "The party has forced a victory";
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatMoveText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatMoveText()
{  
  ASSERT(combatData.GetCurrCombatant() != NO_DUDE);

  int moves = combatData.GetCombatant(combatData.GetCurrCombatant()).GetAdjMaxMovement() - 
                combatData.GetCombatant(combatData.GetCurrCombatant()).movement;

  if ( moves < 0 ) moves = 0;

  CString temp;
  temp.Format("MOVES LEFT: %u", moves);
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatAttackText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatAttackText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  
  int target = combatData.GetCombatant(dude).GetCurrTarget();
  CString msg("attacks");
  CString temp;

  if (combatData.GetCombatant(dude).GetType()==MONSTER_TYPE)
  {    
    int inv_item = combatData.GetCombatant(dude).myItems.GetReady(WeaponHand);
    if (inv_item != NO_READY_ITEM)
    {
      // using a weapon attack
      GLOBAL_ITEM_ID giID = combatData.GetCombatant(dude).myItems.GetItem(inv_item);
      ITEM_DATA *idata = itemData.GetItemData(giID);
      if (idata!=NULL) msg = idata->attackMsg;
    }
    else
    {
      // only use monster attack msgs when using natural, non-weapon, attacks
      int nbrattacks = combatData.GetCombatant(dude).GetNbrAttacks();
      int availattacks = combatData.GetCombatant(dude).availAttacks;
      int currAttack = nbrattacks - availattacks;

      ASSERT( (currAttack>=0) && (currAttack<nbrattacks) );
      if ((currAttack < 0)||(currAttack>=nbrattacks)) currAttack=0;
      monsterData.GetMonsterAttackMsg(combatData.GetCombatant(dude).origKey, currAttack, msg);
    }
  }
  else
  {
    int inv_item = combatData.GetCombatant(dude).myItems.GetReady(WeaponHand);
    if (inv_item != NO_READY_ITEM)
    {      
      GLOBAL_ITEM_ID giID = combatData.GetCombatant(dude).myItems.GetItem(inv_item);
      ITEM_DATA *idata = itemData.GetItemData(giID);
      if (idata!=NULL) msg = idata->attackMsg;      
    }
  }

  if (msg.GetLength()==0) msg = "attacks";
  temp.Format("%s %s %s", 
              combatData.GetCombatant(dude).GetName(),
              msg,
              combatData.GetCombatant(target).GetName());

  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatGuardText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatGuardText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  temp.Format("%s is guarding...",
              combatData.GetCombatant(dude).GetName());
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatBandageText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatBandageText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  int bandagetarget = combatData.GetCombatant(dude).bandageWho;
  if (bandagetarget >= 0)
  {    
    temp.Format("%s is bandaging %s...",
                combatData.GetCombatant(dude).GetName(),
                combatData.GetCombatant(bandagetarget).GetName());
    combatData.GetCombatant(dude).bandageWho = -1;
  }
  else
    temp.Format("%s is bandaging...",
                combatData.GetCombatant(dude).GetName());
  
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatTurnUndeadText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatTurnUndeadText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  temp.Format("%s turns undead...",
              combatData.GetCombatant(dude).GetName());
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatFleeText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatFleeText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  

  CString temp;
  temp.Format("%s runs away...",
              combatData.GetCombatant(dude).GetName());
  return temp;
}

//*****************************************************************************
//    NAME: FormatCombatCastText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatCombatCastText()
{ 
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  
  //GLOBAL_SPELL_ID charspellbookkey = combatData.m_aCombatants[dude].GetSpellKeyBeingCast();
  CHARACTER_SPELL cspell;
  if (!combatData.GetCombatant(dude).GetSpellBeingCast(cspell))
    return CString("");

  CString name(spellData.GetSpellName(cspell.spell));
  if (name.IsEmpty()) name = "Unknown!";
  CString temp;
  temp.Format("%s begins casting %s...",
              combatData.GetCombatant(dude).GetName(),
              name);
  return temp;
}

//*****************************************************************************
//    NAME: FormatActivateSpellText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatActivateSpellText()
{
  int dude = combatData.GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  
  //GLOBAL_SPELL_ID charspellbookkey = combatData.m_aCombatants[dude].GetSpellKeyBeingCast();
  CHARACTER_SPELL cspell;
  if (!combatData.GetCombatant(dude).GetSpellBeingCast(cspell))
    return CString("");

  CString name(spellData.GetSpellName(cspell.spell));
  if (name.IsEmpty()) name = "Unknown!";
  CString temp;
  temp.Format("%s casts %s...",
              combatData.GetCombatant(dude).GetName(),
              name);
  return temp;
}

//*****************************************************************************
//    NAME: FormatSpellHitText
//
// PURPOSE: 
//
//*****************************************************************************
CString FormatSpellHitText()
{
  CString msg("");

  COMBATANT *pTarget=NULL;
  COMBATANT *pCaster = combatData.getActiveCastingCharacter();
  ASSERT(pCaster!=NULL);  
  if (pCaster==NULL) return msg;
  if (pCaster->targets.m_SelectingUnits)
  {
    int target = pCaster->targets.m_targets[0];
    pTarget = combatData.getCombatantPtr(target);
    ASSERT(pTarget!=NULL);  
    if (pTarget==NULL) return msg;
  }//else target is a map coordinate

  //SPELLBOOK_SPELL_ID charspellbookkey = pCaster->GetSpellBookKeyBeingCast();
  CHARACTER_SPELL cspell;
  if (!pCaster->GetSpellBeingCast(cspell))
    return msg;
  SPELL_DATA *pSpell = spellData.GetSpellData(cspell.spell);
  ASSERT(pSpell!=NULL);  
  if (pSpell==NULL) return msg;

  msg = pSpell->CastMsg;
  msg.Replace("/s", pSpell->Name);
  msg.Replace("/c", pCaster->GetName());
  if (pTarget!=NULL)
    msg.Replace("/t", pTarget->GetName());
  else
    msg.Replace("/t", "");

  return msg;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::FormatSpecAbMsg
//
// PURPOSE: 
//
//*****************************************************************************
CString COMBAT_DATA::FormatSpecAbMsg(DWORD sa_state)
{
  int dude = GetCurrCombatant();
  ASSERT(dude != NO_DUDE);  
  return (m_aCombatants[dude].FormatSpecAbMsg(sa_state));
}

//*****************************************************************************
//    NAME: COMBAT_DATA::FormatTimeDelayMsg
//
// PURPOSE: 
//
//*****************************************************************************
CString COMBAT_DATA::FormatTimeDelayMsg(overallCombatState state)
{
  if (state == OCS_None)
    state = (overallCombatState)GetCombatantState();

  CString msg="";
  switch (state)
  {
  case OCS_Casting:
    msg = FormatCombatCastText();
    break;
  case OCS_ActivateSpell:
    msg=FormatActivateSpellText();
    break;
  case OCS_Attacking:
    msg = FormatCombatAttackText();
    break;
  case OCS_Guarding: 
    msg = FormatCombatGuardText();
    break;
  case OCS_Bandaging:
    msg = FormatCombatBandageText();
    break;
  case OCS_Using:
    break;
  case OCS_Moving: // shouldn't happen
    ASSERT(FALSE);
    //msg = FormatCombatMoveText();
    break;
  case OCS_Turning:
    msg = FormatCombatTurnUndeadText();
    break;
  case OCS_Fleeing:
    msg = FormatCombatFleeText();
    break;
  case OCS_Delaying:
    msg = FormatCombatDelayText();
    break;
  case OCS_EndingTurn:
    msg = FormatCombatEndTurnText();
    break;
  case OCS_ForcingWin:
    msg = FormatCombatForceWinText();
    break;
  }
  return msg;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::Clear
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::Clear()
{
  m_bStartingNewCombatRound = FALSE;
  m_bIsWilderness      = FALSE;
  m_bCombatActive      = FALSE;
  ResetAim();
  ClearSounds();
  m_hCharDeathSound      = -1;
  m_hCharMoveSound       = -1;
  m_hTurnUndeadSound     = -1;
  m_bLastRound         = FALSE;
  m_bCombatOver        = FALSE;
  m_eCombatVictor         = NoWinner;
  m_iExpPointTotal        = 0;
  m_iStartTerrainX        = -1;
  m_iStartTerrainY        = -1;
  m_iCursorX              = -1; 
  m_iCursorY              = -1;
  m_iPartyStartX          = -1;
  m_iPartyStartY          = -1;
  m_iCurrInitiative       = -1;
  m_iCurrRound            = -1;
  m_iCurrTurn             = -1;
  m_iNumCombatants        = 0; 
  m_bNeedUpdate        = FALSE;
  m_bMonsterNoMove     = FALSE;
  m_bMonsterAlwaysMiss = FALSE;
  m_bMonsterAlwaysHit  = FALSE; 
  m_bMonsterNoCast     = FALSE;
  m_bMonsterNoFlee     = FALSE;
  m_bMonsterPlyrControl= FALSE;
  m_bMonsterNoDeath    = FALSE;
  m_iNumMonFlee           = 0;
  m_iNumMonSlain          = 0;
  m_iNumFrndFlee          = 0;
  m_iNumFrndSlain         = 0;
  m_bPartyAlwaysMiss   = FALSE; 
  m_bPartyAlwaysHit    = FALSE;
  m_bPartyNeverDies    = FALSE;
  m_bNoMagic           = FALSE;
  m_iCurrCaster           = -1;
  QComb.Clear();
  //m_aCombatants.RemoveAll();
  //m_aTempCOMBATANTS.RemoveAll();
  m_aCombatants.SetSize(MAX_COMBATANTS);
  m_aTempCOMBATANTS.SetSize(MAX_COMBATANTS);
}

//*****************************************************************************
//    NAME: COMBAT_DATA::GetCombatantState
//
// PURPOSE: 
//
//*****************************************************************************
individualCombatantState COMBAT_DATA::GetCombatantState()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].GetState(); 
  else
    return ICS_None;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::AddTarget
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::AddTarget(int target) 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].AddTarget(target); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::GetCurrTarget
//
// PURPOSE: 
//
//*****************************************************************************
int COMBAT_DATA::GetCurrTarget() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].GetCurrTarget(); 
  else
    return NO_DUDE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::GetNextTarget
//
// PURPOSE: 
//
//*****************************************************************************
int COMBAT_DATA::GetNextTarget() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].GetNextTarget(); 
  else
    return NO_DUDE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::RemoveCurrTarget
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::RemoveCurrTarget()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].RemoveCurrTarget(); 
}

//*****************************************************************************
//    NAME: COMBAT_DATA::AttacksRemaining
//
// PURPOSE: 
//
//*****************************************************************************
double COMBAT_DATA::AttacksRemaining()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].AttacksRemaining(); 
  else
    return 0.0;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::IsAttacking
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::IsAttacking() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].IsAttacking(); 
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::CanTarget
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanTarget() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanTarget(); 
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::CanMove
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanMove()  
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanMove(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanCast
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanCast()  
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanCast(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanUse
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanUse()   
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)    
    return m_aCombatants[curr].CanUse(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanGuard
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanGuard() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanGuard(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanAttack
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanAttack(int target) 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].canAttack(target); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanTurnUndead
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanTurnUndead()  
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanTurnUndead(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::CanDelay
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::CanDelay() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].CanDelay(); 
  else
    return FALSE;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::EndTurn
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::EndTurn(individualCombatantState newState) 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].EndTurn(newState);     
  }
}
//*****************************************************************************
//    NAME: COMBAT_DATA::StopMoving
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::StopMoving() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].StopMoving(); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::Quick
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::Quick(BOOL Enable) 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].Quick(Enable); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::Guard
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::Guard() 
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].Guard(); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::StartInitialSpellCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::StartInitialSpellCasting(GLOBAL_SPELL_ID key)
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr!=NO_DUDE);
  if (curr != NO_DUDE)
    m_aCombatants[curr].StartInitialSpellCasting(key);
}
//*****************************************************************************
//    NAME: COMBAT_DATA::EndInitialSpellCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::EndInitialSpellCasting()
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr!=NO_DUDE);
  if (curr != NO_DUDE)
    m_aCombatants[curr].EndInitialSpellCasting();
}

//*****************************************************************************
//    NAME: COMBAT_DATA::StopSpellCasting
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::StopSpellCasting()
{
  int curr = m_iCurrCaster;//GetCurrCombatant();
  ASSERT(curr!=NO_DUDE);
  if (curr != NO_DUDE)
  {
    m_aCombatants[curr].StopCasting(TRUE); // end turn
    //m_aCombatants[curr].EndTurn();
  }
}

//*****************************************************************************
//    NAME: COMBAT_DATA::StartTurnUndead
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::StartTurnUndead()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].TurnUndead();
}

//*****************************************************************************
//    NAME: COMBAT_DATA::TurnUndead
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::TurnUndead() 
{
  int curr = GetCurrCombatant();
  if (curr == NO_DUDE) return;

  if (!m_aCombatants[curr].IsPartyMember()) 
  {
    m_aCombatants[curr].EndTurn();
    return;
  }
  
  int level = m_aCombatants[curr].GetCurrentLevel(ClericFlag);
  if (isPaladin(m_aCombatants[curr].GetClass()))
  {
    int plevel = m_aCombatants[curr].GetCurrentLevel(PaladinFlag)-2;
    level = max(level,plevel);
  }

  menu.setHorzOrient();
  menu.setUseActive(FALSE); // no highlighting

  for (int i=0; i<m_iNumCombatants; i++)
  {
    if ((m_aCombatants[i].IsPartyMember())||(m_aCombatants[i].GetIsFriendly())) continue;

    int result = GetUndeadTurnValue(level, m_aCombatants[i].GetAdjUndeadType(), m_eUndeadTurnModifier);
    switch (result)
    {
    case UNDEAD_NOAFFECT:
    case UNDEAD_TURNED: 
    case UNDEAD_DESTROY:
      break;
    default:
      // should be minimum dice roll for success
      if ((result>=1)&&(result<=20))
      {
        if (result >= rollDice(20,1,0.0))
          result = UNDEAD_TURNED;
      }
      break;
    }

    CString turnMsg("");
    switch (result)
    {
    case UNDEAD_TURNED: // undead will flee the combat map (status=Running, then Fled)
      menu.setMenu(EmptyMenuData); // starts empty
      PlaceCursorOnDude(i,FALSE); // calls ensure visible
      //EnsureVisibleTarget(i);        
      UpdateCombatTimeDelayMsg();
      theApp.AppSleep(225); // give time to see normal icon before death animation


      turnMsg.Format("%s is turned by %s...", 
                     m_aCombatants[i].GetName(),m_aCombatants[curr].GetName());
      menu.changeMenuItem(1, turnMsg);
      UpdateCombatTimeDelayMsg();
      m_aCombatants[i].PlayCombatTurnUndead();

      theApp.AppSleep(500); // give time to see normal icon

      m_aCombatants[i].EndTurn();
      m_aCombatants[i].SetStatus(Running);
      m_aCombatants[i].isTurned=TRUE;
      m_aCombatants[i].m_iLastAttacker = curr; // so it knows who to flee from
      TRACE("Turn Undead: %i is turned and fleeing\n", m_aCombatants[i].self);
      break;
    case UNDEAD_DESTROY: // undead will be removed from the map (status=Gone)
      menu.setMenu(EmptyMenuData); // starts empty
      PlaceCursorOnDude(i,FALSE); // calls ensure visible
      //EnsureVisibleTarget(i);
      UpdateCombatTimeDelayMsg();
      theApp.AppSleep(225); // give time to see normal icon before death animation
      
      turnMsg.Format("%s is destroyed by %s...", 
                     m_aCombatants[i].GetName(),m_aCombatants[curr].GetName());
      menu.changeMenuItem(1, turnMsg);
      UpdateCombatMoveScreen();        

      m_aCombatants[i].EndTurn();
      m_aCombatants[i].SetStatus(Gone);
      m_aCombatants[i].SetHitPoints(-10);
      m_aCombatants[i].blitDeadSprite(); // built-in delay at the end, plays death sound
      placeCombatant(m_aCombatants[i].x, m_aCombatants[i].y, NO_DUDE, m_aCombatants[i].width, m_aCombatants[i].height);
      TRACE("Turn Undead: %i is turned and destroyed\n", m_aCombatants[i].self);
      break;
    default:
      // do nothing
      break;
    }
  }
  PlaceCursorOnCurrentDude();
  m_aCombatants[curr].EndTurn();
}
//*****************************************************************************
//    NAME: COMBAT_DATA::DelayAction
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::DelayAction() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].DelayAction(); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::AttackOver
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::AttackOver() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].AttackOver(); 
}
//*****************************************************************************
//    NAME: COMBAT_DATA::MakeAttack
//
// PURPOSE: 
//
//*****************************************************************************
//void COMBAT_DATA::MakeAttack() 
//{ 
//  int curr = GetCurrCombatant();
//  if (curr != NO_DUDE)
//    m_aCombatants[curr].makeAttack(m_aCombatants[curr].GetCurrTarget()); 
//}
//*****************************************************************************
//    NAME: COMBAT_DATA::StartAttack
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::StartAttack() 
{ 
  int curr = GetCurrCombatant();
  int currTarg;
  currTarg = m_aCombatants[curr].GetCurrTarget();

  if ( (curr != NO_DUDE) && (currTarg != NO_DUDE))
    return m_aCombatants[curr].StartAttack(currTarg); 
  return FALSE;
}


//*****************************************************************************
//    NAME: COMBAT_DATA::GetCombatState
//
// PURPOSE: 
//
//*****************************************************************************
overallCombatState COMBAT_DATA::GetCombatState() 
{ 
  ASSERT( m_iNumCombatants > 0 );

  // first check for some special cases
  // that override the current combatant state
  //
  if (IsCombatOver())
  {
    return OCS_CombatOver;
  }

  if (m_bStartingNewCombatRound)
  {    
    return OCS_StartNewRound;
  }

  if ((IsNewCombatant()) && (!IsFreeAttacker()))
  {
    return OCS_CombatRoundDelay;
  }

  return (overallCombatState)GetCombatantState();
}

// have we changed to the next active combatant
//*****************************************************************************
//    NAME: COMBAT_DATA::IsNewCombatant
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::IsNewCombatant() 
{ 
  int dude = GetCurrCombatant();
  return ((m_iPrevRndCombatant != dude));  
}

// is there a queued combatant that is waiting to make a free
// attack.
//*****************************************************************************
//    NAME: COMBAT_DATA::IsFreeAttacker
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::IsFreeAttacker() 
{ 
  return ((QComb.Top() != NO_DUDE) && (!QComb.ChangeStats())); 
}

//*****************************************************************************
//    NAME: COMBAT_DATA::HandleCombatRoundMsgExpired
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::HandleCombatRoundMsgExpired()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].HandleCombatRoundMsgExpired(); 
  m_iPrevRndCombatant = curr;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::OnAuto
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::OnAuto() 
{ 
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].OnAuto(); 
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::toggleReadyItem
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::toggleReadyItem(int item)
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].toggleReadyItem(item); 
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::delCharacterItem
//
// PURPOSE: 
//
//*****************************************************************************
BOOL COMBAT_DATA::delCharacterItem(int index, int qty)
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    return m_aCombatants[curr].delCharacterItem(index, qty); 
  else
    return FALSE;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::determineAvailAttacks
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::determineAvailAttacks()
{
  int curr = GetCurrCombatant();
  if (curr != NO_DUDE)
    m_aCombatants[curr].determineAvailAttacks(); 
}

//*****************************************************************************
//    NAME: COMBAT_DATA::PlayCombatDeath
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::PlayCombatDeath() 
{ 
  if (!PlaySoundEffects) return;
  if (!ValidSound(m_hCharDeathSound))
    m_hCharDeathSound = LoadSound(m_pEvent->DeathSound);
  if (!PlaySound(m_hCharDeathSound))
    m_hCharDeathSound=-1;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::PlayCombatMove
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::PlayCombatMove() 
{ 
  if (!PlaySoundEffects) return;
  if (!ValidSound(m_hCharMoveSound))
    m_hCharMoveSound = LoadSound(m_pEvent->MoveSound);
  if (!PlaySound(m_hCharMoveSound))
    m_hCharMoveSound=-1;
}
//*****************************************************************************
//    NAME: COMBAT_DATA::PlayCombatTurnUndead
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::PlayCombatTurnUndead() 
{ 
  if (!PlaySoundEffects) return;
  if (!ValidSound(m_hTurnUndeadSound))
    m_hTurnUndeadSound = LoadSound(m_pEvent->TurnUndeadSound);
  if (!PlaySound(m_hTurnUndeadSound))
    m_hTurnUndeadSound=-1;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::LoadSounds
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::LoadSounds()
{
  ClearSounds();
  m_hCharDeathSound  = LoadSound(m_pEvent->DeathSound);
  m_hCharMoveSound   = LoadSound(m_pEvent->MoveSound);
  m_hTurnUndeadSound = LoadSound(m_pEvent->TurnUndeadSound);
}
//*****************************************************************************
//    NAME: COMBAT_DATA::ClearSounds
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::ClearSounds()
{
  if (m_hCharDeathSound >= 0)
    ClearSound(m_hCharDeathSound);
  if (m_hCharMoveSound >= 0)
    ClearSound(m_hCharMoveSound);
  if (m_hTurnUndeadSound >= 0)
    ClearSound(m_hTurnUndeadSound);
  m_hCharDeathSound = -1;
  m_hCharMoveSound = -1;
  m_hTurnUndeadSound=-1;
}

//
// 1 round = 1 minute
// 1 round has 10 initiative steps
// 10 rounds = 1 turn
//
//*****************************************************************************
//    NAME: COMBAT_DATA::GetCurrCombatant
//
// PURPOSE: 
//
//*****************************************************************************
int COMBAT_DATA::GetCurrCombatant() 
{ 
  return QComb.Top(); 
}

//*****************************************************************************
//    NAME: CurrCombatTurn
//
// PURPOSE: 
//
//*****************************************************************************
int CurrCombatTurn() 
{ 
  return combatData.m_iCurrTurn; 
}

//*****************************************************************************
//    NAME: CurrCombatRound
//
// PURPOSE: 
//
//*****************************************************************************
int CurrCombatRound()
{
  return combatData.m_iCurrRound;
}

//*****************************************************************************
//    NAME: CurrCombatInitiative
//
// PURPOSE: 
//
//*****************************************************************************
int CurrCombatInitiative() 
{ 
  return combatData.m_iCurrInitiative; 
}

//*****************************************************************************
//    NAME: COMBAT_DATA::getCombatPartyCharacter
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT& COMBAT_DATA::getCombatPartyCharacter(int PartyIdx)
{
  if ((PartyIdx < 0) || (PartyIdx >= party.numCharacters))
    return getActiveCombatCharacter();

  for (int i=0; i<m_iNumCombatants; i++)
  {
    if ((m_aCombatants[i].IsPartyMember()) && (m_aCombatants[i].origKey == PartyIdx))
    {
      if (m_aCombatants[GetCurrCombatant()].GetType() == MONSTER_TYPE)
        WriteDebugString("getActiveCombatCharacter() called for monster combatant\n");
      return m_aCombatants[i];
    }
  }

  ASSERT( FALSE ); // shouldn't get here
  return m_aCombatants[0];
}

//*****************************************************************************
//    NAME: *COMBAT_DATA::getCombatantPtr
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *COMBAT_DATA::getCombatantPtr(int index)
{
  if ((index < 0) || (index >= m_iNumCombatants))
  {
    //ASSERT(FALSE);
    return NULL;
  }
  return &m_aCombatants[index];
}

//*****************************************************************************
//    NAME: COMBAT_DATA::ProcessTimeSensitiveData
//
// PURPOSE: 
//
//*****************************************************************************
static int lastUpdateRound=-1;
static int lastUpdateInitiative=-1;
bool COMBAT_DATA::ProcessTimeSensitiveData(LONGLONG currTime)
{
  if (IsCombatOver()) return false;
  // update once per combat round
  int currRnd = CurrCombatRound();
  int currInit = CurrCombatInitiative();
  bool UpdateScreen = false;

  bool update = ((lastUpdateRound != currRnd) || (lastUpdateInitiative != currInit));
  if (update)
  {     
    if ((lastUpdateRound >= 0) || (lastUpdateInitiative >= 0))
    {
      int roundDelta = currRnd-lastUpdateRound;

      if (roundDelta > 0) // 1 round per minute
      {
        activeSpellData.ProcessTimeSensitiveData(roundDelta);
        pendingSpellData.ProcessTimeSensitiveData(roundDelta, currInit);        
        party.incrementClock(roundDelta);        

        for (int i=0; i<m_iNumCombatants; i++)
        {          
          m_aCombatants[i].CheckForExpiredSpellEffects();
          
          if (m_aCombatants[i].GetAdjSpecAb(SA_Poisoned))
          {
            double currhp = m_aCombatants[i].GetHitPoints();
            if (currhp > 0)
            {
              double rateofloss = 0.25;
              BOOL IsSlowed = m_aCombatants[i].GetAdjSpecAb(SA_SlowPoison);
              if (IsSlowed)              
                rateofloss = 0.12;
            
              double hploss = currhp * rateofloss;
              hploss += 0.5; // round up
              hploss = floor(hploss);
              if (hploss < 1.0)
                hploss = 1.0;
              int newhptotal = currhp - hploss;
              if ((IsSlowed) && (newhptotal < 1))
                newhptotal = 1;   
              hploss = currhp - newhptotal;              
              if (hploss > 0)
              {
                PlaceCursorOnDude(i,FALSE); // calls ensure visible
                CString msg;
                msg.Format("%s takes poison damage", m_aCombatants[i].GetName());
                if (m_aCombatants[i].IsSpellPending())
                  msg += ", pending spell is lost";
                FormatCombatText(combatTextData, msg);
                UpdateCombatTimeDelayMsg();
                m_aCombatants[i].PlayHit();
                m_aCombatants[i].TakeDamage(hploss,FALSE);
                theApp.AppSleep(GetMsgTimeDelay());
                ClearFormattedText(combatTextData);
              }
            }
          }
        }
      }
    }
    lastUpdateRound=currRnd;
    lastUpdateInitiative = currInit;
  }

  return UpdateScreen;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::GetContext
//
// PURPOSE: 
//
//*****************************************************************************
ActorType COMBAT_DATA::GetContext(int i)
{
  if ((i>=0)&&(i<m_iNumCombatants)) return m_aCombatants[i].GetContext();
  else return NULL_ACTOR;
}

// force immediate party victory
//*****************************************************************************
//    NAME: COMBAT_DATA::ForcePartyVictory
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::ForcePartyVictory()
{
  int partycnt = 0;
  for (int i=0; i<m_iNumCombatants; i++)
  {
    if ((!m_aCombatants[i].GetIsFriendly()) && (m_aCombatants[i].charOnCombatMap()) && (!m_aCombatants[i].IsPartyMember()))
    {
      PlaceCursorOnDude(i,FALSE); // calls ensure visible
      UpdateCombatTimeDelayMsg();      
      m_aCombatants[i].TakeDamage(2000000000, FALSE);
      theApp.AppSleep(GetMsgTimeDelay());
    }
    else
    {
      if (m_aCombatants[i].IsAlive())
        partycnt++;
    }
  }
  if (partycnt > 0) m_bLastRound = TRUE;
  PlaceCursorOnCurrentDude();
}

// apply the spell effects to targets
//*****************************************************************************
//    NAME: COMBAT_DATA::InvokePendingSpellOnCombatant
//
// PURPOSE: 
//
//*****************************************************************************
int COMBAT_DATA::InvokePendingSpellOnCombatant(int caster, int targ, int activeSpellKey, GLOBAL_SPELL_ID spellDbKey)
{  
  if (caster == NO_DUDE) return activeSpellKey;  
  if (targ == NO_DUDE) return activeSpellKey;
  
  COMBATANT &cdude = m_aCombatants[caster];
  COMBATANT &tdude = m_aCombatants[targ];
 
  GLOBAL_SPELL_ID spelldbkey = spellDbKey;

  if (!spelldbkey.IsValidSpell())
  {
    if (cdude.GetType()!=MONSTER_TYPE)
    {
      CHARACTER_SPELL cspell;
      if (!cdude.GetSpellBeingCast(cspell)) 
      {
        ASSERT(FALSE);
        return activeSpellKey;
      }
      spelldbkey = cspell.spell;
    }
    else
    {
      // monsters don't have spell books
      spelldbkey = cdude.GetSpellKeyBeingCast();

      // better check this code when it's finally ready to be
      // executed for the first time. I'm not certain that the key will be
      // correct.
      // Seems ok after I fixed a couple things  PRS. 19 Dec 2009  ASSERT(FALSE);
    }
  }

  SPELL_DATA *pSdata = spellData.GetSpellData(spelldbkey);
  if (pSdata==NULL) 
  {
    ASSERT(FALSE);
    return activeSpellKey;
  }

  WriteDebugString("InvokePendingSpellOnCombatant(): caster %i, targ %i, key %i, spell %s\n", 
                    caster, targ, activeSpellKey, pSdata->Name);
  
  activeSpellKey = cdude.InvokeSpellOnTarget(tdude, activeSpellKey, spelldbkey);
  
  return activeSpellKey;
}

//*****************************************************************************
//    NAME: COMBAT_DATA::InvokePendingSpellOnMap
//
// PURPOSE: Casting spell with area effect, and it has no current targets
//          in map area. Still need to create an active spell record.
//
//*****************************************************************************
int COMBAT_DATA::InvokePendingSpellOnMap(int caster, int activeSpellKey)
{
  WriteDebugString("InvokePendingSpellOnMap(): caster %i, key %i\n", 
                   caster, activeSpellKey);
  
  if (caster == NO_DUDE) return -1;  
  
  COMBATANT &cdude = m_aCombatants[caster];
 
  GLOBAL_SPELL_ID spelldbkey; // Starts clear = -1;
  if (cdude.GetType()!=MONSTER_TYPE)
  {
    CHARACTER_SPELL cspell;
    if (!cdude.GetSpellBeingCast(cspell)) 
    {
      ASSERT(FALSE);
      return -1;
    }
    spelldbkey = cspell.spell;
  }
  else
  {
    // monsters don't have spell books
    spelldbkey = cdude.GetSpellKeyBeingCast();
  }

  SPELL_DATA *pSdata = spellData.GetSpellData(spelldbkey);
  if (pSdata==NULL) 
  {
    ASSERT(FALSE);
    return -1;
  }

  if (activeSpellKey < 0)
  {
    if (pSdata->Duration_Rate != Permanent)
      activeSpellKey=activeSpellData.GetNextKey();
  }

  if (pSdata->Duration_Rate != Permanent)
  {            
    ASSERT( activeSpellKey >= 0 );    
    ASSERT( activeSpellData.Get(activeSpellKey) == NULL );
      
    ACTIVE_SPELL aspell;
    aspell.caster = cdude.GetContext(pSdata->ClassMask);
    aspell.key = activeSpellKey;
    aspell.spellKey = pSdata->m_gsID; // spell db key
    aspell.CountTime = 0;
    aspell.StopTime = 0;
    aspell.Lingers = pSdata->Lingers;
    aspell.lingerData.OnceOnly = pSdata->LingerOnceOnly;    

    
    // add affected map locations to active spell record
    aspell.lingerData.mapdata.RemoveAll();
    POSITION pos = cdude.targets.m_maptarglist.GetHeadPosition();
    while (pos != NULL)
      aspell.lingerData.mapdata.AddTail(cdude.targets.m_maptarglist.GetNext(pos));
    
    // add targets to active spell record so that linger spell checks
    // won't re-target same dudes once they move (unless allowed by OnceOnly flag)
    for (int i=0;i<cdude.targets.m_targets.GetSize();i++)
      aspell.lingerData.affectedtargs.AddTail(cdude.targets.m_targets[i]);
    
    WriteDebugString("Adding active spell cast by %i\n", aspell.caster.Instance);

    double val = 0.0;
    // determine duration value
    val = cdude.EvalDiceAsClass(pSdata->Duration, pSdata->ClassMask);
    ASSERT( val > 0.0 );                 

    // convert duration value into game minutes
    switch (pSdata->Duration_Rate)
    {
    case inRounds:
      val = max(1,val); // 1 round min
      aspell.StopTime = val + party.getElapsedMinutes();
      break;
    case byNbrAttacks:
    case byDamageTaken: 
      aspell.StopTime = val;
      break;
    case inHours:
      val = val * 60.0;
      val = max(1,val); // 1 round min
      aspell.StopTime = val + party.getElapsedMinutes();      
      break;
    case inDays:
      val = val * 24.0 * 60.0;
      val = max(1,val); // 1 round min
      aspell.StopTime = val + party.getElapsedMinutes();      
      break;
    default: 
      ASSERT(FALSE); 
      break;
    }

    activeSpellData.AddWithCurrKey(aspell);
  }

  return activeSpellKey;
}


//*****************************************************************************
//    NAME: COMBAT_DATA::RemoveSpellEffect
//
// PURPOSE: 
//
//*****************************************************************************
void COMBAT_DATA::RemoveSpellEffect(DWORD parent, SPELL_DATA *pSpell)
{
  for (int i=0; i<m_iNumCombatants; i++)
    m_aCombatants[i].RemoveSpellEffect(parent,pSpell);
}

//*****************************************************************************
//    NAME: *GetLastAttackerOf
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetLastAttackerOf(COMBATANT *pDude)
{
  if (pDude == NULL) return NULL;
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (combatData.GetCombatant(i).m_iLastAttacked == pDude->self)
    {
      return (&combatData.GetCombatant(i));
    }
  }
  return NULL;
}

//*****************************************************************************
//    NAME: *GetLeastDamagedFriendly
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetLeastDamagedFriendly()
{
  int HighestHP = -1;
  int HighestHPIndex = -1;  

  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (combatData.GetCombatant(i).GetIsFriendly())
    {
      int hp = combatData.GetCombatant(i).GetAdjHitPoints();
      if (hp > HighestHP)
      {
        HighestHP = hp;
        HighestHPIndex = i;
      }      
    }
  }

  if (HighestHPIndex >= 0)
    return (&combatData.GetCombatant(HighestHPIndex));

  return NULL;  
}

//*****************************************************************************
//    NAME: *GetLeastDamagedEnemy
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetLeastDamagedEnemy()
{
  int HighestHP = -1;
  int HighestHPIndex = -1;  
  
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (!combatData.GetCombatant(i).GetIsFriendly())
    {
      int hp = combatData.GetCombatant(i).GetAdjHitPoints();
      if (hp > HighestHP)
      {
        HighestHP = hp;
        HighestHPIndex = i;
      }      
    }
  }
  
  if (HighestHPIndex >= 0)
    return (&combatData.GetCombatant(HighestHPIndex));
  
  return NULL;  
}

//*****************************************************************************
//    NAME: *GetMostDamagedFriendly
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetMostDamagedFriendly()
{
  int LowestHP = 999999999;
  int LowestHPIndex = -1;  
  
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (combatData.GetCombatant(i).GetIsFriendly())
    {
      int hp = combatData.GetCombatant(i).GetAdjHitPoints();
      if (hp < LowestHP)
      {
        LowestHP = hp;
        LowestHPIndex = i;
      }      
    }
  }
  
  if (LowestHPIndex >= 0)
    return (&combatData.GetCombatant(LowestHPIndex));
  
  return NULL;  
}

//*****************************************************************************
//    NAME: *GetMostDamagedEnemy
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetMostDamagedEnemy()
{
  int LowestHP = 999999999;
  int LowestHPIndex = -1;  
  
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (!combatData.GetCombatant(i).GetIsFriendly())
    {
      int hp = combatData.GetCombatant(i).GetAdjHitPoints();
      if (hp < LowestHP)
      {
        LowestHP = hp;
        LowestHPIndex = i;
      }      
    }
  }
  
  if (LowestHPIndex >= 0)
    return (&combatData.GetCombatant(LowestHPIndex));
  
  return NULL;  
}


//*****************************************************************************
//    NAME: *GetNearestTo
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetNearestTo(COMBATANT *pDude)
{
  if (pDude == NULL) return NULL;

  int NearestDist = 999999999;
  int NearestIndex = -1;  
  
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    int dist = distance(pDude->self, 
                        pDude->x, 
                        pDude->y,
                        combatData.GetCombatant(i).self,
                        combatData.GetCombatant(i).x,
                        combatData.GetCombatant(i).y);
    if (dist < NearestDist)
    {
      NearestDist = dist;
      NearestIndex = i;
    }      
  }
  
  if (NearestIndex >= 0)
    return (&combatData.GetCombatant(NearestIndex));
  
  return NULL;  
}

//****************************************************************************
//    NAME  GetCombatantX and GetCombatantY
//                given: combatant index
//****************************************************************************
int GetCombatantX(int index)
{
  return combatData.GetCombatant(index).x;
}

int GetCombatantY(int index)
{
  return combatData.GetCombatant(index).y;
}



//*****************************************************************************
//    NAME: *GetNearestEnemyTo
//
// PURPOSE: 
//
//*****************************************************************************
COMBATANT *GetNearestEnemyTo(COMBATANT *pDude)
{
  if (pDude == NULL) return NULL;
  
  int NearestDist = 999999999;
  int NearestIndex = -1;  
  
  for (int i=0; i<combatData.NumCombatants(); i++)
  {
    if (!combatData.GetCombatant(i).GetIsFriendly())
    {
      int dist = distance(pDude->self, 
                          pDude->x, 
                          pDude->y,
                          combatData.GetCombatant(i).self,
                          combatData.GetCombatant(i).x,
                          combatData.GetCombatant(i).y);

      if (dist < NearestDist)
      {
        NearestDist = dist;
        NearestIndex = i;
      }      
    }
  }
  
  if (NearestIndex >= 0)
    return (&combatData.GetCombatant(NearestIndex));
  
  return NULL;  
}

//*****************************************************************************
//    NAME: GetDistanceBetween_ByPartyIndex
//
// PURPOSE: 
//
//*****************************************************************************
int COMBAT_DATA::GetDistanceBetween_ByPartyIndex(int PartyMember1Idx, int PartyMember2Idx)
{
  if (PartyMember1Idx == PartyMember2Idx)
    return 0; // distance from me to me

  int CombatantIdx1 = -1;
  int CombatantIdx2 = -1;

  for (int i=0; i<m_iNumCombatants; i++)
  {
    if (    (combatData.m_aCombatants[i].IsPartyMember())
        && ((combatData.m_aCombatants[i].GetType() == CHAR_TYPE) || (combatData.m_aCombatants[i].GetType() == NPC_TYPE))
        &&  (combatData.m_aCombatants[i].origKey >= 0))
    {
      if (combatData.m_aCombatants[i].origKey == PartyMember1Idx)
        CombatantIdx1 = i;
      else if (combatData.m_aCombatants[i].origKey == PartyMember2Idx)
        CombatantIdx2 = i;
    }
  }

  if ((CombatantIdx1 >= 0) && (CombatantIdx2 >= 0))
  {
    int dist = distance(combatData.m_aCombatants[CombatantIdx1].self, 
                        combatData.m_aCombatants[CombatantIdx1].x,
                        combatData.m_aCombatants[CombatantIdx1].y, 

                        combatData.m_aCombatants[CombatantIdx2].self, 
                        combatData.m_aCombatants[CombatantIdx2].x,
                        combatData.m_aCombatants[CombatantIdx2].y);

    return (dist);
  }

  ASSERT(FALSE);
  return 1000;
}
