/******************************************************************************
* Filename: Combatants.h
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/
#ifndef __COMBATANTS_H__
#define __COMBATANTS_H__

//#include "SharedQueue.h"
#include "GameEvent.h"
#include "Combatant.h"

enum combatVictorType { NoWinner, PartyWins, PartyRanAway, MonsterWins };

const int MAX_COMBATANTS = 100;
const int MAX_CHARS_IN_ROW = 3; // how many chars wide the battle line can be
const int MAX_COMBAT_IDLE_ROUNDS = 20;

enum overallCombatState 
{ 
  OCS_None,
  OCS_Casting,
  OCS_Attacking, 
  OCS_Guarding, 
  OCS_Bandaging, 
  OCS_Using,
  OCS_Moving,
  OCS_Turning,
  OCS_Fleeing,
  OCS_Fled,
  OCS_CombatRoundDelay,
  OCS_CombatOver,
  OCS_Delaying,
  OCS_EndingTurn,
  OCS_ForcingWin,
  OCS_StartNewRound,
  OCS_ActivateSpell,
};

const int NUM_COMBATANTS_STATES = 17;
extern char *CombatantsStateText[NUM_COMBATANTS_STATES];

//*****************************************************************************
//*****************************************************************************
//*****************************************************************************

struct COMBAT_DATA 
{
  COMBAT_DATA() { Clear(); }
  ~COMBAT_DATA() {  }
  void Clear();
  void InitCombatData(COMBAT_EVENT_DATA *event);
  void AddCombatants();
  void AddCharsToCombatants();
  void AddMonstersToCombatants();
  void AddMonsterToCombatants(const CString& monsterName, int friendly);
  void AddMonsterToCombatants(int monsterDataIndex);

  void DetermineIfCombatOver();
  void DetermineVictoryExpPoints();
  void getNumCombatants(int* pNumParty, int* pNumMons);
  COMBATANT& getActiveCombatCharacter();  
  COMBATANT& getCombatPartyCharacter(int PartyIdx);
  COMBATANT *getCombatantPtr(int index);

  COMBATANT *getActiveCastingCharacter();
  void SetActiveCastingCharacter(int dude);

  int getNextCombatant();

  void ForcePartyVictory(); // force immediate party victory
  void DetermineCombatInitiative();

  individualCombatantState GetCombatantState();
  void determineInitCombatPos();
  void determineInitCombatPos(COMBATANT *pCombatant, int i, BOOL *reset, BOOL ignoreCurrentCombatant);
  int determinePartyOrientation(int x, int y);
  void GetNextDirection(eventDirType *pCurrDir, int *pFacing);
  int GetMonsterDistance(eventDirType dir);
  void getNextMonsterCombatPos(COMBATANT &monster);
  void GetMonsterCoords(int &x, int &y, int dist, int dir);
  void getNextCharCombatPos(COMBATANT* pDude, BOOL* pReset, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacter      (COMBATANT* pDude, int   x, int   y, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterLeft  (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterRight (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterNorth (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterSouth (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterHorz  (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);
  BOOL PlaceCharacterVert  (COMBATANT* pDude, int* px, int* py, BOOL ignoreCurrentCombatant);

  BOOL IsCombatOver() { return m_bCombatOver; }

  BOOL IsNewCombatant();
  BOOL IsFreeAttacker();

  void /* PRS July 2009 BOOL */ UpdateCombat();
  void CheckForDeadGuys();
  void DisplayCombatants();
  void displayCursor(BOOL coverFullIcon);
  void HandleCombatRoundMsgExpired();
  void HandleTimeDelayMsgExpired();
  int  HandleTimeDelayMsgBegin();
  int  HandleCurrState();

  void RestoreToParty();

  void CheckDyingCombatants();
  void CheckMorale();
  BOOL CheckIdleTime();
  void StartNewRound();
  
  void AddTarget(int target) ;
  int GetCurrTarget() ;
  int GetNextTarget() ;
  BOOL CanTarget() ;
  int GetNextAim();
  int GetPrevAim();
  int GetCurrAim() { return m_iAimPos; }
  void ResetAim(); 
  void RemoveCurrTarget();
  BOOL IsValidTarget(COMBATANT *pAttacker, COMBATANT *pTarget);
  
  BOOL CanSeeParty(int x, int y);
  BOOL IsAttacking();
  double AttacksRemaining();
  BOOL CanMove();  
  BOOL CanCast();  
  BOOL CanUse();   
  BOOL CanGuard(); 
  BOOL CanAttack(int target) ;
  BOOL CanTurnUndead();  
  BOOL CanDelay(); 
  BOOL CanBandage();
  void Bandage();
  void EndTurn(individualCombatantState newState=ICS_None); 
  void StopMoving(); 
  void Quick(BOOL Enable); 
  void Guard(); 
  void TurnUndead();
  void StartTurnUndead();
  void StartInitialSpellCasting(GLOBAL_SPELL_ID key);
  void EndInitialSpellCasting();
  void StopSpellCasting();
  void DelayAction(); 
  void AttackOver(); 
  //void MakeAttack(); 
  BOOL StartAttack(); 

  int GetDistanceBetween_ByPartyIndex(int PartyMember1Idx, int PartyMember2Idx);

  int InvokePendingSpellOnCombatant(int caster, int targ, int activeSpellKey, GLOBAL_SPELL_ID spellDbKey=GLOBAL_SPELL_ID());
  int InvokePendingSpellOnMap(int caster, int activeSpellKey);    
  void RemoveSpellEffect(DWORD parent, SPELL_DATA *pSpell);

  BOOL NeedMissileAnimation();
  void InitMissileAnimation();
  BOOL NeedHitAnimation();
  void InitHitAnimation();
  BOOL NeedSpellCastAnimation(int targ);
  void InitSpellCastAnimation(int targ);
  BOOL NeedSpellInRouteAnimation(int x, int y);
  void InitSpellInRouteAnimation(int x, int y);
  BOOL NeedSpellCoverageAnimation(int x, int y);
  void InitSpellCoverageAnimation(int x, int y);
  BOOL NeedSpellHitAnimation(int targ);
  void InitSpellHitAnimation(int targ);
  BOOL NeedSpellLingerAnimation();
  void InitSpellLingerAnimation(int mapx, int mapy);

  BOOL toggleReadyItem(int item);
  BOOL delCharacterItem(int index, int qty);
  void determineAvailAttacks();
  
  bool ProcessTimeSensitiveData(LONGLONG currTime);

  overallCombatState GetCombatState();

  void getCharWeaponText(int index, CString &wpn, CString &dmg);
  BOOL OnAuto(); 
  void SetUpdateFlag() { m_bNeedUpdate = TRUE; }

  CString FormatTimeDelayMsg(overallCombatState state=OCS_None);
  CString FormatSpecAbMsg(DWORD sa_state);

  void moveCursorLeft();
  void moveCursorRight();
  void moveCursorForward();
  void moveCursorBackward();
  void PlaceCursorOnCurrentDude(BOOL ForceCenter=FALSE); // on currently active dude
  void PlaceCursorOnDude(int dude,BOOL ForceCenter); // on specified dude

  void moveLeft();
  void moveRight();
  void moveForward();
  void moveBackward();
  void moveNW();
  void moveNE();
  void moveSW();
  void moveSE();
  BOOL CurrMoveWouldFleeMap();

  void PlayCombatDeath();
  void PlayCombatMove();
  void PlayCombatTurnUndead();

  void LoadSounds();
  void ClearSounds();

  int GetCurrCombatant();

  COMBAT_EVENT_DATA *m_pEvent;

  int m_hCharDeathSound;
  int m_hCharMoveSound;
  int m_hTurnUndeadSound;

  BOOL m_bStartingNewCombatRound;
  BOOL m_bCombatActive;
  BOOL m_bLastRound;
  BOOL m_bCombatOver;
  BOOL m_bNeedUpdate;
  int m_iExpPointTotal;
  int m_iCurrCaster;

  eventTurnUndeadModType m_eUndeadTurnModifier;
 	combatVictorType       m_eCombatVictor;

  int m_iMoveToX      , m_iMoveToY;
	int m_iStartTerrainX, m_iStartTerrainY;
	int m_iCursorX      , m_iCursorY;
  int m_iPartyStartX  , m_iPartyStartY;
  int m_iPrevRndCombatant;
	int m_iCurrInitiative;
	int m_iCurrRound;
  int m_iCurrTurn;
  int m_iAimPos;

  eventSurpriseType m_eSurprise;

  int m_iActiveScripter;
  ActorType GetContext(int i);

  // used for morale checks
  int m_iNumMonFlee;
  int m_iNumMonSlain;
  int m_iNumFrndFlee;
  int m_iNumFrndSlain;

  // flags that alter normal combat behavior
  BOOL m_bMonsterNoMove;
  BOOL m_bMonsterAlwaysMiss;
  BOOL m_bMonsterAlwaysHit;
  BOOL m_bMonsterNoCast;
  BOOL m_bMonsterPlyrControl;
  BOOL m_bMonsterNoFlee;
  BOOL m_bMonsterNoDeath;
  BOOL m_bPartyAlwaysMiss;
  BOOL m_bPartyAlwaysHit;
  BOOL m_bNoMagic;
  BOOL m_bPartyNeverDies; 
  BOOL m_bPartyNoExperience;
  
  // which combat map format is used
  BOOL m_bIsWilderness;

  QueuedCombatantData QComb;

  // used for clipping
  RECT m_combatRect;
  RECT m_fullRect;

  private:
	  int m_iNumCombatants;
	  CArray<COMBATANT,  COMBATANT&> m_aCombatants;
	  CArray<COMBATANT,  COMBATANT&> m_aTempCOMBATANTS;
  public:
    COMBATANT &GetCombatant(int dude){return m_aCombatants[dude];};
    int NumCombatants(void){return m_iNumCombatants;};
};

CString FormatCombatMoveText();
CString FormatSpellHitText();

//extern COMBAT_MSGS combatMsgs;
extern COMBAT_DATA combatData;


#endif //__COMBATANTS_H__