/******************************************************************************
* Filename: Dbutils.cpp
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include "..\Shared\stdafx.h"
//#include "..\Shared\Version.h"
#include "resource.h"

#include <afxpriv.h>
#include "UAFWinEd.h"
#include "items.h"
#include "monster.h"
#include "spell.h"
#include "class.h"
#include "itemdb.h"
#include "DbUtils.h"
#include "ConfigFile.h"
#include "ItemEditor.h"
#include "MonsterEditor.h"
#include "SpellEditor.h"
#include "ImportWarning.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDbUtils dialog

CDbUtils::CDbUtils(CWnd* pParent /*=NULL*/)
	: CDialog(CDbUtils::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDbUtils)
	m_SpellSize = _T("");
	m_MonsterSize = _T("");
	m_ItemSize = _T("");
  m_SpecAbSize = _T("");
	m_ClassSize = _T("");
	m_BaseclassSize = _T("");
	m_RaceSize = _T("");
	m_AbilitySize = _T("");
	//}}AFX_DATA_INIT
}


void CDbUtils::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDbUtils)
	DDX_Text(pDX, IDC_SPELLSIZE, m_SpellSize);
	DDX_Text(pDX, IDC_MONSTERSIZE, m_MonsterSize);
	DDX_Text(pDX, IDC_ITEMSIZE, m_ItemSize);
	DDX_Text(pDX, IDC_SPECABSIZE, m_SpecAbSize);
	DDX_Text(pDX, IDC_CLASSSIZE, m_ClassSize);
	DDX_Text(pDX, IDC_BASECLASSSIZE, m_BaseclassSize);
	DDX_Text(pDX, IDC_RACESIZE, m_RaceSize);
	DDX_Text(pDX, IDC_ABILITYSIZE, m_AbilitySize);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDbUtils, CDialog)
	//{{AFX_MSG_MAP(CDbUtils)
	ON_BN_CLICKED(IDC_CONVERTITEMDB, OnConvertitemdb)
	ON_BN_CLICKED(IDC_CONVERTMONSTERDB, OnConvertmonsterdb)
	ON_BN_CLICKED(IDC_CONVERTSPELLDB, OnConvertspelldb)
	ON_BN_CLICKED(IDC_EXPORTITEMDB, OnExportitemdb)
	ON_BN_CLICKED(IDC_EXPORTMONSTERDB, OnExportmonsterdb)
	ON_BN_CLICKED(IDC_EXPORTSPELLDB, OnExportspelldb)
	ON_BN_CLICKED(IDC_LOADDEFITEMS, OnLoaddefitems)
	ON_BN_CLICKED(IDC_LOADDEFMONSTERS, OnLoaddefmonsters)
	ON_BN_CLICKED(IDC_LOADDEFSPELLS, OnLoaddefspells)
	ON_BN_CLICKED(IDC_EDITITEMTEXT, OnEdititemtext)
	ON_BN_CLICKED(IDC_EDITMONSTERTEXT, OnEditmonstertext)
	ON_BN_CLICKED(IDC_EDITSPELLTEXT, OnEditspelltext)
	ON_BN_CLICKED(IDC_CONVERTCLASSDB, OnConvertclassdb)
	ON_BN_CLICKED(IDC_CONVERTBASECLASSDB, OnConvertbaseclassdb)
	ON_BN_CLICKED(IDC_CONVERTRACEDB, OnConvertracedb)
	ON_BN_CLICKED(IDC_CONVERTABILITYDB, OnConvertabilitydb)
	ON_BN_CLICKED(IDC_IMPORTALLTEXT, OnImportalltext)
#ifdef OLD_SPECIAL_ABILITIES
#else
	ON_BN_CLICKED(IDC_EXPORTSPECABDB, OnExportspecabdb)
	ON_BN_CLICKED(IDC_IMPORTSPECABDB, OnImportSpecAbDB)
#endif
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDbUtils message handlers

bool CDbUtils::LiveDangerously(void)
{
   CImportWarning dlg;
   int answer;
   if (dlg.DoModal() != IDOK)
   {
	   MsgBoxInfo("That is a wise choice");
	   return false;
   };
   if ( (sscanf(dlg.m_numeric,"%d",&answer) != 1) || (answer!=dlg.m_i))
   {
       MsgBoxInfo("Wrong answer....sorry");
       return false;
   };
   return true;
}


void CDbUtils::OnLoaddefitems() 
{
   CString filename;
   filename = rte.DataDir()+ITEM_DB_NAME;
   if (GetFilename(filename, "DAT", TRUE))
   {   
      AfxGetApp()->DoWaitCursor(1);
      loadData(itemData, filename);   
      AfxGetApp()->DoWaitCursor(-1);

     if (itemData.GetCount() == 0)
     {
        MsgBoxError("Operation Failed!", "Dat File Error");
        TRACE("Empty Item DB\n");
     }
     else 
     { 
        SET_MODIFIED;
        CString temp;
        temp.Format("Num Items in DB = %i\n", itemData.GetCount());
        TRACE(temp);
        MsgBoxInfo("Operation Successful", "Dat File Message");
     }
     m_ItemSize.Format("%i", itemData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnLoaddefmonsters() 
{
   CString filename;
   filename = rte.DataDir()+MONSTER_DB_NAME;
   if (GetFilename(filename, "DAT", TRUE))
   {   
     AfxGetApp()->DoWaitCursor(1);
     loadData(monsterData, filename);
     AfxGetApp()->DoWaitCursor(-1);

     if (monsterData.GetCount() <= 0)
     {
        MsgBoxError("Operation Failed!", "Dat File Error");
        TRACE("Empty Monster DB\n");
     }
     else 
     {
        SET_MODIFIED;
        CString temp;
        temp.Format("Num Monsters in DB = %i\n", monsterData.GetCount());
        TRACE(temp);
        MsgBoxInfo("Operation Successful", "Dat File Message");
     }   

     m_MonsterSize.Format("%i", monsterData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnLoaddefspells() 
{
   CString filename;
   filename = rte.DataDir()+SPELL_DB_NAME;
   if (GetFilename(filename, "DAT", TRUE))
   {   
     AfxGetApp()->DoWaitCursor(1);
     loadData(spellData, filename);
     AfxGetApp()->DoWaitCursor(-1);

     if (spellData.GetCount() <= 0)
     {
       MsgBoxError("Operation Failed!", "Dat File Error");
       TRACE("Empty Spell DB\n");
     }
     else 
     {
       SET_MODIFIED;
       CString temp;
       temp.Format("Num Spells in DB = %i\n", spellData.GetCount());
       TRACE(temp);
       MsgBoxInfo("Operation Successful", "Dat File Message");
     }

     m_SpellSize.Format("%i", spellData.GetCount());
     UpdateData(FALSE);
   }
}

BOOL CDbUtils::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
   m_ItemSize.Format("%i", itemData.GetCount());
   m_MonsterSize.Format("%i", monsterData.GetCount());
   m_SpellSize.Format("%i", spellData.GetCount());
   m_ClassSize.Format("%i", classData.GetCount());
   m_BaseclassSize.Format("%i", baseclassData.GetCount());
   m_RaceSize.Format("%i", raceData.GetCount());
   m_AbilitySize.Format("%i", abilityData.GetCount());
   m_SpecAbSize.Format("%i", specialAbilitiesData.GetCount());

   UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDbUtils::OnConvertitemdb() 
{
   CString filename;

   if (!LiveDangerously()) return;

   filename = rte.DataDir()+ITEM_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadItemDB(filename, &itemData);
     AfxGetApp()->DoWaitCursor(-1);
     CString msg;
     switch (status)
     {
     case 0:
       msg = "Items database file read successfully";
       break;
     case 1:
       msg = "Minor errors in Items database";
       break;
     case -1:
       msg = "Cannot read Items database file";
       break;
     case -2:
       msg = "Items database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading items database file";
       break;
     default:
       msg = "Unknown error reading items database file";
       break;
     };   

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");

     m_ItemSize.Format("%i", itemData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnConvertmonsterdb() 
{
   CString filename;

   if (!LiveDangerously()) return;
   filename = rte.DataDir()+MONSTER_CONFIG_NAME;

   if (GetFilename(filename, "TXT", TRUE))
   {
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadMonsterDB(filename, &monsterData);
     AfxGetApp()->DoWaitCursor(-1);
     CString msg;
     switch (status)
     {
     case 0:
       msg = "Monsters database file read successfully";
       break;
     case 1:
       msg = "Minor errors in Monsters database";
       break;
     case -1:
       msg = "Cannot read Monsters database file";
       break;
     case -2:
       msg = "Monsters database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading monsters database file";
       break;
     default:
       msg = "Unknown error reading monsters database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");

     m_MonsterSize.Format("%i", monsterData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnConvertspelldb() 
{
   CString filename;

   if (!LiveDangerously()) return;

   filename = rte.DataDir()+SPELL_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadSpellDB(filename, &spellData);
     AfxGetApp()->DoWaitCursor(-1);

     CString msg;
     switch (status)
     {
     case 0:
       msg = "Spell database read successfully";
       break;
     case 1:
       msg = "Minor errors in Spells database";
       break;
     case -1:
       msg = "Cannot read Spells database file";
       break;
     case -2:
       msg = "Spells database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading spells database file";
       break;
     default:
       msg = "Unknown error reading spells database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");
 
     m_SpellSize.Format("%i", spellData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnConvertclassdb() 
{
   CString filename;
   filename = rte.DataDir()+CLASS_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadClassDB(filename, &classData);
     AfxGetApp()->DoWaitCursor(-1);

     CString msg;
     switch (status)
     {
     case 0:
       msg = "Class database read successfully";
       break;
     case 1:
       msg = "Minor errors in Class database";
       break;
     case -1:
       msg = "Cannot read Class database file";
       break;
     case -2:
       msg = "Class database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading Class database file";
       break;
     default:
       msg = "Unknown error reading Class database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");
 
     m_ClassSize.Format("%i", classData.GetCount());
     UpdateData(FALSE);
   }
}


void CDbUtils::OnExportitemdb() 
{
   CString filename;
   filename = rte.DataDir()+ITEM_CONFIG_NAME;
   if (GetFilename(filename, "TXT", FALSE))
   {
     CONFIG_FILE ConfFile;
     AfxGetApp()->DoWaitCursor(1);
     BOOL success = (WriteItemDB(filename, &itemData, &ConfFile)==0);
     AfxGetApp()->DoWaitCursor(-1);

     if (!success)
        MsgBoxError("Operation Failed!", "Database Error");
     else
        MsgBoxInfo("Operation Successful", "Database Message");
   }
}

void CDbUtils::OnExportmonsterdb() 
{
   CString filename;
   filename = rte.DataDir()+MONSTER_CONFIG_NAME;
   if (GetFilename(filename, "TXT", FALSE))
   {
     CONFIG_FILE ConfFile;
     AfxGetApp()->DoWaitCursor(1);
     BOOL success = (WriteMonsterDB(filename, &monsterData, &ConfFile)==0);
     AfxGetApp()->DoWaitCursor(-1);

     if (!success)
        MsgBoxError("Operation Failed!", "Database Error");
     else
        MsgBoxInfo("Operation Successful", "Database Message");
   }
}

#ifdef OLD_SPECIAL_ABILITIES
#else
void CDbUtils::OnExportspecabdb() 
{
   CString filename;
   if (GetFilename(filename, "TXT", FALSE))
   {
     CONFIG_FILE ConfFile;
     AfxGetApp()->DoWaitCursor(1);
     BOOL success = (WriteSpecAbDB(filename, &specialAbilitiesData, &ConfFile)==0);
     AfxGetApp()->DoWaitCursor(-1);

     if (!success)
        MsgBoxError("Operation Failed!", "Database Error");
     else
        MsgBoxInfo("Operation Successful", "Database Message");
   }
}

void CDbUtils::OnImportSpecAbDB() 
{
   CString filename;

   filename = rte.DataDir()+SPECAB_CONFIG_NAME;

   if (!LiveDangerously()) return;

   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadSpecialAbilitiesDB(filename, &specialAbilitiesData);
     AfxGetApp()->DoWaitCursor(-1);
     CString msg;
     switch (status)
     {
     case 0:
       msg = "Special Abilities database file read successfully";
       break;
     case 1:
       msg = "Minor errors in Special Abilities database";
       break;
     case -1:
       msg = "Cannot read Special Abilities database file";
       break;
     case -2:
       msg = "Special Abilities database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading Special Abilities database file";
       break;
     default:
       msg = "Unknown error reading Special Abilities database file";
       break;
     };   

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");

     m_SpecAbSize.Format("%i", specialAbilitiesData.GetCount());
     UpdateData(FALSE);
   }
};

#endif

void CDbUtils::OnExportspelldb() 
{
   CString filename;
   filename = rte.DataDir()+SPELL_CONFIG_NAME;
   if (GetFilename(filename, "TXT", FALSE))
   {
     CONFIG_FILE ConfFile;
     AfxGetApp()->DoWaitCursor(1);
     BOOL success = (WriteSpellDB(filename, &spellData, &ConfFile)==0);
     AfxGetApp()->DoWaitCursor(-1);

     if (!success)
        MsgBoxError("Operation Failed!", "Database Error");
     else
        MsgBoxInfo("Operation Successful", "Database Message");
   }
}

void CDbUtils::OnEdititemtext() 
{
	// TODO: Add your control notification handler code here
  CString filename;
   filename = rte.DataDir()+ITEM_CONFIG_NAME;
  if (!GetFilename(filename, "TXT", TRUE))
    return;

  ITEM_DATA_TYPE tempItemData;
  CONFIG_FILE ConfFile;

  ReadItemDB(filename,&tempItemData,&ConfFile);

  CItemEditor dlg;
  dlg.m_pItemData=&tempItemData;
  if (dlg.DoModal()==IDCANCEL) return;  // Forget all changes
  // Rename old file to .bak
  // First form a .BAK filename
  if (backup(filename)!=0) return; // Message already printed.
  WriteItemDB(filename,&tempItemData, &ConfFile);
}

void CDbUtils::OnEditmonstertext() 
{
	// TODO: Add your control notification handler code here
  CString filename;
   filename = rte.DataDir()+MONSTER_CONFIG_NAME;
  if (!GetFilename(filename, "TXT", TRUE))
    return;

  MONSTER_DATA_TYPE tempMonsterData;
  CONFIG_FILE ConfFile;

  ReadMonsterDB(filename,&tempMonsterData,&ConfFile);

  CMonsterEditor dlg;
  dlg.m_pMonsterData=&tempMonsterData;
  if (dlg.DoModal()==IDCANCEL) return;  // Forget all changes
  // Rename old file to .bak
  // First form a .BAK filename
  if (backup(filename)!=0) return; // Message already printed.
  WriteMonsterDB(filename,&tempMonsterData, &ConfFile);
  
}


void CDbUtils::OnEditspelltext() 
{
	// TODO: Add your control notification handler code here
  CString filename;
   filename = rte.DataDir()+ITEM_CONFIG_NAME;
  if (!GetFilename(filename, "TXT", TRUE))
    return;

  SPELL_DATA_TYPE tempSpellData;
  CONFIG_FILE ConfFile;

  ReadSpellDB(filename,&tempSpellData,&ConfFile);

  CSpellEditor dlg;
  dlg.m_pSpellData=&tempSpellData;
  if (dlg.DoModal()==IDCANCEL) return;  // Forget all changes
  // Rename old file to .bak
  // First form a .BAK filename
  if (backup(filename)!=0) return; // Message already printed.
  WriteSpellDB(filename,&tempSpellData, &ConfFile);
}



void CDbUtils::OnConvertbaseclassdb() 
{
	// TODO: Add your control notification handler code here
	
   CString filename;
   filename = rte.DataDir()+CLASS_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadBaseclassDB(filename, &baseclassData);
     AfxGetApp()->DoWaitCursor(-1);

     CString msg;
     switch (status)
     {
     case 0:
       msg = "Baseclass database read successfully";
       break;
     case 1:
       msg = "Minor errors in Baseclass database";
       break;
     case -1:
       msg = "Cannot read Baseclass database file";
       break;
     case -2:
       msg = "Baseclass database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading Baseclass database file";
       break;
     default:
       msg = "Unknown error reading Baseclass database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");
 
     m_BaseclassSize.Format("%i", baseclassData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnConvertracedb() 
{
	// TODO: Add your control notification handler code here
	
   CString filename;
   filename = rte.DataDir()+RACE_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadRaceDB(filename, &raceData);
     AfxGetApp()->DoWaitCursor(-1);

     CString msg;
     switch (status)
     {
     case 0:
       msg = "Race database read successfully";
       break;
     case 1:
       msg = "Minor errors in Race database";
       break;
     case -1:
       msg = "Cannot read Race database file";
       break;
     case -2:
       msg = "Race database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading Race database file";
       break;
     default:
       msg = "Unknown error reading Race database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo(msg, "Database Status");
 
     m_RaceSize.Format("%i", raceData.GetCount());
     UpdateData(FALSE);
   }
	
}

void CDbUtils::OnConvertabilitydb() 
{
	// TODO: Add your control notification handler code here
	
   CString filename;
   filename = rte.DataDir()+ABILITY_CONFIG_NAME;
   if (GetFilename(filename, "TXT", TRUE))
   {
     SET_MODIFIED;
     AfxGetApp()->DoWaitCursor(1);
     int status = ReadAbilityDB(filename, &abilityData);
     AfxGetApp()->DoWaitCursor(-1);

     CString msg;
     switch (status)
     {
     case 0:
       msg = "Ability database read successfully";
       break;
     case 1:
       msg = "Minor errors in Ability database";
       break;
     case -1:
       msg = "Cannot read Ability database file";
       break;
     case -2:
       msg = "Ability database file appears to be the wrong kind of file";
       break;
     case -3:
       msg = "Internal program error reading Ability database file";
       break;
     default:
       msg = "Unknown error reading Ability database file";
       break;
     };

     WriteDebugString("%s\n", msg);
     MsgBoxInfo( msg, "Database Status");
 
     m_AbilitySize.Format("%i", abilityData.GetCount());
     UpdateData(FALSE);
   }
}

void CDbUtils::OnImportalltext() 
{
  CUAFWinEdApp *pApp = reinterpret_cast<CUAFWinEdApp*>(AfxGetApp());
  pApp->LoadDefaultDBFiles();     	
}
