/******************************************************************************
* Filename: ItemDB.cpp
* Copyright (c) 2000, UAF Development Team (email CocoaSpud@hotmail.com)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include "..\Shared\stdafx.h"
//#include "..\Shared\Version.h"
#include "resource.h"

//#include "UAFWinEd.h"

//#include "externs.h"
#include "SoundMgr.h"
#include "items.h"
#include "spell.h"
#include "char.h"
#include "monster.h"
//#include "picslot.h"

//#include <math.h>
#include "class.h"
#include "itemdb.h"
#include "ConfigFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int long2log(int n)
{ // convert to integer whose decimal form is:
  //   Exxx  where xxx is an integer and E is a power of 10.
  int E;
  for (E=0; n>999; E+=1000, n/=10)
    ;
  return E+n;
}

void ReadWriteSpecialAbilities(CONFIG_PKT *pkt, SPECIAL_ABILITIES *pSpecAb)
{
  if (pkt->IsStoring)
  {
    const CStringPAIR *pEntry;
    if (pkt->listNum == 0) pkt->listPos = pSpecAb->GetHeadPosition();

    if (pkt->listPos == NULL)
    {
      pkt->status = CONFIG_STAT_end;
      return;
    };
    pEntry = pSpecAb->GetNextData(pkt->listPos);
    pkt->value = pEntry->Key();;
    if (!pEntry->Value().IsEmpty())
    {
      pkt->value += ',';
      pkt->value += pEntry->Value();
    };
    if (pkt->listPos != NULL)
    {
      pkt->status = CONFIG_STAT_more;
    };
  }
  else
  {
    int index;
    CString name, param;
    index = pkt->value.Find(',');
    if (index < 0)
    {
      name = pkt->value;
      param = "";
    }
    else
    {
      name = pkt->value.Left(index);
      param = pkt->value.Right(pkt->value.GetLength()-index-1);
    };
    pSpecAb->InsertAbility(name, param,"","");
  };
}



ITEM_DATA tempITEM; // Where we build item.
ITEM_DATA_TYPE *pItemData=NULL;  // Where data will be put

// Handle clearing and storing of entire ITEM_DATA object
static void itemObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempITEM.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempITEM.Name.CompareNoCase("template")!=0)
    {
      if (pItemData->FindName(tempITEM.Name).IsValidItem())
      {
        CString msg;
        msg.Format("Duplicate name in items.txt = %s\n",tempITEM.Name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pItemData->AddItem(tempITEM);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pItemData->GetHeadPosition();
    tempITEM.Clear();
    tempITEM.Name="template";
    pkt->objectName=tempITEM.Name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempITEM=pItemData->GetAt(position);
      pItemData->GetNext(position);
      pkt->objectName=tempITEM.Name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}


static void DitemName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempITEM.Name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    pkt->objectName);
  tempITEM.Name = pkt->objectName;
}

static void DitemExperience(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_integer(pkt->value,tempITEM.Experience);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000000,
    tempITEM.Experience);
}

static void DitemCost(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Cost);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000000,
    tempITEM.Cost);
}

static void DitemBundleQuantity(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Bundle_Qty);
    return;
  };
	pkt->status=CONFIG_DECODE_long(
		pkt->value,
		pkt->errorMsg,
		1,1000,
		tempITEM.Bundle_Qty, 1);

  if (tempITEM.Bundle_Qty < 1)
    tempITEM.Bundle_Qty = 1;   
}

static void DitemEncumbrance(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Encumbrance);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000000,
    tempITEM.Encumbrance);
}

static void DitemAttackBonus(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Attack_Bonus);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    -2000000000,2000000000,
    tempITEM.Attack_Bonus);
}

static void DitemACbase(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Protection_Base);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    -2000000000,2000000000,
    tempITEM.Protection_Base);
}

static void DitemACbonus(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Protection_Bonus);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    -2000000000,2000000000,
    tempITEM.Protection_Bonus);
}

static void DitemSmallDamageDice(CONFIG_PKT *pkt)
{
  unsigned int dice=1, sides=1;
  int bonus=0;
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dice(
        pkt->value,
        tempITEM.Nbr_Dice_Sm,
        tempITEM.Dmg_Dice_Sm,
        tempITEM.Dmg_Bonus_Sm);
    return;
  };
  pkt->status=CONFIG_DECODE_dice(
    pkt->value,
    pkt->errorMsg,
    dice, sides, bonus);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempITEM.Nbr_Dice_Sm=dice;
    tempITEM.Dmg_Dice_Sm=sides;
    tempITEM.Dmg_Bonus_Sm=bonus;
  };
}

static void DitemLargeDamageDice(CONFIG_PKT *pkt)
{
  unsigned int dice=1, sides=1;
  int bonus=0;
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dice(
          pkt->value,
          tempITEM.Nbr_Dice_Lg,
          tempITEM.Dmg_Dice_Lg,
          tempITEM.Dmg_Bonus_Lg);
    return;
  };
  pkt->status=CONFIG_DECODE_dice(
    pkt->value,
    pkt->errorMsg,
    dice, sides, bonus);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempITEM.Nbr_Dice_Lg=dice;
    tempITEM.Dmg_Dice_Lg=sides;
    tempITEM.Dmg_Bonus_Lg=bonus;
  };
}

static void DitemHandsToUse(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Hands_to_Use);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2,
    tempITEM.Hands_to_Use,1);
}

static void DitemSpellLevel(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Spell_Level);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    1,9,
    tempITEM.Spell_Level,0);
}

static void DitemNumberCharges(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Num_Charges);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000000,
    tempITEM.Num_Charges,0);
}

static void DitemRange(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempITEM.Range);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000,
    tempITEM.Range,1);
}

static void DitemCursed(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_BOOL(pkt->value,tempITEM.Cursed) ;
    return;
  };
  pkt->status=CONFIG_DECODE_BOOL(
    pkt->value,
    pkt->errorMsg,
    tempITEM.Cursed, FALSE);
}
static void DitemNonLethal(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_BOOL(pkt->value,tempITEM.IsNonLethal);
    return;
  };
  pkt->status=CONFIG_DECODE_BOOL(
    pkt->value,
    pkt->errorMsg,
    tempITEM.IsNonLethal, TRUE);
}

static void DitemUsable(CONFIG_PKT *pkt)
{
  BOOL usable;
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_BOOL(pkt->value,tempITEM.IsUsable());
    return;
  };
  pkt->status=CONFIG_DECODE_BOOL(
    pkt->value,
    pkt->errorMsg,
    usable,FALSE);
  tempITEM.IsUsable(usable);
}

static void DitemScribable(CONFIG_PKT *pkt)
{
  BOOL scribable;
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_BOOL(pkt->value,tempITEM.IsScribable());
    return;
  };
  pkt->status=CONFIG_DECODE_BOOL(
    pkt->value,
    pkt->errorMsg,
    scribable,FALSE);
  tempITEM.IsScribable(scribable);
}

static void DitemLocationCarried(CONFIG_PKT *pkt)
{
  itemLocationType L;
  int i;
  static CONFIGID carriedIDs[] =
  {
    CONFIGID( "weapon hand" ,WeaponHand),
    CONFIGID( "shield"      ,ShieldHand),
    CONFIGID( "armor"       ,BodyArmor),
    CONFIGID( "on hands"    ,Hands),
    CONFIGID( "head"        ,Head),
    CONFIGID( "waist"       ,Waist),
    CONFIGID( "body"        ,BodyRobe),
    CONFIGID( "back"        ,Back),
    CONFIGID( "feet"        ,Feet),
    CONFIGID( "fingers"     ,Fingers),
    CONFIGID( "quiver"      ,AmmoQuiver),
    CONFIGID( "NotReadied"  ,CannotBeReadied),
    CONFIGID( ""          ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,carriedIDs,tempITEM.Location_Carried);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    carriedIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    L=(itemLocationType)i;
    tempITEM.Location_Carried=L;
  };
}

static void DitemRechargeRate(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID rechargeIDs[] =
  {
    CONFIGID( "never" ,irrNever),
    CONFIGID( "daily" ,irrDaily),
    CONFIGID( ""          ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,rechargeIDs,tempITEM.Recharge_Rate);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    rechargeIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempITEM.Recharge_Rate=(itemRechargeRate)i;
  };
}

static void DitemWeaponType(CONFIG_PKT *pkt)
{
  weaponClassType W;
  int i;
  static CONFIGID typeIDs[] =
  {
    CONFIGID( "not weapon"            ,NotWeapon),
    CONFIGID( "hand held blunt"       ,HandBlunt),
    CONFIGID( "hand held cutting"     ,HandCutting),
    CONFIGID( "hand held or thrown"   ,HandThrow),
    CONFIGID( "sling"                 ,SlingNoAmmo),
    CONFIGID( "bow"                   ,Bow),
    CONFIGID( "crossbow"              ,Crossbow),
    CONFIGID( "thrown only"           ,Throw),
    CONFIGID( "ammo"                  ,Ammo),
    CONFIGID( "SpellCaster"           ,SpellCaster),
    CONFIGID( ""          ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,typeIDs,tempITEM.Wpn_Type);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    typeIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    W=(weaponClassType)i;
    tempITEM.Wpn_Type=W;
  };
}

static void DitemSpellClass(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID classIDs[] =
  {
    CONFIGID( "MU"      ,1),
    CONFIGID( "Cleric"    ,2),
    CONFIGID( ""          ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,classIDs,tempITEM.Spell_Class);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    classIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempITEM.Spell_Class=i;
  };
}

static void DitemClass(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID classIDs[] =
  {
    CONFIGID( "MU"      ,MagicUserFlag),
    CONFIGID( "Cleric"  ,ClericFlag),
    CONFIGID( "thief"   ,ThiefFlag),
    CONFIGID( "fighter" ,FighterFlag),
    CONFIGID( "Paladin" ,PaladinFlag),
    CONFIGID( "ranger"  ,RangerFlag),
    CONFIGID( "Druid"   ,DruidFlag),
    CONFIGID( "bit7"    ,0x80),
    CONFIGID( "bit8"    ,0x100),
    CONFIGID( "bit9"    ,0x200),
    CONFIGID( "any"     ,AllClasses),
    CONFIGID( ""    ,0)
  };
  if (pkt->IsStoring)
  {
    if (tempITEM.Usable_by_Class==AllClasses) 
    {
      pkt->value="any";
    }
    else
    {
    CONFIG_ENCODE_flags(pkt->value,
                        classIDs,
                        tempITEM.Usable_by_Class);
    };
    return;
  };
  if (pkt->value=="") {
    tempITEM.Usable_by_Class=AllClasses;
  }
  else
  {
    pkt->status=CONFIG_DECODE_flags (
      pkt->value,
      pkt->errorMsg,
      classIDs,
      i);
    if (pkt->status==CONFIG_STAT_ok)
    {
      tempITEM.Usable_by_Class=i;
    };
  };
}


#ifdef OLD_SPECIAL_ABILITIES
static void DitemSpecialAbility(CONFIG_PKT *pkt)
{
  CList<int, int> *list;
  static CONFIGID abilityIDs[] =
  {
    CONFIGID("none"                 ,SA_None),
    CONFIGID("bless"                ,SA_Bless),
    CONFIGID("Curse"                ,SA_Curse),
    CONFIGID("Undead Fear"          ,SA_UndeadFear),
    CONFIGID("Enlarge"              ,SA_Enlarge),
    CONFIGID("Reduce"               ,SA_Reduce),
    CONFIGID("Charm Person"         ,SA_CharmPerson),
    CONFIGID("Detect Magic"         ,SA_DetectMagic),
    CONFIGID("Reflect Gaze Attack"  ,SA_ReflectGazeAttack),
    CONFIGID("Prot from Evil"       ,SA_ProtFromEvil),
    CONFIGID("Prot from Good"       ,SA_ProtFromGood),
    CONFIGID("Shield"               ,SA_Shield),
    CONFIGID("Sleep"                ,SA_Sleep),
    CONFIGID("Fog"                  ,SA_Fog),
    CONFIGID("Entangle"             ,SA_Entangle),
    CONFIGID("Invisible to Animals" ,SA_InvisibleToAnimals),
    CONFIGID("Invisible to Undead"  ,SA_InvisibleToUndead),
    CONFIGID("Non-Undead Fear"      ,SA_NonUndeadFear),
    CONFIGID("Sanctuary"            ,SA_Sanctuary),
    CONFIGID("Shillelagh"           ,SA_Shillelagh),
    CONFIGID("Displacement"         ,SA_Displacement),
    CONFIGID("Wizardry"             ,SA_Wizardry),
    CONFIGID("Hold Person"          ,SA_HoldPerson),
    CONFIGID("Vorpal Attack"        ,SA_VorpalAttack),
    CONFIGID("Silenced"             ,SA_Silence),
    CONFIGID("Poisoned"             ,SA_Poisoned),
    CONFIGID("Slow Poison"          ,SA_SlowPoison),
    CONFIGID("Mirror Image"         ,SA_MirrorImage),
    CONFIGID("Invisible"            ,SA_Invisible),
    CONFIGID("Enfeebled"            ,SA_Enfeebled),
    CONFIGID("Blindness"            ,SA_Blindness),
    CONFIGID("Diseased"             ,SA_Diseased),
    CONFIGID(""          ,0)
  };
  if (pkt->IsStoring)
  {
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempITEM.specAbs.GetHeadPosition();
    }

    if ((pkt->listNum>NUM_SPECIAL_ABILITIES)||(pkt->listPos==NULL))
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    specialAbilitiesType sa = tempITEM.specAbs.GetAt(pkt->listPos);
    CONFIG_ENCODE_enum(
          pkt->value,
          abilityIDs,
          sa);

    tempITEM.specAbs.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;

    /*
    if (pkt->listNum<tempITEM.specAbs.specAbs.GetSize())
    {
      CONFIG_ENCODE_enum(
            pkt->value,
            abilityIDs,
            tempITEM.specAbs.specAbs[pkt->listNum]);
      pkt->status=CONFIG_STAT_more;
    } 
    else 
    {
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
    }
    return;
    */
  };
  pkt->status=CONFIG_DECODE_enumList (
    pkt->value,
    pkt->errorMsg,
    abilityIDs,
    &list);
  if ((list!=NULL) && (pkt->status==CONFIG_STAT_ok))
  {
    POSITION pos;
    tempITEM.specAbs.Clear();
    pos=list->GetHeadPosition();
    while (pos!=NULL)
    {
      tempITEM.specAbs.EnableSpecAb((specialAbilitiesType)list->GetAt(pos));
    };
  };
}

#else
static void DitemSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempITEM.specAbs);
}
#endif

static void DitemIDname(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempITEM.Id_Name;
    return;
  };
  if (pkt->value.CompareNoCase("NA") == 0)
    tempITEM.Id_Name = "";
  else
	  pkt->status=CONFIG_DECODE_string(
		  pkt->value,
		  pkt->errorMsg,
		  tempITEM.Id_Name);
}
static void DitemAmmoType(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempITEM.AmmoType;
    return;
  };
  if (   (pkt->value.CompareNoCase("NA")==0)
      || (pkt->value.CompareNoCase("None")==0)
      || (pkt->value==""))
    tempITEM.AmmoType = "";
  else
	  pkt->status=CONFIG_DECODE_string(
		  pkt->value,
		  pkt->errorMsg,
		  tempITEM.AmmoType);
}


static void DitemSpellEffect(CONFIG_PKT *pkt)
{
  CString effect;
  if (pkt->IsStoring)
  {
    if ((!tempITEM.m_gsID.IsValidSpell()))
    {
      pkt->value="none";
      return;
    };
    // We need to find the tempITEM.Spell'th entry in spellData.
    POSITION pos=spellData.GetHeadPosition();
    DWORD index = 0;
    while (pos!=NULL)
    {
      if (index==tempITEM.m_gsID.IllegalConversion()) 
      {
        pkt->value=spellData.GetAt(pos).Name;
        return;
      };
      // PRS 20July2009  What is this?  Should we NextPos(pos) here somewhere???
      index++;
    };
    // There was no such spell!!!
    pkt->value.Format("Unknown spell index = 0x%08x",tempITEM.m_gsID);
    return;
  };

  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    tempITEM.m_gsID.Clear();
  }
  else
  {
    pkt->status=CONFIG_DECODE_string(
      pkt->value,
      pkt->errorMsg,
      effect);
    if (pkt->status==CONFIG_STAT_ok)
    {
      POSITION pos=spellData.GetHeadPosition();
      int index=0;
      while (pos!=NULL) {
        if (spellData.GetAt(pos).Name==effect) break;
        index++;
        spellData.GetNext(pos);
      };
      if (pos==NULL)
      {
        pkt->errorMsg.Format(
          "No such spell as %s is defined",effect);
        pkt->status=CONFIG_STAT_value;
      }
      else
      {
        // PRS July 2009 tempITEM.Spell=index;
        tempITEM.m_gsID.IllegalConversion(index);
      };
    };
  };
}

static void DitemROFperRound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_float(pkt->value,tempITEM.ROF_Per_Round);
    return;
  };
  pkt->status=CONFIG_DECODE_float(
    pkt->value,
    pkt->errorMsg,
    tempITEM.ROF_Per_Round);
}

static void DitemHitSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempITEM.HitSound;
    if (pkt->value == "") pkt->value = "none";          
    return;
  };

	if ((pkt->value == "") || (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		tempITEM.HitSound = "DEFAULT";
	}
	else if (pkt->value.CompareNoCase("none")==0)
	{
		tempITEM.HitSound="";
	}
	else
	{
		pkt->status=CONFIG_DECODE_file(
			pkt->value,
			pkt->errorMsg,
			tempITEM.HitSound);
	};
}

static void DitemMissSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempITEM.MissSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
	if ((pkt->value=="") || (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		tempITEM.MissSound = "DEFAULT";
	}
	else if (pkt->value.CompareNoCase("none")==0)
	{
		tempITEM.MissSound="";
	}
	else
	{
		pkt->status=CONFIG_DECODE_file(
			pkt->value,
			pkt->errorMsg,
			tempITEM.MissSound);
	};
}
static void DitemLaunchSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempITEM.LaunchSound;
    if (pkt->value == "") pkt->value = "none";          
    return;
  };

	if ((pkt->value == "") || (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		tempITEM.LaunchSound = "DEFAULT";
	}
	else if (pkt->value.CompareNoCase("none")==0)
	{
		tempITEM.LaunchSound="";
	}
	else
	{
		pkt->status=CONFIG_DECODE_file(
			pkt->value,
			pkt->errorMsg,
			tempITEM.LaunchSound);
	};
}

static void DitemSpriteData(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempITEM.MissileArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempITEM.MissileArt.filename;
      return;
    };
    if (tempITEM.MissileArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempITEM.MissileArt.filename,
      tempITEM.MissileArt.timeDelay,
      tempITEM.MissileArt.NumFrames,
      tempITEM.MissileArt.FrameWidth,
      tempITEM.MissileArt.FrameHeight,
      tempITEM.MissileArt.style,
      tempITEM.MissileArt.MaxLoops,
      tempITEM.MissileArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempITEM.MissileArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempITEM.MissileArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempITEM.MissileArt.filename,
      tempITEM.MissileArt.timeDelay,
      tempITEM.MissileArt.NumFrames,
      tempITEM.MissileArt.FrameWidth,
      tempITEM.MissileArt.FrameHeight,
      tempITEM.MissileArt.style,
      tempITEM.MissileArt.MaxLoops,
      tempITEM.MissileArt.flags);
  };
  tempITEM.MissileArt.picType = SpriteDib;
}
static void DitemHitSpriteData(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempITEM.HitArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempITEM.HitArt.filename;
      return;
    };
    if (tempITEM.HitArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempITEM.HitArt.filename,
      tempITEM.HitArt.timeDelay,
      tempITEM.HitArt.NumFrames,
      tempITEM.HitArt.FrameWidth,
      tempITEM.HitArt.FrameHeight,
      tempITEM.HitArt.style,
      tempITEM.HitArt.MaxLoops,
      tempITEM.HitArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempITEM.HitArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempITEM.HitArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempITEM.HitArt.filename,
      tempITEM.HitArt.timeDelay,
      tempITEM.HitArt.NumFrames,
      tempITEM.HitArt.FrameWidth,
      tempITEM.HitArt.FrameHeight,
      tempITEM.HitArt.style,
      tempITEM.HitArt.MaxLoops,
      tempITEM.HitArt.flags);
  };
  tempITEM.HitArt.picType = SpriteDib;
}

#undef df
#undef dh
#define df(a) Ditem##a
#define dh(a) IDS_HELP_item_##a

// default define
#define DEFID(a,b,c)    CONFIGID(a, CONFIG_DATA_##b, df(c), dh(c), 1, false, false)
// used to specify max number
#define DEFIDn(a,b,c,d) CONFIGID(a, CONFIG_DATA_##b, df(c), dh(c), d, false, false)
// used to disable writes to text file
#define DEFIDz(a,b,c,d) CONFIGID(a, CONFIG_DATA_##b, df(c), dh(c), 1, d, false)
// multi-line items are associated by sequential numbering
#define DEFIDg(a,b,c) CONFIGID(a, CONFIG_DATA_##b, df(c), dh(c), 1, false, true)

CONFIGID itemIDs[] =
{
CONFIGID( "" ,CONFIG_DATA_object ,itemObject),
DEFID ( "name"                    ,string      ,Name),
DEFID ( "id name"                 ,string      ,IDname),
DEFID ( "launch sound"            ,file        ,LaunchSound),
DEFID ( "hit sound"               ,file        ,HitSound),
DEFID ( "miss sound"              ,file        ,MissSound),
DEFID ( "missile sprite"          ,spritedata  ,SpriteData),
DEFID ( "hit sprite"              ,spritedata  ,HitSpriteData),
DEFID ( "ROF per round"           ,float       ,ROFperRound),
DEFID ( "combat range"            ,long        ,Range),
DEFID ( "cost"                    ,long        ,Cost),
DEFID ( "hands to carry"          ,long        ,HandsToUse),
DEFID ( "class"                   ,flags       ,Class),
DEFID ( "small/medium damage dice",dice        ,SmallDamageDice),
DEFID ( "large damage dice"       ,dice        ,LargeDamageDice),
DEFID ( "AC base"                 ,long        ,ACbase),
DEFID ( "AC bonus"                ,long        ,ACbonus),
DEFID ( "attack bonus"            ,long        ,AttackBonus),
DEFID ( "experience"              ,long        ,Experience),
DEFID ( "encumbrance"             ,long        ,Encumbrance),
DEFID ( "cursed"                  ,BOOL        ,Cursed),
DEFID ( "non lethal"              ,BOOL        ,NonLethal),
DEFID ( "location carried"        ,enum        ,LocationCarried),
DEFID ( "bundle quantity"         ,long        ,BundleQuantity),
DEFID ( "Special Ability"         ,string      ,SpecialAbility),
DEFID ( "weapon type"             ,enum        ,WeaponType),
DEFID ( "ammo type"               ,string      ,AmmoType),
DEFID ( "usable"                  ,BOOL        ,Usable),
DEFID ( "scribtable"              ,BOOL        ,Scribable),
DEFID ( "number of charges"       ,long        ,NumberCharges),
DEFID ( "recharge rate"           ,enum        ,RechargeRate),
DEFID ( "spell effect"            ,string      ,SpellEffect),
DEFID ( "spell level"             ,long        ,SpellLevel),
DEFID ( "spell class"             ,enum        ,SpellClass),
CONFIGID( "" ,CONFIG_DATA_end ,NULL)
};


int ReadItemDB(CString &file, 
               ITEM_DATA_TYPE *itemsPointer,
               CONFIG_FILE *pConfigFile)
{
  return ReadItemDB((LPCSTR)file, itemsPointer, pConfigFile);
}


int ReadItemDB(LPCSTR file, 
               ITEM_DATA_TYPE *itemsPointer,
               CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CONFIG_FILE myConfigFile;  // In case none is provided.
  CFile dataFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  CArchive ar( &dataFile, CArchive::load);
  ar.ReadString(string);
  if (string != "// Item database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
//    AfxGetApp()->WinHelp(IDS_HELP_Monster_Name);
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old itemData
  itemsPointer->Clear();
  // Read and store new itemData
  ASSERT (pItemData==NULL); // Nobody should be using it
  pItemData=itemsPointer;
  status=pConfigFile->DecodeFile(itemIDs, true);
  pItemData=NULL; // We are done.
  // substitute default filenames for resources using "DEFAULT"
  itemsPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}

int WriteItemDB(LPCSTR file, ITEM_DATA_TYPE *itemsPointer, CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pItemData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
  itemsPointer->CollapseDefaultFilenames();
  
  pItemData=itemsPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Item database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the items file!!!\r\n");
  status=pConfigFile->WriteFile(ar, itemIDs);


  // substitute default filenames for resources using "DEFAULT"
  itemsPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pItemData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}



//*****************************************************************************
// NAME: ReadMonsterDB
//
// PURPOSE:  
//*****************************************************************************
MONSTER_DATA tempMONSTER;  // Where we build monster
MONSTER_DATA_TYPE *pMonsterData=NULL;

// Handle clearing and storing of entire MONSTER_DATA object
static void monsterObject(CONFIG_PKT *pkt) {
  static POSITION position=NULL;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempMONSTER.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    ASSERT(pMonsterData!=NULL);
    // don't add templates to database
    if (tempMONSTER.Name.CompareNoCase("template")!=0)
    {
      if (pMonsterData->FindName(tempMONSTER.Name)!=0)
      {
        CString msg;
        msg.Format("Duplicate name in monsters.txt = %s\n",tempMONSTER.Name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
      {
        tempMONSTER.GuessUndeadStatus();
        pMonsterData->AddMonster(tempMONSTER);
      }
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pMonsterData->GetHeadPosition();
    tempMONSTER.Clear();
    tempMONSTER.Name="template";
    pkt->objectName=tempMONSTER.Name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempMONSTER=pMonsterData->GetAt(position);
      pMonsterData->GetNext(position);
      pkt->objectName=tempMONSTER.Name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}

static void DmonsterName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempMONSTER.Name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempMONSTER.Name);
  pkt->objectName=tempMONSTER.Name;
}

static void DmonsterIconFile(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempMONSTER.Icon.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempMONSTER.Icon.filename;
      return;
    };
    if (tempMONSTER.Icon.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempMONSTER.Icon.filename,
      tempMONSTER.Icon.timeDelay,
      tempMONSTER.Icon.NumFrames,
      tempMONSTER.Icon.FrameWidth,
      tempMONSTER.Icon.FrameHeight,
      tempMONSTER.Icon.style,
      tempMONSTER.Icon.MaxLoops,
      tempMONSTER.Icon.flags);
    return;
  };

	if ((pkt->value == "") || (pkt->value.CompareNoCase("DEFAULT")==0))
	{
    tempMONSTER.Icon.filename="DEFAULT";
    tempMONSTER.Icon.picType = IconDib;
    tempMONSTER.Icon.SetDefaults();
	}
	else if ((pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempMONSTER.Icon.Clear();
	}
	else
	{
    // cm_a_ogre.pcx,0,2,48,48,0
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempMONSTER.Icon.filename,
      tempMONSTER.Icon.timeDelay,
      tempMONSTER.Icon.NumFrames,
      tempMONSTER.Icon.FrameWidth,
      tempMONSTER.Icon.FrameHeight,
      tempMONSTER.Icon.style,
      tempMONSTER.Icon.MaxLoops,
      tempMONSTER.Icon.flags);

    if (pkt->status!=CONFIG_STAT_ok)
    {
      tempMONSTER.Icon.picType = IconDib;
      tempMONSTER.Icon.SetDefaults();
      pkt->status=CONFIG_STAT_ok;
    }
  };
  tempMONSTER.Icon.picType = IconDib;
}

static void DmonsterHitSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempMONSTER.HitSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
		pkt->status=CONFIG_DECODE_file(
				pkt->value,
				pkt->errorMsg,
				tempMONSTER.HitSound);
		if (tempMONSTER.HitSound.CompareNoCase("none")==0) 
								tempMONSTER.HitSound="";
		else if (tempMONSTER.HitSound=="")
								tempMONSTER.HitSound="DEFAULT";
}

static void DmonsterMissSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempMONSTER.MissSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
		pkt->status=CONFIG_DECODE_file(
				pkt->value,
				pkt->errorMsg,
				tempMONSTER.MissSound);
		if (tempMONSTER.MissSound.CompareNoCase("none")==0) 
								tempMONSTER.MissSound="";
		else if (tempMONSTER.MissSound=="")
								tempMONSTER.MissSound="DEFAULT";
}
static void DmonsterMoveSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempMONSTER.MoveSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
		pkt->status=CONFIG_DECODE_file(
				pkt->value,
				pkt->errorMsg,
				tempMONSTER.MoveSound);
		if (tempMONSTER.MoveSound.CompareNoCase("none")==0) 
								tempMONSTER.MoveSound="";
		else if (tempMONSTER.MoveSound=="")
								tempMONSTER.MoveSound="DEFAULT";
}
static void DmonsterDeathSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CString path;
    pkt->value = tempMONSTER.DeathSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
		pkt->status=CONFIG_DECODE_file(
				pkt->value,
				pkt->errorMsg,
				tempMONSTER.DeathSound);
		if (tempMONSTER.DeathSound.CompareNoCase("none")==0) 
								tempMONSTER.DeathSound="";
		else if (tempMONSTER.DeathSound=="")
								tempMONSTER.DeathSound="DEFAULT";
}

static void DmonsterIntelligence(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.Intelligence);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000,
    tempMONSTER.Intelligence,0);
}

static void DmonsterArmorClass(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.Armor_Class);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    -200000,2000000,
    tempMONSTER.Armor_Class,0);
}

static void DmonsterMovementRate(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value, tempMONSTER.Movement);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000,
    tempMONSTER.Movement,1);
}

static void DmonsterHitDice(CONFIG_PKT *pkt)
{
  double temp;
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_float(pkt->value, double(tempMONSTER.Hit_Dice));
    return;
  };
  pkt->status=CONFIG_DECODE_float(
    pkt->value,
    pkt->errorMsg,
    temp,2.71828);
  tempMONSTER.Hit_Dice=(float)temp;
}
static void DmonsterUseHitDice(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID ffIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,ffIDs,tempMONSTER.UseHitDice);
    return;
  }
  pkt->status=CONFIG_DECODE_enum (pkt->value,
                                  pkt->errorMsg,
                                  ffIDs,
                                  i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempMONSTER.UseHitDice=i;
  };
}
static void DmonsterHitDiceBonus(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.Hit_Dice_Bonus);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000,
    tempMONSTER.Hit_Dice_Bonus,0);
}

static void DmonsterTHAC0(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.THACO);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    -100,100,
    tempMONSTER.THACO,10);
}

static void DmonsterNbrAttacks(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring) 
  {
    pkt->value = "";
    return;
  }

  //pkt->errorMsg = "Nbr Attacks is no longer supported, use Attack records instead";
  //pkt->status = CONFIG_STAT_value;

  long Nbr_Attacks=0;
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,1000,
    Nbr_Attacks,1);
  // No certainty about the order in which Nbr_Attacks
  // and AttackDice will be read from the file.
  //
  // if Nbr_Attacks is first, add the attack records
  //
  // if AttackDice is first, there will be just one
  // attack record in the array, but it will need
  // to be duplicated for Nbr_Attacks.
  //
  // if Nbr_Attacks second, set the proper array size
  // and duplicate the first attack record to all
  // others.
  //

  // need at least one attack
  if (Nbr_Attacks <= 0) Nbr_Attacks=1;
  // set the array size to proper value
  if (Nbr_Attacks != tempMONSTER.attackData.GetCount())
    tempMONSTER.attackData.data.SetSize(Nbr_Attacks);
  // dup first record to all others of size > 1
  for (int i=1;i<tempMONSTER.attackData.GetCount();i++)
    tempMONSTER.attackData.data.SetAt(i,tempMONSTER.attackData.GetAtIndex(0));
}

static void DmonsterAttackDice(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring) 
  {
    pkt->value = "";
    return;
  }
  //pkt->errorMsg = "Attack Dice is no longer supported, use Attack records instead";
  //pkt->status = CONFIG_STAT_value;

  unsigned int dice=1, sides=6;
  int bonus=0;
  pkt->status=CONFIG_DECODE_dice(
    pkt->value,
    pkt->errorMsg,
    dice, sides, bonus);

  // No certainty about the order in which Nbr_Attacks
  // and AttackDice will be read from the file.
  //
  // if Nbr_Attacks is first, add the attack records
  //
  // if AttackDice is first, there will be just one
  // attack record in the array, but it will need
  // to be duplicated for Nbr_Attacks.
  //
  // if Nbr_Attacks second, set the proper array size
  // and duplicate the first attack record to all
  // others.
  //
  if (pkt->status==CONFIG_STAT_ok)
  {
    MONSTER_ATTACK_DETAILS tmp;
    tmp.sides = sides;
    tmp.nbr = dice;
    tmp.bonus = bonus;
    tmp.attackMsg = "attacks";

    if (tempMONSTER.attackData.GetCount()==0)
    {
      // read AttackDice first, just add this
      // record and it will be duplicated
      // Nbr_Attack times when that value is
      // determined later.
      tempMONSTER.attackData.Add(tmp);
    }
    else
    {
      // Nbr_Attacks was read first, so update
      // all existing records with the current
      // AttackDice data.
      for (int i=0;i<tempMONSTER.attackData.GetCount();i++)
        tempMONSTER.attackData.data.SetAt(i,tmp);
    }
  }
}

static void DmonsterMagicResistance(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.Magic_Resistance);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,1000,
    tempMONSTER.Magic_Resistance,0);
}

static void DmonsterSize(CONFIG_PKT *pkt)
{
  creatureSizeType S;
  int i;
  static CONFIGID sizeIDs[] =
  {
    CONFIGID( "small"     ,Small),
    CONFIGID( "medium"    ,Medium),
    CONFIGID( "large"     ,Large),
    CONFIGID( ""          ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,sizeIDs,tempMONSTER.Size);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    sizeIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    S=(creatureSizeType)i;
    tempMONSTER.Size=S;
  };
}

static void DmonsterMorale(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.Morale);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,1000,
    tempMONSTER.Morale,0);
}

static void DmonsterXPValue(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempMONSTER.XP_Value);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,10000000,
    tempMONSTER.XP_Value,0);
}

#ifdef OLD_SPECIAL_ABILITIES

static void DmonsterSpecialAbility(CONFIG_PKT *pkt)
{
  specialAbilitiesType S;
  int i;
  static CONFIGID abilityIDs[] =
  {
    CONFIGID("none"                 ,SA_None),
    CONFIGID("bless"                ,SA_Bless),
    CONFIGID("Curse"                ,SA_Curse),
    CONFIGID("Undead Fear"          ,SA_UndeadFear),
    CONFIGID("Enlarge"              ,SA_Enlarge),
    CONFIGID("Reduce"               ,SA_Reduce),
    CONFIGID("Charm Person"         ,SA_CharmPerson),
    CONFIGID("Detect Magic"         ,SA_DetectMagic),
    CONFIGID("Reflect Gaze Attack"  ,SA_ReflectGazeAttack),
    CONFIGID("Prot from Evil"       ,SA_ProtFromEvil),
    CONFIGID("Prot from Good"       ,SA_ProtFromGood),
    CONFIGID("Shield"               ,SA_Shield),
    CONFIGID("Sleep"                ,SA_Sleep),
    CONFIGID("Fog"                  ,SA_Fog),
    CONFIGID("Entangle"             ,SA_Entangle),
    CONFIGID("Invisible to Animals" ,SA_InvisibleToAnimals),
    CONFIGID("Invisible to Undead"  ,SA_InvisibleToUndead),
    CONFIGID("Non-Undead Fear"      ,SA_NonUndeadFear),
    CONFIGID("Sanctuary"            ,SA_Sanctuary),
    CONFIGID("Shillelagh"           ,SA_Shillelagh),
    CONFIGID("Displacement"         ,SA_Displacement),
    CONFIGID("Wizardry"             ,SA_Wizardry),
    CONFIGID("Hold Person"          ,SA_HoldPerson),
    CONFIGID("Vorpal Attack"        ,SA_VorpalAttack),    
    CONFIGID("Silenced"             ,SA_Silence),
    CONFIGID("Poisoned"             ,SA_Poisoned),
    CONFIGID("Slow Poison"          ,SA_SlowPoison),
    CONFIGID("Mirror Image"         ,SA_MirrorImage),
    CONFIGID("Invisible"            ,SA_Invisible),
    CONFIGID("Enfeebled"            ,SA_Enfeebled),
    CONFIGID("Blindness"            ,SA_Blindness),
    CONFIGID("Diseased"             ,SA_Diseased),
    CONFIGID(""          ,0)
  };
  
  if (pkt->IsStoring)
  {
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempMONSTER.specAbs.GetHeadPosition();
    }

    if ((pkt->listNum>NUM_SPECIAL_ABILITIES)||(pkt->listPos==NULL))
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    specialAbilitiesType sa = tempMONSTER.specAbs.GetAt(pkt->listPos);
    CONFIG_ENCODE_enum(
          pkt->value,
          abilityIDs,
          sa);

    tempMONSTER.specAbs.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;

    /*
    if (pkt->listNum >= tempMONSTER.specAbs.specAbs.GetSize())
    {
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    };
    CONFIG_ENCODE_enum(
              pkt->value,
              abilityIDs,
              tempMONSTER.specAbs.specAbs[pkt->listNum]);
    pkt->status=CONFIG_STAT_more;
    return;
    */
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    abilityIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    S=(specialAbilitiesType)i;
    tempMONSTER.specAbs.EnableSpecAb(S);
  };
}
#else
static void DmonsterSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempMONSTER.specAbs);
}
#endif


static void DmonsterUndeadType(CONFIG_PKT *pkt)
{
  int i;
  //enum undeadType { NotUndead, Skeleton, Zombie, Ghoul, Shadow, Wight, Ghast, Wraith, Mummy,
  //						Spectre, Vampire, Ghost, Lich, Special };

  static CONFIGID abilityIDs[] =
  {
        CONFIGID("none"     ,NotUndead),
        CONFIGID("Skeleton" ,Skeleton),
        CONFIGID("Zombie"   ,Zombie),
        CONFIGID("Ghoul"    ,Ghoul),
        CONFIGID("Shadow"   ,Shadow),
        CONFIGID("Wight"    ,Wight),
        CONFIGID("Ghast"    ,Ghast),
        CONFIGID("Wraith"   ,Wraith),
        CONFIGID("Mummy"    ,Mummy),
        CONFIGID("Spectre"  ,Spectre),
        CONFIGID("Vampire"  ,Vampire),
        CONFIGID("Ghost"    ,Ghost),
        CONFIGID("Lich"     ,Lich),
        CONFIGID("Special"  ,Special),
        CONFIGID( ""        ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(
              pkt->value,
              abilityIDs,
              tempMONSTER.undeadStatus);
     return;
  }
  
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    abilityIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
    tempMONSTER.undeadStatus = (undeadType)i;
}

static void DmonsterClasses(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID classIDs[] =
  {
    CONFIGID( "MU"      ,MagicUserFlag),
    CONFIGID( "Cleric"  ,ClericFlag),
    CONFIGID( "thief"   ,ThiefFlag),
    CONFIGID( "fighter" ,FighterFlag),
    CONFIGID( "Paladin" ,PaladinFlag),
    CONFIGID( "ranger"  ,RangerFlag),
    CONFIGID( "Druid"   ,DruidFlag),
    CONFIGID( "any"     ,AllClasses),
    CONFIGID( ""    ,0)
  };
  if (pkt->IsStoring)
  {
    if (tempMONSTER.ItemMask==AllClasses)
    {
      pkt->value="any";
    }
    else
    {
      CONFIG_ENCODE_flags(pkt->value,classIDs,tempMONSTER.ItemMask);
    };
    return;
  };
  if (pkt->value=="") {
    tempMONSTER.ItemMask=0;
  }
  else
  {
    pkt->status=CONFIG_DECODE_flags (
      pkt->value,
      pkt->errorMsg,
      classIDs,
      i);
    if (pkt->status==CONFIG_STAT_ok)
    {
      tempMONSTER.ItemMask=i;
    } else tempMONSTER.ItemMask=0;
  };
}

static void DmonsterForm(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID formIDs[] =
  {
    CONFIGID( "mammal"      ,1),
    CONFIGID( "animal"      ,2),
    CONFIGID( "snake"       ,4),
    CONFIGID( "giant"       ,8),
    CONFIGID( "large"       ,16),
    //CONFIGID( "undead"      ,32),
    CONFIGID( "none"        ,0),
    CONFIGID( ""            ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,formIDs,tempMONSTER.Form_Type);
    return;
  };
  if (pkt->value=="") tempMONSTER.Form_Type=1;
  else
  {
    pkt->status=CONFIG_DECODE_flags (
      pkt->value,
      pkt->errorMsg,
      formIDs,
      i);
    if (pkt->status==CONFIG_STAT_ok)
    {
      tempMONSTER.Form_Type=i;
    };
  };
}

static void DmonsterImmunity(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID immunityIDs[] =
  {
    CONFIGID( "poison"        ,1),
    CONFIGID( "death magic"   ,2),
    CONFIGID( "confusion"     ,4),
    CONFIGID( "vorpal weapon" ,8),
    CONFIGID( "none"          ,0),
    CONFIGID( ""              ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,immunityIDs,tempMONSTER.Immunity_Type);
    return;
  };
  if (pkt->value=="") tempMONSTER.Form_Type=1;
  else
  {
    pkt->status=CONFIG_DECODE_flags (
      pkt->value,
      pkt->errorMsg,
      immunityIDs,
      i);
    if (pkt->status==CONFIG_STAT_ok)
    {
      tempMONSTER.Immunity_Type=i;
    };
  };
}


static void DmonsterPenalty(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID penaltyIDs[] =
  {
    CONFIGID( "DwarfAC"     ,1),
    CONFIGID( "GnomeAC"     ,2),
    CONFIGID( "DwarfTHAC0"  ,4),
    CONFIGID( "GnomeTHAC0"  ,8),
    CONFIGID( "RangerDmg"   ,16),
    CONFIGID( "none"        ,0),
    CONFIGID( ""            ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,penaltyIDs,tempMONSTER.Penalty_Type);
    return;
  };
  pkt->status=CONFIG_DECODE_flags (
    pkt->value,
    pkt->errorMsg,
    penaltyIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempMONSTER.Penalty_Type=i;
  };
}

static void DmonsterMiscOptions(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID miscIDs[] =
  {
    CONFIGID( "can be held/charmed"     ,1),
    CONFIGID( "affected by dispel evil" ,2),
    CONFIGID( "none"                    ,0),
    CONFIGID( ""                        ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,miscIDs,tempMONSTER.MiscOptions_Type);
    return;
  };
  pkt->status=CONFIG_DECODE_flags (
    pkt->value,
    pkt->errorMsg,
    miscIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempMONSTER.MiscOptions_Type=i;
  };
}

static void DmonsterItem(CONFIG_PKT *pkt)
{
  CString iname;
  if (pkt->IsStoring)
  {
    if (tempMONSTER.myItems.GetCount()==0)
    {
      pkt->value="none";
      return;
    }

    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->listPos = tempMONSTER.myItems.GetHeadPosition();
      pkt->value = "";
    }

    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    GLOBAL_ITEM_ID giID = tempMONSTER.myItems.GetAt(pkt->listPos).m_giID;
    pkt->value = itemData.GetItemName(giID);

    tempMONSTER.myItems.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  };

  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    return;
  }
  else
  {
    pkt->status=CONFIG_DECODE_string(
        pkt->value,
        pkt->errorMsg,
        iname);

    if (pkt->status==CONFIG_STAT_ok)
    {
      POSITION pos=itemData.GetHeadPosition();
      bool found=false;
      while ((pos!=NULL)&&(!found)) 
      {
        if (itemData.GetAt(pos).Name.CompareNoCase(iname)==0)
        {
          ITEM_DATA idata = itemData.GetAt(pos);
          tempMONSTER.myItems.addItem(idata.m_giID,1,idata.Num_Charges,FALSE,idata.Cost);
          found=true;
        }
        itemData.GetNext(pos);
      }

      if (!found)
      {
        pkt->errorMsg.Format("No such item called %s is defined", iname);
        pkt->status=CONFIG_STAT_value;
      }
    }
  }
}

static void DmonsterAttack(CONFIG_PKT *pkt)
{
  CString iname;
  if (pkt->IsStoring)
  {
    if (tempMONSTER.attackData.GetCount()==0)
    {
      pkt->value = "none";
      pkt->status=CONFIG_STAT_ok;
      return;
    }

    if (pkt->listNum>=tempMONSTER.attackData.GetCount())
    {
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    CONFIG_ENCODE_MonsterAttack(
            pkt->value,
            &tempMONSTER.attackData.data[pkt->listNum]);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    return;
  }
  else
  {
    MONSTER_ATTACK_DETAILS tmp;
    pkt->status=CONFIG_DECODE_MonsterAttack (
                      pkt->value,
                      pkt->errorMsg,
                      &tmp);
    if (pkt->status==CONFIG_STAT_ok)
      tempMONSTER.attackData.Add(tmp);
  }
}

#undef df
#undef dh
#define df(a) Dmonster##a
#define dh(a) IDS_HELP_monster_##a
// defined above // #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))
CONFIGID monsterIDs[] =
{
CONFIGID( "" ,CONFIG_DATA_object ,monsterObject),
DEFID( "name"                    ,string ,Name),
DEFID( "icon file"               ,spritedata   ,IconFile),
DEFID( "miss sound"              ,file   ,MissSound),
DEFID( "hit sound"               ,file   ,HitSound),
DEFID( "move sound"              ,file   ,MoveSound),
DEFID( "death sound"             ,file   ,DeathSound),
DEFID( "intelligence"            ,long   ,Intelligence),
DEFID( "armor class"             ,long   ,ArmorClass),
DEFID( "movement rate"           ,long   ,MovementRate),
DEFID( "hit dice"                ,float  ,HitDice),
DEFID( "treat HD as hit points"  ,enum   ,UseHitDice),
DEFID( "hit dice bonus"          ,long   ,HitDiceBonus),
DEFID( "THAC0"                   ,long   ,THAC0),
DEFIDz( "Number of Attacks"      ,long   ,NbrAttacks, true), // read-only
DEFIDz( "Attack dice"            ,dice   ,AttackDice, true), // read-only
DEFID( "magic resistance"        ,long   ,MagicResistance),
DEFID( "size"                    ,enum   ,Size),
DEFID( "morale"                  ,long   ,Morale),
DEFID( "experience point value"  ,long   ,XPValue),
DEFID( "Special Ability"         ,string ,SpecialAbility),
DEFID( "classes"                 ,flags  ,Classes),
DEFID( "form"                    ,flags  ,Form),
DEFID( "penalty"                 ,flags  ,Penalty),
DEFID( "immunity"                ,flags  ,Immunity),
DEFID( "Misc Options"            ,flags  ,MiscOptions),
DEFID( "item"                    ,string ,Item),
DEFID( "attack"                  ,string ,Attack),
DEFID( "Undead"                  ,enum   ,UndeadType),
CONFIGID( "" ,CONFIG_DATA_end ,NULL)
};



int ReadMonsterDB(CString& file, 
               MONSTER_DATA_TYPE *monstersPointer,
               CONFIG_FILE *pConfigFile)
{
  return ReadMonsterDB((LPCSTR)file,monstersPointer,pConfigFile);
}


int ReadMonsterDB(LPCSTR file, 
               MONSTER_DATA_TYPE *monstersPointer,
               CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CONFIG_FILE myConfigFile;  // In case none is provided.
  CFile dataFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  CArchive ar( &dataFile, CArchive::load);
  ar.ReadString(string);
  if (string != "// Monster database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
//    AfxGetApp()->WinHelp(IDS_HELP_Monster_Name);
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old monsterData
  monstersPointer->Clear();
  // Read and store new monsterData
  ASSERT (pMonsterData==NULL); // Nobody should be using it
  pMonsterData=monstersPointer;
  status=pConfigFile->DecodeFile(monsterIDs, true);
  pMonsterData=NULL; // We are done.
  // substitute default filenames for resources using "DEFAULT"
  monstersPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


SPECABILITY_DEF tempSPECABILTIY; // Where we build special Ability.
A_SPECABILITY_DEF_L *pSpecAbData=NULL;  // Where data will be put

/*
int ReadSpecialAbilitiesDB(CString &file, 
               ITEM_DATA_TYPE *itemsPointer,
               CONFIG_FILE *pConfigFile)
{
  return ReadSpecialAbilitiesDB((LPCSTR)file, itemsPointer, pConfigFile);
}
*/

bool IsComment(const CString& str)
{
  if (str.GetLength() < 2) return false;
  if (str[0] != '\\') return false;
  if (str[1] != '\\') return false;
  return true;
}

bool IsContinuation(const CString& str)
{
  if (str.GetLength() < 1) return false;
  return str[0] == '-';
}


CONFIG_ITEM_STATUS GetCompleteLine(CONFIG_FILE *pConfig, int objNum, int *lineNum, CString& line)
{
  int numLine;
  CString tLine, error;
  numLine = pConfig->NumberOfItems(objNum);
  while (*lineNum < numLine)
  {
    // Find start-of-line
    tLine = pConfig->GetLineText(objNum, *lineNum);
    (*lineNum)++;
    if (IsComment(tLine)) continue;
    line = tLine;
    while (*lineNum < numLine)
    {
      // Add all continuation lines
      tLine = pConfig->GetLineText(objNum, *lineNum);
      if (!IsContinuation(tLine)) return CONFIG_STAT_ok;
      line += "\r\n" + tLine.Right(tLine.GetLength()-1);
      (*lineNum)++;
    };
  };
  return CONFIG_STAT_ok;
}

int DecodeSpecAbObject(CONFIG_FILE *pConfig, int objNum, SPECABILITY_DEF *pSpecDef, CAR& car)
{
  CString line, error, name, value;
  CONFIG_ITEM_STATUS stat;
  int lineNum, help;
  pConfig->GetLineInfo(objNum, 0, line, stat, &help, &error);
  if (line != "\\(BEGIN)") return CONFIG_STAT_syntax;
  lineNum = 1;
  while (lineNum < pConfig->NumberOfItems(objNum))
  {
    int col;
    int type;
    stat = GetCompleteLine(pConfig, objNum, &lineNum, line);
    if (line.IsEmpty()) continue;
    if (line.Left(6) == "\\(END)") return CONFIG_STAT_ok;
    col = line.Find('=');
    if (col < 0) return CONFIG_STAT_syntax;
    name = line.Left(col);
    name.TrimLeft();
    name.TrimRight();
    value = line.Right(line.GetLength() - col - 1);
    value.TrimLeft();
    value.TrimRight();
    if (name.GetLength() < 1) return CONFIG_STAT_syntax;
    if (name[0] == '[')
    {
      type = SPECAB_SCRIPT;
      if (name.GetLength() < 3) return CONFIG_STAT_syntax;
      if (name[name.GetLength()-1] != ']') return CONFIG_STAT_syntax;
      name = name.Mid(1, name.GetLength()-2);
    }
    else
    if (name[0] == '(')
    {
      type = SPECAB_VARIABLE;
      if (name.GetLength() < 3) return CONFIG_STAT_syntax;
      if (name[name.GetLength()-1] != ')') return CONFIG_STAT_syntax;
      name = name.Mid(1, name.GetLength()-2);
    }
    else
      type = SPECAB_CONSTANT;
    name = car.CanonicalString(name);
    value = car.CanonicalString(value);
    pSpecDef->Insert(name, value, type);
  };
  return CONFIG_STAT_ok;
}
  

int ReadSpecialAbilitiesDB(LPCSTR file, 
               A_SPECABILITY_DEF_L *specAbPointer,
               CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CONFIG_FILE myConfigFile;  // In case none is provided.
  CFile dataFile;
  SPECABILITY_DEF specDef;
  int status = CONFIG_STAT_ok;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  CAR car( &dataFile, CArchive::load);
  car.ReadString(string);
  if (string != "// Special Abilities database file")
  {
    car.Close();
    dataFile.Close();
    return -2;
  };
//    AfxGetApp()->WinHelp(IDS_HELP_Monster_Name);


  pConfigFile->ReadFile(car);
  // Now we begin the decode process
  // Get rid of old itemData
  specAbPointer->Clear();
  // Read and store new itemData
  ASSERT (pSpecAbData==NULL); // Nobody should be using it
  pSpecAbData=specAbPointer;
  //status=pConfigFile->DecodeFile(specialAbilitiesIDs, true);
  {
    int objNum;
    int s;
    for (objNum=1; pConfigFile->Exists(objNum); objNum++)
    {
      CString name;
      s = DecodeSpecAbObject(pConfigFile, objNum, &specDef, car);
      if (s > status) status = s;
      name = specDef.Lookup("name");
      if (name.IsEmpty())
      {
        s = CONFIG_STAT_semantic;
        if (s > status) status = s;
      }
      else
      {
        SPECABILITY_DEF *pEntry;
        pEntry = specAbPointer->InsertAbility(name);
        specDef.Delete("name");
        pEntry->Swap(specDef);
        specDef.Clear();
      };
    };
  };
  pSpecAbData=NULL; // We are done.
  car.Close();
  dataFile.Close();
  // substitute default filenames for resources using "DEFAULT"
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}





#ifdef OLD_SPECIAL_ABILITIES
#else
int WriteSpecAbDB(LPCSTR file, 
                   A_SPECABILITY_DEF_L *specAbDef, 
                   CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status = CONFIG_STAT_ok;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  //ASSERT (pSpecAbData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
  //monstersPointer->CollapseDefaultFilenames();
  
  //pSpecAbData=specAbDef;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Special Abilities database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the special abilities file!!!\r\n");
  ar.WriteString("// Sample:\n");
  ar.WriteString("// \\(BEGIN)\n");
  ar.WriteString("// name = Bless\n");
  ar.WriteString("// [Activation Script] = $PUBLIC $FUNC SA_Bless  // Script names in []\n"); 
  ar.WriteString("// -{   //  Continuations start with hyphen\n");
  ar.WriteString("// -  $RETURN $SET_CHAR_BLESS($IndexOf($Target()),true)\n");
  ar.WriteString("// -} SA_Bless;\n");
  ar.WriteString("// (parameterA) = 5    // parameter names in ()\n");
  ar.WriteString("// \\(END)\n\n");

  //status=pConfigFile->WriteFile(ar, monsterIDs);

  // substitute default filenames for resources using "DEFAULT"
  // monstersPointer->ExpandDefaultFilenames();

  POSITION pos;
  const SPECABILITY_DEF *pSpecAbDef;
  pos = specAbDef->GetStartPosition();
  while (pos != NULL)
  {
    pSpecAbDef = specAbDef->GetNext(pos);
    ar.WriteString("\n\\(BEGIN)\n");
    ar.WriteString("name = ");
    ar.WriteString(pSpecAbDef->Name());
    ar.WriteString("\n");
    {
      POSITION pos;
      pos = pSpecAbDef->GetStartPosition();
      while (pos != NULL)
      {
        {
          CString indentedValue;
          const ASLENTRY *pASL;
          pSpecAbDef->GetNextEntry(pos, &pASL);
          switch(pASL->Flags())
          {
          case SPECAB_SCRIPT: ar.WriteString("["); break;
          case SPECAB_VARIABLE: ar.WriteString("("); break;
          };
          ar.WriteString(pASL->Key());
          switch(pASL->Flags())
          {
          case SPECAB_SCRIPT: ar.WriteString("]"); break;
          case SPECAB_VARIABLE: ar.WriteString(")"); break;
          };
          ar.WriteString(" = ");
          indentedValue = pASL->Value();
          indentedValue.Replace("\n", "\n-");
          ar.WriteString(indentedValue);
          ar.WriteString("\n");
        };
      };
    };
    ar.WriteString("\\(END)\n");
  };

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pMonsterData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}
#endif


int WriteMonsterDB(LPCSTR file, 
                   MONSTER_DATA_TYPE *monstersPointer, 
                   CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pMonsterData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
  monstersPointer->CollapseDefaultFilenames();
  
  pMonsterData=monstersPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Monster database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the monsters file!!!\r\n");

  status=pConfigFile->WriteFile(ar, monsterIDs);

  // substitute default filenames for resources using "DEFAULT"
  monstersPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pMonsterData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}





//*****************************************************************************
// NAME: ReadSpellDB
//
// PURPOSE:  
//*****************************************************************************
SPELL_DATA tempSPELL;
SPELL_DATA_TYPE *pSpellData=NULL;


int ReadSpellDB(CString &file,
                SPELL_DATA_TYPE *spellsPointer,
                CONFIG_FILE *pConfigFile)
{
  return ReadSpellDB((LPCSTR)file, spellsPointer, pConfigFile);
}

// Handle clearing and storing of entire SPELL_DATA object
static void spellObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempSPELL.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempSPELL.Name.CompareNoCase("template")!=0)
    {
      if (pSpellData->FindName(tempSPELL.Name).IsValidSpell())
      {
        CString msg;
        msg.Format("Duplicate name in spells.txt = %s\n",tempSPELL.Name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pSpellData->AddSpell(tempSPELL);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pSpellData->GetHeadPosition();
    tempSPELL.Clear();
    tempSPELL.Name="template";
    pkt->objectName=tempSPELL.Name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempSPELL=pSpellData->GetAt(position);
      pSpellData->GetNext(position);
      pkt->objectName=tempSPELL.Name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}



static void DspellName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempSPELL.Name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempSPELL.Name);
  pkt->objectName=tempSPELL.Name;
}
static void DspellCastMsg(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempSPELL.CastMsg;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempSPELL.CastMsg);
  pkt->objectName=tempSPELL.CastMsg;
}
static void DspellCastingSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = tempSPELL.CastSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  };
    pkt->status=CONFIG_DECODE_file(
        pkt->value,
        pkt->errorMsg,
        tempSPELL.CastSound,
        "DEFAULT");
    if (tempSPELL.CastSound.CompareNoCase("none")==0)
      tempSPELL.CastSound="";
    else if (tempSPELL.CastSound=="")
      tempSPELL.CastSound="DEFAULT";
}

static void DspellMissileSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = tempSPELL.MissileSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  }

  pkt->status=CONFIG_DECODE_file( pkt->value,
                                  pkt->errorMsg,
                                  tempSPELL.MissileSound,
                                  "DEFAULT");

  if (tempSPELL.MissileSound.CompareNoCase("none")==0)
    tempSPELL.MissileSound="";
  else if (tempSPELL.MissileSound=="")
    tempSPELL.MissileSound="DEFAULT";
}

static void DspellCoverageSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = tempSPELL.CoverageSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  }

  pkt->status=CONFIG_DECODE_file( pkt->value,
                                  pkt->errorMsg,
                                  tempSPELL.CoverageSound,
                                  "DEFAULT");

  if (tempSPELL.CoverageSound.CompareNoCase("none")==0)
    tempSPELL.CoverageSound="";
  else if (tempSPELL.CoverageSound=="")
    tempSPELL.CoverageSound="DEFAULT";
}
static void DspellHitSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = tempSPELL.HitSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  }

  pkt->status=CONFIG_DECODE_file( pkt->value,
                                  pkt->errorMsg,
                                  tempSPELL.HitSound,
                                  "DEFAULT");

  if (tempSPELL.HitSound.CompareNoCase("none")==0)
    tempSPELL.HitSound="";
  else if (tempSPELL.HitSound=="")
    tempSPELL.HitSound="DEFAULT";
}
static void DspellLingerSound(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = tempSPELL.LingerSound;
    if (pkt->value == "") pkt->value = "none";
    return;
  }

  pkt->status=CONFIG_DECODE_file( pkt->value,
                                  pkt->errorMsg,
                                  tempSPELL.LingerSound,
                                  "DEFAULT");

  if (tempSPELL.LingerSound.CompareNoCase("none")==0)
    tempSPELL.LingerSound="";
  else if (tempSPELL.LingerSound=="")
    tempSPELL.LingerSound="DEFAULT";
}

static void DspellClass(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID classIDs[] =
  {
    CONFIGID( "MU"      ,MagicUserFlag),
    CONFIGID( "Cleric"  ,ClericFlag),
    //CONFIGID( "Paladin" ,PaladinFlag),
    //CONFIGID( "Ranger"  ,RangerFlag),
    //CONFIGID( "Druid"   ,DruidFlag),
    CONFIGID( ""        ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,classIDs,tempSPELL.ClassMask);
    return;
  };
  if (pkt->value=="")
  {
    tempSPELL.ClassMask=MagicUserFlag;
    tempSPELL.CastMask = MagicUserFlag; 
    return;
  };
  pkt->status=CONFIG_DECODE_flags (
    pkt->value,
    pkt->errorMsg,
    classIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.ClassMask=i;

    // as a default in case 'CastMask' is not specified in the file
    tempSPELL.CastMask = i; 
  };
}

static void DspellCastByClass(CONFIG_PKT *pkt)
{
  unsigned int i;
  static CONFIGID classIDs[] =
  {
    CONFIGID( "MU"      ,MagicUserFlag),
    CONFIGID( "Cleric"  ,ClericFlag),
    CONFIGID( "Paladin" ,PaladinFlag),
    CONFIGID( "Ranger"  ,RangerFlag),
    CONFIGID( "Druid"   ,DruidFlag),
    CONFIGID( "Thief"   ,ThiefFlag),
    CONFIGID( "Fighter" ,FighterFlag),
    CONFIGID( ""        ,0)
  };
  
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_flags(pkt->value,classIDs,tempSPELL.CastMask);
    return;
  }

  if (pkt->value=="")
  {
    if (tempSPELL.ClassMask != 0)
      tempSPELL.CastMask=tempSPELL.ClassMask;
    else
      tempSPELL.CastMask = MagicUserFlag;

    return;
  }

  pkt->status=CONFIG_DECODE_flags (
    pkt->value,
    pkt->errorMsg,
    classIDs,
    i);

  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.CastMask=i;

    if (tempSPELL.ClassMask != 0)
    {
      if (tempSPELL.ClassMask == MagicUserFlag)
      {
        tempSPELL.CastMask = MagicUserFlag; // force MU flag
      }
      else
      {
        tempSPELL.CastMask &= ~MagicUserFlag; // remove MU flag
        if (tempSPELL.CastMask == 0)
          tempSPELL.CastMask = tempSPELL.ClassMask;
      }
    }      
  }
}

static void DspellFriendlyFire(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID ffIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,ffIDs,tempSPELL.Friendly_Fire_Ok);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    ffIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Friendly_Fire_Ok=i;
  };
}
static void DspellCumulative(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID cuIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,cuIDs,tempSPELL.IsCumulative);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    cuIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.IsCumulative=i;
  };
}
static void DspellUsableByParty(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID ubpIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,ubpIDs,tempSPELL.UsableByParty);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    ubpIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.UsableByParty=i;
  };
}
static void DspellCanScribe(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID scribeIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,scribeIDs,tempSPELL.CanScribeAtLevelChange);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    scribeIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.CanScribeAtLevelChange=i;
  };
}

static void DspellAutoScribe(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID scribeIDs[] =
  {
    CONFIGID( "no"    ,0),
    CONFIGID( "yes"   ,1),
    CONFIGID( ""      ,0)
  };

  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,scribeIDs,tempSPELL.AutoScribeAtLevelChange);
    return;
  }

  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    scribeIDs,
    i);

  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.AutoScribeAtLevelChange=i;
  }
}

static void DspellCombatOnly(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID combatIDs[] =
  {
    CONFIGID( "yes"   ,1),
    CONFIGID( "no"    ,0),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    //CONFIG_ENCODE_enum(pkt->value,combatIDs,tempSPELL.Combat_Only);
    CONFIG_ENCODE_enum(pkt->value,combatIDs,tempSPELL.NotInCamp()?0:1);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    combatIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    //tempSPELL.Combat_Only=i;
    tempSPELL.NotInCamp(i==0);
  };
}

static void DspellCampOnly(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID campIDs[] =
  {
    CONFIGID( "yes"   ,1),
    CONFIGID( "no"    ,0),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    //CONFIG_ENCODE_enum(pkt->value,combatIDs,tempSPELL.Combat_Only);
    CONFIG_ENCODE_enum(pkt->value,campIDs,tempSPELL.NotInCombat()?0:1);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    campIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    //tempSPELL.Combat_Only=i;
    tempSPELL.NotInCombat(i==0);
  };
}

static void DspellCanBeDispelled(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID combatIDs[] =
  {
    CONFIGID( "yes"   ,1),
    CONFIGID( "no"    ,0),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,combatIDs,tempSPELL.CanBeDispelled);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    combatIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.CanBeDispelled=i;
  };
}

static void DspellLingers(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID lingerIDs[] =
  {
    CONFIGID( "yes"   ,1),
    CONFIGID( "no"    ,0),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,lingerIDs,tempSPELL.Lingers);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    lingerIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Lingers=i;
  };
}

static void DspellLingerAffectsOnce(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID lingerIDs[] =
  {
    CONFIGID( "yes"   ,1),
      CONFIGID( "no"    ,0),
      CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,lingerIDs,tempSPELL.LingerOnceOnly);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    lingerIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.LingerOnceOnly=i;
  };
}

static void DspellCastingTimeUnits(CONFIG_PKT *pkt)
{
  int i;
  //enum spellCastingTimeType { sctImmediate, sctInitiative, sctRounds, sctTurns };
  static CONFIGID casttimeunitIDs[] = {
    CONFIGID( "immediate"  ,sctImmediate),
    CONFIGID( "initiative" ,sctInitiative),
    CONFIGID( "rounds"     ,sctRounds),
    CONFIGID( "turns"      ,sctTurns),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,casttimeunitIDs,tempSPELL.Casting_Time_Type);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    casttimeunitIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Casting_Time_Type=(spellCastingTimeType)i;
  };
}

static void DspellSaveVersus(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID saveIDs[] = {
    CONFIGID( "spell"           ,Sp),
    CONFIGID( "Par/Pois/DM"     ,ParPoiDM),
    CONFIGID( "Pet/Poly"        ,PetPoly),
    CONFIGID( "Rod/Staff/Wand"  ,RodStaffWand),
    CONFIGID( "Breath Weapon"   ,BreathWeapon),
    CONFIGID( ""      ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,saveIDs,tempSPELL.Save_Vs);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    saveIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Save_Vs=(spellSaveVsType)i;
  };
}

static void DspellSaveResult(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID saveIDs[] = {
    CONFIGID( "none"        ,NoSave),
    CONFIGID( "negates"     ,SaveNegates),
    CONFIGID( "half damage" ,SaveForHalf),
    CONFIGID( "use THAC0"   ,UseTHACO),
    CONFIGID( ""            ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,saveIDs,tempSPELL.Save_Result);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    saveIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Save_Result=(spellSaveEffectType)i;
  };
}

static void DspellSpellAttribute(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.spell_asl.GetStartPosition();
    }
    
    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }    
    
    //CString sakey;
    //CString savalue;
    {
      const ASLENTRY *pEntry;
      tempSPELL.spell_asl.GetNextAssoc(pkt->listPos, &pEntry);

      pkt->value = *pEntry->pKey();
      if (!(*pEntry->pValue()).IsEmpty())
      { 
        pkt->value += ',';
        pkt->value += *pEntry->pValue();
      }

      pkt->status = CONFIG_STAT_more;
      return;   
    };
  };

  CString temp;
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp);

  tempSPELL.spell_asl.Update(temp, "1", ASLF_EDITOR);
  pkt->objectName=temp;  
}


#ifdef OLD_SPECIAL_ABILITIES
static void DspellSpecialAbility(CONFIG_PKT *pkt)
{
  specialAbilitiesType S;
  int i;
  static CONFIGID abilityIDs[] =
  {
    CONFIGID("none"                 ,SA_None),
    CONFIGID("bless"                ,SA_Bless),
    CONFIGID("Curse"                ,SA_Curse),
    CONFIGID("Undead Fear"          ,SA_UndeadFear),
    CONFIGID("Enlarge"              ,SA_Enlarge),
    CONFIGID("Reduce"               ,SA_Reduce),
    CONFIGID("Charm Person"         ,SA_CharmPerson),
    CONFIGID("Detect Magic"         ,SA_DetectMagic),
    CONFIGID("Reflect Gaze Attack"  ,SA_ReflectGazeAttack),
    CONFIGID("Prot from Evil"       ,SA_ProtFromEvil),
    CONFIGID("Prot from Good"       ,SA_ProtFromGood),
    CONFIGID("Shield"               ,SA_Shield),
    CONFIGID("Sleep"                ,SA_Sleep),
    CONFIGID("Fog"                  ,SA_Fog),
    CONFIGID("Entangle"             ,SA_Entangle),
    CONFIGID("Invisible to Animals" ,SA_InvisibleToAnimals),
    CONFIGID("Invisible to Undead"  ,SA_InvisibleToUndead),
    CONFIGID("Non-Undead Fear"      ,SA_NonUndeadFear),
    CONFIGID("Sanctuary"            ,SA_Sanctuary),
    CONFIGID("Shillelagh"           ,SA_Shillelagh),
    CONFIGID("Displacement"         ,SA_Displacement),
    CONFIGID("Wizardry"             ,SA_Wizardry),
    CONFIGID("Hold Person"          ,SA_HoldPerson),
    CONFIGID("Vorpal Attack"        ,SA_VorpalAttack),    
    CONFIGID("Silenced"             ,SA_Silence),
    CONFIGID("Poisoned"             ,SA_Poisoned),
    CONFIGID("Slow Poison"          ,SA_SlowPoison),
    CONFIGID("Mirror Image"         ,SA_MirrorImage),
    CONFIGID("Invisible"            ,SA_Invisible),
    CONFIGID("Enfeebled"            ,SA_Enfeebled),
    CONFIGID("Blindness"            ,SA_Blindness),
    CONFIGID("Diseased"             ,SA_Diseased),
    CONFIGID(""          ,0)
  };
  if (pkt->IsStoring)
  {
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.specAbs.GetHeadPosition();
    }

    if ((pkt->listNum>NUM_SPECIAL_ABILITIES)||(pkt->listPos==NULL))
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    specialAbilitiesType sa = tempSPELL.specAbs.GetAt(pkt->listPos);
    CONFIG_ENCODE_enum(
          pkt->value,
          abilityIDs,
          sa);

    tempSPELL.specAbs.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    abilityIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    S=(specialAbilitiesType)i;
    tempSPELL.specAbs.EnableSpecAb(S);
  };
}

#else
static void DspellSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempSPELL.specAbs);
}
#endif

static void DspellTargetingType(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID targetIDs[] = 
  {
    CONFIGID( "self"              ,Self),
    CONFIGID( "selectByCount"     ,SelectedByCount),
    CONFIGID( "party"             ,WholeParty),
    CONFIGID( "touch"             ,TouchedTargets),
    CONFIGID( "areacircle"        ,AreaCircle),
    CONFIGID( "arealinepickstart" ,AreaLinePickStart),
    CONFIGID( "arealinepickend"   ,AreaLinePickEnd),
    CONFIGID( "areasquare"        ,AreaSquare),
    CONFIGID( "areacone"          ,AreaCone),
    CONFIGID( "SelectByHitDice"   ,SelectByHitDice),
    CONFIGID( ""                  ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,targetIDs,tempSPELL.Targeting);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    targetIDs,
    i);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempSPELL.Targeting=(spellTargetingType)i;
  };
}

static void DspellDurationUnits(CONFIG_PKT *pkt)
{
  int i;
  static CONFIGID unitIDs[] = {
    CONFIGID( "instantaneous" ,Permanent),
    CONFIGID( "rounds"        ,inRounds),
    CONFIGID( "damage taken"  ,byDamageTaken),
    CONFIGID( "hours"         ,inHours),
    CONFIGID( "days"          ,inDays),
    CONFIGID( "nbr attacks"   ,byNbrAttacks),
    CONFIGID( ""              ,0)
  };
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_enum(pkt->value,unitIDs,tempSPELL.Duration_Rate);
    return;
  };
  pkt->status=CONFIG_DECODE_enum (
    pkt->value,
    pkt->errorMsg,
    unitIDs,
    i);

  if (pkt->status==CONFIG_STAT_ok)
    tempSPELL.Duration_Rate=(spellDurationType)i;
  else
    tempSPELL.Duration_Rate = Permanent;
}

static void DspellLevel(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempSPELL.Level);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,9,
    tempSPELL.Level,1);
}

static void DspellCastingTime(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempSPELL.Casting_Time);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,1000,
    tempSPELL.Casting_Time,1);
}

static void DspellNumberOfTargets(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempSPELL.NumTargets);
    return;
  };
  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    tempSPELL.NumTargets.Clear();
  }
  else
  {
    pkt->status=CONFIG_DECODE_dicePlus(
      pkt->value,
      pkt->errorMsg,
      &tempSPELL.NumTargets,
          0,0,0,0,0);
  }
}

static void DspellDuration(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempSPELL.Duration);
    return;
  };
  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    tempSPELL.Duration.Clear();
  }
  else
  {
    pkt->status=CONFIG_DECODE_dicePlus(
      pkt->value,
      pkt->errorMsg,
      &tempSPELL.Duration,
          0,0,0,0,0);
  }
}

static void DspellTargetDuration(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempSPELL.EffectDuration);
    return;
  };
  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    tempSPELL.EffectDuration.Clear();
  }
  else
  {
    pkt->status=CONFIG_DECODE_dicePlus(
      pkt->value,
      pkt->errorMsg,
      &tempSPELL.EffectDuration,
      0,0,0,0,0);
  }
}

static void DspellTargetRange(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempSPELL.TargetRange);
    return;
  };
  if ((pkt->value=="")||(pkt->value.CompareNoCase("none")==0))
  {
    tempSPELL.TargetRange.Clear();
  }
  else
  {
    pkt->status=CONFIG_DECODE_dicePlus(
      pkt->value,
      pkt->errorMsg,
      &tempSPELL.TargetRange,
          0,0,0,0,0);
  }
}

static void DspellAttackBonus(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    // not supported now
    pkt->value = "";
    return;
  };
  DICEPLUS dp;
  pkt->status=CONFIG_DECODE_dicePlus(
                  pkt->value,
                  pkt->errorMsg,
                  &dp,
                  0,0,0,0,0);
  if (pkt->status==CONFIG_STAT_ok)
  {
    //if (dp.m_numDice>0)
    if (!dp.m_Text.IsEmpty())
    {
      SPELL_EFFECTS_DATA seData;
      seData.affectedAttr = "$CHAR_THACO";      
      seData.changeData=dp;
      seData.flags = SPELL_EFFECTS_DATA::EFFECT_TARGET;
      tempSPELL.AddEffect(seData);
    }
  };
}

static void DspellDamage(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    // not supported now
    pkt->value = "";
    return;
  };
  DICEPLUS dp;
  pkt->status=CONFIG_DECODE_dicePlus(
                  pkt->value,
                  pkt->errorMsg,
                  &dp,
                  0,0,0,0,0);
  if (pkt->status==CONFIG_STAT_ok)
  {
    if (!dp.m_Text.IsEmpty())
    {
      CString diceformat;
      SPELL_EFFECTS_DATA seData;
      seData.affectedAttr = "$CHAR_DAMAGEBONUS";
      seData.changeData=dp;
      seData.flags = SPELL_EFFECTS_DATA::EFFECT_TARGET;
      tempSPELL.AddEffect(seData);
    }
  };
}

static void DspellProtectionBonus(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    // not supported now
    pkt->value = "";
    return;
  };
  DICEPLUS dp;
  pkt->status=CONFIG_DECODE_dicePlus(
                  pkt->value,
                  pkt->errorMsg,
                  &dp,
                  0,0,0,0,0);
  if (pkt->status==CONFIG_STAT_ok)
  {
    //if (dp.m_numDice>0)
    if (!dp.m_Text.IsEmpty())
    {
      SPELL_EFFECTS_DATA seData;
      seData.affectedAttr = "$CHAR_AC";
      seData.changeData=dp;
      seData.flags = SPELL_EFFECTS_DATA::EFFECT_TARGET;
      tempSPELL.AddEffect(seData);
    }
  };
}

static void DspellHealing(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    // not supported now
    pkt->value = "";
    return;
  };
  DICEPLUS dp;
  pkt->status=CONFIG_DECODE_dicePlus(
                  pkt->value,
                  pkt->errorMsg,
                  &dp,
                  0,0,0,0,0);
  if (pkt->status==CONFIG_STAT_ok)
  {
    //if (dp.m_numDice>0)
    if (!dp.m_Text.IsEmpty())
    {
      SPELL_EFFECTS_DATA seData;
      seData.affectedAttr = "$CHAR_HITPOINTS";
      seData.changeData=dp;
      seData.flags = SPELL_EFFECTS_DATA::EFFECT_TARGET;
      tempSPELL.AddEffect(seData);
    }
  };
}

static void DspellPrice(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_long(pkt->value,tempSPELL.Cast_Cost);
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,2000000000,
    tempSPELL.Cast_Cost,0);
}

static void DspellCastArt(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.CastArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempSPELL.CastArt.filename;
      return;
    };
    if (tempSPELL.CastArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempSPELL.CastArt.filename,
      tempSPELL.CastArt.timeDelay,
      tempSPELL.CastArt.NumFrames,
      tempSPELL.CastArt.FrameWidth,
      tempSPELL.CastArt.FrameHeight,
      tempSPELL.CastArt.style,
      tempSPELL.CastArt.MaxLoops,
      tempSPELL.CastArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempSPELL.CastArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempSPELL.CastArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempSPELL.CastArt.filename,
      tempSPELL.CastArt.timeDelay,
      tempSPELL.CastArt.NumFrames,
      tempSPELL.CastArt.FrameWidth,
      tempSPELL.CastArt.FrameHeight,
      tempSPELL.CastArt.style,
      tempSPELL.CastArt.MaxLoops,
      tempSPELL.CastArt.flags);
  };
  tempSPELL.CastArt.picType = SpriteDib;
}

static void DspellMissileArt(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.MissileArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempSPELL.MissileArt.filename;
      return;
    };
    if (tempSPELL.MissileArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempSPELL.MissileArt.filename,
      tempSPELL.MissileArt.timeDelay,
      tempSPELL.MissileArt.NumFrames,
      tempSPELL.MissileArt.FrameWidth,
      tempSPELL.MissileArt.FrameHeight,
      tempSPELL.MissileArt.style,
      tempSPELL.MissileArt.MaxLoops,
      tempSPELL.MissileArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempSPELL.MissileArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempSPELL.MissileArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempSPELL.MissileArt.filename,
      tempSPELL.MissileArt.timeDelay,
      tempSPELL.MissileArt.NumFrames,
      tempSPELL.MissileArt.FrameWidth,
      tempSPELL.MissileArt.FrameHeight,
      tempSPELL.MissileArt.style,
      tempSPELL.MissileArt.MaxLoops,
      tempSPELL.MissileArt.flags);
  };
  tempSPELL.MissileArt.picType = SpriteDib;
}
static void DspellTargCoverArt(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.TargCoverageArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempSPELL.TargCoverageArt.filename;
      return;
    };
    if (tempSPELL.TargCoverageArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempSPELL.TargCoverageArt.filename,
      tempSPELL.TargCoverageArt.timeDelay,
      tempSPELL.TargCoverageArt.NumFrames,
      tempSPELL.TargCoverageArt.FrameWidth,
      tempSPELL.TargCoverageArt.FrameHeight,
      tempSPELL.TargCoverageArt.style,
      tempSPELL.TargCoverageArt.MaxLoops,
      tempSPELL.TargCoverageArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempSPELL.TargCoverageArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempSPELL.TargCoverageArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempSPELL.TargCoverageArt.filename,
      tempSPELL.TargCoverageArt.timeDelay,
      tempSPELL.TargCoverageArt.NumFrames,
      tempSPELL.TargCoverageArt.FrameWidth,
      tempSPELL.TargCoverageArt.FrameHeight,
      tempSPELL.TargCoverageArt.style,
      tempSPELL.TargCoverageArt.MaxLoops,
      tempSPELL.TargCoverageArt.flags);
  };
  tempSPELL.TargCoverageArt.picType = SpriteDib;
}
static void DspellTargHitArt(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.TargHitArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempSPELL.TargHitArt.filename;
      return;
    };
    if (tempSPELL.TargHitArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempSPELL.TargHitArt.filename,
      tempSPELL.TargHitArt.timeDelay,
      tempSPELL.TargHitArt.NumFrames,
      tempSPELL.TargHitArt.FrameWidth,
      tempSPELL.TargHitArt.FrameHeight,
      tempSPELL.TargHitArt.style,
      tempSPELL.TargHitArt.MaxLoops,
      tempSPELL.TargHitArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempSPELL.TargHitArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempSPELL.TargHitArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempSPELL.TargHitArt.filename,
      tempSPELL.TargHitArt.timeDelay,
      tempSPELL.TargHitArt.NumFrames,
      tempSPELL.TargHitArt.FrameWidth,
      tempSPELL.TargHitArt.FrameHeight,
      tempSPELL.TargHitArt.style,
      tempSPELL.TargHitArt.MaxLoops,
      tempSPELL.TargHitArt.flags);
  };
  tempSPELL.TargHitArt.picType = SpriteDib;
}
static void DspellTargLingerArt(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.TargLingerArt.filename.CompareNoCase("DEFAULT")==0)
    {
      pkt->value=tempSPELL.TargLingerArt.filename;
      return;
    };
    if (tempSPELL.TargLingerArt.filename=="")
    {
      pkt->value="none";
      return;
    };
    CONFIG_ENCODE_SpriteData(
      pkt->value,
			tempSPELL.TargLingerArt.filename,
      tempSPELL.TargLingerArt.timeDelay,
      tempSPELL.TargLingerArt.NumFrames,
      tempSPELL.TargLingerArt.FrameWidth,
      tempSPELL.TargLingerArt.FrameHeight,
      tempSPELL.TargLingerArt.style,
      tempSPELL.TargLingerArt.MaxLoops,
      tempSPELL.TargLingerArt.flags);
    return;
  };
	if (pkt->value.CompareNoCase("DEFAULT")==0)
	{
    tempSPELL.TargLingerArt.filename = "DEFAULT";
	}
	else if ((pkt->value == "") ||
           (pkt->value.CompareNoCase("NA") == 0) ||
           (pkt->value.CompareNoCase("none") == 0))
	{
		tempSPELL.TargLingerArt.Clear();
	}
	else
	{
		pkt->status=CONFIG_DECODE_SpriteData(
			pkt->value,
			pkt->errorMsg,
			tempSPELL.TargLingerArt.filename,
      tempSPELL.TargLingerArt.timeDelay,
      tempSPELL.TargLingerArt.NumFrames,
      tempSPELL.TargLingerArt.FrameWidth,
      tempSPELL.TargLingerArt.FrameHeight,
      tempSPELL.TargLingerArt.style,
      tempSPELL.TargLingerArt.MaxLoops,
      tempSPELL.TargLingerArt.flags);
  };
  tempSPELL.TargLingerArt.picType = SpriteDib;
}

static void DspellSpellActivateScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="";
      return;
    }

    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }

    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    pkt->value = tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).activationScript;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');

    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  // loading
	if ((pkt->value == "") ||
      (pkt->value.CompareNoCase("NA") == 0) ||
      (pkt->value.CompareNoCase("none") == 0) ||
      (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		return;
	}
	else
	{
    CString temp;
    pkt->status = 
      CONFIG_DECODE_string(pkt->value,
                           pkt->errorMsg,
                           temp);
    pkt->objectName=temp;

    // an attempt to reformat the script for easier viewing
    temp.Replace("{", "\r\n{\r\n");
    temp.Replace("};", "\r\n};\r\n");

    int count = 1;
    POSITION pos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    while (pos != NULL)
    {
      if (count == pkt->keyNum)
      {
        tempSPELL.m_EffectedAttributes.GetAt(pos).activationScript = temp;
        return;
      }
      count++;
      tempSPELL.m_EffectedAttributes.GetNext(pos);
    }
    // a spell script was given before the associated spell effect
    ASSERT(FALSE);
  }  
}

static void DspellSpellModifyScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="";
      return;
    }

    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }

    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    pkt->value = tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).modificationScript;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');

    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  // loading
	if ((pkt->value == "") ||
      (pkt->value.CompareNoCase("NA") == 0) ||
      (pkt->value.CompareNoCase("none") == 0) ||
      (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		return;
	}
	else
	{
    CString temp;
    pkt->status = 
      CONFIG_DECODE_string(pkt->value,
                           pkt->errorMsg,
                           temp);
    pkt->objectName=temp;

    // an attempt to reformat the script for easier viewing
    temp.Replace("{", "\r\n{\r\n");
    temp.Replace("};", "\r\n};\r\n");


    int count = 1;
    POSITION pos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    while (pos != NULL)
    {
      if (count == pkt->keyNum)
      {
        tempSPELL.m_EffectedAttributes.GetAt(pos).modificationScript = temp;
        return;
      }
      count++;
      tempSPELL.m_EffectedAttributes.GetNext(pos);
    }
    // a spell script was given before the associated spell effect
    ASSERT(FALSE);
  }  
}

static void DspellSpellSavingThrowScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="";
      return;
    }

    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }

    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    pkt->value = tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).savingThrowScript;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');

    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  // loading
	if ((pkt->value == "") ||
      (pkt->value.CompareNoCase("NA") == 0) ||
      (pkt->value.CompareNoCase("none") == 0) ||
      (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		return;
	}
	else
	{
    CString temp;
    pkt->status = 
      CONFIG_DECODE_string(pkt->value,
                           pkt->errorMsg,
                           temp);
    pkt->objectName=temp;

    // an attempt to reformat the script for easier viewing
    temp.Replace("{", "\r\n{\r\n");
    temp.Replace("};", "\r\n};\r\n");

    int count = 1;
    POSITION pos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    while (pos != NULL)
    {
      if (count == pkt->keyNum)
      {
        tempSPELL.m_EffectedAttributes.GetAt(pos).savingThrowScript = temp;
        return;
      }
      count++;
      tempSPELL.m_EffectedAttributes.GetNext(pos);
    }
    // a spell script was given before the associated spell effect
    ASSERT(FALSE);
  }  
}

static void DspellSpellSavingThrowFailedScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="";
      return;
    }
    
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }
    
    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }
    
    pkt->value = tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).savingThrowFailedScript;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');
    
    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }
  
  // loading
  if ((pkt->value == "") ||
    (pkt->value.CompareNoCase("NA") == 0) ||
    (pkt->value.CompareNoCase("none") == 0) ||
    (pkt->value.CompareNoCase("DEFAULT")==0))
  {
    return;
  }
  else
  {
    CString temp;
    pkt->status = 
      CONFIG_DECODE_string(pkt->value,
      pkt->errorMsg,
      temp);
    pkt->objectName=temp;
    
    // an attempt to reformat the script for easier viewing
    temp.Replace("{", "\r\n{\r\n");
    temp.Replace("};", "\r\n};\r\n");
    
    int count = 1;
    POSITION pos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    while (pos != NULL)
    {
      if (count == pkt->keyNum)
      {
        tempSPELL.m_EffectedAttributes.GetAt(pos).savingThrowFailedScript = temp;
        return;
      }
      count++;
      tempSPELL.m_EffectedAttributes.GetNext(pos);
    }
    // a spell script was given before the associated spell effect
    ASSERT(FALSE);
  }  
}

static void DspellSpellSavingThrowSucceededScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="";
      return;
    }
    
    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }
    
    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }
    
    pkt->value = tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).savingThrowSucceededScript;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');
    
    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }
  
  // loading
  if ((pkt->value == "") ||
    (pkt->value.CompareNoCase("NA") == 0) ||
    (pkt->value.CompareNoCase("none") == 0) ||
    (pkt->value.CompareNoCase("DEFAULT")==0))
  {
    return;
  }
  else
  {
    CString temp;
    pkt->status = 
      CONFIG_DECODE_string(pkt->value,
      pkt->errorMsg,
      temp);
    pkt->objectName=temp;
    
    // an attempt to reformat the script for easier viewing
    temp.Replace("{", "\r\n{\r\n");
    temp.Replace("};", "\r\n};\r\n");
    
    int count = 1;
    POSITION pos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    while (pos != NULL)
    {
      if (count == pkt->keyNum)
      {
        tempSPELL.m_EffectedAttributes.GetAt(pos).savingThrowSucceededScript = temp;
        return;
      }
      count++;
      tempSPELL.m_EffectedAttributes.GetNext(pos);
    }
    // a spell script was given before the associated spell effect
    ASSERT(FALSE);
  }  
}

static void DspellSpellEffect(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    if (tempSPELL.m_EffectedAttributes.GetCount()==0)
    {
      pkt->value="none";
      return;
    }

    if (pkt->listNum==0)
    {
      // only happens on first call,
      pkt->value = "";
      pkt->listPos = tempSPELL.m_EffectedAttributes.GetHeadPosition();
    }

    if (pkt->listPos==NULL)
    { 
      pkt->value = "";
      pkt->status=CONFIG_STAT_end;
      return;
    }

    if (tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos).affectedAttr!="")
      CONFIG_ENCODE_SpellEffect(pkt->value, &tempSPELL.m_EffectedAttributes.GetAt(pkt->listPos));

    tempSPELL.m_EffectedAttributes.GetNext(pkt->listPos);
    pkt->status=CONFIG_STAT_more;
    return;
  }

  // loading
	if ((pkt->value == "") ||
      (pkt->value.CompareNoCase("NA") == 0) ||
      (pkt->value.CompareNoCase("none") == 0) ||
      (pkt->value.CompareNoCase("DEFAULT")==0))
	{
		return;
	}
	else
	{
    SPELL_EFFECTS_DATA data;
		pkt->status=CONFIG_DECODE_SpellEffect(
			            pkt->value,
			            pkt->errorMsg,
			            &data);
    if (pkt->status==CONFIG_STAT_ok)
      tempSPELL.AddEffect(data);
  }  
}

static void DspellSpellBeginScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempSPELL.SpellBeginSource;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');

    return;
  };

  CString temp;
  pkt->status = 
    CONFIG_DECODE_string(pkt->value,
                         pkt->errorMsg,
                         temp);
  pkt->objectName=temp;

  // an attempt to reformat the script for easier viewing
  temp.Replace("{", "\r\n{\r\n");
  temp.Replace("};", "\r\n};\r\n");

  tempSPELL.SpellBeginSource = temp;
}

static void DspellSpellEndScript(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempSPELL.SpellEndSource;
    pkt->value.Remove('\n');
    pkt->value.Remove('\r');
    pkt->value.Remove('\t');
    pkt->value.Remove('\v');

    return;
  };

  CString temp;
  pkt->status = 
    CONFIG_DECODE_string(pkt->value,
                         pkt->errorMsg,
                         temp);
  pkt->objectName=temp;

  // an attempt to reformat the script for easier viewing
  temp.Replace("{", "\r\n{\r\n");
  temp.Replace("};", "\r\n};\r\n");

  tempSPELL.SpellEndSource = temp;
}

#undef df
#undef dh
#define df(a) Dspell##a
#define dh(a) IDS_HELP_spell_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID spellIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,spellObject),
DEFID ("name"               ,string     ,Name),
DEFID ("casting sound"      ,file       ,CastingSound),
DEFID ("cast msg"           ,string     ,CastMsg),
DEFID ("missile sound"      ,file       ,MissileSound),
DEFID ("coverage sound"     ,file       ,CoverageSound),
DEFID ("hit sound"          ,file       ,HitSound),
DEFID ("linger sound"       ,file       ,LingerSound),
DEFID ("cast art"           ,spritedata ,CastArt),
DEFID ("missile art"        ,spritedata ,MissileArt),
DEFID ("target cover art"   ,spritedata ,TargCoverArt),
DEFID ("target hit art"     ,spritedata ,TargHitArt),
DEFID ("target linger art"  ,spritedata ,TargLingerArt),
DEFID ("class"              ,flags      ,Class),
DEFID ("cast by"            ,flags      ,CastByClass),
DEFID ("friendly fire ok"   ,enum       ,FriendlyFire),
DEFID ("cumulative"         ,enum       ,Cumulative),
DEFID ("usable by party"    ,enum       ,UsableByParty),
DEFID ("can scribe"         ,enum       ,CanScribe),
DEFID ("auto scribe"        ,enum       ,AutoScribe),
DEFID ("combat only"        ,enum       ,CombatOnly),
DEFID ("can be dispelled"   ,enum       ,CanBeDispelled),
DEFID ("lingers"            ,enum       ,Lingers),
DEFID ("linger affects once",enum       ,LingerAffectsOnce),
DEFID ("save versus"        ,enum       ,SaveVersus),
DEFID ("save result"        ,enum       ,SaveResult),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
DEFID ("targeting type"     ,enum       ,TargetingType),
DEFID ("level"              ,long       ,Level),
DEFID ("casting time"       ,long       ,CastingTime),
DEFID ("casting time units" ,enum       ,CastingTimeUnits),
DEFID ("Number of targets"  ,dicePlus   ,NumberOfTargets),
DEFID ("duration"           ,dicePlus   ,Duration),
DEFID ("duration units"     ,enum       ,DurationUnits),
DEFID ("target duration"    ,dicePlus   ,TargetDuration),
DEFID ("target range"       ,dicePlus   ,TargetRange),
DEFIDz ("attack bonus"      ,exDice     ,AttackBonus, true), // read-only
DEFIDz ("damage"            ,exDice     ,Damage, true), // read-only
DEFIDz ("protection bonus"  ,exDice     ,ProtectionBonus, true), // read-only
DEFIDz ("healing"           ,exDice     ,Healing, true), // read-only
DEFID ("price"              ,long       ,Price),
DEFIDg ("spell effect"      ,spellEffect,SpellEffect),
DEFIDg ("activate script"   ,string     ,SpellActivateScript),
DEFIDg ("modify script"     ,string     ,SpellModifyScript),
DEFIDg ("saving throw script",string    ,SpellSavingThrowScript),
DEFIDg ("saving throw failed script",string    ,SpellSavingThrowFailedScript),
DEFIDg ("saving throw succeeded script",string    ,SpellSavingThrowSucceededScript),
DEFID ("spell attribute"    ,string     ,SpellAttribute),
DEFID ("begin script"       ,string     ,SpellBeginScript),
DEFID ("end script"         ,string     ,SpellEndScript),
CONFIGID( "",CONFIG_DATA_end,NULL)
};

int ReadSpellDB(LPCSTR file,
                SPELL_DATA_TYPE *spellsPointer,
                CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Spell database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old spellData
  spellsPointer->Clear();
  // Read and store new spellData
  ASSERT(pSpellData==NULL); // Nobody should be using it.
  pSpellData=spellsPointer;
  status=pConfigFile->DecodeFile(spellIDs, true);
  pSpellData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
  spellsPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteSpellDB(LPCSTR file, 
                 SPELL_DATA_TYPE *spellsPointer, 
                 CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pSpellData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
  spellsPointer->CollapseDefaultFilenames();
  
  pSpellData=spellsPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Spell database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the spells file!!!\r\n");
  status=pConfigFile->WriteFile(ar, spellIDs);

  // substitute default filenames for resources using "DEFAULT"
  spellsPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pSpellData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}




//*****************************************************************************
// NAME: ReadRaceDB
//
// PURPOSE:  
//*****************************************************************************

RACE_DATA tempRACE; // Where we build race.
RACE_DATA_TYPE *pRaceData=NULL;  // Where data will be put

// Handle clearing and storing of entire RACE_DATA object
static void raceObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempRACE.Clear();
    tempRACE.m_race_asl.Insert(RACE_ALLOWED_CLASS_ATTRIBUTE,
                                     "",   //An empty list..else anything goes.
                                     ASLF_READONLY|ASLF_DESIGN);
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempRACE.m_name.CompareNoCase("template")!=0)
    {
      if (pRaceData->FindName(tempRACE.m_name)!=raceUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in races.txt = %s\n",tempRACE.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pRaceData->AddRace(tempRACE);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pRaceData->GetHeadPosition();
    tempRACE.Clear();
    tempRACE.m_name="template";
    pkt->objectName=tempRACE.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempRACE=pRaceData->GetAt(position);
      pRaceData->GetNext(position);
      pkt->objectName=tempRACE.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}

static void DraceName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempRACE.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempRACE.m_name);
  pkt->objectName=tempRACE.m_name;
}

static void DraceWeight (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempRACE.m_weight);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
    pkt->value,
    pkt->errorMsg,
    &tempRACE.m_weight,
        0,0,0);
}

static void DraceHeight (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempRACE.m_height);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
    pkt->value,
    pkt->errorMsg,
    &tempRACE.m_height,
    0,0,0);
}

static void DraceAge (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempRACE.m_age);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
    pkt->value,
    pkt->errorMsg,
    &tempRACE.m_age,
    0,0,0);
}

static void DraceMaxAge (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value,tempRACE.m_maxAge);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
    pkt->value,
    pkt->errorMsg,
    &tempRACE.m_maxAge,
    0,0,0);
}

static void DraceBaseMovement (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(pkt->value, tempRACE.m_baseMovement);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
              pkt->value,
              pkt->errorMsg,
              &tempRACE.m_baseMovement,
              0,0,0);
}

static void DraceAbilityReq (CONFIG_PKT *pkt)
{
  ABILITY_LIMITS temp;
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DraceAbilityReq(IsStoring) not implemented");
    return;
  };
  pkt->status=CONFIG_DECODE_AbilityRequirement(
    pkt->value,
    pkt->errorMsg,
    &temp);
  if (pkt->status==CONFIG_STAT_ok)
        tempRACE.AddAbilityRequirement(temp);
}

static void DraceMultiClass (CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DraceMultiClass(IsStoring) not implemented");
    return;
  };
  tempRACE.AddAllowedClass(pkt->value);
  pkt->status=CONFIG_STAT_ok;
}

static void DraceSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempRACE.m_specAbs);
}


#undef df
#undef dh
#define df(a) Drace##a
#define dh(a) IDS_HELP_race_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID raceIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,raceObject),
DEFID ("name"               ,string     ,Name),
DEFID ("weight"             ,dicePlus   ,Weight),
DEFID ("height"             ,dicePlus   ,Height),
DEFID ("age"                ,dicePlus   ,Age),
DEFID ("maxage"             ,dicePlus   ,MaxAge),
DEFID ("BaseMovement"       ,dicePlus   ,BaseMovement),
DEFID ("AbilityRequirement" ,AbilityReq ,AbilityReq),
DEFID ("AllowedMultiClass"  ,string     ,MultiClass),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
CONFIGID( "",CONFIG_DATA_end,NULL)
};


int ReadRaceDB(CString &file, 
               RACE_DATA_TYPE *racePointer,
               CONFIG_FILE *pConfigFile)
{
  return ReadRaceDB((LPCSTR)file, racePointer, pConfigFile);
}


int ReadRaceDB(LPCSTR file,
                RACE_DATA_TYPE *racePointer,
                CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Race database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old raceData
  racePointer->Clear();
  // Read and store new raceData
  ASSERT(pRaceData==NULL); // Nobody should be using it.
  pRaceData=racePointer;
  status=pConfigFile->DecodeFile(raceIDs, true);
  pRaceData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  racePointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteRaceDB(LPCSTR file, 
                RACE_DATA_TYPE *racePointer, 
                CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pRaceData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  racePointer->CollapseDefaultFilenames();
  
  pRaceData=racePointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Race database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the races file!!!\r\n");
  status=pConfigFile->WriteFile(ar, raceIDs);

  // substitute default filenames for resources using "DEFAULT"
//  racePointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pRaceData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}



// ====================================================================
// 
//  Manipulate ability database
//
// ====================================================================



ABILITY_DATA tempABILITY; // Where we build ability
ABILITY_DATA_TYPE *pAbilityData=NULL;  // Where data will be put

// Handle clearing and storing of entire ABILITY_DATA object
static void abilityObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempABILITY.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempABILITY.m_name.CompareNoCase("template")!=0)
    {
      if (pAbilityData->FindName(tempABILITY.m_name)!=abilityUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in ability.txt = %s\n",tempABILITY.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
      {
        tempABILITY.m_roll.Compile();
        pAbilityData->AddAbility(tempABILITY);
      }
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pAbilityData->GetHeadPosition();
    tempABILITY.Clear();
    tempABILITY.m_name="template";
    pkt->objectName=tempABILITY.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempABILITY=pAbilityData->GetAt(position);
      pAbilityData->GetNext(position);
      pkt->objectName=tempABILITY.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}


static void DabilityName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempABILITY.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempABILITY.m_name);
  pkt->objectName=tempABILITY.m_name;
}

static void DabilityAbbrev(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempABILITY.m_abbreviation;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempABILITY.m_abbreviation);
}

static void DabilityRoll(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_dicePlus(
      pkt->value,
      tempABILITY.m_roll);
    return;
  };
  pkt->status=CONFIG_DECODE_dicePlus(
    pkt->value,
    pkt->errorMsg,
    &tempABILITY.m_roll);
}

static void DabilityModify(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    CONFIG_ENCODE_integer(
      pkt->value,
      tempABILITY.m_modify);
    return;
  };
  int temp;
  pkt->status=CONFIG_DECODE_integer(
    pkt->value,
    pkt->errorMsg,
    0,255,
    temp);
  tempABILITY.m_modify=(BYTE)temp;
}

static void DabilityAdjust(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
//    CONFIG_ENCODE_adjust(
//      pkt->value,
//      tempABILITY.m_adjust);
    return;
  };

  pkt->value.TrimLeft();
  pkt->value.TrimRight();
  pkt->value.Remove(' ');

  tempABILITY.AddAdjust(pkt->value);

  /*
  ADJUSTMENTListType list;
  pkt->status=CONFIG_DECODE_adjust(
    pkt->value,
    pkt->errorMsg,
    list);
  if (pkt->status==CONFIG_STAT_ok)
  {
    for (int i=0;i<list.GetSize();i++)
      tempABILITY.AddAdjust(list[i]);
  };
  */
}


static void DabilitySpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempABILITY.m_specAbs);
}


#undef df
#undef dh
#define df(a) Dability##a
#define dh(a) IDS_HELP_ability_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID abilityIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,abilityObject),
DEFID ("name"               ,string     ,Name),
DEFID ("Abbreviation"       ,string     ,Abbrev),
DEFID ("Roll"               ,dicePlus   ,Roll),
DEFID ("Modify"             ,long       ,Modify),
DEFID ("Adjust"             ,adjust     ,Adjust),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
CONFIGID( "",CONFIG_DATA_end,NULL)
};

int ReadAbilityDB(CString &file, 
                  ABILITY_DATA_TYPE *abilityPointer,
                  CONFIG_FILE *pConfigFile)
{
  return ReadAbilityDB((LPCSTR)file, abilityPointer, pConfigFile);
}



int ReadAbilityDB(LPCSTR file,
                 ABILITY_DATA_TYPE *abilityPointer,
                 CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Ability database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old abilityData
  abilityPointer->Clear();
  // Read and store new abilityData
  ASSERT(pAbilityData==NULL); // Nobody should be using it.
  pAbilityData=abilityPointer;
  status=pConfigFile->DecodeFile(abilityIDs, true);
  pAbilityData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  abilityPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteAbilityDB(LPCSTR file, 
                ABILITY_DATA_TYPE *abilityPointer, 
                CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pAbilityData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  abilityPointer->CollapseDefaultFilenames();
  
  pAbilityData=abilityPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Ability database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the races file!!!\r\n");
  status=pConfigFile->WriteFile(ar, abilityIDs);

  // substitute default filenames for resources using "DEFAULT"
//  abilityPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pAbilityData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}

// ====================================================================
// 
//  Manipulate BaseClass database
//
// ====================================================================



BASE_CLASS_DATA tempBASECLASS; // Where we build baseClass
BASE_CLASS_DATA_TYPE *pBaseclassData=NULL;  // Where data will be put

// Handle clearing and storing of entire BASE_CLASS_DATA object
static void baseclassObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempBASECLASS.Clear();
    tempBASECLASS.m_allowedAlignments |= 0x8000; // Not set yet.
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    tempBASECLASS.m_allowedAlignments&=0x7fff;
    if (tempBASECLASS.m_name.CompareNoCase("template")!=0)
    {
      if (pBaseclassData->FindName(tempBASECLASS.m_name)!=baseclassUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in baseclass.txt = %s\n",tempBASECLASS.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pBaseclassData->AddBaseclass(tempBASECLASS);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pBaseclassData->GetHeadPosition();
    tempBASECLASS.Clear();
    tempBASECLASS.m_name="template";
    pkt->objectName=tempBASECLASS.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempBASECLASS=pBaseclassData->GetAt(position);
      pBaseclassData->GetNext(position);
      pkt->objectName=tempBASECLASS.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}


static void DbaseclassName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempBASECLASS.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempBASECLASS.m_name);
  pkt->objectName=tempBASECLASS.m_name;
}

static void DbaseclassPrimeReq(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempBASECLASS.m_primeReq[0].m_abilityName;
    return;
  };
  CString temp;
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempBASECLASS.AddPrimeReq(temp);
  };
}

static void DbaseclassAbilityReq (CONFIG_PKT *pkt)
{
  ABILITY_LIMITS temp;
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DbaseclassAbilityReq(IsStoring) not implemented");
    return;
  };
  pkt->status=CONFIG_DECODE_AbilityRequirement(
    pkt->value,
    pkt->errorMsg,
    &temp);
  if (pkt->status==CONFIG_STAT_ok)
        tempBASECLASS.AddAbilityRequirement(temp);
}

static void DbaseclassAllowRace (CONFIG_PKT *pkt)
{
  RACE_REFERENCE temp;
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DbaseclassAllowRace(IsStoring) not implemented");
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp.m_raceName);
  if (pkt->status==CONFIG_STAT_ok)
        tempBASECLASS.AddAllowedRace(temp);
}

static void DbaseclassExpLevel (CONFIG_PKT *pkt)
{
  long temp;
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DbaseclassExpLevel(IsStoring) not implemented\n");
    return;
  };
  pkt->status=CONFIG_DECODE_long(
    pkt->value,
    pkt->errorMsg,
    0,999999999,
    temp);
  if (pkt->status==CONFIG_STAT_ok)
    tempBASECLASS.AddExpLevel(temp);
        //tempBASECLASS.AddExpLevel(long2log(temp));
}


static void DbaseclassAllowAlign(CONFIG_PKT *pkt)
{
  static char alignments[9][16] =
  {
    "Lawful Good",
    "Neutral Good",
    "Chaotic Good",
    "Lawful Neutral",
    "True Neutral",
    "Chaotic Neutral",
    "Lawful Evil",
    "Neutral Evil",
    "Chaotic Evil"
  };
    
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DbaseclassAllowAlign(IsStoring) not implemented\n");
    return;
  };
  CString temp;
  pkt->status = CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp);
  if (pkt->status == CONFIG_STAT_ok)
  {
    for (int i=0; i<9; i++)
    {
      if (temp.CompareNoCase(alignments[i])==0)
      {
        if (tempBASECLASS.m_allowedAlignments&0x8000)
          tempBASECLASS.m_allowedAlignments=0;
        tempBASECLASS.m_allowedAlignments |= 1<<i;
        return;
      };
    };
    if (temp.CompareNoCase("any")==0)
    {
      tempBASECLASS.m_allowedAlignments=0x1ff;
    }
    else
    {
      pkt->status=CONFIG_STAT_value;
    };
  };
}


#undef df
#undef dh
#define df(a) Dbaseclass##a
#define dh(a) IDS_HELP_baseclass_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID baseclassIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,baseclassObject),
DEFID ("name"               ,string     ,Name),
DEFIDn("Prime Requisite"    ,string     ,PrimeReq ,9),
DEFID ("AbilityRequirement" ,AbilityReq ,AbilityReq),
DEFID ("Allowed Race"       ,string     ,AllowRace),
DEFID ("Allowed Alignment"  ,string     ,AllowAlign),
DEFID ("ExpLevel"           ,integer    ,ExpLevel),
CONFIGID( "",CONFIG_DATA_end,NULL)
};

int ReadBaseclassDB(CString &file, 
                    BASE_CLASS_DATA_TYPE *baseclassPointer,
                    CONFIG_FILE *pConfigFile)
{
  return ReadBaseclassDB((LPCSTR)file, baseclassPointer, pConfigFile);
}



int ReadBaseclassDB(LPCSTR file,
                    BASE_CLASS_DATA_TYPE *baseclassPointer,
                    CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// BaseClass database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old baseclassData
  baseclassPointer->Clear();
  // Read and store new baseclassData
  ASSERT(pBaseclassData==NULL); // Nobody should be using it.
  pBaseclassData=baseclassPointer;
  status=pConfigFile->DecodeFile(baseclassIDs, true);
  pBaseclassData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  baseclassPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteBaseclassDB(LPCSTR file, 
                     BASE_CLASS_DATA_TYPE *baseclassPointer, 
                     CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pBaseclassData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  baseclassPointer->CollapseDefaultFilenames();
  
  pBaseclassData=baseclassPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// BaseClass database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the races file!!!\r\n");
  status=pConfigFile->WriteFile(ar, baseclassIDs);

  // substitute default filenames for resources using "DEFAULT"
//  baseclassPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pBaseclassData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}


// ====================================================================
// 
//  Manipulater class database
//
// ====================================================================


CLASS_DATA tempCLASS; // Where we build class
CLASS_DATA_TYPE *pClassData=NULL;  // Where data will be put

// Handle clearing and storing of entire CLASS_DATA object
static void classObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempCLASS.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempCLASS.m_name.CompareNoCase("template")!=0)
    {
      if (pClassData->FindName(tempCLASS.m_name)!=classUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in class.txt = %s\n",tempCLASS.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pClassData->AddClass(tempCLASS);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pRaceData->GetHeadPosition();
    tempCLASS.Clear();
    tempCLASS.m_name="template";
    pkt->objectName=tempCLASS.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempCLASS=pClassData->GetAt(position);
      pClassData->GetNext(position);
      pkt->objectName=tempCLASS.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}

static void DclassName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempCLASS.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempCLASS.m_name);
  pkt->objectName=tempCLASS.m_name;
}

static void DclassBaseclass(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DclassBaseclass(IsStoring) not implemented");
    return;
  };
  BASECLASS_REFERENCE temp;
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp.m_baseclassName);
  if (pkt->status==CONFIG_STAT_ok)
  {
    tempCLASS.AddBaseclass(temp);
  };
}

static void DclassSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempCLASS.m_specialAbilities);
}




#undef df
#undef dh
#define df(a) Dclass##a
#define dh(a) IDS_HELP_class_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID classIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,classObject),
DEFID ("name"               ,string     ,Name),
DEFID ("Baseclass"          ,string     ,Baseclass),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
CONFIGID( "",CONFIG_DATA_end,NULL)
};

int ReadClassDB(CString &file, 
                CLASS_DATA_TYPE *classPointer,
                CONFIG_FILE *pConfigFile)
{
  return ReadClassDB((LPCSTR)file, classPointer, pConfigFile);
}


int ReadClassDB(LPCSTR file,
                CLASS_DATA_TYPE *classPointer,
                CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Class database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old classData
  classPointer->Clear();
  // Read and store new classData
  ASSERT(pClassData==NULL); // Nobody should be using it.
  pClassData=classPointer;
  status=pConfigFile->DecodeFile(classIDs, true);
  pClassData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  classPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteClassDB(LPCSTR file, 
                 CLASS_DATA_TYPE *classPointer, 
                 CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pAbilityData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  classPointer->CollapseDefaultFilenames();
  
  pClassData=classPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Class database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the classes file!!!\r\n");
  status=pConfigFile->WriteFile(ar, classIDs);

  // substitute default filenames for resources using "DEFAULT"
//  classPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pClassData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}

// ====================================================================
// 
//  Manipulate trait database
//
// ====================================================================



TRAIT_DATA tempTRAIT; // Where we build trait
TRAIT_DATA_TYPE *pTraitData=NULL;  // Where data will be put

// Handle clearing and storing of entire TRAIT_DATA object
static void traitObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempTRAIT.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempTRAIT.m_name.CompareNoCase("template")!=0)
    {
      if (pTraitData->FindName(tempTRAIT.m_name)!=traitUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in trait.txt = %s\n",tempTRAIT.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
      {
        tempTRAIT.m_roll.Compile();
        pTraitData->AddTrait(tempTRAIT);
      }
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pTraitData->GetHeadPosition();
    tempTRAIT.Clear();
    tempTRAIT.m_name="template";
    pkt->objectName=tempTRAIT.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempTRAIT=pTraitData->GetAt(position);
      pTraitData->GetNext(position);
      pkt->objectName=tempTRAIT.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}


static void DtraitName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempTRAIT.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempTRAIT.m_name);
  pkt->objectName=tempTRAIT.m_name;
}

static void DtraitAdjust(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DtraitAdjust(IsStoring) not implemented\n");
    return;
  };

  pkt->value.TrimLeft();
  pkt->value.TrimRight();
  pkt->value.Remove(' ');
  tempTRAIT.AddAdjust(pkt->value);

  /*
  ADJUSTMENTListType list;
  pkt->status=CONFIG_DECODE_adjust(
    pkt->value,
    pkt->errorMsg,
    list);
  if (pkt->status==CONFIG_STAT_ok) 
  {
    for (int i=0;i<list.GetSize();i++)
      tempTRAIT.AddAdjust(list[i]);
  }
  */
}

static void DtraitSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempTRAIT.m_specAbs);
}



#undef df
#undef dh
#define df(a) Dtrait##a
#define dh(a) IDS_HELP_trait_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID traitIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,traitObject),
DEFID ("name"               ,string     ,Name),
DEFID ("adjust"             ,adjust     ,Adjust),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
CONFIGID( "",CONFIG_DATA_end,NULL)
};



int ReadTraitDB(LPCSTR file,
                TRAIT_DATA_TYPE *traitPointer,
                CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Trait database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old traitData
  traitPointer->Clear();
  // Read and store new traitData
  ASSERT(pTraitData==NULL); // Nobody should be using it.
  pTraitData=traitPointer;
  status=pConfigFile->DecodeFile(traitIDs, true);
  pTraitData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  traitPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteTraitDB(LPCSTR file, 
                 TRAIT_DATA_TYPE *traitPointer, 
                 CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pTraitData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  traitPointer->CollapseDefaultFilenames();
  
  pTraitData=traitPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Trait database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the races file!!!\r\n");
  status=pConfigFile->WriteFile(ar, traitIDs);

  // substitute default filenames for resources using "DEFAULT"
//  traitPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internl error encoding and writing file");
    return -1;
  };
  pTraitData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}

// ====================================================================
// 
//  Manipulate spellgroup database
//
// ====================================================================



SPELLGROUP_DATA tempSPELLGROUP; // Where we build Spell Group
SPELLGROUP_DATA_TYPE *pSpellgroupData=NULL;  // Where data will be put

// Handle clearing and storing of entire SPELLGROUP_DATA object
static void spellgroupObject(CONFIG_PKT *pkt) {
  static POSITION position;
  switch (pkt->func)
  {
  case CONFIG_FUNC_beginObject:
    tempSPELLGROUP.Clear();
    break;
  case CONFIG_FUNC_storeObject:
    // don't add templates to database
    if (tempSPELLGROUP.m_name.CompareNoCase("template")!=0)
    {
      if (pSpellgroupData->FindName(tempSPELLGROUP.m_name)!=spellgroupUnknown)
      {
        CString msg;
        msg.Format("Duplicate name in spellgroups.txt = %s\n",tempSPELLGROUP.m_name);
        WriteDebugString(msg);
        pkt->status=CONFIG_STAT_key;
      }
      else
        pSpellgroupData->AddSpellgroup(tempSPELLGROUP);
    }
    break;
  case CONFIG_FUNC_getFirstObject:
    position=pSpellgroupData->GetHeadPosition();
    tempSPELLGROUP.Clear();
    tempSPELLGROUP.m_name="template";
    pkt->objectName=tempSPELLGROUP.m_name;
    break;
  case CONFIG_FUNC_getNextObject:
    if (position==NULL) {
      pkt->status=CONFIG_STAT_end;
    }
    else
    {
      tempSPELLGROUP=pSpellgroupData->GetAt(position);
      pSpellgroupData->GetNext(position);
      pkt->objectName=tempSPELLGROUP.m_name;
    };
    break;
  default:
    pkt->status=CONFIG_STAT_intError;
  };
}


static void DspellgroupName(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value=tempSPELLGROUP.m_name;
    return;
  };
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    tempSPELLGROUP.m_name);
  pkt->objectName=tempSPELLGROUP.m_name;
}

static void DspellgroupSpell(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DspellgroupSpell(IsStoring) not implemented\n");
    return;
  };
  CString temp;
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp);
  if (pkt->status==CONFIG_STAT_ok) tempSPELLGROUP.AddSpell(temp);
}

static void DspellgroupSpellgroup(CONFIG_PKT *pkt)
{
  if (pkt->IsStoring)
  {
    pkt->value = "";
    WriteDebugString("DspellgroupSpellgroup(IsStoring) not implemented\n");
    return;
  };
  CString temp;
  pkt->status=CONFIG_DECODE_string(
    pkt->value,
    pkt->errorMsg,
    temp);
  if (pkt->status==CONFIG_STAT_ok) tempSPELLGROUP.AddSpellgroup(temp);
}

static void DspellgroupSpecialAbility(CONFIG_PKT *pkt)
{
  ReadWriteSpecialAbilities(pkt, &tempSPELLGROUP.m_specAbs);
}



#undef df
#undef dh
#define df(a) Dspellgroup##a
#define dh(a) IDS_HELP_spellgroup_##a
// Defined above //  #define DEFID(a,b,c) CONFIGID(a,CONFIG_DATA_##b,df(c),dh(c))

CONFIGID spellgroupIDs[] =
{
CONFIGID ("",CONFIG_DATA_object,spellgroupObject),
DEFID ("name"               ,string     ,Name),
DEFID ("spell"              ,string     ,Spell),
DEFID ("spellgroup"         ,string     ,Spellgroup),
DEFID ("Special Ability"    ,string     ,SpecialAbility),
CONFIGID( "",CONFIG_DATA_end,NULL)
};



int ReadSpellgroupDB(LPCSTR file,
                     SPELLGROUP_DATA_TYPE *spellgroupPointer,
                     CONFIG_FILE *pConfigFile)
{
  // Returns  1 if minor errors.
  // Returns  0 if everything is perfect
  // Returns -1 if cannot read file.
  // Returns -2 if file appears to be wrong kind of file.
  // Returns -3 if internal program error occurs.
  CFile dataFile;
  CONFIG_FILE myConfigFile;
  CONFIG_ITEM_STATUS status;
  CString string;
  if (pConfigFile==NULL) pConfigFile = &myConfigFile;
  CArchive ar( &dataFile, CArchive::load);
  if (!dataFile.Open(file, CFile::modeRead)) return -1;
  ar.ReadString(string);
  if (string != "// Spell Group database file")
  {
    ar.Close();
    dataFile.Close();
    return -2;
  };
  pConfigFile->ReadFile(ar);
  ar.Close();
  dataFile.Close();
  // Now we begin the decode process
  // Get rid of old spellgroupData
  spellgroupPointer->Clear();
  // Read and store new spellgroupData
  ASSERT(pSpellgroupData==NULL); // Nobody should be using it.
  pSpellgroupData=spellgroupPointer;
  status=pConfigFile->DecodeFile(spellgroupIDs, true);
  pSpellgroupData=NULL; // We are done with it.
  // substitute default filenames for resources using "DEFAULT"
//  traitPointer->ExpandDefaultFilenames();
  if (status>=CONFIG_STAT_intError) return -3;
  if (status>CONFIG_STAT_ok) return 1;
  return 0;
}


int WriteSpellgroupDB(LPCSTR file, 
                      SPELLGROUP_DATA_TYPE *spellgroupPointer, 
                      CONFIG_FILE *pConfigFile)
{
  // Returns  0 if everything is perfect
  // Returns -1 if cannot write file.
  CFile dataFile;
  CString string;
  CONFIG_ITEM_STATUS status;
  SetFileAttributes(file, FILE_ATTRIBUTE_NORMAL);
  if (!dataFile.Open(file, CFile::modeCreate|CFile::modeWrite)) return -1;
  ASSERT (pSpellgroupData==NULL); // Nobody should be using it
  
  // substitute "DEFAULT" for all resources using default filenames
//  traitPointer->CollapseDefaultFilenames();
  
  pSpellgroupData=spellgroupPointer;
  CArchive ar( &dataFile, CArchive::store);
  ar.WriteString("// Spell Group database file\r\n");
  ar.WriteString("// The line above **must** be the first line of the Spell Group file!!!\r\n");
  status=pConfigFile->WriteFile(ar, spellgroupIDs);

  // substitute default filenames for resources using "DEFAULT"
//  traitPointer->ExpandDefaultFilenames();

  if (status==CONFIG_STAT_intError)
  {
    MsgBoxError("Internal error encoding and writing file");
    return -1;
  };
  pSpellgroupData=NULL; // We are done.
  ar.Close();
  dataFile.Close();
  return 0;
}



